c#########################################################################
c DISCUS Configuration file
c#########################################################################
c (1) Crystal dimensions
c
c	MAXSCAT : Maximum number of different atomtypes
c	NMAX    : Maximum number of atoms
c	MAXAT_CELL : Maximum number of atoms per unit cell
c
	integer*4	MAXSCAT,NMAX,MAXAT_CELL
c
	parameter	(NMAX       = 1000000)
	parameter	(MAXSCAT    = 25)
	parameter	(MAXAT_CELL = 100)
c
c#########################################################################
c (1b) Molecules
c
c	Currently the array sizes for molecules are set in the
c	file 'molecule.inc'.
c
c#########################################################################
c (2) Fourier transform
c
c	MAXQXY  : Maximum number of points in Q 
c	CFPKT   : Number of points in SIN(THETA/LAMBDA) lookup table
c	CFINC   : Increment for SIN(THETA/LAMBDA) table. The lookup
c	          table ranges from 0 to (CFPKT+1)*CFINC
c
	integer		MAXQXY,CFPKT
	real		CFINC
c
	parameter 	(MAXQXY = 301*301)
	parameter	(CFPKT  = 3999)
	parameter	(CFINC  = 0.001)
c
c#########################################################################
c (3) Reverse-Monte-Carlo
c
c	RMC_MAX_Q      : Maximum number of input data points. 
c                      : This is the value for all points of all planes,
c                      : the size of a single plane must be <= MAXQXY !
c
c	RMC_MAX_SQ     : Size of Fourier-Array (RMC_MAX_Q * SYM)
c
c	RMC_MAX_PLANES : Maximum number of input data planes
c	RMC_MAX_SYM    : Maximum number of symmetrically equivalent planes
c	RMC_MAX_ATOM   : Maximum number of atoms moved in one RMC move
c	RMC_MAX_LOTS   : Maximum number of 'lots' allowed
c
	integer		RMC_MAX_Q,RMC_MAX_PLANES,RMC_MAX_SYM
	integer		RMC_MAX_ATOM,RMC_MAX_LOTS,RMC_MAX_SQ
c
	parameter	(RMC_MAX_PLANES = 1)
	parameter	(RMC_MAX_SYM    = 1)
	parameter	(RMC_MAX_Q      = 5000)
	parameter	(RMC_MAX_SQ     = RMC_MAX_SYM*RMC_MAX_Q)
	parameter	(RMC_MAX_ATOM   = 10)
	parameter	(RMC_MAX_LOTS   = 1)
c
c#########################################################################
c (4) CHEM and MC 
c
c	Note that the array sizes defined here are used in the 
c	CHEM sublevel as well as the Monte-Carlo (MC, AMC) levels.
c
c	CHEM_MAX_VEC  : Maximum number of neighbour vector definitions
c	CHEM_MAX_RAN  : Maximum number of neighbour range definitions
c	CHEM_MAX_ANG  : Maximum number of neighbour angle definitions
c	CHEM_MAX_ENV  : Maximum number of neighbours in environment def.
c	CHEM_MAX_COR  : Maximum number of correlations
c	CHEM_MAX_BIN  : Maximum number of points for histograms
c	CHEM_MAX_ATOM : Maximum number of different atoms
c	CHEM_MAX_NEIG : Maximum number of neighbouring atoms/molecules
c	CHEM_MAX_CENT : Maximum number of central atoms
c
        integer         CHEM_MAX_VEC,CHEM_MAX_COR,CHEM_MAX_ANG
	integer		CHEM_MAX_RAN
	integer		CHEM_MAX_ENV
        integer         CHEM_MAX_BIN,CHEM_MAX_ATOM
	integer		CHEM_MAX_NEIG,MAX_ATOM_ENV
	integer		CHEM_MAX_CENT
c
        parameter       (CHEM_MAX_VEC  = 200 )
        parameter       (CHEM_MAX_RAN  = 200 )
        parameter       (CHEM_MAX_ANG  = 200 )
        parameter       (CHEM_MAX_ENV  = 5000)
        parameter       (CHEM_MAX_COR  = 200 )
        parameter       (CHEM_MAX_BIN  = 1000)
        parameter       (CHEM_MAX_ATOM = 20  )
c	Warning!	MAXPAR_RES  in "param.inc" must always be of 
c	                            identical size to
c			CHEM_MAX_NEIG in "config.inc"
        parameter       (CHEM_MAX_NEIG = 5000)
 	parameter  	(MAX_ATOM_ENV  = CHEM_MAX_NEIG)
	parameter	(CHEM_MAX_CENT = 200 )
c
c#########################################################################
c (5) Microdomains
c
c	MMAX         : Maximum number of atoms within a microdomain
c	MK_MAX_SCAT  : Maximum number of atom types within a microdomain
c	MC_MAX_MICRO : Maximum number of micro domains within the crystal
c
	integer		MMAX,MK_MAX_SCAT,MC_MAX_MICRO
c
	parameter	(MMAX         = 10000)
	parameter	(MK_MAX_SCAT  = MAXSCAT)
	parameter	(MC_MAX_MICRO = NMAX)
c
c#########################################################################
c (6) Stacking faults
c
c	ST_MAXQXY    : Maximum number of points of Fourier (<= MAXQXY !)
c	ST_MAXLAYER  : Maximum number of layers
c	ST_MAXTYPE   : Maximum number of layer types
c
	integer		ST_MAXQXY,ST_MAXLAYER,ST_MAXTYPE
	integer		ST_MMAX,ST_MAX_SCAT
c
	parameter	(ST_MAXQXY    = MAXQXY)
	parameter	(ST_MAXLAYER  = 5000)
	parameter	(ST_MAXTYPE   = 6)
	parameter	(ST_MMAX      = 1000)
	parameter	(ST_MAX_SCAT  = MAXSCAT)
c
c#########################################################################
c (7) Pair distribution function
c
c	MAXDAT       : Maximum number of points in PDF
c	MAXBND       : Maximum extend of periodic boundary array
c 
	integer		MAXDAT,MAXBND
c
	parameter	(MAXDAT = 2500)
	parameter	(MAXBND = 1000)
c
c#########################################################################
c (8) Powder module
c
c	POW_MAXPKT   : Maximum number of points in powder pattern
c
	integer        POW_MAXPKT
c
	parameter      (POW_MAXPKT=100000)
