	subroutine fourier
c+
c	This subroutine 'fourier' calculates the Fourier transform
c	of the given crystal structure. The algorithm to speed up
c	the explicite Fourier is based on the program 'DIFFUSE' by
c	B.D. Butler. See also: B.D. Butler & T.R. Welberry, (1992).
c	J. Appl. Cryst. 25, 391-399. 
c
c	Version : 1.02
c	Date    : 12 January 1998
c
c	Authors : Th. Proffen (tproffen@lanl.gov)
c	          R.B. Neder  (reinhard.neder@mail.uni-wuerzburg.de)
c
c-                  
	implicit        none
c
	include        'config.inc'
	include        'crystal.inc'
	include        'crystal.cmm'
	include        'doact.inc'
	include        'learn.inc'
	include        'macro.inc'
	include        'output.inc'
	include	       'diffuse.inc'
	include        'prompt.inc'
	include        'errlist.inc'
c
	integer         maxw
	parameter      (maxw=MAXSCAT)
	logical         lold
	parameter      (lold=.false.)
c
	character*5     befehl
	character*11    prom
	character*200   zeile,cpara(maxw)
	character*200   line
	integer         lpara(maxw)
	integer         i,j,k,ianz,lp,length
	integer         indxg,lbef
	integer         jj(maxw)
	real            werte(maxw),divis(2)
c
	logical         str_comp
c
10	continue
c
	call no_error
	divis(1) = float(max(1,inc(1)-1))
	divis(2) = float(max(1,inc(2)-1))
c
	prom=prompt//'/fourier'
	call get_command (line,length,befehl,lbef,zeile,lp,prom)
	if(ier_num.eq.0) then
	if(line(1:1).eq.' ' .or. line(1:1).eq.'#')  goto 10
c
c------	search for "="
c
	indxg=index(line,'=')
	if(indxg.ne.0 .and. .not.(str_comp(befehl,'echo',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'syst',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                          str_comp(befehl,'?   ',2,lbef,4)) )then
c
c------	--evaluatean expression and assign the value to a variabble
c
	  call do_math(line,indxg,length)
	else
c
c------ execute a macro file
c
        if(befehl(1:1).eq.'@') then
	  if(length.ge.2) then
	    call file_kdo(line(2:length),length-1)
	  else
	    ier_num = -13
	    ier_typ = ER_MAC
	  endif
c
c------	Define the ascissa 'absc'
c
	elseif(str_comp(befehl,'absc',1,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ianz.eq.1) then
            if(cpara(1).eq.'h') then
              extr_abs=1
            elseif(cpara(1).eq.'k') then
              extr_abs=2
            elseif(cpara(1).eq.'l') then
              extr_abs=3
            else
              ier_num = -6
	      ier_typ = ER_COMM
            endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------	continues a macro 'continue'
c
	elseif(str_comp(befehl,'continue',1,lbef,8)) then
	  call macro_continue(zeile,lp)
c
c------	define the anomalous scattering curve for an element 'delf'
c
	elseif(str_comp(befehl,'delf',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ianz.eq.3 ) then
	    i = 1
	    call get_iscat(i,cpara,lpara,werte,maxw,lold)
	    if(ier_num.eq.0) then
	      if(werte(1).gt.0) then
	        do k=1,i
	          jj(k) = nint(werte(1))
	        enddo
	        cpara(1) = '0.0'
	        lpara(1) = 3
                call ber_params(ianz,cpara,lpara,werte,maxw)
                if(ier_num.eq.0) then
	          do k=1,i
	            cr_delfr(jj(k))    = werte(2)
	            cr_delfi(jj(k))    = werte(3)
	            cr_delf_int(jj(k)) = .false.
	          enddo
	        endif
	      else
	        ier_num = -27
	        ier_typ = ER_APPL
	      endif
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------	Switch dispersion on/off 'disp' second parameter 'anom' or 'off'
c
	elseif(str_comp(befehl,'disp',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	  if(ier_num.eq.0) then
	    if(ianz.eq.1) then
	      if    (cpara(1)(1:1).eq.'a') then
	        ano=.true.
	      elseif(cpara(1)(1:1).eq.'o') then
	        ano=.false.
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------ Echo a string, just for interactive check in a macro 'echo'
c
	elseif(str_comp(befehl,'echo',2,lbef,4)) then
	  call echo(zeile,lp)
c
c------	 Evaluate an expression, just for interactive check 'eval'
c
	elseif(str_comp(befehl,'eval',2,lbef,4)) then
	  call do_eval(zeile,lp)
c
c------	Terminate Fourier 'exit'
c
	elseif(str_comp(befehl,'exit',3,lbef,4)) then
	  goto 9999
c
c------	help 'help' , '?'
c
	elseif(str_comp(befehl,'help',2,lbef,4) .or.
     &	       str_comp(befehl,'?   ',1,lbef,4)     ) then
	  if(str_comp(zeile,'errors',2,lp,6) ) then
	    lp = lp + 7
	    call do_hel('discus '//zeile,lp)
	  else
	    lp = lp+12
	    call do_hel('discus four '//zeile,lp)
	  endif
c
c------	define the whole layer 'laye'
c
	elseif(str_comp(befehl,'laye',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ier_num.eq.0) then
            if(ianz.eq.11) then
              call ber_params(ianz,cpara,lpara,werte,maxw)
              if(ier_num.eq.0) then
	        do j=1,3
	          do i=1,3
	            eck(i,j)=werte((j-1)*3+i)
	          enddo
	        enddo
	        inc(1)=nint(werte(10))
	        inc(2)=nint(werte(11))
	        divis(1) = float(max(1,inc(1)-1))
	        divis(2) = float(max(1,inc(2)-1))
	        do i=1,3              
	          vi(i,1)=(eck(i,2)-eck(i,1))/divis(1)
	          vi(i,2)=(eck(i,3)-eck(i,1))/divis(2)
	        enddo
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	define the corners 'll', 'lr', 'ul'
c
	elseif(str_comp(befehl,'ll  ',2,lbef,4) .or.
     &	       str_comp(befehl,'lr  ',2,lbef,4) .or.
     &	       str_comp(befehl,'ul  ',2,lbef,4)     ) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ier_num.eq.0) then
            if(ianz.eq.3) then
              call ber_params(ianz,cpara,lpara,werte,maxw)
              if(ier_num.eq.0) then
	        if (str_comp(befehl,'ll  ',2,lbef,4)) j=1
	        if (str_comp(befehl,'lr  ',2,lbef,4)) j=2
	        if (str_comp(befehl,'ul  ',2,lbef,4)) j=3
	        do i=1,3
	          eck(i,j)=werte(i)
	        enddo
	        do i=1,3              
	          vi(i,1)=(eck(i,2)-eck(i,1))/divis(1)
	          vi(i,2)=(eck(i,3)-eck(i,1))/divis(2)
	        enddo
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	set Fourier volume (lots)
c
	elseif(str_comp(befehl,'lots',3,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.eq.0) then
	    if (ianz.eq.1) then
	      call do_cap (cpara(1))
	      if (cpara(1)(1:1).eq.'O') then
	        ilots = LOT_OFF
	        nlots = 1
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
	    elseif (ianz.eq.6) then
	      call do_cap (cpara(1))
	      if (cpara(1)(1:1).eq.'B') then
	        ilots = LOT_BOX
	      elseif (cpara(1)(1:1).eq.'E') then
	        ilots = LOT_ELI
	      else
	        ier_num = -2
	        ier_typ = ER_FOUR
	      endif
	      if (ier_num.eq.0) then
	        call del_params (1,ianz,cpara,lpara,maxw)
	        call ber_params(ianz-1,cpara,lpara,werte,maxw)
	        if (ier_num.eq.0) then
	          ls_xyz(1) = nint(werte(1))
	          ls_xyz(2) = nint(werte(2))
	          ls_xyz(3) = nint(werte(3))
	          nlots     = nint(werte(4))
	          call do_cap(cpara(5))
	          lperiod = (cpara(5)(1:1).eq.'Y')
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	define the number of points along the abscissa 'na'
c
	elseif(str_comp(befehl,'nabs',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ier_num.eq.0) then
            if(ianz.eq.1) then
              call ber_params(ianz,cpara,lpara,werte,maxw)
              if(ier_num.eq.0) then
	        if(werte(1).gt.0) then
	          inc(1)   = nint(werte(1))
	          divis(1) = float(max(1,inc(1)-1))
	          do i=1,3              
	            vi(i,1)=(eck(i,2)-eck(i,1))/divis(1)
	            vi(i,2)=(eck(i,3)-eck(i,1))/divis(2)
	          enddo
	        else
	          ier_num = -12
	          ier_typ = ER_APPL
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	switch to neutron diffraction 'neut'
c
	elseif(str_comp(befehl,'neut',2,lbef,4)) then
	  lxray = .false.
c
c------	define the number of points along the ordinate 'no'
c
	elseif(str_comp(befehl,'nord',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ier_num.eq.0) then
            if(ianz.eq.1) then
              call ber_params(ianz,cpara,lpara,werte,maxw)
              if(ier_num.eq.0) then
	        if(werte(1).gt.0) then
	          inc(2)   = nint(werte(1))
	          divis(2) = float(max(1,inc(2)-1))
	          do i=1,3              
	            vi(i,1)=(eck(i,2)-eck(i,1))/divis(1)
	            vi(i,2)=(eck(i,3)-eck(i,1))/divis(2)
	          enddo
	        else
	          ier_num = -12
	          ier_typ = ER_APPL
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	define the ordinate  'ordi'
c
	elseif(str_comp(befehl,'ordi',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ianz.eq.1) then
            if(cpara(1).eq.'h') then
              extr_ord=1
            elseif(cpara(1).eq.'k') then
              extr_ord=2
            elseif(cpara(1).eq.'l') then
              extr_ord=3
            else
              ier_num = -6
	      ier_typ = ER_COMM
            endif
	  endif
c
c------	start the Fourier transform 'run'
c
	elseif(str_comp(befehl,'run ',1,lbef,4)) then
	  if(inc(1)*inc(2).le.MAXQXY) then
	    call dlink(lxray,ano,lambda,rlambda)
	    if(ier_num.eq.0) then
	      four_log = .true.
	      call four_run
	    endif
	  else
	    ier_num = -8
	    ier_typ = ER_APPL
	  endif
c
c------	define the scattering curve for an element 'scat'
c
	elseif(str_comp(befehl,'scat',2,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ianz.eq.10 ) then
	    i = 1
	    call get_iscat(i,cpara,lpara,werte,maxw,lold)
	    if(ier_num.eq.0) then
	      if(werte(1).gt.0) then
	        do k=1,i
	          jj(k) = nint(werte(1))
	        enddo
	        cpara(1) = '0.0'
	        lpara(1) = 3
                call ber_params(ianz,cpara,lpara,werte,maxw)
                if(ier_num.eq.0) then
	          do k=1,i
	            do i=2,9
	              cr_scat(i,jj(k)) = werte(i)
	            enddo
	            cr_scat(1,jj(k))   = werte(10)
	            cr_scat_int(jj(k)) = .false.
	          enddo
	        endif
	      else
	        ier_num = -27
	        ier_typ = ER_APPL
	      endif
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------	set desired mode of Fourier transform 'set'
c
	elseif(str_comp(befehl,'set',2,lbef,3)) then
	  call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.eq.0) then
	    if (ianz.ge.1 .and. ianz.le.2) then
	      if(str_comp(cpara(1),'aver',1,lpara(1),4)) then
	        if (ianz.eq.1) then
	          fave = 0.0
	        else
	          call del_params (1,ianz,cpara,lpara,maxw)
	          call ber_params (ianz,cpara,lpara,werte,maxw)
	          if (ier_num.eq.0) then
	            if (werte(1).ge.0.0 .and. werte(1).le.100.0) then
	              fave = werte(1)*0.01
	            else
	              ier_num = -1
	              ier_typ = ER_FOUR
	            endif
	          endif
	        endif
	      else
	        ier_num = -1
	        ier_typ = ER_FOUR
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------	Show the current settings for the Fourier 'show'
c
	elseif(str_comp(befehl,'show',2,lbef,4)) then
	  call dlink(lxray,ano,lambda,rlambda)
	  call four_show
c
c------	Switch usage of temperature coefficients on/off 'temp'
c
	elseif(str_comp(befehl,'temp',1,lbef,4)) then
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
          if(ier_num.eq.0) then
	    if(ianz.eq.1) then
	      if(cpara(1)(1:2).eq.'ig') then
	        ldbw=.false.
	      elseif(cpara(1)(1:2).eq.'us') then
	        ldbw=.true.
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c-------Operating System Kommandos 'syst'
c
	elseif(str_comp(befehl,'syst',2,lbef,4)) then
	  if(zeile.ne. ' ') then
	    call do_operating(zeile(1:lp),lp)
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ waiting for user input
c
	elseif(str_comp(befehl,'wait',3,lbef,4)) then
	  call do_input(zeile,lp)
c
c------	set the wave length to be used 'wvle'
c
	elseif(str_comp(befehl,'wvle',1,lbef,4)) then
	  call do_cap (zeile)
          call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	  if(ianz.eq.1) then
	    if(ichar('A') .le. ichar(cpara(1)(1:1)) .and.
     &	       ichar(cpara(1)(1:1)) .le. ichar('Z')      ) then
	      lambda = cpara(1)
	    else
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      rlambda=werte(1)
	      lambda=' '
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------	switch to x-ray diffraction 'xray'
c
	elseif(str_comp(befehl,'xray',1,lbef,4)) then
	  lxray = .true.
	else
	  ier_num = -8
	  ier_typ = ER_COMM
	endif
	endif
	endif
c
c------ end of command list
c
	if(ier_num.ne.0) then
	  call errlist
	  if(lmakro) then
	    call macro_close
	  endif
	  if(lblock) then
	    ier_num = -11
	    ier_typ = ER_COMM
	    return
	  endif
	  call no_error
	endif
	goto 10
9999	continue
2000	format(a)
c
      end
c*****7*****************************************************************
	subroutine four_run
c+
c	Main routine to calculate the Fourier transform for the 
c	given plane in reciprocal space. Based on the program
c	'diffuse' written by B.D. Butler.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
	include		'errlist.inc'
c
	real		ss,seknds,dnorm
	integer		lbeg(3),csize(3)
	integer		iscat,nlot,ncell,i
c
	ier_num = 0
c
c------ preset some values
c
	ss = seknds(0.0)
	call four_layer
c
c------ zero some arrays
c
	do i=1,num(1)*num(2)
	   csf(i) = cmplx(0.0d0,0.0d0)
	  acsf(i) = cmplx(0.0d0,0.0d0)
	   dsi(i) = 0.0d0
	enddo
c
c------ preset some tables, calculate average structure
c
	call four_cexpt
	call four_stltab
	if (ier_num.ne.0) return
	call four_formtab
	call four_csize (cr_icc,csize,lperiod,ls_xyz)
	call four_aver (ilots,fave)
c
c------ loop over crystal regions (lots)
c
	do nlot=1,nlots
	  do i=1,num(1)*num(2)
	    csf(i) = cmplx(0.0d0,0.0d0)
	  enddo
c
	  call four_ranloc (csize,lbeg)
	  if(four_log) then
	    if (ilots.ne.LOT_OFF) then
  	      write (output_io,2000) nlot,nlots,(lbeg(i),i=1,3)
	    else
  	      write (output_io,2010) nlot,nlots
	    endif
	  endif
c
c------ - loop over all different atom types
c
	  do iscat=1,cr_nscat
	    call four_getatm (iscat,ilots,lbeg,csize,ncell)
	    call four_strucf (iscat,.true.)
c
c------ --- Add this part of the structur factor to the total
c
	    do i=1,num(1)*num(2)
	      csf(i) = csf(i) + tcsf(i)
	    enddo
	    if(four_log) then
	      write (output_io,3000) cr_at_lis(iscat),nxat
	    endif
	  enddo
c
c------ - subtract average structure factor, add intensity
c
	  do i=1,num(1)*num(2)
	    csf(i) = csf(i) - acsf(i)
	    dsi(i) = dsi(i) + real(csf(i)*conjg(csf(i)))
	  enddo
	enddo
c
c------ if we had lots, normalise the intensities
c
	if (nlots.ne.1) then
	  dnorm = float(cr_icc(1)*cr_icc(2)*cr_icc(3)) / 
     &	          float(ncell*nlots)
	  dnorm = dnorm**2
	  do i=1,num(1)*num(2)
	    dsi(i)=dnorm*dsi(i)
	  enddo
	endif
c
	call four_qinfo
	ss = seknds(ss)
	if(four_log) then
	  write (output_io,4000) ss
	endif
c
2000	format (/,' Lot # ',I4,'/',I4,' : starting at unit cell (',
     &	            3(I3,1X),')')
2010	format (/,' Lot # ',I4,'/',I4,' : complete crystal')
3000	format (  '                   Atom typ = ',A4,7X,'(# ',I9,' )') 
4000	format (/,' Elapsed time    : ',G12.6,' sec')
	end
c*****7*****************************************************************
	subroutine four_strucf (iscat,lform)
c+
c	Here the complex structure factor of 'nxat' identical atoms
c	from array 'xat' is computed.
c-
	implicit	none
c
	include		'config.inc'
	include		'diffuse.inc'
c
	real*8		xarg0,xincu,xincv
	integer		iscat
	integer		i,ii,j,k,iarg,iarg0,iincu,iincv,iadd
	logical		lform
c
	integer		IAND,ISHFT
c
c------ zero fourier array
c
	do i=1,num(1)*num(2)
	  tcsf(i)=cmplx(0.0d0,0.0d0)
	enddo
c
c------ Loop over all atoms in 'xat'
c
	do k=1,nxat
	  xarg0 =  xm(1)*xat(k,1)+ xm(2)*xat(k,2)+ xm(3)*xat(k,3)
	  xincu = uin(1)*xat(k,1)+uin(2)*xat(k,2)+uin(3)*xat(k,3)
	  xincv = vin(1)*xat(k,1)+vin(2)*xat(k,2)+vin(3)*xat(k,3)
c
	  iarg0 = nint( 64*I2PI*( xarg0-int(xarg0)+1.0d0))
	  iincu = nint( 64*I2PI*( xincu-int(xincu)+1.0d0))
	  iincv = nint( 64*I2PI*( xincv-int(xincv)+1.0d0))
	  iarg  = iarg0
c
c------ - Loop over all points in Q. 'iadd' is the address of the
c------ - complex exponent table. 'IADD' divides out the 64 and
c------ - ISHFT acts as MOD so that the argument stays in the table
c------ - boundaries.
c
	  ii = 0
c
	  do j=1,num(1)
	    do i=1,num(2)
	      iadd = ISHFT(iarg,-6)
	      iadd = IAND (iadd,MASK)
	      ii = ii + 1
	      tcsf(ii) = tcsf(ii) + cex(iadd)
	      iarg = iarg + iincv
	    enddo
	    iarg = iarg0 + iincu*j
	  enddo  
	enddo
c
c------ Now we multiply with formfactor
c
	if (lform) then
	  do i=1,num(1)*num(2)
	    tcsf(i) = tcsf(i)*cfact(istl(i),iscat)
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine four_aver (lots,ave)
c+
c	This routine calculates the average structure factor <F>.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
	include		'random.inc'
c
	real		ran1,norm,ave
	integer		isite,iatom,iscat,icell(3)
	integer		lots,scell,ncell,j,ii,jj,kk
	logical		sel
c
	if (ave.eq.0.0) then
	  return
	else
	  if(four_log) then
	    write (output_io,1000) 100.0*ave
	  endif
	  scell = cr_icc(1)*cr_icc(2)*cr_icc(3)
	  ncell = 0
c
c------ - Loop over all unit cells
c
	  do ii=1,cr_icc(1)
	  do jj=1,cr_icc(2)
	  do kk=1,cr_icc(3)
	    icell(1)=ii
	    icell(2)=jj
	    icell(3)=kk
c
c------ --- get only 'ave'% of those unit cells and compute average
c------ --- unit cell
c
	    sel = (ran1(idum).le.ave)
	    if (sel) then
	      ncell = ncell + 1
c
c------ ----- Loop over all atom types
c
	      do iscat=1,cr_nscat
	        nxat=0
c
c------ ------- Loop over all sites within unit cell
c
	        do isite=1,cr_ncatoms
	          call celltoindex (icell,isite,iatom)
	          if (cr_iscat(iatom).eq.iscat) then
	            nxat=nxat+1
	            do j=1,3
	              xat(nxat,j) = cr_pos(j,iatom)-
     &	                            float(icell(j)-1)-cr_dim0(j,1)
	            enddo
	          endif
	        enddo
	        call four_strucf (iscat,.true.)
	        do j=1,num(1)*num(2)
	          acsf(j) = acsf(j) + tcsf(j)
	        enddo
	      enddo
	    endif
	  enddo
	  enddo
	  enddo
c
c------ - now compute the interference function of the lot shape
c
	  call four_getav (lots)
	  call four_strucf (0,.false.)
	  norm = 1.0/ncell
	  do j=1,num(1)*num(2)
	    acsf(j) = acsf(j)*tcsf(j)*cmplx(dble(norm),0.0d0)
	  enddo
c
c------ - write how much of the crystal we actually used
c
	  if(four_log) then
	    write (output_io,2000) (float(ncell)/float(scell))*100.0
	  endif
c
	endif
c
1000	format (' Calculating <F> with ',F5.1,' % of the crystal ...')
2000	format (' Used ',F5.1,' % of the crystal to calulate <F> ...')
	end
c*****7*****************************************************************
	subroutine four_getatm (iscat,lots,lbeg,csize,ncell)
c+
c	This routine creates an atom list of atoms of type 'iscat'
c	which are within the current lot.
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
c
	real		offset(3)
	real		x0(3),xtest(3)
	integer*4	cr_end
	integer		lbeg(3),csize(3),icell(3),jcell(3)
	integer		ncell,iscat,lots,i,j,ir,ii,jj,kk,is,ia
c
	nxat   = 0
	ncell  = 0
	cr_end = cr_ncatoms*cr_icc(1)*cr_icc(2)*cr_icc(3) + 1
c
c------ No lots required, return all atoms of type 'iscat'
c
	if (lots.eq.LOT_OFF) then
	  ncell = cr_icc(1)*cr_icc(2)*cr_icc(3)
	  do i=1,cr_natoms
	    if (cr_iscat(i).eq.iscat) then
	      nxat=nxat+1
	      do j=1,3
	        xat(nxat,j)=cr_pos(j,i)
	      enddo
	    endif
	  enddo
c
c------ Box shaped lot
c
	elseif (lots.eq.LOT_BOX) then
	  do kk=0,ls_xyz(3)-1
	    icell(3)=kk+lbeg(3)
	    offset(3)=cr_dim0(3,1)+lbeg(3)-1
	    if (icell(3).gt.cr_icc(3)) then
	      icell(3)=icell(3)-cr_icc(3)
	      offset(3)=offset(3)-float(cr_icc(3))
	    endif
	    do jj=0,ls_xyz(2)-1
	      icell(2)=jj+lbeg(2)
	      offset(2)=cr_dim0(2,1)+lbeg(2)-1
	      if (icell(2).gt.cr_icc(2)) then
	        icell(2)=icell(2)-cr_icc(2)
	        offset(2)=offset(2)-float(cr_icc(2))
	      endif
	      do ii=0,ls_xyz(1)-1
	        icell(1)=ii+lbeg(1)
	        offset(1)=cr_dim0(1,1)+lbeg(1)-1
	        if (icell(1).gt.cr_icc(1)) then
	          icell(1)=icell(1)-cr_icc(1)
	          offset(1)=offset(1)-float(cr_icc(1))
	        endif
c
	        ncell=ncell+1
	        do is=1,cr_ncatoms
	          call celltoindex (icell,is,ia)
	          if (cr_iscat(ia).eq.iscat) then
	            nxat=nxat+1
	            do j=1,3
	              xat(nxat,j)=cr_pos(j,ia)-offset(j)
	            enddo
	          endif
	        enddo
c
	        do ir=cr_end,cr_natoms
	          do j=1,3	
	            jcell(j)=icell(j)+nint(cr_dim(j,1))-1
	          enddo
	          if (int(cr_pos(1,ir)).eq.jcell(1) .and.
     &	              int(cr_pos(2,ir)).eq.jcell(2) .and.
     &	              int(cr_pos(3,ir)).eq.jcell(3) .and.
     &	              cr_iscat(ir).eq.iscat              ) then
	            nxat=nxat+1
	            do j=1,3
	              xat(nxat,j)=cr_pos(j,ir)-offset(j)
	            enddo
	          endif
	        enddo
c
	      enddo
	    enddo
	  enddo
c
c------ Ellipsoid shaped lot
c
	elseif (lots.eq.LOT_ELI) then
	  do ii=1,3
	    x0(ii)=float(ls_xyz(ii))/2.0
	  enddo
c
	  do kk=0,ls_xyz(3)-1
	    icell(3)=kk+lbeg(3)
	    offset(3)=cr_dim0(3,1)+lbeg(3)-1
	    xtest(3)=(float(kk)-x0(3)+0.5)**2/x0(3)**2
	    if (icell(3).gt.cr_icc(3)) then
	      icell(3)=icell(3)-cr_icc(3)
	      offset(3)=offset(3)-float(cr_icc(3))
	    endif
	    do jj=0,ls_xyz(2)-1
	      icell(2)=jj+lbeg(2)
	      offset(2)=cr_dim0(2,1)+lbeg(2)-1
	      xtest(2)=(float(jj)-x0(2)+0.5)**2/x0(2)**2
	      if (icell(2).gt.cr_icc(2)) then
	        icell(2)=icell(2)-cr_icc(2)
	        offset(2)=offset(2)-float(cr_icc(2))
	      endif
	      do ii=0,ls_xyz(1)-1
	        icell(1)=ii+lbeg(1)
	        offset(1)=cr_dim0(1,1)+lbeg(1)-1
	        xtest(1)=(float(ii)-x0(1)+0.5)**2/x0(1)**2
	        if (icell(1).gt.cr_icc(1)) then
	          icell(1)=icell(1)-cr_icc(1)
	          offset(1)=offset(1)-float(cr_icc(1))
	        endif
c
	        if ((xtest(1)+xtest(2)+xtest(3)).le.1.0) then
	          ncell=ncell+1
	          do is=1,cr_ncatoms
	            call celltoindex (icell,is,ia)
	            if (cr_iscat(ia).eq.iscat) then
	              nxat=nxat+1
	              do j=1,3
	                xat(nxat,j)=cr_pos(j,ia)-offset(j)
	              enddo
	            endif
	          enddo
c
	          do ir=cr_end,cr_natoms
	            do j=1,3	
	              jcell(j)=icell(j)+nint(cr_dim(j,1))-1
	            enddo
	            if (int(cr_pos(1,ir)).eq.jcell(1) .and.
     &	                int(cr_pos(2,ir)).eq.jcell(2) .and.
     &	                int(cr_pos(3,ir)).eq.jcell(3) .and.
     &	                cr_iscat(ir).eq.iscat              ) then
	              nxat=nxat+1
	              do j=1,3
	                xat(nxat,j)=cr_pos(j,ir)-offset(j)
	              enddo
	            endif
	          enddo
	        endif
c
	      enddo
	    enddo
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine four_getav (lots)
c+
c	This routine computes the lattice inside the lot
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
c
	real		xtest(3),x0(3)
	integer		lots,ii,jj,kk
c
	nxat=0
c
c------ No lots required, return complete lattice
c
	if (lots.eq.LOT_OFF) then
	  do ii=0,cr_icc(1)-1
	    do jj=0,cr_icc(2)-1
	      do kk=0,cr_icc(3)-1
	        nxat=nxat+1
	        xat(nxat,1)=float(ii)+cr_dim0(1,1)
	        xat(nxat,2)=float(jj)+cr_dim0(2,1)
	        xat(nxat,3)=float(kk)+cr_dim0(3,1)
	      enddo
	    enddo
	  enddo
c
c------ Box shaped lot
c
	elseif (lots.eq.LOT_BOX) then
	  do kk=0,ls_xyz(3)-1
	    do jj=0,ls_xyz(2)-1
	      do ii=0,ls_xyz(1)-1
	        nxat=nxat+1
	        xat(nxat,1)=float(ii)
	        xat(nxat,2)=float(jj)
	        xat(nxat,3)=float(kk)
	      enddo
	    enddo
	  enddo
c
c------ Ellipsoid shaped lot
c
	elseif (lots.eq.LOT_ELI) then
	  do ii=1,3
	    x0(ii)=float(ls_xyz(ii))/2.0
	  enddo
c
	  do kk=0,ls_xyz(3)-1
	    xtest(3)=(float(kk)-x0(3)+0.5)**2/x0(3)**2
	    do jj=0,ls_xyz(2)-1
	      xtest(2)=(float(jj)-x0(2)+0.5)**2/x0(2)**2
	      do ii=0,ls_xyz(1)-1
	        xtest(1)=(float(ii)-x0(1)+0.5)**2/x0(1)**2
	        if ((xtest(1)+xtest(2)+xtest(3)).le.1.0) then
	          nxat=nxat+1
	          xat(nxat,1)=float(ii)
	          xat(nxat,2)=float(jj)
	          xat(nxat,3)=float(kk)
	        endif
	      enddo
	    enddo
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine four_ranloc (csize,lbeg)
c+
c	Returns a random cell from within the simulated crystal
c	which is limited by 'csize'.
c-
	implicit	none
c
	include		'random.inc'
c
	integer		csize(3),lbeg(3)
	real		ran1
c
	lbeg(1) = int(ran1(idum)*csize(1)) +1
	lbeg(2) = int(ran1(idum)*csize(2)) +1
	lbeg(3) = int(ran1(idum)*csize(3)) +1
c
	end
c*****7*****************************************************************
	subroutine four_csize (cr_icc,csize,lperiod,ls_xyz)
c+
c	Limits crystal size in case of no periodic boundary 
c	conditions.
c-
	implicit	none
c
	integer		cr_icc(3),csize(3),ls_xyz(3)
	logical		lperiod
c
	if (lperiod) then
	  csize(1) = cr_icc(1)
	  csize(2) = cr_icc(2)
	  csize(3) = cr_icc(3)
	else
	  csize(1) = cr_icc(1) - ls_xyz(1)
	  csize(2) = cr_icc(2) - ls_xyz(2)
	  csize(3) = cr_icc(3) - ls_xyz(3)
	endif
c
	end
c*****7*****************************************************************
	subroutine four_layer
c+
c	This routine writes the corners of the plane to be calculated
c	in the 'diffuse.inc' variables.
c-
	implicit	none
c
	include		'config.inc'
	include		'diffuse.inc'
c
	integer		i
c
	do i=1,3
	  xm (i) = eck(i,1)
	  uin(i) = vi (i,1)
	  vin(i) = vi (i,2)
	enddo
c
	do i=1,2
	  num(i) = inc(i)
	enddo
c
	end
c*****7*****************************************************************
	subroutine four_cexpt
c+
c	This routine initialises the complex exponent table and
c	is called only at the first Fourier run.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'diffuse.inc'
c
	real*8		twopi,xmult,xarg,xt
	integer		i
c
	if (.not. ffour) then
	  write (output_io,1000)
c
	  xt    = 1.0d0 / float(I2PI)
	  twopi = 8.0d0 * datan(1.0d0)
c
	  do i=0,MASK
	    xmult  = float(i)*xt
	    xarg   = twopi*xmult
	    cex(i) = cmplx(cos(xarg),sin(xarg))
	  enddo
	  ffour = .true.
	endif
c
1000	format (' Computing complex exponent table ...')
	end
c*****7*****************************************************************
	subroutine four_stltab
c+
c	Sets up an integer array containing the corresponding integers
c	to the formfactor table for each sin(theta)/lambda.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
	include		'errlist.inc'
c
	real		quad,q2,h(3)
	integer		i,j,k
c
	if(four_log) then
	  write (output_io,1000)
	endif
c
	do j=1,num(2)
	  do i=1,num(1)
	    do k=1,3
	      h(k)=xm(k) + uin(k)*float(i-1) + vin(k)*float(j-1)
	    enddo
	    q2=quad(h,h,cr_rten)/4.0
	    k=(i-1)*num(2)+j
	    istl(k)=nint(sqrt(q2)*(1.0/CFINC))
c
	    if (istl(k).gt.CFPKT) then
	      ier_num = -3
	      ier_typ = ER_FOUR
	      return
	    endif
c
	  enddo
	enddo
c
1000	format (' Computing sin(theta)/lambda table ...')
	end
c*****7*****************************************************************
	subroutine four_formtab
c+
c	This routine sets up the complex formfactor lookup table
c	for all atom types. The range in sin(theta)/lambda is
c	0 -> 2 in steps of 0.001. These values can be changed
c	in the 'diffuse.inc' file.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
c
	real		q2,sb,sf,sfp,sfpp
	integer		iq,iscat
c
	real		form
c
	if(four_log) then
	  write (output_io,1000)
	endif
c
	do iscat=1,cr_nscat
	  do iq=0,CFPKT
	    q2 = (float(iq)*CFINC)**2
	    sf = form(iscat,cr_scat,lxray,q2)
c
	    if (ano) then
	      sfp  = cr_delfr(cr_iscat(iscat))
	      sfpp = cr_delfi(cr_iscat(iscat))
	    else
	      sfp  = 0.0
	      sfpp = 0.0
	    endif
c
	    if (ldbw) then
	      sb = exp(-cr_dw(cr_iscat(iscat))*q2)
	    else
	      sb = 1.0
	    endif
c
	    cfact(iq,iscat)=cmplx(sb*(sf+sfp),sb*sfpp)
	  enddo
	enddo
c
1000	format (' Computing formfactor lookup table ...')
	end
c*****7*****************************************************************
	subroutine four_qinfo
c+
c	Gives information about max/min values for diffuse and
c	Bragg scattering.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'diffuse.inc'
c
	integer		i,j,k,nd
	real		h(3),dsum,dsum2
	logical		lbragg
c
	braggmax = -9999.0
	braggmin = 1E19
	diffumax = -9999.0
	diffumin = 1E19
c
	nd    = 0
	dsum  = 0.0
	dsum2 = 0.0
c
	do j=1,num(2)
	  do i=1,num(1)
	    lbragg = .true.
	    do k=1,3
	      h(k) = eck(k,1) + vi(k,1)*float(i-1) + vi(k,2)*float(j-1)
	      lbragg = lbragg .and. amod(h(k),1.0).eq.0.0
	    enddo
	    k  = (i-1)*num(2)+j
	    if (lbragg) then
	      braggmax = max(braggmax,dsi(k))
	      braggmin = min(braggmin,dsi(k))
	    else
	      diffumax = max(diffumax,dsi(k))
	      diffumin = min(diffumin,dsi(k))
c
	      dsum  = dsum  + dsi(k)
	      dsum2 = dsum2 + dsi(k)**2
	      nd    = nd + 1
	    endif
	  enddo
	enddo
c
	if (nd.ne.0) then
	  diffuave = dsum / float(nd)
	  diffusig = sqrt(dsum2/float(nd) - diffuave**2)
	else
	  diffuave = 0.0
	  diffusig = 0.0
	endif
c
	if(four_log) then
	  write (output_io,1000)
	  if (braggmax.ge.0.0) then
	    write (output_io,1010) braggmin,braggmax
	  endif
	  write (output_io,1020) diffumin,diffumax
	  write (output_io,1030) diffuave,diffusig
	endif
c	
1000	format (/,' ')
1010	format (  ' Bragg scat.     : ',G12.6,'  -> ',G12.6)
1020	format (  ' Diffuse scat.   : ',G12.6,'  -> ',G12.6)
1030	format (  '      Average    : ',G12.6,'  +- ',G12.6)
	end
c*****7*****************************************************************
	subroutine four_show
c+
c	prints summary of current fourier settings
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'diffuse.inc'
	include         'errlist.inc'
	include		'output.inc'
c
	character*7	radiation
        character*1     extr_achs(0:3)
	real		u(3),v(3),w(3)
	real		dvi1,dvi2,dvi3,dvi4,dvi5
	integer		i,j
	logical		lspace
c
	real		do_blen,do_bang
c
        data extr_achs/' ','h','k','l'/
c
	if (fave.eq.0.0) then
	   write (output_io,1010)
	 else
	   write (output_io,1020) fave*100.0
	 endif
c
	if     (ilots.eq.LOT_OFF) then
	  write (output_io,1100)
	elseif (ilots.eq.LOT_BOX) then
	  write (output_io,1110) nlots
	  write (output_io,1130) (ls_xyz(i),i=1,3),lperiod
	elseif (ilots.eq.LOT_ELI) then
	  write (output_io,1120) nlots
	  write (output_io,1130) (ls_xyz(i),i=1,3),lperiod
	endif
c
	radiation = 'neutron'
	if (lxray) radiation = 'x-ray'
	if (lambda.eq.' ') then
	  write (output_io,1200) radiation,rlambda
	else
	  write (output_io,1210) radiation,lambda,rlambda
	endif
c	
	if (ldbw) then
	  write (output_io,1300) 'used'
	else
	  write (output_io,1300) 'ignored'
	endif
c
	if (ano) then
	  write (output_io,1310) 'used'
	else
	  write (output_io,1310) 'ignored'
	endif
c
	do i=1,3
	  u(i) = vi(i,1)
	  v(i) = 0.0
	  w(i) = vi(i,2)
	enddo
	lspace = .false.
	dvi1   = do_blen(lspace,u,v)
	dvi2   = do_blen(lspace,w,v)
	if(inc(1).gt.1 .and. inc(2).gt.1) then
	  dvi3   = do_bang(lspace,u,v,w)
	else
	  dvi3 = 0.0
	endif
	if(dvi3.gt.0) then
	  dvi4   = dvi2/dvi1
	  if(abs(u(extr_abs)).gt.0.0  .and.
     &	     abs(w(extr_ord)).gt.0.0) then
	    dvi5 = (dvi2/w(extr_ord))/(dvi1/u(extr_abs))
	  else
	    ier_num = -4
	    ier_typ = ER_FOUR
	  endif
	else
	  dvi4   = 0.0
	  dvi5   = 0.0
	endif
c
	write (output_io,1400) ((eck(i,j),i=1,3),j=1,3)
	write (output_io,1410) (vi(i,1),i=1,3),dvi1,(vi(i,2),i=1,3),dvi2
	write (output_io,1420) (inc(i),i=1,2),extr_achs(extr_abs),
     &	                              extr_achs(extr_ord)
	write (output_io,1430) dvi3,dvi4,dvi5
c
1010	format (  ' Fourier technique    : turbo Fourier')
1020	format (  ' Fourier technique    : turbo Fourier, minus <F>',
     &	          ' (based on ',F5.1,'% of cryst.)')
1100	format (  '   Fourier volume     : complete crystal')
1110	format (  '   Fourier volume     : ',I4,' box shaped lots')
1120	format (  '   Fourier volume     : ',I4,' ellipsoid shaped lots')
1130	format (  '   Lot size           : ',I3,' x ',I3,' x ',I3,
     &	           ' unit cells (periodic boundaries = ',L1,')')
1200	format (  '   Radiation          : ',A,', wavelength = ',
     &	              F7.4,' A')
1210	format (  '   Radiation          : ',A,', wavelength = ',A4,
     &	          ' = ',F7.4,' A')
1300	format (  '   Temp. factors      : ',A)
1310	format (  '   Anomalous scat.    : ',A)
1400	format (/,' Reciprocal layer     : ',/
     &	          '   lower left  corner : ',3(2x,f9.4),/
     &	          '   lower right corner : ',3(2x,f9.4),/
     &	          '   upper left  corner : ',3(2x,f9.4))
1410	format (  '   hor. increment     : ',3(2x,f9.4),2x,
     &	              ' -> ',f9.4,' A**-1',/
     &	          '   vert. increment    : ',3(2x,f9.4),2x,
     &	              ' -> ',f9.4,' A**-1')
1420	format (  '   # of points        :  ',I4,' x',I4,'  (',
     &	              A1,',',A1,')')
1430	format (  '   Angle              : ',2x,f9.4,' degrees',/
     &	          '   Ratio/Aver  v/h    : ',2(2x,f9.4))
	end
c*****7*****************************************************************
	real function form (ll,scat,lxray,h2)
c+
c 	calculates the form factor     
c-
	implicit	 none
c
	integer	         idim,maxscat
	parameter       (idim=3,maxscat=150)
c
	logical          lxray
	integer	         ll,i
	real             h2,scat(9,0:maxscat)
c
	form=scat(1,ll)
	if (lxray) then
	  do i=1,4
	    form=form + scat(2*i,ll)*exp(-scat(2*i+1,ll)*h2)
	  enddo
	endif
	end 
c*****7*****************************************************************
	real function quad(h,k,rten)
c+
c     	Calculates the scalar product of h and k.
c     	1/d**2 = h(i)*k(j)*rten(i,j)
c-
	implicit	 none
c
	integer          idim
	parameter(idim=3)
c
	integer          i,j
	real             h(idim),k(idim),rten(idim,idim)
c
	quad=0.0
	do i=1,idim 
	  do j=1,idim 
	    quad=quad + h(i) * k(j) * rten(i,j)
	  enddo
	enddo
	end 
c*****7*****************************************************************
	subroutine dlink(lxray,ano,lambda,rlambda)
c-
c     	This routine dlinks between the DISCUS package and the routines
c     	that generate the atomic form factors. These routines have been
c     	adapted from the program LAZY.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'charact.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
c
	common /anomal/ delfr,delfi
	common /neu/    fneu
	common /scat/   fa,fb,fc
c
	character*4   	lambda,line
	integer       	element,symwl,kodlp,i,j
	logical       	ano,lxray
	real          	fa(4,8),fb(4,8),fc(8),rlambda
	real          	delfr,delfi,fneu
	real          	wave
c
	ier_num = 0
	ier_typ = ER_NONE
c
	write(line,100) lambda
	read (line,100) symwl
	if(.not. lxray) then
	  kodlp=2
	else
	  kodlp=1
	  if(lambda.ne.'    ') rlambda=wave(symwl)
	endif
c
	if(ier_num.ne.0) return
c
	if(cr_nscat.gt.0) then
	  do i=1,cr_nscat
	    if(cr_scat_int(i) ) then
	      if(.not. lxray) then
	        write(line ,100) cr_at_lis(i)
c
c------	--------For neutron scattering remove the charge from an ion-name 
c
	        line(3:4)='  '
	        j = ichar(line(2:2))
	        if(zero.le. j .and. j.le.nine) then
	          line(2:2) = ' '
	        endif
	        read (line,100) element
	        call anoma(symwl,element,kodlp)
	        cr_scat(1,i)=fneu
	        cr_delfr(i)=0.0
	        cr_delfi(i)=0.0
	        do j=2,9
	          cr_scat(j,i)=0.0
	        enddo
	      else
	        write(line,100) cr_at_lis(i)
	        read (line,100) element
c	        if(ano .and. .not. cr_delf_int(i) ) then
	        if(ano .and.       cr_delf_int(i) ) then
	          call anoma(symwl,element,kodlp)
	          cr_delfr(i)=delfr
	          cr_delfi(i)=delfi
	        endif
	        call fsc(element,1)
	        cr_scat(1,i)=fc(1)
	        do j=1,4
	          cr_scat(2*j  ,i)=fa(j,1)
	          cr_scat(2*j+1,i)=fb(j,1)
	        enddo
	      endif
c
	      if(element.eq.0) then
	        ier_num = -20
	        ier_typ = ER_APPL
	        goto 999
	      endif
	    endif
	  enddo
	else
	  ier_num = -21
	  ier_typ = ER_APPL
	endif
c
999	continue
c
100   	format(a4)
101   	format(a2)
      	end
c*****7*****************************************************************
	subroutine errlist_four
c-
c	Displays error Messages for the error type RMC
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU=-15,IO=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  ' ',      !-15
     &  ' ',      !-14
     &  ' ',      !-13
     &  ' ',      !-12
     &  ' ',      !-11
     &  ' ',      !-10
     &  ' ',      !-9
     &  ' ',      !-8
     &  ' ',      !-7
     &  ' ',      !-6
     &  ' ',      !-5
     &  'Component of increment vector is zero    ',      !-4
     &  'SIN(THETA)/LAMBDA > lookup table limits  ',      !-3
     &  'Invalid lot shape selected               ',      !-2
     &  'Invalid Fourier mode selected            ',      !-1 
     &  ' '       !  0
     &	/
c
	call disp_error ('FOUR',error,iu,io)
	end
C*****7****************************************************************
