c*****7*****************************************************************
	real function do_blen(lspace,u,v)
c-
c	Calculates the length of the vector v-u in the space defined by
c	lspace (.true. = real space , .false. = reciprocal space)
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
c
	logical        lspace
	real           u(3),v(3)
c
	integer        i
	real           w(3)
	real           skalpro
c
	do i=1,3
	  w(i) = v(i) - u(i)
	enddo
c
	if(lspace) then
	  do_blen = sqrt( skalpro(w,w,cr_gten))
	else
	  do_blen = sqrt( skalpro(w,w,cr_rten))
	endif
c
	end
c*****7*****************************************************************
	real function do_bang(lspace,u,v,w)
c-
c	Calculates the angle between vectors v-u and v-w in the space 
c	defined by lspace (.true. = real space , .false. = reciprocal space)
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'wink.inc'
c
	logical        lspace
	real           u(3),v(3),w(3)
c
	integer        i
	real           xx,xy,yy,arg
	real           x(3),y(3)
	real           skalpro
c
	do_bang = 0.0
c
	do i=1,3
	  x(i) = v(i) - u(i)
	  y(i) = v(i) - w(i)
	enddo
c
	if(lspace) then
	  xx = sqrt( skalpro(x,x,cr_gten))
	  xy =       skalpro(x,y,cr_gten)
	  yy = sqrt( skalpro(y,y,cr_gten))
	else
	  xx = sqrt( skalpro(x,x,cr_rten))
	  xy =       skalpro(x,y,cr_rten)
	  yy = sqrt( skalpro(y,y,cr_rten))
	endif
	if(xx.gt.0 .and. yy.gt.0) then
	  arg = xy/(xx*yy)
	  if (abs(arg).lt.1.0) then
	    do_bang = acos(arg)/rad
	  elseif(arg.gt.1.0) then
	    do_bang = 0.0
	  elseif(arg.le.-1.0) then
	    do_bang = 180.0
	  endif
	else
	  ier_num = -32
	  ier_typ = ER_APPL
	  do_bang = 0.0
	endif
c
	end
c*****7*********************************************************
        real function skalpro(h,k,rten)
c-
c     	Calulates the SCALARPRODUCT of two vectors
c     	1/D**2 = H(I)*K(J)*RTEN(I,J)
c-
	implicit       none
c
	integer        idim
        parameter     (idim=3)
        real           h(idim),k(idim),rten(idim,idim)
c
	integer      i,j
c
        skalpro=0.0
        do i=1,idim
          do j=1,idim
            skalpro=skalpro + h(i) * k(j) * rten(i,j)
	  enddo
	enddo
c
        end
c*****7*****************************************************************
	subroutine vekprod(u,v,ww,eps,rten)
c-
c	calculates the VECTORPRODUCT in general triclinic space
c	with  EPS and RTEN in direct space
c	with REPS and GTEN in reciprocal space
c+
	implicit       none
c
	integer        i,j,k,l
	REAL           U(3),V(3),WW(3),EPS(3,3,3),RTEN(3,3)
c
	do i=1,3
	  ww(i)=0.0
	  do j=1,3
	    do k=1,3
	      do l=1,3
	        ww(i)=ww(i) + eps(j,k,l)*u(k)*v(l)*rten(j,i)
	      enddo
	    enddo
	  enddo
	enddo
c
	end
C*****7*****************************************************************
	subroutine d2r(line,laenge,lspace)
c-
c	Transforms a vector from direct to reciprocal space or vice versa
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'param.inc'
c
	integer        maxw
	parameter     (maxw=3)
c
	character*200  cpara(maxw),line
	integer        lpara(maxw)
	integer        ianz
	integer        i,laenge
	logical        lspace
	real           werte(maxw)
	real           u(3),v(3)
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.eq.0) then
	  if(ianz.eq.3) then
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if(ier_num.eq.0) then
	      do i =1,3
	        u(i) = werte(i  )
	      enddo
	      if(lspace) then
	        call trans(u,cr_gten,v,3)
	      else
	        call trans(u,cr_rten,v,3)
	      endif
c
c------	----Write resulting vector into res_para
c
	      res_para(0) = 3
	      do i=1,3
	        res_para(i) = v(i)
	      enddo
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
	end
C*****7*****************************************************************
	subroutine vprod(line,laenge)
c-
c	Calculates the vector product v X u
c	both input vectors and the result can be in direct or
c	reciprocal spce
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'param.inc'
c
	integer        maxw
	parameter     (maxw=7)
c
	character*200  cpara(maxw),line
	integer        lpara(maxw)
	integer        ianz
	integer        i,laenge
	real           werte(maxw)
	real           u(3),v(3),w(3)
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.eq.0) then
	  if(ianz.ge.6) then
c
c------	----Set default to direct space
c
	    if(ianz.eq.6) then
	      cpara(7) = 'ddd'
	      lpara(7) = 3
	    endif
	      ianz = 6
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if(ier_num.eq.0) then
	      do i =1,3
	        u(i) = werte(i  )
	        v(i) = werte(i+3)
	      enddo
	      if(cpara(7)(1:1).eq.'r') then
c
c------	------------Transform vector 1 into direct space
c
	        call trans(u,cr_rten,w,3)
	        do i=1,3
	          u(i) = w(i)
	        enddo
	      endif
	      if(cpara(7)(2:2).eq.'r') then
c
c------	------------Transform vector 2 into direct space
c
	        call trans(v,cr_rten,w,3)
	        do i=1,3
	          v(i) = w(i)
	        enddo
	      endif
c
c------	------Calculate vektorprodukt, always in direct space
c
	      call vekprod(u,v,w,cr_eps,cr_rten)
c
	      if(cpara(7)(3:3).eq.'r') then
c
c------	------------Transform result vector into reciprocal space
c
	        call trans(w,cr_gten,u,3)
	        do i=1,3
	          w(i) = u(i)
	        enddo
	      endif
c
c------	----------Write result vector into res_para
c
	      res_para(0) = 3
	      do i=1,3
	        res_para(i) = w(i)
	      enddo
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
C*****7*****************************************************************
	subroutine do_proj(line,laenge)
c-
c	Calculates the projection of a vector onto another vector and
c	onto the plane normal to the second vector.
c	Both input vectors and the result can be in direct or
c	reciprocal spce
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'param.inc'
c
	integer        maxw
	parameter     (maxw=7)
c
	character*200  cpara(maxw),line
	integer        lpara(maxw)
	integer        ianz
	integer        i,laenge
	real           werte(maxw)
	real           u(3),v(3),w(3),h(3),x(3)
	real           vv,uv,uh,hh
c
	real           skalpro
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.eq.0) then
	  if(ianz.ge.6) then
c
c------	----Set default to direct space
c
	    if(ianz.eq.6) then
	      cpara(7) = 'dddd'
	      lpara(7) = 4
	    endif
	      ianz = 6
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if(ier_num.eq.0) then
	      do i =1,3
	        u(i) = werte(i  )
	        v(i) = werte(i+3)
	      enddo
	      if(cpara(7)(1:1).eq.'r') then
c
c------	------------Transform vector 1 into direct space
c
	        call trans(u,cr_rten,w,3)
	        do i=1,3
	          u(i) = w(i)
	        enddo
	      endif
	      if(cpara(7)(2:2).eq.'r') then
c
c------	------------Transform vector 2 into direct space
c
	        call trans(v,cr_rten,w,3)
	        do i=1,3
	          v(i) = w(i)
	        enddo
	      endif
c
c------	------Calculate projection onto second vector, always in direct space
c
	      uv = skalpro(u,v,cr_gten)
	      vv = skalpro(v,v,cr_gten)
	      do i=1,3
	        w(i) = v(i)*uv/vv
	      enddo
c
	      if(cpara(7)(3:3).eq.'r') then
c
c------	------------Transform result vector into reciprocal space
c
	        call trans(w,cr_gten,x,3)
	        do i=1,3
	          w(i) = x(i)
	        enddo
	      endif
c
c------	----------Write result vector into res_para
c
	      res_para(0) = 3
	      do i=1,3
	        res_para(i) = w(i)
	      enddo
c
c------	------Calculate projection onto plane normal to second vector,
c	      always in direct space
c
c
c------	------------Transform vector 2 into reciprocal space
c
	        call trans(v,cr_gten,h,3)
	        uh = 0.0
	        do i=1,3
	          uh = uh + u(i)*h(i)
	        enddo
	        hh = skalpro(h,h,cr_rten)
	        do i=1,3
	          w(i) = u(i) - uh*v(i)/hh
	        enddo
c
	      if(cpara(7)(4:4).eq.'r') then
c
c------	------------Transform result vector into reciprocal space
c
	        call trans(w,cr_gten,x,3)
	        do i=1,3
	          w(i) = x(i)
	        enddo
	      endif
c
c------	----------Write result vector into res_para
c
	      res_para(0) = 6
	      do i=1,3
	        res_para(i+3) = w(i)
	      enddo
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
