c*****7*****************************************************************
	subroutine do_micro
c-
c     	This subroutine reads a microdomain from file,requests basic 
c     	vectors for the distribution of the microdomains and replaces 
c     	all atoms found within a microdomain with the respective atom
c	of the microdomain.
c
c	Version : 1.0
c	Date    : 05 Oct 96
c
c	Author  : R.B. Neder (reinhard.neder@mail.uni-wuerzburg.de)
c-
	implicit      none
c
	include      'config.inc'
	include      'crystal.inc'
	include      'crystal.cmm'
	include      'doact.inc'
	include      'errlist.inc'
	include      'learn.inc'
	include      'macro.inc'
	include      'microdis.inc'
	include      'prompt.inc'
c
	integer        maxw
	parameter     (maxw=9)
c
	character*5    befehl
	character*12   prom
	character*200 line,zeile,cpara(maxw)
	integer        lpara(maxw),lp
	integer        i,j,ii,ianz,m,l,k,laenge,lbef
	integer        indxg
	logical        lend,lm(4),lda,linit
	logical        micro_run
	real           werte(maxw)
	real           chance
c
	logical        str_comp
c
	data linit     /.true./
c
	lend=.false.

	if(linit) then
	  do k=1,MAX_MD_TY
	    md_corr(k,k) = 1.0
	  enddo
	  do k=1,4
	    md_orient(k,k,1) = 1.0
	    lm(k)=.false.
	  enddo
	  linit = .false.
	endif
c
	do while (.not. lend)
	  call no_error
	  prom=prompt//'/micro'
	  call get_command (line,laenge,befehl,lbef,zeile,lp,prom)
	  if(ier_num.eq.0) then
	  if(line.ne.' ' .and. line(1:1).ne.'#') then
c
c------	----search for "="
c
	    indxg=index(line,'=')
	    if(indxg.ne.0 
     &	              .and. .not.(str_comp(befehl,'echo',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'syst',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                          str_comp(befehl,'?   ',2,lbef,4)) )then
c
c------	------evaluatean expression and assign the value to a variabble
c
	      call do_math(line,indxg,laenge)
	    else
c
c------ ----execute a macro file
c
              if(befehl(1:1).eq.'@') then
	        if(laenge.ge.2) then
	          call file_kdo(line(2:laenge),laenge-1)
	        else
	          ier_num = -13
	          ier_typ = ER_MAC
	        endif
c
c------	----set unit to Angstrom 'angs'
c
	    elseif(str_comp(befehl,'angs',2,lbef,4)) then
	      mv_unit = MD_ANG
c
c------ ----list asymmetric unit 'asym'
c
	    elseif(str_comp(befehl,'asym',2,lbef,4)) then
              call show_asym
c
c------	----Base vectors of microdomain distribution 'base'
c
	    elseif(str_comp(befehl,'base',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.9) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            ii=0
	            do i=1,3
	              do j=1,3
	              ii=ii+1
	              md_base(j,i)=werte(ii)
	              enddo
	            enddo
	            call invmat(md_rbase,md_base)
	            call transmat(md_rbase,3)
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ ----set current boundary type 'boundary'
c
	    elseif(str_comp(befehl,'boun',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1 .or. ianz.eq.4) then
	          if    (cpara(1)(1:2).eq.'bl') then
c------	------------Boundary type 'block'
	            mv_bound = MD_BLOCK
	          elseif(cpara(1)(1:2).eq.'fa' .and. ianz.eq.4) then
c------	------------Boundary type 'face'
	            mv_bound = MD_FACE
	            cpara(1) = '0.0'
	            lpara(1) = 3
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            if(ier_num.eq.0) then
	              mv_hkl_n = mv_hkl_n + 1
	              do i=1,3
	                mv_hkl(i,mv_hkl_n) = werte(i+1)
	              enddo
	              mvv_rad   (mv_hkl_n) = mv_rad
	              mvv_rad_si(mv_hkl_n) = mv_rad_si
	            endif
	          elseif(cpara(1)(1:2).eq.'fu') then
c------	------------Boundary type 'fuzzy'
	            mv_bound = MD_FUZZY
	          elseif(cpara(1)(1:2).eq.'sp') then
c------	------------Boundary type 'sphere'
	            mv_bound = MD_SPHERE
	          else
	            ier_num = -30
	            ier_typ = ER_APPL
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----Read a single collumn of the correlation matrix 'ccolumn'
c
	    elseif(str_comp(befehl,'ccol',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.md_num+1-md_num_cre) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            i            = nint(werte(1))
	            if(md_num_cre.lt.i .and.
     &	               i.le.md_num     .and.
     &	               i.le.MAX_MD_TY) then
	              do j=md_num_cre + 1,md_num
	                md_corr(j,i) = werte(j+1)
	              enddo
	            else
	              ier_num = -13
	              ier_typ = ER_APPL
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----Read a single element of the correlation matrix 'celement'
c
	    elseif(str_comp(befehl,'cele',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.3) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            if((md_num_cre.lt.i .and.
     &	                i.le.md_num     .and.
     &	                i.le.MAX_MD_TY       ) .or. 
     &	               (md_num_cre.lt.j .and.
     &	                j.le.md_num     .and.
     &	                j.le.MAX_MD_TY       )      ) then
	              i            = nint(werte(1))
	              j            = nint(werte(2))
	              md_corr(i,j) = werte(3)
	            else
	              ier_num = -13
	              ier_typ = ER_APPL
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------ ----list atoms present in the crystal 'chem'
c
	    elseif(str_comp(befehl,'chem',2,lbef,4)) then
              call show_chem
c
c------ ----set current content type 'content'
c
	    elseif(str_comp(befehl,'content',5,lbef,7)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1 .or. ianz.eq.2) then
	          if    (cpara(1)(1:2).eq.'ce' .and. ianz.eq.2) then
c------	------------content type 'cell'
	            mv_cont  = MD_CELL
	            mv_file  = cpara(2)
	          elseif(cpara(1)(1:2).eq.'fr') then
c------	------------content type 'free'
	            mv_cont = MD_FREE
	            mv_file  = ' '
	          elseif(cpara(1)(1:2).eq.'st' .and. ianz.eq.2) then
c------	------------content type 'structure'
	            mv_cont = MD_STRUC
	            mv_file = cpara(2)
	            laenge  = lpara(2)
	            call rem_bl(mv_file,laenge)
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	          if(mv_file.ne.' ') then
	            inquire(file=mv_file,exist=lda)
	            if(.not.lda) then
	              ier_num = -1
	              ier_typ = ER_IO
	              mv_file = ' '
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	--continues a macro 'continue'
c
	    elseif(str_comp(befehl,'continue',5,lbef,8)) then
	      call macro_continue(zeile,lp)
c
c------	----Create list of microdomain origins 'create'
c
	    elseif(str_comp(befehl,'crea',3,lbef,4)) then
	      if(md_num.gt.md_num_cre) then
	        call create_md_list
	      else
	        ier_num = -33
	        ier_typ = ER_APPL
	      endif
c
c------	----Read a single row of the correlation matrix 'crow'
c
	    elseif(str_comp(befehl,'crow',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.md_num+1-md_num_cre) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            j            = nint(werte(1))
	            if(md_num_cre.lt.j .and.
     &	               j.le.md_num     .and.
     &	               j.le.MAX_MD_TY) then
	            do i=md_num_cre+1,md_num
	              md_corr(j,i) = werte(i+1)
	            enddo
	            else
	              ier_num = -13
	              ier_typ = ER_APPL
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----set the density of microdomains for random distribution 'density'
c	    density in md's/unit cell
c
	    elseif(str_comp(befehl,'dens',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            md_dens = werte(1)
	          endif
	        endif
	      endif
c
c------	----set the distribution type 'distribution'
c
	    elseif(str_comp(befehl,'dist',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1 .or. ianz.eq.2) then
	          if    (cpara(1)(1:1) .eq. 'l') then
	            md_distr = MD_LATTICE
	          elseif(cpara(1)(1:1) .eq. 'f') then
	            md_distr = MD_FILE
	            md_infile= cpara(2)
	          elseif(cpara(1)(1:1) .eq. 'p') then
	            md_distr = MD_PARA
	          elseif(cpara(1)(1:1) .eq. 'r') then
	            md_distr = MD_RAND
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----set unit to direct lattice constants 'dlu'
c
	    elseif(str_comp(befehl,'dlu ',2,lbef,4)) then
	      mv_unit = MD_DLU
c
c------	 ---Echo a string, just for interactive check in a macro 'echo'
c
	    elseif(str_comp(befehl,'echo',2,lbef,4)) then
	      call echo(zeile,lp)
c
c------	 ---Evaluate an expression, just for interactive check 'eval'
c
	    elseif(str_comp(befehl,'eval',2,lbef,4)) then
	      call do_eval(zeile,lp)
c
c------	----exit 'exit'
c
	    elseif(str_comp(befehl,'exit',2,lbef,4)) then
	      lend=.true.
c
c------	----help 'help' , '?'
c
	    elseif(str_comp(befehl,'help',1,lbef,4) .or.
     &	           str_comp(befehl,'?   ',1,lbef,4)     ) then
	      if(str_comp(zeile,'errors',2,lp,6) ) then
	        lp = lp + 7
	        call do_hel('discus '//zeile,lp)
	      else
	        lp = lp + 13
	        call do_hel('discus micro '//zeile,lp)
	      endif
c
c------	----Fitting of microdomain distribution to grid of direct lattice 'grid'
c
	    elseif(str_comp(befehl,'grid',1,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          if(cpara(1)(1:1).eq.'f') then
	            md_grid = MD_G_FREE
	          elseif(cpara(1)(1:1).eq.'h') then
	            md_grid = MD_G_HOST
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----Initialize microdomain setup
c
	    elseif(str_comp(befehl,'init',2,lbef,4)) then
	      do i=1,4
	        lm(i)=.false.
	      enddo
	      call micro_reset
c
c------	----Insert current type of microdomain into list 'insert'
c
	    elseif(str_comp(befehl,'inse',2,lbef,4)) then
	      if(md_num.lt.MAX_MD_TY) then
	        if(mv_cont.eq.MD_FREE .or. mv_file.ne.' ') then
	          md_num             = md_num + 1
	          md_ty_fl(1,md_num) = mv_bound 
	          md_ty_fl(2,md_num) = mv_cont
	          md_ty_fl(3,md_num) = mv_orient
	          md_co_fi(  md_num) = mv_file
	          if    (md_ty_fl(1,md_num).eq.MD_BLOCK) then
	            md_ty_fl (2,md_num) = MD_STRUC
	            md_rad   (0,md_num) = mv_rad
	            md_rad   (1,md_num) = mv_rad
	            md_rad_si(1,md_num) = 0.0
	            md_hkl_n (  md_num) = 0
	          elseif(md_ty_fl(1,md_num).eq.MD_FUZZY) then
	            md_ty_fl (2,md_num) = MD_STRUC
	            md_rad   (0,md_num) = mv_rad
	            md_rad   (1,md_num) = mv_rad
	            md_rad_si(1,md_num) = 0.0
	            md_hkl_n (  md_num) = 0
	          elseif(md_ty_fl(1,md_num).eq.MD_FACE) then
	            md_hkl_n(  md_num) = mv_hkl_n
	            md_rad  (0,md_num) = 0.0
	            do i=1,mv_hkl_n
	              do j=1,3
	                md_hkl(j,i,md_num) = mv_hkl(j,i)
	              enddo
	              md_rad   (i,md_num) = mvv_rad   (i)
	              md_rad_si(i,md_num) = mvv_rad_si(i)
	              md_rad   (0,md_num) = max(md_rad(0,md_num),
     &	                                        md_rad(i,md_num))
	            enddo
	          elseif(md_ty_fl(1,md_num).eq.MD_SPHERE) then
	            md_rad   (0,md_num) = mv_rad
	            md_rad   (1,md_num) = mv_rad
	            md_rad_si(1,md_num) = mv_rad_si
	            md_hkl_n (  md_num) = 0
	          endif
	        else
	          ier_num = -15
	          ier_typ = ER_APPL
	        endif
	      else
	        ier_num = -29
	        ier_typ = ER_APPL
	      endif
c
c------	----Switch logging of microdomain position on/off 'log '
c
	    elseif(str_comp(befehl,'log ',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0.and. ianz.eq.2) then
	        if(cpara(1)(1:1).eq.'p') then
	          if(    cpara(2)(1:2).eq.'of') then
	            md_log_pos = .false.
	          elseif(cpara(2)(1:2).eq.'on') then
	            md_log_pos = .true.
	          endif
	        else
	          ier_num = -16
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----Read a single row of the orientation  matrix 'mrow'
c
	    elseif(str_comp(befehl,'mrow',2,lbef,4)) then
	      if(md_ori_n.lt.MAX_MD_ORI) then
	        call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	        if(ier_num.eq.0) then
	          if(ianz.eq.5) then
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            if(ier_num.eq.0) then
	              j            = nint(werte(1))
	              if( 0.lt.j .and. j.le.3) then
	                do i=1,4
	                  md_orient(j,i,md_ori_n+1) = werte(i+1)
	                enddo
	                lm(j) = .true.
	                if(lm(1).and.lm(2).and.lm(3)) then
	                  do i=1,4
	                    md_orient(4,i,md_ori_n+1) = 0.0
	                  enddo
	                  md_orient(4,4,md_ori_n+1) = 1.0
	                  md_ori_n = md_ori_n + 1
	                  do i=1,4
	                    lm(i) = .false.
	                  enddo
	                endif
	              else
	                ier_num = -17
	                ier_typ = ER_APPL
	              endif
	            endif
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        endif
	      else
	        ier_num = -1 
	        ier_typ = ER_APPL
	      endif
c
c------	----set orientation for the current type 'orientation'
c
	    elseif(str_comp(befehl,'orie',1,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          j = nint(werte(1))
	          if(0.lt.j .and. j.le.md_ori_n) then
	            mv_orient = j
	          else
	            ier_num = -18
	            ier_typ = ER_APPL
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----place a microdomain at given position 'place'
c
	    elseif(str_comp(befehl,'plac',1,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.4) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            if(mc_num.lt.MC_MAX_MICRO) then
	              if(0.lt.nint(werte(4)) .and.
     &	                 nint(werte(4)).le.md_num) then
	                mc_num = mc_num + 1
	                do i=1,3
	                  mc_orig(i,mc_num) = werte(i)
	                enddo
	                mc_type(mc_num) = nint(werte(4))
	                if    (md_ty_fl(1,mc_type(mc_num)) .eq.
     &	                       MD_BLOCK) then
	                  mc_rad(mc_num) = 0.0
	                elseif(md_ty_fl(1,mc_type(mc_num)) .eq.
     &	                       MD_FUZZY) then
	                  mc_rad(mc_num) = 0.0
	                elseif(md_ty_fl(1,mc_type(mc_num)) .eq.
     &	                       MD_FACE) then
	                  mc_rad(mc_num) = 
     &	                      chance(md_rad_si(1,mc_type(mc_num)))
	                elseif(md_ty_fl(1,mc_type(mc_num)) .eq.
     &	                       MD_SPHERE) then
	                  mc_rad(mc_num) = 
     &	                      chance(md_rad_si(1,mc_type(mc_num)))
	                endif
c
c------	calculate inverse orientation matrices
c
	                call micro_inv
	                if(ier_num.eq.0) then
c
c------	                check whether microdomains form a closed form
c
	                  call micro_hkl_closed
	                  if(ier_num.eq.0) then
	                    call micro_hkl
	                    if(ier_num.eq.0) then
	                      micro_run = .false.
	                      call microdomains(micro_run)
	                    endif
	                  endif
	                endif
	              else
	                ier_num = -6
	                ier_typ = ER_APPL
	              endif
	            else
	              ier_num = -5
	              ier_typ = ER_APPL
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----set radius for the current boundary 'radius'
c
	    elseif(str_comp(befehl,'radi',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          if(mv_unit.eq.MD_DLU) then
	            werte(1) = werte(1) / cr_a0(1)
	            werte(2) = werte(2) / cr_a0(1)
	          endif
c
c	----------set radius
c
	          mv_rad  = werte(1)
c
c	----------set radius distribution type
c
	          if(ianz.eq.2) then
	            mv_rad_si = werte(2)
	          else
	            mv_rad_si = 0.0
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----remove one or all MD from the list 'remove'
c
	    elseif(str_comp(befehl,'remo',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(cpara(1).eq.'all') then
	        i      = md_num_cre + 1
	        ii     = md_num
	        md_num = md_num_cre
	      else
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
c-----	----------automatically purge MD type
	          i      = nint(werte(1))
	          if(md_num_cre.lt.i .and. i.le.md_num) then
	            ii     = md_num - 1
	            md_num = md_num - 1
	            do l=i,ii
	              md_ty_fl(1,l) = md_ty_fl(1,l+1)
	              md_ty_fl(2,l) = md_ty_fl(2,l+1)
	              md_ty_fl(3,l) = md_ty_fl(3,l+1)
	              md_ty_fl(4,l) = md_ty_fl(4,l+1)
	              md_hkl_n(  l) = md_hkl_n(  l+1)
	              do j=1,MAX_MD_HKL
	                do k=1,3
	                  md_hkl   (k,j,l) = md_hkl(k,j,l+1)
	                enddo
	                md_rad   (j,l) = md_rad   (j,l+1)
	                md_rad_si(j,l) = md_rad_si(j,l+1)
	              enddo
	            enddo
	            i      = md_num + 1
	            ii     = md_num + 1
	          else
	            ier_num = -40
	            ier_typ = ER_APPL
	          endif
	        endif
	      endif
	      if(ier_num.eq.0) then
c-----	--------Set MD parameters to 0 or ' '
	        do l=i,ii
	          md_ty_fl(1,l) = 0
	          md_ty_fl(2,l) = 0
	          md_ty_fl(3,l) = 0
	          md_ty_fl(4,l) = 0
	          md_hkl_n(  l) = 0
	          do j=1,MAX_MD_HKL
	            do k=1,3
	              md_hkl   (k,j,l) = 0
	            enddo
	            md_rad   (j,l) = 0
	            md_rad_si(j,l) = 0
	          enddo
	        enddo
	      endif
c
c------	----Reciprocal Base vectors of microdomain distribution 'rbase'
c
	    elseif(str_comp(befehl,'rbas',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.9) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            ii=0
	            do i=1,3
	              do j=1,3
	              ii=ii+1
	              md_rbase(j,i)=werte(ii)
	              enddo
	            enddo
	            call invmat(md_base,md_rbase)
	            call transmat(md_base,3)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----Start the distribution of microdomains 'run'
c
	    elseif(str_comp(befehl,'run ',2,lbef,4)) then
CDBG
CDBG	write (output_io,*) ' mc_num     ',mc_num
CDBG	write (output_io,*) ' mc_num_run ',mc_num_run
	      call micro_hkl
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.0) then
CDBG
CDBG	write (output_io,*) ' mc_num     ',mc_num
CDBG	write (output_io,*) ' mc_num_run ',mc_num_run
	          call micro_purge
	          if(ier_num.eq.0) then
	            micro_run = .true.
CDBG
CDBG	write (output_io,*) ' mc_num     ',mc_num
CDBG	write (output_io,*) ' mc_num_run ',mc_num_run
	            call microdomains(micro_run)
CDBG
CDBG	write (output_io,*) ' mc_num     ',mc_num
CDBG	write (output_io,*) ' mc_num_run ',mc_num_run
	          endif
	        elseif(ianz.eq.1) then
	          if(str_comp(cpara(1),'purge',1,lpara(1),5)) then
	            call micro_purge
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----Minimum separations between elements (atoms or microdomains) 'sep'
c
	    elseif(str_comp(befehl,'sepa',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.2) then
	          if(cpara(1)(1:1).eq.'f') then
	            cpara(1) = '0.0'
	            lpara(1) = 3
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            if(ier_num.eq.0) then
	              md_sep_fuz = werte(2)
	            endif
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----Set general parameters                 'set '
c
	    elseif(str_comp(befehl,'set',3,lbef,3)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.2) then
	          if(str_comp(cpara(1),'overlap',1,lpara(1),7)) then
	            if(str_comp(cpara(2),'warning',1,lpara(2),7)) then
	              md_overlap= MD_WARNING
	            elseif(str_comp(cpara(2),'error',1,lpara(2),6)) then
	              md_overlap = MD_ERROR
	            else
	              ier_num = -6
	              ier_typ = ER_COMM
	            endif
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----Constant shift of microdomain origins 'shift'
c
	    elseif(str_comp(befehl,'shif',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.3) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            do i=1,3
	              md_shift(i) = werte(i)
	            enddo
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----show current settings 'show'
c
	    elseif(str_comp(befehl,'show',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.0 .or. ianz.eq.1) then
	          call show_microdist(ianz,cpara,maxw)
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------	----base vector sigma 'sigma', 'six', 'siy', 'siz'
c
	    elseif(str_comp(befehl,'si',2,lbef,2)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          if(ianz.eq.9) then
	            ii=0
	            do i=1,3
	              do m=1,3
	                ii=ii+1
	                md_si(i,m)=werte(ii)
	                md_si(i,m)=md_si(i,m)*md_si(i,m)
	              enddo
	            enddo
	          elseif(ianz.eq.3) then
	            i=0
	            if(befehl(3:3).eq.'x') then
	              i=1
	            elseif(befehl(3:3).eq.'y') then
	              i=2
	            elseif(befehl(3:3).eq.'z') then
	              i=3
	            endif
	            if(i.ne.0) then
	              ii=0
	              do m=1,3
	                ii=ii+1
	                md_si(i,m)=werte(ii)
	                md_si(i,m)=md_si(i,m)*md_si(i,m)
	              enddo
	            else
	              ier_num = -8
	              ier_typ = ER_COMM
	            endif
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------- -Operating System Kommandos 'syst'
c
	  elseif(str_comp(befehl,'syst',2,lbef,4)) then
	    if(zeile.ne. ' ') then
	      call do_operating(zeile(1:lp),lp)
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
c
c
c------  -----waiting for user input
c
	    elseif(str_comp(befehl,'wait',3,lbef,4)) then
	      call do_input(zeile,lp)
c
c------	----unknown command
c
	    else
	      ier_num = -8
	      ier_typ = ER_COMM
	    endif
	  endif
	  endif
	  endif
c
c------	--Goto's and jumps are terrible, yet FORTRTAN does not have a break
c------	--Jump here if an error occured
c
9998	  continue
	  if(ier_num.ne.0) then
	    call errlist
	    if(lmakro) then
	      call macro_close
	      prompt_status = PROMPT_ON
	    endif
	    if(lblock) then
	      ier_num = -11
	      ier_typ = ER_COMM
	      return
	    endif
	    call no_error
	  endif
	enddo
c
9999	continue
c
	end
c*****7****************************************************************
	subroutine create_md_list
c-
c	Creates the list of microdomain origins.
c	If the distribution type is 'LATTICE' or "PARA",
c	the origins of the microdomains are located at the nodes of
c	a perfect lattice or a paracrystalline distributed lattice.
c	If the intergrowth type is 'HOST', the origins are
c	restricted to integer multiples of the base vectors, if
c	the intergrowth type is 'FREE', the origins are incoherently
c	intergrown in the matrix of the crystal.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'micro.inc'
	include       'microdis.inc'
	include       'wink.inc'
c
	integer        i1,i2,j1,j2,k1,k2
	integer        j,j_start,j_end
c
	j_start = max(1,md_num_cre)
	j_end   = md_num
c
c------	calculate inverse orientation matrices
c
	call micro_inv
	if(ier_num.eq.0) then
c
c------	check whether microdomains form a closed form
c
	  call micro_hkl_closed
c
	  if(ier_num.eq.0) then
	    call micro_corr_check
	    if(ier_num.eq.0) then
c
c------	------Read all microdomain types once to determine the dimensions
c
	      do j=j_start,j_end
CDBG
CDBG	write (*,*) ' starte md_get_dim'
CDBG	write (*,*) 'j_start,j,j_end',j_start,j,j_end
	        call md_get_dim(j)
CDBG	write (*,*) ' fertig md_get_dim'
CDBG	write (*,*) 'md_dim ',md_dim(1,1),md_dim(1,2)
CDBG	write (*,*) '       ',md_dim(2,1),md_dim(2,2)
CDBG	write (*,*) '       ',md_dim(3,1),md_dim(3,2)
c
c------	------Now read the microdomain structure
c
	      enddo
	      if(md_distr.eq.MD_LATTICE .or. 
     &	         md_distr.eq.MD_PARA         ) then
C.........DETERMINE MINIMAL AND MAXIMAL INDICES
	        CALL INDMIC(md_rbase,cr_dim,md_dim,i1,i2,J1,J2,k1,k2)
C.......CREATE LIST OF ORIGINS OF microDOMAINS
	        call micro_dist(cr_dim,cr_spcgr,cr_a0,cr_win,
     &	                        i1,i2,j1,j2,k1,k2)
	      elseif(md_distr.eq.MD_RAND) then
	        call micro_rand(cr_spcgr,cr_a0,cr_win,cr_dim)
	      elseif(md_distr.eq.MD_FILE) then
	        call micro_dist_file(cr_spcgr,cr_a0,cr_win,cr_dim)
	      else
	        ier_num = -31
	        ier_typ = ER_APPL
	      endif
	    endif
	  endif
	endif
c
c------	set number of already created microdomains
c
	if(ier_num.eq.0) then
	  md_num_cre = md_num
	endif
c
	end
c*****7****************************************************************2
	subroutine md_get_dim(j)
c-
c	Reads microdomain type j to get minimum and maximum dimensions
c+
	implicit      none
c
	include       'config.inc'
CDBG	include       'crystal.inc'
CDBG	include       'crystal.cmm'
	include       'errlist.inc'
	include       'micro.inc'
	include       'microdis.inc'
c
	integer		sav_ncell(3)
	integer		sav_ncatoms
	logical		sav_r_ncell
c
	character*40	strucfile
	integer        j,i
c
c
c
c------	----Initialize counters for atom numbers
c
	    mk_natoms = 0
	    mk_nscat  = 0
	    ma_natoms = 0
	    mk_at_lis(0) = 'VOID'
	    mk_dw(0)     = 0.0
	    do i = 0,MK_MAX_SCAT
	      mk_dw(i)     = 0.0
	      mk_at_lis(i) = ' '
	      ma_dw(i)     = 0.0
	      ma_at_lis(i) = ' '
	    enddo
	    sav_ncell(1) = 1
	    sav_ncell(2) = 1
	    sav_ncell(3) = 1
	    sav_ncatoms  = 0
	    sav_r_ncell  = .false.
	    strucfile = md_co_fi(j)
	call readstru(MMAX,MK_MAX_SCAT,
     &	            strucfile           ,mk_name,mk_spcgr,
     &	            mk_a0,mk_win,mk_natoms,mk_nscat,
     &	            mk_dw,mk_at_lis,mk_pos,mk_iscat,
     &	            mk_dim,ma_natoms,ma_at_lis,ma_dw,
     &	            ma_pos,ma_iscat,
     &	            sav_ncell,sav_r_ncell,sav_ncatoms)
	if(ier_num.ne.0) then
	  ier_msg(1) = 'Error occured while reading'
	  ier_msg(2) = md_co_fi(mc_type(j))
	  return
	endif
c
	do i=1,3
	  md_dim(i,1) = min(md_dim(i,1),mk_dim(i,1))
	  md_dim(i,2) = max(md_dim(i,2),mk_dim(i,2))
	enddo
c
	end
c*****7****************************************************************2
	subroutine microdomains(micro_run)
c-
c	Replaces the atoms in the original structure with a correlated
c	distribution of microdomains
c+
	implicit      	none
c
	logical         micro_run
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include         'errlist.inc'
	include 	'micro.inc'
	include 	'microdis.inc'
	include 	'wink.inc'
c
	integer        maxw
	parameter     (maxw=5)
	real           EPS
	parameter     (EPS=0.000010)
c
	integer		sav_ncell(3)
	integer		sav_ncatoms
	logical		sav_r_ncell
C
	integer        i,j,m,n,ianz,natoms_old
	integer        k
	integer        j_start,j_end
	logical        lins
	real           werte(maxw),distance,u(3),uu(4),vv(4)
	real           w(3)
	real           dist_orig
c
	if(micro_run) then
c
c------	--Normal microdomain 'run' mode, loop over all microdomain
c	  origins that have not yet been used for modifying the crystal
c
	  j_start = mc_num_run + 1
	  j_end   = mc_num
	else
c
c------	--Place mode, only a single microdomain is placed into the
c	  crystal
c
	  j_start = mc_num
	  j_end   = mc_num
	endif
c
c
c------	Check all atoms as to whether they are inside any microdomain
c
	do i=1,cr_natoms
	  u(1) = cr_pos(1,i)
	  u(2) = cr_pos(2,i)
	  u(3) = cr_pos(3,i)
c
c	  j = mc_num_run + 1
	  call micro_check_all(u,i,j_start,lins)
c
c------	--Remove all atoms, unless they are in a microdomain of FREE content
c
	  if( lins ) then
	    if(md_ty_fl(2,mc_type(iabs(cr_md_next(i)))).ne.MD_FREE )then
	      cr_iscat(i)   = 0
	      cr_ndel       = cr_ndel+1
c	      cr_pos  (1,i) = 0.0
c	      cr_pos  (2,i) = 0.0
c	      cr_pos  (3,i) = 0.0
	    endif
	  endif
c
	enddo
c
C------	Add all atoms within microdomains
c
	ianz=5
	do j=j_start,j_end
c
	  if(    md_ty_fl(2,mc_type(j)).eq.MD_STRUC) then
c
c------	----Initialize counters for atom numbers
c
	    mk_natoms = 0
	    mk_nscat  = 0
	    ma_natoms = 0
	    mk_at_lis(0) = 'VOID'
	    mk_dw(0)     = 0.0
	    do i = 1,MK_MAX_SCAT
	      mk_at_lis(i) = ' '
	    enddo
c
c------	----Now read the microdomain structure
c
	    call readstru(MMAX,MK_MAX_SCAT,
     &	                    md_co_fi(mc_type(j)),mk_name,mk_spcgr,
     &	                    mk_a0,mk_win,mk_natoms,mk_nscat,
     &	                    mk_dw,mk_at_lis,mk_pos,mk_iscat,
     &	                    mk_dim,ma_natoms,ma_at_lis,ma_dw,
     &	                    ma_pos,ma_iscat,
     &	                    sav_ncell,sav_r_ncell,sav_ncatoms)
	    if(ier_num.ne.0) then
	      ier_msg(1) = 'Error occured while reading'
	      ier_msg(2) = md_co_fi(mc_type(j))
	      return
	    endif
	    natoms_old = cr_natoms
	    do i=1,mk_natoms
c------	------Apply orientation operation to the atom
	      uu(1) = mk_pos(1,i)
	      uu(2) = mk_pos(2,i)
	      uu(3) = mk_pos(3,i)
	      uu(4) = 1.0
	      do m=1,4
	        vv(m) = 0.0
	        do n=1,4
	          vv(m) = vv(m) +
     &	                  md_orient(m,n,md_ty_fl(3,mc_type(j)))*uu(n)
	        enddo
	      enddo
	      u(1)=mc_orig(1,j) + vv(1)
	      u(2)=mc_orig(2,j) + vv(2)
	      u(3)=mc_orig(3,j) + vv(3)
c------	------Check whether atom is inside boundaries
	      if(md_ty_fl(1,mc_type(j)).ne.MD_FUZZY) then
	        call micro_check(u,j,lins,distance,dist_orig)
	      else
	        lins = .true.
	      endif
	      if(lins) then
                werte(2) = u(1)
                werte(3) = u(2)
                werte(4) = u(3)
	        werte(5)=mk_dw(mk_iscat(i))
c------	--------Check whether atom is inside crystal
c	        Allow for some rounding errors
	        IF(cr_dim(1,1)-EPS .LE. werte(2)         .AND.
     &	           werte(2)        .LE. cr_dim(1,2)+EPS  .AND.
     &	           cr_dim(2,1)-EPS .LE. werte(3)         .AND.
     &	           werte(3)        .LE. cr_dim(2,2)+EPS  .AND.
     &	           cr_dim(3,1)-EPS .LE. werte(4)         .AND.
     &	           werte(4)        .LE. cr_dim(3,2)+EPS )  THEN
c
c------	---------Check whether new atom is inside more than one MD
c
	          k = cr_natoms + 1
	          call micro_check_all(u,k,j_start,lins)
	          if(ier_num.ne.0) then
	            if(md_overlap.eq.MD_ERROR) then
	              return
	            elseif(md_overlap.eq.MD_WARNING) then
	              call errlist
	            endif
	          endif
	          call do_ins_atom(mk_at_lis(mk_iscat(i)),werte,ianz)
	          cr_md_dist(cr_natoms) = distance
	          cr_md_next(cr_natoms) = -j
	        endif
	      endif
	    enddo
	    if    (md_ty_fl(1,mc_type(j)).eq.MD_BLOCK) then
c------	------Remove old atoms inside block
	      w(1) = mc_orig(1,j)
	      w(2) = mc_orig(2,j)
	      w(3) = mc_orig(3,j)
	      call micro_block_rem(mk_dim,natoms_old,w)
	    elseif(md_ty_fl(1,mc_type(j)).eq.MD_FUZZY) then
c------	------Remove old atoms inside block
	      w(1) = mc_orig(1,j)
	      w(2) = mc_orig(2,j)
	      w(3) = mc_orig(3,j)
	      call micro_fuzzy_rem(mk_dim,natoms_old,mk_natoms,
     &	                           md_sep_fuz,w)
	    endif
	  elseif(md_ty_fl(2,mc_type(j)).eq.MD_CELL) then
	    continue
	  else
	    continue
	  endif
	enddo
c
c------	set number of microdomains and microdomain types that have been
c	used to modify the structure
c
	if(micro_run) then
	  mc_num_run = mc_num
	  md_num_run = md_num
	else
c
c------	--Place mode, update the number of microdomains that have 
c	  modified the crystal only if all microdomains previous to
c	  the current singly placed microdomain had been used already.
c
	  if(mc_num_run+1.eq.mc_num) then
	    mc_num_run = mc_num
	  endif
	endif
c
	end
c*****7*****************************************************************
	subroutine micro_check_all(u,i,j_start,lins)
c-
c	Checks a direct space psoition <u> as to whether it
c	is inside any microdomain.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
	include 	'microdis.inc'
c
	integer        j_start
c
	integer        i,j
c
	logical        lins,linnen
	real           distance,u(3),dist_orig,dist_def
c
	dist_def = 1.e9
	linnen   = .false.
c
c-----	check with respect to all microdomains
c
	do j=j_start,mc_num
c	  if(       md_ty_fl(2,mc_type(j)) .ne. MD_FREE
c     &	      .and. md_ty_fl(1,mc_type(j)) .ne. MD_FUZZY) then
	  if(       md_ty_fl(1,mc_type(j)) .ne. MD_FUZZY) then
c
c-----	----check with respect to microdomain number j
c
	    call micro_check(u,j,lins,distance,dist_orig)
	    if(linnen .or. lins) then
c
c-----	------atom has been inside before or is inside current microdomain
c
	      if(lins) then
c
c-----	--------atom is inside current microdomain
c
c
c------	--------If atoms has been inside before, produce warning or
c	        error message
c
	        if(linnen) then
	          ier_num = -69
	          ier_typ = ER_APPL
	          write(ier_msg(1),2000) 'Atom No       :',i
	          write(ier_msg(2),2000) 'Microdomain No:',
     &	            -cr_md_next(i)
	          write(ier_msg(3),2000) 'Microdomain No:',j
	          if(md_overlap.eq.MD_ERROR) then
	            return
	          endif
	        endif
c
	        if(cr_md_next(i).eq.0  .or.
     &	           dist_def     .gt.dist_orig) then
c
c-----	----------atom is closer to microdomain origin than before
c
	          dist_def      = dist_orig
	          cr_md_dist(i) = distance
	          cr_md_next(i) = -j
	        endif
	      endif
	      linnen = .true.
	    else
c
c-----	----atom is outside
c
	      if(cr_md_next(i).eq. 0  .or. 
     &           cr_md_dist(i).gt.distance) then
c
c-----	--------atom is closer to microdomain surface than before
c
	        cr_md_dist(i) = distance
	        cr_md_next(i) = j
	      endif
	    endif
c
	  endif
	enddo
c
	lins = linnen
	j    = cr_md_next(i)
c
CDBG
CDBG	if(i.eq.38) then
CDBG	write (output_io,5555) i,cr_pos(1,i),cr_pos(2,i),cr_pos(3,i),
CDBG     &	                cr_md_next(i),cr_md_dist(i)
CDBG5555	format(' Pos: ',i5,3(2x,f8.3), '    md : ',i7,2x,f8.3)
CDBG	endif
c
2000	format(a15,i10)
	end
c*****7*****************************************************************
	subroutine micro_check(u,j,lins,distance,dist_orig)
c-
c	Checks whether the atom, given by its position vector u
c	is inside the microdomain j
c+
	implicit      none
c
	include      'config.inc'
	include      'microdis.inc'
c
	integer       j
	logical       lins
	real          distance,u(3),dist_orig
c
	if    ( md_ty_fl(1,mc_type(j)) .eq. MD_BLOCK  .or.
     &	        md_ty_fl(1,mc_type(j)) .eq. MD_FACE   .or.
     &	        md_ty_fl(1,mc_type(j)) .eq. MD_FUZZY      ) then
c
c------	--Check current atom with regard to all faces
c
	  call micro_check_hkl(u,j,lins,distance,dist_orig)
	elseif( md_ty_fl(1,mc_type(j)) .eq. MD_SPHERE ) then
c
c------	--Check current atom with regard to radius of sphere
c
	  call micro_check_rad(u,j,lins,distance,dist_orig)
	endif
c
	end
c*****7*****************************************************************
	subroutine micro_check_rad(u,j,lins,distance,dist_orig)
c-
c	This subroutine checks the atom number i with regard to microdomain j.
c	if the distance atom - microdomain origin is less or equal to the
c	radius, lins will be set to true.
c	If the atom is inside the microdomain, the distance is reported
c	negative, else positive.
c+
	implicit      none
c
	include      'config.inc'
	include      'microdis.inc'
c
	integer       j
	logical       lins
	real          distance,u(3),dist_orig
c
	integer       k
	logical       lspace
	real          v(3)
	real          do_blen
c
	do k=1,3
	  v(k) = mc_orig(k,j)
	enddo
        lspace    = .true.
        dist_orig = do_blen(lspace,u,v)
        distance  = do_blen(lspace,u,v)-(md_rad(1,mc_type(j))+mc_rad(j))
c
	if(distance .le. 0) then
	  lins     = .true.
	else
	  lins = .false.
	endif
c
	end
c*****7*****************************************************************
	subroutine micro_check_hkl(u,j,lins,distance,dist_orig)
c-
c	This subroutine checks the atom with position vector u with
c	respect to microdomain j.
c	To each of the faces of the microdomain, the symmetry operation
c	corresponding to the orientation is applied.
c	The position of the atom is then checked as to whether it is on the
c	same side of the face as the origin or not (see Sands , Chapter 2.9)
c
c	If (1 - u * hkl) is greater than 0 the point u is on the same side
c	of the plane as the origin. Otherwise it is beyond the plane from  the
c	origin.
c+
	implicit      none
c
	include      'config.inc'
	include      'crystal.inc'
	include      'crystal.cmm'
	include      'microdis.inc'
c
	integer       j
	logical       lins
	real          distance,u(3)
c
	integer       k,l,m,n
	logical       lspace
	real          v(3),w(3),a,dinside,doutside,dist_orig
	real          uu(4),vv(4),d
	real          do_blen
c
	l         =  0
	lins      =  .true.
c
c------	calculate the distance atom microdomain origin
c
	do k=1,3
	  v(k) = mc_orig(k,j)
	enddo
        lspace    = .true.
        dist_orig = do_blen(lspace,u,v)
c
	lspace    =  .false.
	doutside  = -1.0e9
	dinside   =  1.0e9
c
c------	Calculate the vector origin --> atom
	do k=1,3
	  uu(k) = u(k) - mc_orig(k,j)
	enddo
	uu(4) = 1.0
c------	Apply inverse orientation operation to the vector to be considered
	do m=1,4
	  vv(m) = 0.0
	  do n=1,4
	    vv(m) = vv(m) + md_or_inv(m,n,md_ty_fl(3,mc_type(j)))*uu(n)
	  enddo
	enddo
c
c	do while( lins .and. l.lt.md_hkl_n(mc_type(j)) )
c	  l = l + 1
	do l=1,md_hkl_n(mc_type(j))
c
c------	--Loop while the atom is considered inside and there are still hkl's
c	  to be checked
c
c
c-----	--Now check the vector
c
	  d = 0.0
	  do k=1,3
	    d    = d + vv(k) * md_hkl(k,l,mc_type(j))
	    w(k) = md_hkl(k,l,mc_type(j))
	  enddo
c
	  a        = (1-d)/do_blen(lspace,w,v) + mc_rad(j)
c
	  lins     = lins .and. 0.0 .le. a
	  if(a.ge.0) then
	    dinside  = min(dinside,a)
	  else
	    doutside = max(doutside,-a)
	  endif
	enddo
c
	if(lins) then
	  distance = -dinside
	else
	  distance =  doutside
	endif
c
	end
c*****7*****************************************************************
	subroutine micro_block_rem(mk_dim,natoms_old,w)
c-
c	Removes all atoms up to number natoms_old that are inside
c	the block defined by mk_dim.
c+
	implicit      none
c
	include      'config.inc'
	include      'crystal.inc'
	include      'crystal.cmm'
c
	integer       natoms_old,i
	real          mk_dim(3,2),w(3)
c
	do i=1,natoms_old
	  if(cr_iscat(i).ne.0) then
	    if(mk_dim(1,1)+w(1) .le. cr_pos(1,i)       .and. 
     &	       cr_pos(1,i)      .le. mk_dim(1,2)+w(1)  .and.
     &	       mk_dim(2,1)+w(2) .le. cr_pos(2,i)       .and. 
     &	       cr_pos(2,i)      .le. mk_dim(2,2)+w(2)  .and.
     &	       mk_dim(3,1)+w(3) .le. cr_pos(3,i)       .and. 
     &	       cr_pos(3,i)      .le. mk_dim(3,2)+w(3)       ) then
	      cr_iscat(i)   = 0
	      cr_ndel       = cr_ndel+1
	    endif
	  endif
	enddo
c
	end
c*****7*****************************************************************
	subroutine micro_fuzzy_rem(mk_dim,natoms_old,mk_natoms,
     &	                           md_sep_fuz,w)
c-
c	Removes all atoms up to number natoms_old that are inside
c	the microdomain of fuzzy boundary type 
c+
	implicit      none
c
	include      'config.inc'
	include      'crystal.inc'
	include      'crystal.cmm'
c
	integer       natoms_old,i,j,k
	integer       mk_natoms
	logical       lspace
	real          md_sep_fuz
	real          mk_dim(3,2)
	real          u(3),v(3),w(3)
	real          a,b,c
c
	real          do_blen
c
	lspace = .true.
c
c------	Set up extra space around microdomain to allow for outside
c	influence of separation
c
	a = 1.5*md_sep_fuz/cr_a0(1)
	b = 1.5*md_sep_fuz/cr_a0(2)
	c = 1.5*md_sep_fuz/cr_a0(3)
c
c------	Loop over all atoms previously inside the crystal
c
	do i=1,natoms_old
	  if(cr_iscat(i).ne.0) then
	    do k=1,3
	      u(k) = cr_pos(k,i)
	    enddo
	    if(mk_dim(1,1)+w(1)-a .le. cr_pos(1,i)        .and. 
     &	       cr_pos(1,i)        .le. mk_dim(1,2)+w(1)+a .and.
     &	       mk_dim(2,1)+w(2)-b .le. cr_pos(2,i)        .and. 
     &	       cr_pos(2,i)        .le. mk_dim(2,2)+w(2)+b .and.
     &	       mk_dim(3,1)+w(3)-c .le. cr_pos(3,i)        .and. 
     &	       cr_pos(3,i)        .le. mk_dim(3,2)+w(3)+c    ) then
	      do j=cr_natoms-mk_natoms+1,cr_natoms
	        do k=1,3
	          v(k) = cr_pos(k,j)
	        enddo
	        if(do_blen(lspace,u,v).le.md_sep_fuz) then
c
c------	----------Atom is too close to microdomain atom, remove.
c
	          cr_iscat(i)   = 0
	          cr_ndel       = cr_ndel+1
	        endif
	      enddo
	    endif
	  endif
	enddo
c
	end
c*****7*****************************************************************
	subroutine micro_dist(cr_dim,cr_spcgr,cr_a0,cr_win,
     &	                      i1,i2,j1,j2,k1,k2)
c-
c	Creates a paracrystalline distribution with lattice
c	md_base  lattice of the distribution
c	si       FW of the distribution for each base vector
c	mc_orig   Arrray of origins of all unit cells
c	mc_num   number of unit cells
c	md_distr Gaussian distribution (MD_PARA    )
c	         Lattice  distribution (MD_LATTICE)
c	         Random   distribution (MD_RAND)
c	lmd      force distribution onto the grid
c	         of the sublattice (.true.)
c	         allow any position of unit cell origin (.false.)
c	clatt    lattice type of the sublattice
c+
	implicit      none
c
	include      'config.inc'
	include      'microdis.inc'
	include      'errlist.inc'
c
	character*16   cr_spcgr
	integer        ii(3),nijk(3),i1,i2,j1,j2,k1,k2,ni,nj,nk
	integer        i,j,k,l,m
	logical        lprob_corr,lprob_rand
	real           chance
	real           xx(3),aa,ab,ac,rdiv
	real           cr_a0(3),cr_win(3)
	real           cr_dim(3,2)
	real           prob(MAX_MD_TY)
	real           prob_n(MAX_MD_TY)
	real           ad,ptot_n
c
	data lprob_corr,lprob_rand / .true.,.false./
c
	if(md_log_pos) then
	  open(99,file='fort.99',status='unknown')
	  open(98,file='fort.98',status='unknown')
	  write(99,3000) 
	  write(99,3010) cr_spcgr
	  write(99,3020) cr_a0,cr_win
	endif
c
	ni=i2-i1
	nj=j2-j1
	nk=k2-k1
	if(     ni.eq.0 .and. nj.eq.0 .and. nk.eq.0) then
	  nijk(1) = 0
	  nijk(2) = 0
	  nijk(3) = 0
	elseif( ni.eq.0 .and. nj.eq.0 .and. nk.gt.0) then
	  nijk(1) = 0
	  nijk(2) = 0
	  nijk(3) = 1
	elseif( ni.eq.0 .and. nj.gt.0 .and. nk.eq.0) then
	  nijk(1) = 0
	  nijk(2) = 1
	  nijk(3) = 0
	elseif( ni.gt.0 .and. nj.eq.0 .and. nk.eq.0) then
	  nijk(1) = 1
	  nijk(2) = 0
	  nijk(3) = 0
	elseif( ni.eq.0 .and. nj.gt.0 .and. nk.gt.0) then
	  nijk(1) = 0
	  nijk(2) =             nk + 1
	  nijk(3) = 1
	elseif( ni.gt.0 .and. nj.eq.0 .and. nk.gt.0) then
	  nijk(1) =             nk + 1
	  nijk(2) = 0
	  nijk(3) = 1
	elseif( ni.gt.0 .and. nj.gt.0 .and. nk.eq.0) then
	  nijk(1) = nj + 1
	  nijk(2) = 1
	  nijk(3) = 0
	elseif( ni.gt.0 .and. nj.gt.0 .and. nk.gt.0) then
	  nijk(1) = (nj + 1) * (nk + 1)
	  nijk(2) =             nk + 1
	  nijk(3) = 1
	else
	  ier_num = -333
	  ier_typ = ER_APPL
	endif
c	do m=1,3
c	  mc_orig(m,1) = (i1*md_base(m,1)+j1*md_base(m,2)+k1*md_base(m,3))
c
c.Test with reverse order
c	  mc_orig(m,1) = (i2*md_base(m,1)+j2*md_base(m,2)+k2*md_base(m,3))
c      	  ad=                   (chance(md_si(m,1)))
c	  mc_orig(m,1) = mc_orig(m,1)+ad
c	  mc_orig(m,1) = mc_orig(m,1) + md_shift(m)
c	enddo
c
c------	Determine cummulative probabilities for all microdomains 
c
	ptot_n    = 0.0
	do i = md_num_run+1,md_num
	  prob_n(i) = 0.0
	  do j = md_num_run+1,md_num
	    prob_n(i) = prob_n(i) + md_corr(j,i)
	    ptot_n    = ptot_n    + md_corr(j,i)
	  enddo
	enddo
c
c 	write (output_io,*) (mc_orig(i,1),i=1,3)
c 	write (output_io,*) '********'
c
       DO I=I1  ,I2
c
c.Test with reverse order
c      DO I=I2  ,I1, -1
c       II(1) = i2-i
c
        II(1) = I-I1
        xx(1) = FLOAT(I)
c
        DO J=J1  ,J2
c
c        DO J=J2  ,J1, -1
c         ii(2) = j2-j
c
          ii(2) = J-J1
          xx(2) = FLOAT(J)
c
          DO K=K1  ,K2
c         DO K=K2  ,K1, -1
c           ii(3)  = k2-k
c
            ii(3)  = K-K1
            xx(3)  = FLOAT(K)
c
	    if(mc_num.eq.MC_MAX_MICRO) then
	      ier_num = -5
	      ier_typ = ER_APPL
	      return
	    endif
	    mc_num = mc_num + 1
            IF(md_distr .eq. MD_PARA    ) THEN
	      mc_orig(1,mc_num)=0.0
	      mc_orig(2,mc_num)=0.0
	      mc_orig(3,mc_num)=0.0
	      rdiv=0.
c------	------Set the cummulative probabilities back to zero
	      do m=md_num_run+1,md_num
	        prob(m) = 0.0
	      enddo
c
	      do l=1,3
	        if(nijk(l).gt.0) then
	          if(ii(l).gt.0) then
c
c------	------------Add base vector to previous microdomain
c
	            do m=1,3
	              aa                =mc_orig(m,mc_num)         
      	              ab=                mc_orig(m,mc_num-nijk(l)) 
      	              ac=                md_base(m,l)                
      	              ad=                          (chance(md_si(l,m)))
	              mc_orig(m,mc_num)=aa+ab+ac+ad
	              mc_orig(m,mc_num)=mc_orig(m,mc_num)+md_shift(m)
c 	write (output_io,*) aa,ab,ac,ad
c 	write (output_io,*) mc_orig(m,mc_num)
	            enddo
	            call micro_prob(ptot_n,prob,prob_n,lprob_corr,
     &	                            nijk,l)
c
	            rdiv=rdiv+1
	          else
c
c------	------------No previous microdomain exists, calculate base vector 
c
	            do m=1,3
	              mc_orig(m,mc_num)=mc_orig(m,mc_num)+
     &	                                          xx(1)*md_base(m,1)+
     &	                                          xx(2)*md_base(m,2)+
     &	                                          xx(3)*md_base(m,3)
      	              ad=                         chance(md_si(l,m))
	              mc_orig(m,mc_num)=mc_orig(m,mc_num)+ad
	              mc_orig(m,mc_num)=mc_orig(m,mc_num)+md_shift(m)
	            enddo
	            call micro_prob(ptot_n,prob,prob_n,lprob_rand,
     &	                            nijk,l)
	            rdiv=rdiv+1
	          endif
	        endif
	      enddo
c 	write (output_io,*) mc_num, (mc_orig(iii,mc_num),iii=1,3),rdiv
	      rdiv=max(1.,rdiv)
c
            ELSE               
	      do l=1,3
                mc_orig(l,mc_num) = xx(1)*md_base(l,1) +
     &	                            xx(2)*md_base(l,2) +
     &	                            xx(3)*md_base(l,3) +
     &	                                  md_shift(l)
	        if(nijk(l).gt.0) then
	          if(ii(l).gt.0) then
	            call micro_prob(ptot_n,prob,prob_n,
     &	                            lprob_corr,nijk,l)
	          else
	            call micro_prob(ptot_n,prob,prob_n,
     &	                            lprob_rand,nijk,l)
	          endif
	        endif
	      enddo
	      rdiv = 1.
c
	    endif
c
c------	----Determine type of microdomain, coordinates and individual radius
c
	    call micro_type(prob)
	    call micro_coord(rdiv)
	    if    (md_ty_fl(1,mc_type(mc_num)) .eq. MD_BLOCK) then
	      mc_rad(mc_num) = 0.0
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FUZZY) then
	      mc_rad(mc_num) = 0.0
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FACE) then
	      mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_SPHERE) then
	      mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	    endif
	  enddo
	  enddo
	enddo
c
c	write (output_io,*) ' Number of microdomains ',mc_num
c
	if(md_log_pos) then
	  close(98)
	  close(99)
	endif
c
3000	format('Log of microdomain positions created')
3010	format(a)
3020	format(6(1x,f10.6))
	end
C*****7****************************************************************2********
	subroutine micro_rand(cr_spcgr,cr_a0,cr_win,cr_dim)
c-
c	Creates a random distribution of microdomains.
c	The density of microdomains is given by md_dens microdomains per
c	unit cell.
c+
	implicit       none
c
	include        'config.inc'
	include        'microdis.inc'
	include        'random.inc'
	include        'errlist.inc'
c
	character*16   cr_spcgr
	real           cr_a0(3),cr_win(3)
	real           cr_dim(3,2)
c
	integer        MAXBAD
	parameter     (MAXBAD=10)
c
	integer        mx_num,nbad,i,j
	logical        lapart,lspace
	real           crx,cry,crz,distance,rdiv,radsum
	real           u(3),v(3)
	real           prob(MAX_MD_TY),ptot
	real           chance
	real           do_blen
	real           ran1
	real           min_dist
	real           min_sepa
c
	if(md_log_pos) then
	  open(99,file='fort.99',status='unknown')
	  open(98,file='fort.98',status='unknown')
	  write(99,3000)
	  write(99,3010) cr_spcgr
	  write(99,3020) cr_a0,cr_win
	endif
c
	crx    = (cr_dim(1,2)-cr_dim(1,1))
	cry    = (cr_dim(2,2)-cr_dim(2,1))
	crz    = (cr_dim(3,2)-cr_dim(3,1))
	mx_num = (crx + 1) * (cry + 1) * (crz + 1) * md_dens
c
c------	Make sure that the number of microdomains is less than the
c	allowed number of microdomains
c
	if(mx_num.gt.MC_MAX_MICRO) then
	  ier_num = -5
	  ier_typ = ER_APPL
	  return
	endif
c
	rdiv   = 1.0
	lspace = .true.
c
	nbad   = 0
c
c------	Determine cummulative probabilities for all microdomains 
c
	ptot   = 0.0
	do i = md_num_run+1,md_num
	  prob(i) = 0.0
	  do j = md_num_run+1,md_num
	    prob(i) = prob(i) + md_corr(j,i)
	    ptot    = ptot    + md_corr(j,i)
	  enddo
	enddo
c
	min_sepa = 1.e30
	do while (mc_num.lt.mx_num .and. nbad.lt.MAXBAD)
	  mc_num            = mc_num + 1
	  mc_orig(1,mc_num) = cr_dim(1,1) + ran1(idum)*crx
	  mc_orig(2,mc_num) = cr_dim(2,1) + ran1(idum)*cry
	  mc_orig(3,mc_num) = cr_dim(3,1) + ran1(idum)*crz
c
c------	----Determine type of microdomain, coordinates and individual radius
c
	  call micro_type(prob)
	  call micro_coord(rdiv)
	  if    (md_ty_fl(1,mc_type(mc_num)) .eq. MD_BLOCK) then
	    mc_rad(mc_num) = 0.0
	  elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FUZZY) then
	    mc_rad(mc_num) = 0.0
	  elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FACE) then
	    mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	  elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_SPHERE) then
	    mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	  endif
c
	  u(1) = mc_orig(1,mc_num)
	  u(2) = mc_orig(2,mc_num)
	  u(3) = mc_orig(3,mc_num)
c
c------	--Check whether microdomains are separated far enough
c
	  i      = 1
	  lapart = .true.
	  min_dist = 1.e30
	  do while (i.lt.mc_num .and. lapart)
	    v(1)     = mc_orig(1,i)
	    v(2)     = mc_orig(2,i)
	    v(3)     = mc_orig(3,i)
	    distance = do_blen(lspace,u,v)
	    radsum   = md_rad(0,mc_type(mc_num)) + mc_rad(mc_num) +
     &	               md_rad(0,mc_type(i     )) + mc_rad(i)
	    lapart   = lapart .and. (distance.gt.radsum)
	    min_dist = min(min_dist,distance)
	    i        = i + 1
	  enddo
	  if(lapart) then
	    nbad   = 0
	    min_sepa=min(min_sepa,min_dist)
	  else
	    nbad   = nbad + 1
	    mc_num = mc_num - 1
	  endif 
	enddo
c
	if(md_log_pos) then
	  close(98)
	  close(99)
	endif
c
3000	format('Log of microdomain positions created')
3001	format('Log of microdomain positions inside crystal')
3010	format(a)
3020	format(6(1x,f10.6))
c
	end
C*****7****************************************************************2********
	subroutine micro_dist_file(cr_spcgr,cr_a0,cr_win,cr_dim)
c-
c	A 'crystal' file is read and the atom positions are interpreted as
c	origins of the microdomains.
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'micro.inc'
	include       'random.inc'
	include       'errlist.inc'
c
	integer		sav_ncell(3)
	integer		sav_ncatoms
	logical		sav_r_ncell
c
	integer        i,j
	real           rdiv
c
	character*16   cr_spcgr
	real           cr_a0(3),cr_win(3)
	real           cr_dim(3,2)
c
	real           chance
c
	if(md_log_pos) then
	  open(99,file='fort.99',status='unknown')
	  open(98,file='fort.98',status='unknown')
	  write(99,3000)
	  write(99,3010) cr_spcgr
	  write(99,3020) cr_a0,cr_win
	endif
c
c------	Initialize counters for Atom numbers
c
	mk_natoms = 0
	mk_nscat  = 0
	ma_natoms = 0
	mk_at_lis(0) = 'VOID'
	mk_dw(0)     = 0.0
	do i = 1,MK_MAX_SCAT
	  mk_at_lis(i) = ' '
	enddo
c
c------	Now read the pseudo microdomain structure
c
	call readstru(MMAX,MK_MAX_SCAT,
     &	            md_infile      ,mk_name,mk_spcgr,
     &	            mk_a0,mk_win,mk_natoms,mk_nscat,
     &	            mk_dw,mk_at_lis,mk_pos,mk_iscat,
     &	            mk_dim,
     &               ma_natoms,ma_at_lis,ma_dw,ma_pos,ma_iscat,
     &	            sav_ncell,sav_r_ncell,sav_ncatoms)
	if(ier_num.ne.0) then
	  ier_msg(1) = 'Error occured while reading'
	  ier_msg(2) = md_infile
	  return
	endif
c
	if(mk_natoms.gt.MC_MAX_MICRO) then
	  ier_num = -5
	  ier_typ = ER_APPL
	  return
	endif
c
	rdiv = 1.0
c
	j = 0
	do i=1,mk_natoms
	  if(mk_iscat(i).ne.0) then
	    j            = j + 1
	    mc_orig(1,j) = mk_pos(1,i)
	    mc_orig(2,j) = mk_pos(2,i)
	    mc_orig(3,j) = mk_pos(3,i)
	    mc_type(j)   = mk_iscat(i)
	    mc_num       = j
	    call micro_coord(rdiv)
	    if    (md_ty_fl(1,mc_type(mc_num)) .eq. MD_BLOCK) then
	      mc_rad(mc_num) = 0.0
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FUZZY) then
	      mc_rad(mc_num) = 0.0
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_FACE) then
	      mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	    elseif(md_ty_fl(1,mc_type(mc_num)) .eq. MD_SPHERE) then
	      mc_rad(mc_num) = chance(md_rad_si(1,mc_type(mc_num)))
	    endif
	  endif
	enddo
c
	if(md_log_pos) then
	  close(98)
	  close(99)
	endif
c
3000	format('Log of microdomain positions created')
3001	format('Log of microdomain positions inside crystal')
3010	format(a)
3020	format(6(1x,f10.6))
c
	end
C*****7****************************************************************2********
	subroutine micro_coord(rdiv)
c-
c	Determines the coordinates of a microdomain.
c	depending on the fitting of the md to the grid of the direct lattice
c	of the host structure, the coordinates are integer multiples
c	or real multiples of the base vectors of the host. The centering
c	of the space group is taken into account.
c+
	implicit       none
c
	real           rdiv
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'microdis.inc'
c
	integer        m,iii
	real           pf(3,3),pi(3,3),pa(3,3),pb(3,3),pc(3,3)
	real           pp(3,3)
	real           if(3,3),ii(3,3),ia(3,3),ib(3,3),ic(3,3)
	real           ip(3,3)
c
	data pf/ 0.5, 0.5, 0.0,  0.5, 0.0, 0.5,  0.0, 0.5, 0.5/
	data pi/ 0.5, 0.5,-0.5,  0.5,-0.5, 0.5,  0.0,-0.5, 0.5/
	data pa/ 1.0, 0.0, 0.0,  0.0, 0.5,-0.5,  0.0, 0.5, 0.5/
	data pb/ 0.5, 0.0,-0.5,  0.0, 1.0, 0.0,  0.5, 0.0, 0.5/
	data pc/ 0.5,-0.5, 0.0,  0.5, 0.5, 0.0,  0.0, 0.0, 1.0/
	data pp/ 1.0, 0.0, 0.0,  0.0, 1.0, 0.0,  0.0, 0.0, 1.0/
c
	data if/ 1.0, 1.0,-1.0,  1.0,-1.0, 1.0, -1.0, 1.0, 1.0/
	data ii/ 1.0, 1.0, 0.0,  1.0, 0.0, 1.0,  0.0, 1.0, 1.0/
	data ia/ 1.0, 0.0, 0.0,  0.0, 1.0, 1.0,  0.0,-1.0, 1.0/
	data ib/ 1.0, 0.0, 1.0,  0.0, 1.0, 0.0, -1.0, 0.0, 1.0/
	data ic/ 1.0, 1.0, 0.0, -1.0, 1.0, 0.0,  0.0, 0.0, 1.0/
	data ip/ 1.0, 0.0, 0.0,  0.0, 1.0, 0.0,  0.0, 0.0, 1.0/
c
	if( md_grid.eq. MD_G_FREE) then
	  do m=1,3
	    mc_orig(m,mc_num) = (mc_orig(m,mc_num)/rdiv)
	  enddo
	elseif( md_grid.eq. MD_G_HOST) then
	  if    (cr_spcgr(1:1).eq.'P') then
	    call micro_coord_host(ip,pp,rdiv)
	  elseif(cr_spcgr(1:1).eq.'F') then
	    call micro_coord_host(if,pf,rdiv)
	  elseif(cr_spcgr(1:1).eq.'I') then
	    call micro_coord_host(ii,pi,rdiv)
	  elseif(cr_spcgr(1:1).eq.'A') then
	    call micro_coord_host(ia,pa,rdiv)
	  elseif(cr_spcgr(1:1).eq.'B') then
	    call micro_coord_host(ib,pb,rdiv)
	  elseif(cr_spcgr(1:1).eq.'C') then
	    call micro_coord_host(ic,pc,rdiv)
	  endif
	endif
c
c 	write (output_io ,*) mc_num, (mc_orig(iii,mc_num),iii=1,3),rdiv
	if(md_log_pos) then
	  write(99,4)  'E1- ', (mc_orig(iii,mc_num),iii=1,3),
     &	                  float(mc_type(mc_num))
	  write(98,44)         (mc_orig(iii,mc_num),iii=1,2)
	endif
c	read (*,*) iii
4	FORMAT(A4,3(2X,F14.6),5X,F8.4)
44	FORMAT(4x,3(2X,F14.6),5X,F8.4)
c
	end
C*****7*****************************************************************
	subroutine micro_coord_host(latt_i,latt_p,rdiv)
c-
c	Shifts the coordinates of a microdomain to the nearest grid
c	point of the primitive lattice that corresponds to the
c	centered lattice.
c	The position is transformed into coordinates of the primitive
c	lattice. Then the position is shifted to the nearest integer 
c	values and the result transformed back to the original lattice.
c	Since the primitive lattice to centered latticces are oblique,
c	micro_coord_host also checks with respect to the six positions
c	at plu or minus one of the base vectors of the primitve lattice.
c	The closest point is chosen. If several at at equal distance,
c	the site is chosen among these by random number generator.
c
c	I have not encounterd situations, were the closest point on the
c	primitive lattice is at any of the other lattice points listed
c	in the comment for "mult". These are the full set of 27 vectors
c	at +- any one of the three base vectors. If your lattice
c	constants are very oblique, the vector at i.e. (1,1,0) might be
c	closer to the microdomain than (1,0,0) or (0,1,0). If you 
c	suspect this, use the full array for "mult". You might 
c	activate the histogramming mechanism assosiated with variable
c	"ii_close".
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'random.inc'
c
	integer        idim
	parameter     (idim=3)
c
	integer        m
	integer        i
c	integer        ii_close(27)
	integer        n_close
	integer        i_close
	integer        n_mult
	logical        lspace
	real           rdiv
	real           latt_i(idim,idim),latt_p(idim,idim)
	real           u(idim),v(idim)
	real           wint(idim)
	real           wintc(idim)
	real           wtry(idim)
	real           w_close(3,27)
	real           dist,dist_min
c
	real           mult(3, 7)
c	real           mult(3,27)
c
	real           do_blen
	real           ran1
c
	data n_mult/ 7/
c	data n_mult/27/
c
	data mult
     &	/
     &	  -1.,  0.,  0.,
     &	   0., -1.,  0.,
     &	   0.,  0., -1.,
     &	   0.,  0.,  0.,
     &	   0.,  0.,  1.,
     &	   0.,  1.,  0.,
     &	   1.,  0.,  0.
     &	                /
c
c    &	/
c    &	  -1., -1., -1.,
c    &	  -1., -1.,  0.,
c    &	  -1., -1.,  1.,
c    &	  -1.,  0., -1.,
c    &	  -1.,  0.,  0.,
c    &	  -1.,  0.,  1.,
c    &	  -1.,  1., -1.,
c    &	  -1.,  1.,  0.,
c    &	  -1.,  1.,  1.,
c    &	   0., -1., -1.,
c    &	   0., -1.,  0.,
c    &	   0., -1.,  1.,
c    &	   0.,  0., -1.,
c    &	   0.,  0.,  0.,
c    &	   0.,  0.,  1.,
c    &	   0.,  1., -1.,
c    &	   0.,  1.,  0.,
c    &	   0.,  1.,  1.,
c    &	   1., -1., -1.,
c    &	   1., -1.,  0.,
c    &	   1., -1.,  1.,
c    &	   1.,  0., -1.,
c    &	   1.,  0.,  0.,
c    &	   1.,  0.,  1.,
c    &	   1.,  1., -1.,
c    &	   1.,  1.,  0.,
c    &	   1.,  1.,  1.
c    &	                 /

c
	n_close  = 0
	dist_min = 1.e9
	lspace   = .true.
c
	do m=1,idim
	  u(m) = mc_orig(m,mc_num)/rdiv
	enddo
	call matvec(latt_i,u,v,idim)
	do m=1,idim
	  wint(m) = float(nint(v(m)))
	enddo
	call matvec(latt_p,wint,wintc,idim)
c
	do i=1,n_mult
	  wtry(1) = wintc(1) + mult(1,i) * latt_p(1,1) 
     &	                     + mult(2,i) * latt_p(1,2) 
     &	                     + mult(3,i) * latt_p(1,3) 
	  wtry(2) = wintc(2) + mult(1,i) * latt_p(2,1) 
     &	                     + mult(2,i) * latt_p(2,2) 
     &	                     + mult(3,i) * latt_p(2,3) 
	  wtry(3) = wintc(3) + mult(1,i) * latt_p(3,1) 
     &	                     + mult(2,i) * latt_p(3,2) 
     &	                     + mult(3,i) * latt_p(3,3) 
	  dist = do_blen(lspace,wtry,u)
	  if(dist.lt.dist_min) then
	    dist_min           = dist
	    n_close            = 1
	    w_close(1,n_close) = wtry(1)
	    w_close(2,n_close) = wtry(2)
	    w_close(3,n_close) = wtry(3)
c	    ii_close( n_close) = i
	  elseif(dist.eq.dist_min) then
	    n_close = n_close + 1
	    w_close(1,n_close) = wtry(1)
	    w_close(2,n_close) = wtry(2)
	    w_close(3,n_close) = wtry(3)
c	    ii_close( n_close) = i
	  endif
	enddo
c
	i_close = int(ran1(idum) * n_close) + 1
	u(1)    = w_close(1,i_close)
	u(2)    = w_close(2,i_close)
	u(3)    = w_close(3,i_close)
c	write(55,*) ii_close(i_close)
c
	do m=1,idim
	  mc_orig(m,mc_num) = u(m)
	enddo
c
	end
C*****7*****************************************************************
	subroutine micro_prob(ptot_n,prob,prob_n,lprev,nijk,l)
c-
c	Set up the cummulative probabilities. 
c	if a previous microdomain exists, add probabilities
c	according to the correlation matrix, else 
c	an average probality for the
c	specific microdomain type i.e. sum corr(i,m) /md_num
c	is added.
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
c
	integer        m,nijk(3),l
	logical        lprev
	real           prob  (MAX_MD_TY),ptot_n
	real           prob_n(MAX_MD_TY)
c
	if(lprev) then
	  do m=md_num_run+1,md_num
	    prob(m) = prob(m) + md_corr(mc_type(mc_num-nijk(l)),m)
	  enddo
	else
	  do m=md_num_run+1,md_num
	    prob(m) = prob(m) + prob_n(m)/ptot_n
	  enddo
	endif
c
	end
C*****7*****************************************************************
	subroutine micro_type(prob)
c-
c	Determine type of microdomain from weighted probabilities
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'random.inc'
	include       'errlist.inc'
c
	real           prob(MAX_MD_TY)
c
	integer        m
	real           ad,s,ptot
	real           ran1
c
	ptot = 0.0
	do m=md_num_run+1,md_num
	  ptot = ptot + prob(m)
	enddo
	ad = ran1(idum)*ptot
	s  = prob(1)
	m  = md_num_run+1
c
	do while( s.lt.ad .and. m.lt.md_num)
	  m = m + 1
	  s = s + prob(m)
	enddo
	mc_type(mc_num) = m
c	write (output_io,*) ' Md Type(mc_num) ',mc_num,mc_type(mc_num)
c
	end
C*****7*****************************************************************
	subroutine micro_hkl_closed
c-
c	Checks whether a form of faces appears to be closed
c+
	implicit       none
c
	include       'config.inc'
	include       'prompt.inc'
	include       'microdis.inc'
	include       'errlist.inc'
c
c
	integer        i,ii
	logical        lclosed,lins
	real           u(3),distance,dist_orig
c
c------	setup a dummy microdomain at 0,0,0. This microdomain
c	is used for the check
c
	ii           = mc_num + 1
	mc_orig(1,ii) = 0.0
	mc_orig(2,ii) = 0.0
	mc_orig(3,ii) = 0.0
c
c------	loop over all microdomain types. If boundary type is face,
c	perform check
c
	do i=md_num_cre+1,md_num
	  if(md_ty_fl(1,i) .eq. MD_FACE) then
	    lclosed      = .true.
	    mc_type(ii)  = i
c
c------	----Check whether microdomain faces form a closed form
c	    the origin should be inside, the other points outside
c
	    u(1)    =  0.0
	    u(2)    =  0.0
	    u(3)    =  0.0
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. lins
c
	    u(1)    =  1.0e9
	    u(2)    =  0.0
	    u(3)    =  0.0
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    u(1)    = -1.0e9
	    u(2)    =  0.0
	    u(3)    =  0.0
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    u(1)    =  0.0
	    u(2)    =  1.0e9
	    u(3)    =  0.0
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    u(1)    =  0.0
	    u(2)    = -1.0e9
	    u(3)    =  0.0
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    u(1)    =  0.0
	    u(2)    =  0.0
	    u(3)    =  1.0e9
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    u(1)    =  0.0
	    u(2)    =  0.0
	    u(3)    = -1.0e9
	    call micro_check_hkl(u,ii,lins,distance,dist_orig)
	    lclosed = lclosed .and. .not.lins
c
	    if(.not.lclosed) then
	      write (output_io,2000) i
	      ier_num = -34
	      ier_typ = ER_APPL
	      goto 9999
	    endif
	  endif
	enddo
9999	continue
c
2000	format(' Microdomain type number :',i4)
c
	end
C*****7*****************************************************************
	subroutine micro_hkl
c-
c	Replaces the indices of the microdomain boundaries by hkl that 
c	correspond to the actual radius of this face
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'micro.inc'
	include       'errlist.inc'
c
	integer		sav_ncell(3)
	integer		sav_ncatoms
	logical		sav_r_ncell
c
c
	integer        i,j,k
	logical        lspace
	real           u(3),v(3),dstar

	real           do_blen
c
	v(1) = 0.0
	v(2) = 0.0
	v(3) = 0.0
	lspace = .false.
c
	do i=md_num_run+1,md_num
c
	  if    (md_ty_fl(1,i) .eq. MD_BLOCK .or.
     &	         md_ty_fl(1,i) .eq. MD_FUZZY      ) then
c
c------	----Initialize counters for Atom numbers
c
	    mk_natoms = 0
	    mk_nscat  = 0
	    ma_natoms = 0
	    mk_at_lis(0) = 'VOID'
	    mk_dw(0)     = 0.0
	    do j = 1,MK_MAX_SCAT
	      mk_at_lis(j) = ' '
	    enddo
c
c------	----Now read the microdomain structure
c
	    call readstru(MMAX,MK_MAX_SCAT,
     &	                md_co_fi(1)    ,mk_name,mk_spcgr,
     &	                mk_a0,mk_win,mk_natoms,mk_nscat,
     &	                mk_dw,mk_at_lis,mk_pos,mk_iscat,
     &	                mk_dim,
     &                   ma_natoms,ma_at_lis,ma_dw,ma_pos,ma_iscat,
     &	                sav_ncell,sav_r_ncell,sav_ncatoms)
c	    100 Face
	    u(1)          =  mk_dim(1,2) + 0.0001
	    u(2)          =  0.0
	    u(3)          =  0.0
	    lspace        =  .true.
	    md_rad(1,i)   =  max(0.005,do_blen(lspace,u,v))
	    u(1)          =  1.0
	    u(2)          =  0.0
	    u(3)          =  0.0
	    lspace        =  .false.
	    dstar         =  do_blen(lspace,u,v)
	    md_hkl(1,1,i) =  1.0 /dstar/md_rad(1,i)
	    md_hkl(2,1,i) =  0.0
	    md_hkl(3,1,i) =  0.0
c	    -100 Face
	    u(1)          =  mk_dim(1,1) - 0.0001
	    u(2)          =  0.0
	    u(3)          =  0.0
	    lspace        =  .true.
	    md_rad(2,i)   =  max(0.005,do_blen(lspace,u,v))
	    md_hkl(1,2,i) = -1.0 /dstar/md_rad(2,i)
	    md_hkl(2,2,i) =  0.0
	    md_hkl(3,2,i) =  0.0
c
c	    010 Face
	    u(1)          =  0.0
	    u(2)          =  mk_dim(2,2) + 0.0001
	    u(3)          =  0.0
	    lspace        =  .true.
	    md_rad(3,i)   =  max(0.005,do_blen(lspace,u,v))
	    u(1)          =  0.0
	    u(2)          =  1.0
	    u(3)          =  0.0
	    lspace        =  .false.
	    dstar         =  do_blen(lspace,u,v)
	    md_hkl(1,3,i) =  0.0
	    md_hkl(2,3,i) =  1.0 /dstar/md_rad(3,i)
	    md_hkl(3,3,i) =  0.0
c	    0-10 Face
	    u(1)          =  0.0
	    u(2)          =  mk_dim(2,1) - 0.0001
	    u(3)          =  0.0
	    lspace        =  .true.
	    md_rad(4,i)   =  max(0.005,do_blen(lspace,u,v))
	    md_hkl(1,4,i) =  0.0
	    md_hkl(2,4,i) = -1.0 /dstar/md_rad(4,i)
	    md_hkl(3,4,i) =  0.0
c
c	    001 Face
	    u(1)          =  0.0
	    u(2)          =  0.0
	    u(3)          =  mk_dim(3,2) + 0.0001
	    lspace        =  .true.
	    md_rad(5,i)   =  max(0.005,do_blen(lspace,u,v))
	    u(1)          =  0.0
	    u(2)          =  0.0
	    u(3)          =  1.0
	    lspace        =  .false.
	    dstar         =  do_blen(lspace,u,v)
	    md_hkl(1,5,i) =  0.0
	    md_hkl(2,5,i) =  0.0
	    md_hkl(3,5,i) =  1.0 /dstar/md_rad(5,i)
c	    00-1 Face
	    u(1)          =  0.0
	    u(2)          =  0.0
	    u(3)          =  mk_dim(3,1) - 0.0001
	    lspace        =  .true.
	    md_rad(6,i)   =  max(0.005,do_blen(lspace,u,v))
	    md_hkl(1,6,i) =  0.0
	    md_hkl(2,6,i) =  0.0
	    md_hkl(3,6,i) = -1.0 /dstar/md_rad(6,i)
c
	    md_hkl_n(i)   = 6
	  elseif(md_ty_fl(1,i) .eq. MD_FACE) then
	    lspace = .false.
	    do j=1,md_hkl_n(i)
	      do k=1,3
	        u(k) = md_hkl(k,j,i)
	      enddo
	      dstar = do_blen(lspace,u,v)
	      do k=1,3
	        md_hkl(k,j,i) = md_hkl(k,j,i)/dstar/
     &	                        max(0.001,md_rad(j,i))
	      enddo
	    enddo
	  endif
	enddo
c
	end
C*****7*****************************************************************
	subroutine micro_purge
c-
c	Checks whether a microdomain is (at least partially) within
c	the original crystal
c	if not, the microdomain is purged from  the list of microdomains.
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'errlist.inc'
	include       'microdis.inc'
c
	integer        i,j,k,idel,iii
	real           rr,xmin,xmax,ymin,ymax,zmin,zmax
c
	if(md_log_pos) then
	  open(97,file='fort.97',status='unknown')
	  write(97,3001) 
	  write(97,3010) cr_spcgr
	  write(97,3020) cr_a0,cr_win
	endif
c
	idel = 0
	do i=mc_num_run+1,mc_num
	  if    ( md_ty_fl(1,mc_type(i)) .eq. MD_BLOCK  .or.
     &	      md_ty_fl(1,mc_type(i)) .eq. MD_FACE       ) then
	    rr = 0
	    do j=1,md_hkl_n(mc_type(i))
	      rr = max(rr,md_rad(j,mc_type(i)))
	    enddo
	    rr = rr * 2.0
	  elseif( md_ty_fl(1,mc_type(i)) .eq. MD_SPHERE ) then
	    rr = md_rad(1,mc_type(i))
	  elseif( md_ty_fl(1,mc_type(i)) .eq. MD_FUZZY  ) then
	    call md_get_dim(mc_type(i))
	    rr = 0
	    do k = 1,3
	      rr = max(rr,abs(md_dim(k,1)))
	      rr = max(rr,abs(md_dim(k,2)))
	    enddo
	  endif
c
	  if(md_ty_fl(1,mc_type(i)) .ne. MD_FUZZY ) then
c
c------	----The purging is not applied to Fuzzy microdomains, since
c	    their size is to inpredictable
c
	    xmax = mc_orig(1,i) - rr
	    xmin = mc_orig(1,i) + rr
	    ymax = mc_orig(2,i) - rr
	    ymin = mc_orig(2,i) + rr
	    zmax = mc_orig(3,i) - rr
	    zmin = mc_orig(3,i) + rr
	    IF(.not. (  cr_dim(1,1) .LE. xmin          .AND.
     &	              xmax          .LE.   cr_dim(1,2) .AND.
     &	                cr_dim(2,1) .LE. ymin          .AND.
     &	              ymax          .LE.   cr_dim(2,2) .AND.
     &	                cr_dim(3,1) .LE. zmin          .AND.
     &	              zmax          .LE.   cr_dim(3,2) ) )  THEN
	      idel         = idel +1
	    else
	      mc_type(  i-idel) = mc_type(  i)
	      mc_orig(1,i-idel) = mc_orig(1,i)
	      mc_orig(2,i-idel) = mc_orig(2,i)
	      mc_orig(3,i-idel) = mc_orig(3,i)
	      mc_rad (  i-idel) = mc_rad (  i)
 	    endif
 	  endif
	enddo
c
	mc_num = mc_num - idel
c
	if(md_log_pos) then
	  do i=1,mc_num
	    write(97,4)  'E1- ', (mc_orig(iii,i),iii=1,3),
     &	                  float(mc_type(i))
	  enddo
	  close(97)
	endif
c
4	format(A4,3(2X,F14.6),5X,F6.4)
3001	format('Log of microdomain positions inside crystal')
3010	format(a)
3020	format(6(1x,f10.6))
c
	end
c*****7*****************************************************************
      	subroutine indmic(rbase,cr_dim,mk_dim,I1,I2,J1,J2,K1,K2)
c-
c     	Determines the indices for microdomain lattices. The corners
c	of the crystal plus half a layer of microdomains are transformed 
c	into indizes of the lattices. The values are rounded to the
c	next highest integer for the upper boundaries and the next 
c	lowest integer for the lower boundaries.
c
c	The transformation needs the transpose of the reciprocal base matrix.
c
	implicit     	none
c
	integer         i,ii,j,jj,k,kk
	integer         i_low,i_high
	integer         j_low,j_high
	integer         k_low,k_high
	integer 	i1,i2,j1,j2,k1,k2
	real    	cr_dim(3,2),mk_dim(3,2)
      	real 		rbase(3,3),u(3),v(3)
c
c------ Calculate coordinates of the crystal corners in the space
c------ of base vectors
c
	i1 =  1000000
	i2 = -1000000
	j1 =  1000000
	j2 = -1000000
	k1 =  1000000
	k2 = -1000000
CDBG
CDBG	write (output_io,*) 'mk_dim ',mk_dim(1,1),mk_dim(1,2)
CDBG	write (output_io,*) '       ',mk_dim(2,1),mk_dim(2,2)
CDBG	write (output_io,*) '       ',mk_dim(3,1),mk_dim(3,2)
CDBG	write (output_io,*) 'cr_dim ',cr_dim(1,1),cr_dim(1,2)
CDBG	write (output_io,*) '       ',cr_dim(2,1),cr_dim(2,2)
CDBG	write (output_io,*) '       ',cr_dim(3,1),cr_dim(3,2)
c
	call transmat(rbase,3)
c
	do i=-1,1,2
	  ii = 1 + int((i+1.)*0.5)
	  do j=-1,1,2
	    jj = 1 + int((j+1.)*0.5)
	    do k=-1,1,2
	      kk = 1 + int((k+1.)*0.5)
	      v(1)=cr_dim(1,ii)+sign(mk_dim(1,ii),float(i))*0.5
	      v(2)=cr_dim(2,jj)+sign(mk_dim(2,jj),float(j))*0.5
	      v(3)=cr_dim(3,kk)+sign(mk_dim(3,kk),float(k))*0.5
	      call trans(v,rbase,u,3)
CDBG
CDBG	write (output_io,*) ' v       ',v
CDBG	write (output_io,*) ' u       ',u
CDBG	write (output_io,*) ' i,j,k   ',i,j,k
CDBG	write (output_io,*) ' ii,jj,kk',ii,jj,kk
	      if(u(1).ge.0) then
	        i_low  =  int(u(1))
	        i_high = nint(u(1)+sign(0.5,u(1)))
	      else
	        i_low  = nint(u(1)+sign(0.5,u(1)))
	        i_high =  int(u(1))
	      endif
	      if(u(2).ge.0) then
	        j_low  =  int(u(2))
	        j_high = nint(u(2)+sign(0.5,u(2)))
	      else
	        j_low  = nint(u(2)+sign(0.5,u(2)))
	        j_high =  int(u(2))
	      endif
	      if(u(3).ge.0) then
	        k_low  =  int(u(3))
	        k_high = nint(u(3)+sign(0.5,u(3)))
	      else
	        k_low  = nint(u(3)+sign(0.5,u(3)))
	        k_high =  int(u(3))
	      endif
CDBG
CDBG	write (output_io,*) ' i_low,u(1),i_high ',i_low,u(1),i_high
CDBG	write (output_io,*) ' j_low,u(2),j_high ',j_low,u(2),j_high
CDBG	write (output_io,*) ' k_low,u(3),k_high ',k_low,u(3),k_high
	      i1= min(i1,i_low)
	      i2= max(i2,i_high)
	      j1= min(j1,j_low)
	      j2= max(j2,j_high)
	      k1= min(k1,k_low)
	      k2= max(k2,k_high)
	    enddo
	  enddo	
	enddo
c
	call transmat(rbase,3)
c
CDBG
CDBG	write (output_io,*) ' i1,i2,j1,j2,k1,k2 ', i1,i2,j1,j2,k1,k2
	end
c*****7*****************************************************************
	real function chance(a)
c-
c	Determines a gaussian distributed value
c-
	implicit      none
c
	real          a
	real          gasdev
c
1     	continue
	if(a.ne.0.0) then
      	  chance=GASDEV(A)
	else
	  chance=0.0
	endif
      end
c*****7*****************************************************************
	subroutine show_microdist(ianz,cpara,maxw)
c-
c	Displays the current parameters for microdomain distributions
c+
	implicit       none
c
	include        'config.inc'
	include        'prompt.inc'
	include        'microdis.inc'
	include        'errlist.inc'
c
	integer        maxw
	character*200  cpara(maxw)
	integer        ianz
c
	character*10   md_ctype(0:2,0:4)
	integer        i,j,k,m
c
c
	data           md_ctype
     &	/'          ',   ! 0,0
     &	 '          ',   ! 1,0
     &	 '          ',   ! 2,0
     &	 '          ',   ! 0,1
     &	 'Face      ',   ! 1,1
     &	 'Unit cell ',   ! 2,1
     &	 '          ',   ! 0,2
     &	 'Sphere    ',   ! 1,2
     &	 'Structure ',   ! 2,2
     &	 '          ',   ! 0,3
     &	 'Block     ',   ! 1,3
     &	 'Free mod. ',   ! 2,3
     &	 '          ',   ! 0,4
     &	 'Fuzzy     ',   ! 1,4
     &	 '          ' /  ! 2,4
	ier_num = -6
	ier_typ = ER_COMM
c
c------	Distribution
c
	if(ianz.eq.0 .or. cpara(1)(1:2).eq.'di') then
	  write (output_io,3000)
	  write (output_io,3010) (( md_base (j,i),j=1,3),i=1,3)
	  write (output_io,3015) (( md_rbase(j,i),j=1,3),i=1,3)
	  write (output_io,3016)  ( md_shift(i  ),i=1,3)
	  if(md_distr.eq.MD_PARA    ) then
	    write (output_io,3020) ((sqrt(md_si(i,m)),m=1,3),i=1,3)
	  elseif(md_distr.eq.MD_LATTICE) then
	    write (output_io,3030)
	  elseif(md_distr.eq.MD_RAND) then
	    write (output_io,3035) md_dens
	  elseif(md_distr.eq.MD_FILE) then
	    write (output_io,3036) md_infile
	  endif
	  if( md_grid .eq.MD_G_HOST) then
	    write (output_io,3040)
	  elseif(md_grid .eq.MD_G_FREE) then
	    write (output_io,3050)
	  endif
	  ier_num = 0
	  ier_typ = ER_NONE
	endif
c
c------	Defaults
c
	if(ianz.eq.0 .or. cpara(1)(1:2).eq.'de') then
	  write (output_io,3060) md_ctype(1,mv_bound)
	  if(mv_bound.eq.1) then
	    write (output_io,3061)
	    do i=1,mv_hkl_n
	      write (output_io,3065) (mv_hkl(j,i),j=1,3)
	    enddo
	  endif
	  write (output_io,3070) md_ctype(2,mv_cont),
     &	                mv_file,mv_orient,mv_rad,mv_rad_si,md_sep_fuz
	  ier_num = 0
	  ier_typ = ER_NONE
	endif
c
c------	Microdomain types 
c
	if(ianz.eq.0 .or. cpara(1)(1:2).eq.'ty') then
	  write (output_io,4000)
	  do i=1,md_num
	    if    (md_ty_fl(1,i).eq.MD_FACE) then
	      write (output_io,4010) i,md_ctype(1,md_ty_fl(1,i)),
     &	                      md_ctype(2,md_ty_fl(2,i)),
     &	                      md_ty_fl(3,i),
     &	                      md_rad(1,i),
     &	                     (md_hkl(j,1,i),j=1,3),
     &	                      md_rad_si(1,i)
	      do k=2,md_hkl_n(i)
	        write (output_io,4015)  md_rad(k,i), 
     &	                      (md_hkl(j,k,i),j=1,3),
     &	                       md_rad_si(k,i)
	      enddo
	    elseif(md_ty_fl(1,i).eq.MD_SPHERE) then
	      write (output_io,4020) i,md_ctype(1,md_ty_fl(1,i)),
     &	                      md_ctype(2,md_ty_fl(2,i)),
     &	                      md_ty_fl(3,i),
     &	                      md_rad(1,i),
     &	                      md_rad_si(1,i)
	    elseif(md_ty_fl(1,i).eq.MD_BLOCK) then
	      write (output_io,4020) i,md_ctype(1,md_ty_fl(1,i)),
     &	                      md_ctype(2,md_ty_fl(2,i)),
     &	                      md_ty_fl(3,i),
     &	                      md_rad(1,i),
     &	                      md_rad_si(1,i)
	    elseif(md_ty_fl(1,i).eq.MD_FUZZY) then
	      write (output_io,4020) i,md_ctype(1,md_ty_fl(1,i)),
     &	                      md_ctype(2,md_ty_fl(2,i)),
     &	                      md_ty_fl(3,i),
     &	                      md_rad(1,i),
     &	                      md_rad_si(1,i)
	    endif
	    if(md_ty_fl(2,i).ne.MD_FREE) then
	      write (output_io,4040)   md_co_fi(i)
	    endif
	  enddo
	  ier_num = 0
	  ier_typ = ER_NONE
	endif
c
c------	Correlation matrix
c
	if(ianz.eq.0 .or. cpara(1)(1:2).eq.'co') then
	  write (output_io,5000)
	  do i=1,md_num
	    write (output_io,5010) (md_corr(i,j),j=1,md_num)
	  enddo
	  ier_num = 0
	  ier_typ = ER_NONE
	endif
c
c------	Orientation of microdomains
c
	if(ianz.eq.0 .or. cpara(1)(1:2).eq.'or') then
	  write (output_io,6000)
	  do i=1,md_ori_n
	    write (output_io,6020) i
	    do k=1,4
	      write (output_io,6010) (md_orient(k,j,i),j=1,4)
	    enddo
	  enddo
	  ier_num = 0
	  ier_typ = ER_NONE
	endif
	ier_num = 0
	ier_typ = ER_NONE
c
3000	format(/'****************************************************'//
     &	       ' Characterisation of microdomain distribution'/)
3010	format(' Base vectors           : ',3(f8.3,2x)/,
     &	       '                          ',3(f8.3,2x)/,
     &	       '                          ',3(f8.3,2x)/)
3015	format(' Reciprocal base vectors: ',3(f8.3,2x)/,
     &	       '                          ',3(f8.3,2x)/,
     &	       '                          ',3(f8.3,2x)/)
3016	format(' Shift of origin          ',3(f8.3,2x))
3020	format(/' Distribution type      : Paracrystalline Distribution '/
     &	        ' Sigma                  : ',3(f8.3,2x)/
     &	        '                          ',3(f8.3,2x)/,
     &	        '                          ',3(f8.3,2x))
3030	format(/' Distribution type      :Lattice Distribution')
3035	format(/' Distribution type      : Random Distribution'/
     &	        '   density              : ',  f8.3,
     &	        ' microdomains/unit cell')
3036	format(/' Distribution type      :Input from file     '/
     &	        '                        : ',a)
3040	format(' Forced fit to host lattice')
3050	format(' No fit to host lattice')
c
3060	format(/'****************************************************'//
     &	       ' Current Default values for :'/
     &	       ' Boundary                             : ',a)
3061	format(43x,'h         k         l')
3065	format(40x,3(f8.3,2x))
3070	format(/' Content                              : ',a/
     &	        ' Filename                             : ',a/
     &	        ' Number of Orientation                : ',i3/
     &	        ' Radius of boundary surface           : ',f8.3/
     &	        ' Sigma of boundary surface radius     : ',f8.3/
     &	        ' Minimum Separation at fuzzy boundary : ',f8.3/)
c
4000	format(/'****************************************************'//
     &          'Microdomain types'/
     &	       /' Num Boundary     Content Orient.  Radius',
     &	        '     h         k         l')
4010	format(/1x,i3,2(2x,a10),2x,i3,f8.3,2x,3(f8.3,2x)/
     &	        33x,                  f8.3)
4015	format(/33x,                  f8.3,2x,3(f8.3,2x)/
     &	        33x,                  f8.3)
4020	format(/1x,i3,2(2x,a10),2x,i3,f8.3,2x/
     &	        33x,                  f8.3)
4040	format(/18x,a)
c
5000	format(/'****************************************************'//
     &	       /' Correlation matrix'/)
5010	format(8(2x,f8.3))
c
6000	format(/'****************************************************'//
     &	       /' Orientation of Microdomains'/)
6020	format(/' Number : ',i3/)
6010	format('  ( ',2(f8.3,2x),f8.3,' )   ( ',f8.3,' )')
	end
C*****7*****************************************************************
	subroutine micro_inv
c-
c	This subroutine calculates the inverse symmetry operations for
c	the different orientations of the microdomains.
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'errlist.inc'
c
	integer        i,j,k
	real           matrix(4,4)
c
	do i=1,md_ori_n
	  do j=1,4
	    do k=1,4
	      matrix(j,k) = md_orient(j,k,i)
	    enddo
	  enddo
	  call invmat4(matrix)
	  do j=1,4
	    do k=1,4
	      md_or_inv(j,k,i) = matrix(j,k)
	    enddo
	  enddo
	enddo
	end
C*****7*****************************************************************
	subroutine micro_corr_check
c-
c	This subroutine sets the elements of the correlation elements
c	to zero for all microdomains that are undefined.
c	If all elements of the correlation matrix are zero, an error
c	message is supplied.
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
	include       'errlist.inc'
c
	integer        i,j
	real           cc
c
	do i=md_num_cre+1,md_num
	  if(md_ty_fl(1,i).eq.0) then
c
c------	----undefined microdomain, set correlation to 0.0
c
	    do j=md_num_cre+1,md_num
	      md_corr(j,i) = 0.0
	      md_corr(i,j) = 0.0
	    enddo
	  endif
	enddo
c
c------	check for non zero elements in correlation matrix
c
	cc = 0.0
c
	do i=md_num_cre+1,md_num
	  do j=md_num_cre+1,md_num
	    cc = max(cc,md_corr(j,i))
	  enddo
	enddo
	if(cc.gt.0) then
	  ier_num = 0
	  ier_typ = ER_NONE
	else
	  ier_num = -39
	  ier_typ = ER_APPL
	endif
c
	end
C*****7*****************************************************************
	subroutine micro_reset
c-
c	Sets the microdomain status back to zero
c+
	implicit       none
c
	include       'config.inc'
	include       'microdis.inc'
c
	integer        k,i,j
c
	md_num     = 0
	md_num_cre = 0
	md_num_run = 0
	md_distr   = MD_PARA
	md_infile  = ' '
	md_dens    = 0.1
	md_grid    = MD_G_HOST
c
	md_overlap = MD_ERROR
c
	do i=1,4
	  do j=0,MAX_MD_TY
	    md_ty_fl(i,j)=0
	  enddo
	enddo
c
	md_hkl_n(0)=0
	do i=1,MAX_MD_TY
	  md_hkl_n(i)=0
	enddo
c
	md_ori_n   = 1
	md_log_pos = .false.
	md_sep_fuz = 0.5
c
	do i=0,MAX_MD_TY
	  do j=0,MAX_MD_TY
	    md_corr(i,j)=0.0
	  enddo
	  md_corr(i,i)=1.0
	enddo
c
	do i=1,3
	  md_shift(i)=0.0
	  do j=1,3
	    md_base(i,j)=0.0
	    md_si  (i,j)=0.0
	  enddo
	enddo
c
	do i=1,3
	  do j=1,MAX_MD_HKL
	    do k=0,MAX_MD_TY	
	      md_hkl(i,j,k)=0.0
	    enddo
	  enddo
	enddo
c
	do k=1,MAX_MD_ORI
	  do i=1,4
	    do j=1,4
	      md_orient(i,j,k)=0.0
	      md_or_inv(i,j,k)=0.0
	    enddo
	    md_orient(i,i,k)=1.0
	    md_or_inv(i,i,k)=1.0
	  enddo
	enddo
c
	do i=0,MAX_MD_HKL
	  do j=0,MAX_MD_TY
	    md_rad(i,j)=0.0
	  enddo
	enddo
c
	do i=1,MAX_MD_HKL
	  do j=0,MAX_MD_TY
	    md_rad_si(i,j)=0.0
	  enddo
	enddo
c
	do i=1,3
	  do j=1,3
	    md_rbase(i,j)=0.0
	  enddo
	enddo
c
	do i=0,MAX_MD_TY
	  md_co_fi(i)=' '
	enddo
c
	do i=1,3
	  md_dim(i,1)=  100000.0
	  md_dim(i,2)= -100000.0
	enddo
c
c------ common /microlis/
c
	mc_num     = 0
	mc_num_run = 0
c
	do i=0,MC_MAX_MICRO
	  mc_type(i)=0
	enddo
c
	do j=0,MC_MAX_MICRO
	  mc_rad(j)=0.0
	  do i=1,3
	    mc_orig(i,j)=0.0
	  enddo
	enddo
c
	mv_hkl_n = 0
c
	end
C*****7*****************************************************************
	subroutine matvec(matrix,vecin,vecout,idim)
c 
	implicit       none
	integer        idim
	integer        i,j
	real           matrix(idim,idim)
	real           vecin (idim)
	real           vecout(idim)
c
	do i=1,3
	  vecout(i) = 0.0
	  do j=1,idim
	    vecout(i) = vecout(i) + matrix(i,j)*vecin(j)
	  enddo
	enddo
c
	end
C*****7*****************************************************************
	subroutine  typemat(md_base, idim)
c
	implicit	none
c
	include		'prompt.inc'
c	
	integer		idim
	real md_base(idim,idim)
c
	integer		i
c
	write (output_io,1) (md_base(1,i),i=1,3)
	write (output_io,2) (md_base(2,i),i=1,3)
	write (output_io,3) (md_base(3,i),i=1,3)
1	format(' Matrix(1,*) :', 3(f15.8,2x))
2	format(' Matrix(2,*) :', 3(f15.8,2x))
3	format(' Matrix(3,*) :', 3(f15.8,2x),/)
	end
