c
c	Common block for the microdomain distributions
c
c	MAX_MD_TY	Maximum number of different microdomain types
c	MAX_MD_HKL	Maximum number of boundary faces
c	MAX_MD_ORI	Maximum number of different orientations
c
c			symbolic parameters :
c	MD_PARA		The separation between microdomains is a 
c			gaussian distribution
c	MD_LATTICE	The microdomains form a lattice
c	MD_RAND		The microdomains are randomly distributed
c	MD_FILE		The microdomains are read from file
c
c	MD_G_HOST	The origins coincide with the host lattice
c	MD_G_FREE	The origins may be of any value
c
c	MD_FACE		The microdomain is bounded by faces
c	MD_SPHERE	The microdomain is bounded by a sphere
c	MD_BLOCK	The microdomain is bounded by a block
c	MD_FUZZY	The microdomain is bounded by a fuzzy surface
c
c	MD_CELL		The microdomain is generated from a unit cell
c	MD_STRUC	The microdomain is generated from a structure
c	MD_FREE		The microdomain is modified by the user
c
c	MD_ANG		Radii are understood as Anstroem
c	MD_DLU		Radii are understood as multiples of lattice
c			constant a
c
c	MC_MAXMICRO	Maximum nuber of microdomains in the crystal
c                       (set in 'config.inc')
c
c	MD_WARNING      Errors result in warnings, program proceeds
c	MD_ERROS        Errors result in errors, program stops
c
c			Variables describing the distribution
c
c	md_distr	distribution type
c	md_infile         input file for microdomain distribution
c	md_grid		Fit of microdomain origins to host lattice
c	md_hkl_n	number of boundary faces for each type
c	md_num		number of microdomain types
c	md_num_cre	number of microdomain types already created
c	md_num_run	number of microdomain types already run
c	md_ty_fl	flags characterizing each microdomain type
c			(1,*) boundary type
c			(2,*) content type
c			(3,*) orientation number
c			(4,*) reserved for later use
c	md_base		base vectors in real space
c	md_corr		correlation matrix
c	md_dens		density of randomly distributed microdomains
c			of fuzzy microdomains
c	md_hkl		indices of boundary faces for each type
c	md_orient	orientation matrices
c	md_rad		radius for each boundary face for each type
c	md_rad_si	sigma for radii
c	md_rbase	base vectors in reciprocal space
c	md_sep_fuz	 Minumum distance between atoms along the boundary
c	md_si		sigma for base vectors
c	md_shift	constant shift of microdomain origins
c	md_dim          Maximal dimensions of microdomains
c
c			Variables serving as defaults
c
c	mv_bound	current boundary type
c	mv_cont		current content
c	mv_hkl_n	current number of boundary faces
c	mv_orient	current orientation number
c	mv_hkl		indices of bounday faces
c	mv_rad		current default radius
c	mv_rad_si	current default siogma
c	mvv_rad		radii of boundary faces defined already
c	mv_rad_si	sigma of boundary faces
c
c			Variables describing the actual microdomains
c
c	mc_num		number of microdomains in the crystal
c	mc_num_run	number of microdomains already used in the crystal
c	mc_type		type of each microdomain
c	mc_orig		origin of each microdomain
c	mc_rad		individual radius summand
c
c	md_overlap      Statusflag for overlap handling
c
c*****7*****************************************************************
c
	integer        MD_ERROR,MD_WARNING
	parameter     (MD_ERROR   =  0)
	parameter     (MD_WARNING =  1)
c
	integer        MAX_MD_TY
	parameter     (MAX_MD_TY = 12)
c
	integer        MAX_MD_HKL
	parameter     (MAX_MD_HKL = 12)
c
	integer        MAX_MD_ORI
	parameter     (MAX_MD_ORI = 12)
c
	integer        MD_PARA,  MD_LATTICE,  MD_RAND,  MD_FILE
	parameter     (MD_PARA=1,MD_LATTICE=2,MD_RAND=3,MD_FILE=4)
c
	integer        MD_G_HOST,  MD_G_FREE
	parameter     (MD_G_HOST=1,MD_G_FREE=2)
c
	integer        MD_FACE,  MD_SPHERE,  MD_BLOCK  ,MD_FUZZY
	parameter     (MD_FACE=1,MD_SPHERE=2,MD_BLOCk=3,MD_FUZZY=4)
c
	integer        MD_CELL,  MD_STRUC,  MD_FREE
	parameter     (MD_CELL=1,MD_STRUC=2,MD_FREE=3)
c
	integer        MD_ANG,  MD_DLU
	parameter     (MD_ANG=1,MD_DLU=2)
c
c##############################################################################
c
	integer        BL_MD_TY_FL
	parameter     (BL_MD_TY_FL  = 4*(1+MAX_MD_TY) )
c
	integer        BL_MD_CORR
	parameter     (BL_MD_CORR   = (1+MAX_MD_TY)*(1+MAX_MD_TY) )
c
	integer        BL_MD_HKL
	parameter     (BL_MD_HKL    = 3*(MAX_MD_HKL)*(1+MAX_MD_TY) )
c
	integer        BL_MD_ORIENT
	parameter     (BL_MD_ORIENT = 16*MAX_MD_ORI )
c
	integer        BL_MD_RAD
	parameter     (BL_MD_RAD    = (1+MAX_MD_HKL)*(1+MAX_MD_TY) )
c
	integer        BL_MD_RAD_SI
	parameter     (BL_MD_RAD_SI = (  MAX_MD_HKL)*(1+MAX_MD_TY) )
c
	integer        BL_MV_HKL
	parameter     (BL_MV_HKL    = 3*MAX_MD_HKL )
c
	integer        BL_MC_ORIG
	parameter     (BL_MC_ORIG   = 3*(1+MC_MAX_MICRO) )
c
c##############################################################################
c
	character*40   md_co_fi(0:MAX_MD_TY)
	character*40   mv_file
	character*40   md_infile
c
	integer        md_distr
	integer        md_grid
	integer        md_hkl_n  (0:MAX_MD_TY)
	integer        md_num
	integer        md_num_cre
	integer        md_num_run
	integer        md_ori_n
	integer        md_ty_fl(4,0:MAX_MD_TY)
	real           md_dim(3,2)
c
	integer        mv_bound
	integer        mv_cont
	integer        mv_hkl_n
	integer        mv_orient
	integer        mv_unit
c
	integer        md_overlap
c
	logical        md_log_pos
c
	real           md_base   (3,3)
	real           md_corr   (0:MAX_MD_TY ,0:MAX_MD_TY)
	real           md_dens
	real           md_hkl    (3,MAX_MD_HKL,0:MAX_MD_TY)
	real           md_orient (4,4         ,  MAX_MD_ORI)
	real           md_or_inv (4,4         ,  MAX_MD_ORI)
	real           md_rad    (0:MAX_MD_HKL,0:MAX_MD_TY)
	real           md_rad_si (  MAX_MD_HKL,0:MAX_MD_TY)
	real           md_rbase  (3,3)
	real           md_sep_fuz
	real           md_si     (3,3)
	real           md_shift  (3)
c
	real           mv_hkl    (3,MAX_MD_HKL)
	real           mv_rad
	real           mv_rad_si
	real           mvv_rad   (  MAX_MD_HKL)
	real           mvv_rad_si(  MAX_MD_HKL)
c
	integer        mc_num
	integer        mc_num_run
	integer        mc_type(0:MC_MAX_MICRO)
c
	real           mc_orig (3,0:MC_MAX_MICRO)
	real           mc_rad  (  0:MC_MAX_MICRO)
c
	common /microdis/ md_num,md_num_cre,md_num_run,md_distr,
     &	                  md_infile,md_dens,md_grid,
     &	                  md_ty_fl,md_hkl_n,md_ori_n,
     &	                  md_log_pos,md_sep_fuz,
     &	                  md_corr,md_base,md_si,md_shift,
     &	                  md_hkl,md_orient,md_or_inv,md_rad,
     &	                  md_rad_si,md_rbase,md_co_fi,
     &	                  md_overlap,md_dim
c
	common /microdef/ mv_cont,mv_orient,mv_rad,mv_rad_si,
     &	                  mvv_rad,mvv_rad_si,mv_hkl_n,mv_hkl,
     &	                  mv_bound,mv_unit,mv_file
c
	common /microlis/ mc_num,mc_num_run,mc_type,mc_orig,mc_rad
