c*****7*****************************************************************
	subroutine rmc
c-
c	This sublevel includes all commands and functions for the
c	Reverse-Monte-Carlo simulations in DISCUS. 
c
c	Version  : 1.61
c	Date     : 12 January 1998
c
c	Author   : Th. Proffen (tproffen@lanl.gov)
c
c+
	implicit	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'random.inc'
	include 	'doact.inc'
	include 	'macro.inc'
	include 	'errlist.inc'
	include 	'learn.inc'
	include 	'param.inc'
	include 	'prompt.inc'
	include 	'molecule.inc'
	include 	'rmc.inc'
c
	integer       	maxw
	parameter    	(maxw=MAXSCAT)
c
	character*5    	befehl
	character*11   	prom
	character*40   	cdummy
	character*200  	line,zeile,cpara(maxw)
	integer        	lpara(maxw),lp,length
	integer		repl(0:MAXSCAT)
	integer        	indxg,ianz,lbef
	integer        	i,j,idummy
c
	logical         str_comp
c
10	continue
c
	call no_error
	prom=prompt//'/rmc'
	call get_command (line,length,befehl,lbef,zeile,lp,prom)
	if(ier_num.eq.0) then
	if(line.eq.' '.or.line(1:1).eq.'#') goto 10
c
c------ search for "="
c
	indxg=index(line,'=')
	if(indxg.ne.0.and..not.(str_comp(befehl,'echo',2,lbef,4))
     &	             .and..not.(str_comp(befehl,'syst',2,lbef,4))
     &	             .and..not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                        str_comp(befehl,'?   ',2,lbef,4)) ) then
          call do_math(line,indxg,length)
c
c------ execute a macro file
c
        elseif(befehl(1:1).eq.'@') then
          call file_kdo(line(2:length),length-1)
c
c------ continues a macro 'continue'
c
        elseif(str_comp(befehl,'continue',3,lbef,8)) then
          call macro_continue(zeile,lp)
c
c------ selecting/deselecting atoms
c
	elseif(str_comp(befehl,'sele',3,lbef,4) .or.
     &	       str_comp(befehl,'dese',2,lbef,4)     ) then
c
	call atom_select(zeile,lp,rmc_allowed,repl,0,0,maxscat,
     &	                 rmc_sel_atom,idummy,idummy,.false.,
     &	                 str_comp(befehl,'sele',3,lbef,4))
c
c------ selecting/deselecting of molecules
c
	elseif(str_comp(befehl,'msel',2,lbef,4) .or.
     &	       str_comp(befehl,'mdes',2,lbef,4)     ) then
c
	call mole_select(zeile,lp,rmc_allowed,repl,0,0,maxscat,
     &	                 rmc_sel_atom,idummy,idummy,.false.,
     &	                 str_comp(befehl,'msel',2,lbef,4))
c
c------ read experimental data 'data'
c
	elseif(str_comp(befehl,'data',3,lbef,4)) then
	  call rmc_readdata(zeile,lp)
c
c------ Echo a string, just for interactive check in a macro 'echo'
c
	elseif(str_comp(befehl,'echo',2,lbef,4)) then
	  call echo(zeile,lp)
c
c------ Evaluate an expression
c
	elseif(str_comp(befehl,'eval',2,lbef,4)) then
	  call do_eval(zeile,lp)
c
c------	exit 'exit'
c
	elseif(str_comp(befehl,'exit',3,lbef,4)) then
	  goto 9999
c
c------	help 'help','?'
c
	elseif(str_comp(befehl,'help',2,lbef,4) .or.
     &	       str_comp(befehl,'?   ',1,lbef,4)     ) then
	  if(str_comp(zeile,'errors',2,lp,6) ) then
	    lp = lp + 7
	    call do_hel('discus '//zeile,lp)
	  else
	    lp = lp + 12
	    call do_hel('discus rmc '//zeile,lp)
	  endif
c
c------ reset rmc setting 'rese'
c
	elseif(str_comp(befehl,'rese',3,lbef,4)) then
	  rmc_nplane   = 0
	  rmc_calc_f   = .true.
	  rmc_qmin     =  9999.0
	  rmc_qmax     = -9999.0
c
	  do i=1,RMC_MAX_PLANES
	    offq(i)=0
	    do j=1,RMC_MAX_SYM
	      offsq(i,j)=0
	    enddo
	  enddo
c
c------ run 'run'
c
	elseif(str_comp(befehl,'run',2,lbef,3)) then
	  call rmc_run
c
c-----	save structure to file 'save'
c
	elseif(str_comp(befehl,'save',2,lbef,4)) then
	  call rmc_save (zeile,lp)
c
c------ set rmc parameters 'set'
c
	elseif(str_comp(befehl,'set',3,lbef,3)) then
	  call rmc_set(zeile,lp)
c
c------ show rmc parameters 'show'
c
	elseif(str_comp(befehl,'show',3,lbef,4)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.eq.0) then
	    if (ianz.eq.0) then
	      call rmc_show ('ALL')
	    elseif (ianz.eq.1) then
	      call do_cap(cpara(1))
	      if (cpara(1)(1:2).eq.'RV') then
	        call rmc_rvalue
	      else
	        call rmc_show (cpara(1))
	      endif
	    else
	      ier_typ = ER_COMM
	      ier_num = -6
	    endif
	  endif
c
c-------Operating System Kommandos 'syst'
c
	  elseif(str_comp(befehl,'syst',2,lbef,4)) then
	    cdummy=' '
	    if(zeile.ne. ' ') then
	      cdummy(1:lp)=zeile(1:lp)
	      call do_operating(cdummy,lp)
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
c
c------	Waiting for user input
c
	  elseif(str_comp(befehl,'wait',3,lbef,4)) then
	    call do_input(zeile,lp)
c
c------ no command found
c
	else
	  ier_num = -8
	  ier_typ = ER_COMM
	endif
	endif
c
c------ any errors ?
c
	if(ier_num.ne.0) then
	  call errlist
	  if(lmakro) then
	    call macro_close
	    prompt_status = PROMPT_ON
	  endif
	  if(lblock) then
	    ier_num = -11
	    ier_typ = ER_COMM
	    return
	  endif
	  call no_error
	endif
	goto 10
c
9999	continue
c
	end
c*****7*****************************************************************
	subroutine rmc_set (zeile,lp)
c+
c	sets most parameters for 'set' section
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
	include 	'chem.inc'
c
	integer		maxw
	parameter	(maxw=maxscat)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	real		werte(maxw),wwerte(maxw)
	real		mmdis,hklmin(3),hklmax(3),quad
	integer		lpara(maxw)
	integer		ianz,iianz,jjanz,is,js,i,j,ii,ie1,ie2,ip
	logical		flag
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.eq.0) then
	  if (ianz.ge.2) then
	    call do_cap(cpara(1))
c
c------ --- 'set aver': sets the portion of the crystal used to calc <F>
c
	    if (cpara(1)(1:2).eq.'AV') then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      if (ianz.eq.1) then
	        call ber_params (ianz,cpara,lpara,werte,maxw)
	        if (ier_num.eq.0) then
	          if (werte(1).ge.0.0 .and. werte(1).le.100.0) then
	            rmc_ave    = werte(1)*0.01
	            rmc_calc_f = .true.
	          else
	            ier_num = -1
	            ier_typ = ER_FOUR
	          endif
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ --- 'set back': background correction for all planes on/off
c
	     elseif (cpara(1)(1:2).eq.'BA') then
	       call do_cap(cpara(2))
	       rmc_doback = (cpara(2)(1:2).eq.'ON')
	       if (ianz.eq.3 .or. ianz.eq.4) then
	         call del_params (2,ianz,cpara,lpara,maxw)
	         if (ier_num.eq.0) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
	           if (ianz.eq.2) then
	             ip=nint(werte(2))
	             if (ip.gt.0 .and. ip.le.rmc_max_planes) then     
	               rmc_back(ip) = werte(1)
	             else
  	               ier_num = -6
	               ier_typ = ER_COMM
	             endif
	           else
	             do i=1,rmc_max_planes
	               rmc_back(i) = werte(1)
	             enddo
	           endif
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       else
	         do i=1,rmc_max_planes
	           rmc_back(i) = 0.0 
	         enddo
	       endif
c
c------ --- 'set constrain': setting possible skaling/background constrains
c
	     elseif(cpara(1)(1:2).eq.'CO') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.eq.2) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
		   ie1 = nint(werte(1))
		   ie2 = nint(werte(2))
	           if (ie1.gt.0 .and. ie1.le.rmc_max_planes .and.
     &	               ie2.gt.0 .and. ie2.le.rmc_max_planes .and.
     &	               ie2.le.ie1                                ) then
	             rmc_constrain(ie1) = ie2
	           else
	             ier_num = -19
	             ier_typ = ER_RMC
	           endif
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set cycl': setting of number of RMC cycles
c
	     elseif(cpara(1)(1:2).eq.'CY') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.eq.1) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
	           if (ier_num.ne.0) return
	           rmc_maxcyc = int(werte(1))
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set data' : setting data type for input data
c
	     elseif(cpara(1)(1:2).eq.'DA') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.ge.1) then
	           call do_cap(cpara(1))
	           if (cpara(1)(1:1).eq.'N') then
	             rmc_data = rmc_data_nipl
	           elseif (cpara(1)(1:1).eq.'P') then
	             rmc_data = rmc_data_pgm
	           else
	             ier_num = -15
	             ier_typ = ER_RMC
	           endif
	         else
	           ier_num = -6
 	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set dbw'  : debye-waller factor control flag setting
c
	     elseif(cpara(1)(1:2).eq.'DB') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.ge.1) then
	           call do_cap(cpara(1))
	           flag = (cpara(1)(1:2).eq.'ON')
	           if (ianz.gt.1) then
	             call del_params (1,ianz,cpara,lpara,maxw)
	             call  ber_params(ianz,cpara,lpara,werte,maxw)
	             do i=1,ianz
	               ii=nint(werte(i))
	               if (ii.gt.0 .and. ii.le.rmc_nplane) then
	                 rmc_ldbw(ii)=flag
	               else
	                 ier_num = -7
	                 ier_typ = ER_RMC
	               endif
	             enddo
	           else
	             do i=1,rmc_nplane
	               rmc_ldbw(i)=flag
	             enddo
	           endif  
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set disp' : setting of output intervall
c
	     elseif(cpara(1)(1:2).eq.'DI') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.eq.1) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
	           rmc_display = max(1,int(werte(1)))
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set log': Output RMC progress in rmc.log - on/off
c
	    elseif (cpara(1)(1:3).eq.'LOG') then
	      call do_cap(cpara(2))
	      rmc_log = (cpara(2)(1:2).eq.'ON')
c
c------	--- 'set lots' : Setting of RMC lots
c
	    elseif (cpara(1)(1:3).eq.'LOT') then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      if (ier_num.eq.0) then
	        if (ianz.eq.1) then
	          call do_cap (cpara(1))
	          if (cpara(1)(1:1).eq.'O') then
	            rmc_ilots  = LOT_OFF
	            rmc_nlots  = 1
	            rmc_calc_f = .true.
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        elseif (ianz.ge.6) then
	          call do_cap (cpara(1))
	          if (cpara(1)(1:1).eq.'B') then
	            rmc_ilots = LOT_BOX
	            rmc_calc_f = .true.
	          elseif (cpara(1)(1:1).eq.'E') then
	            rmc_ilots = LOT_ELI
	            rmc_calc_f = .true.
	          else
	            ier_num = -2
	            ier_typ = ER_FOUR
	          endif
	          if (ier_num.eq.0) then
	            call del_params (1,ianz,cpara,lpara,maxw)
	            call ber_params (4,cpara,lpara,werte,maxw)
	            if (ier_num.eq.0) then
	              if (nint(werte(4)).le.RMC_MAX_LOTS) then
	                ls_xyz(1) = nint(werte(1))
	                ls_xyz(2) = nint(werte(2))
	                ls_xyz(3) = nint(werte(3))
	                rmc_nlots = nint(werte(4))
	                call do_cap(cpara(5))
	                lperiod = (cpara(5)(1:1).eq.'Y')
	                if (ianz.gt.5) then
	                  call del_params (5,ianz,cpara,lpara,maxw)
	                  call do_build_name (ianz,cpara,lpara,
     &	                                      werte,maxw,1)
	                  if (ier_num.eq.0) then
	                    rmc_ranloc = .FALSE.
	                    rmc_lname = cpara(1)
	                  endif
	                else
	                  rmc_ranloc = .TRUE.
	                endif
	              else
	                ier_num = -21
	                ier_typ = ER_RMC
	              endif
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------ --- 'set mdis' : sets minimal allowed atom distances
c
	     elseif(cpara(1)(1:2).eq.'MD') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.eq.3) then
	           iianz=1
	           call get_iscat(iianz,cpara(1),lpara(1),werte,
     &	                          maxw,.false.)
	           if (ier_num.eq.0) then
	             jjanz=1
	             call get_iscat(jjanz,cpara(2),lpara(2),wwerte,
     &	                            maxw,.false.)
	             if (ier_num.eq.0) then
	               call del_params (2,ianz,cpara,lpara,maxw)
	               call ber_params (1,cpara,lpara,mmdis,1)
	               if (ier_num.eq.0) then
	                 do i=1,iianz
	                   do j=1,jjanz
	                     is=int( werte(i))
	                     js=int(wwerte(j))
	                     call rmc_set_mindist (is,js,mmdis)
	                   enddo
	                 enddo
	               endif
	             endif
	           endif
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set mode': sets operation mode for RMC
c
	     elseif(cpara(1)(1:3).eq.'MOD') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.ne.0) return
	       call rmc_set_mode(rmc_mode,rmc_local,ianz,cpara,
     &	                         lpara,maxw)
c
c------ --- 'set move': sets factor for generated moves
c
	     elseif(cpara(1)(1:3).eq.'MOV') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.ne.0) return
	       call rmc_set_move(rmc_maxmove,rmc_sel_atom,ianz,
     &	                         cpara,werte,lpara,maxw)
c
c------ --- 'set range': set q-range to be used in RMC runs
c
	     elseif (cpara(1)(1:3).eq.'RAN') then
	       if (ianz.eq.2) then
	         call do_cap(cpara(2))
	         if(cpara(2)(1:3).eq.'ALL') then
	           rmc_llim = rmc_qmin
	           rmc_ulim = rmc_qmax
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       elseif (ianz.eq.3) then
	         call del_params (1,ianz,cpara,lpara,maxw)
	         if (ier_num.ne.0) return
	         call ber_params(ianz,cpara,lpara,werte,maxw)
	         rmc_llim = werte(1)/2.0
	         rmc_ulim = werte(2)/2.0
	       elseif (ianz.eq.7) then
	         call del_params (1,ianz,cpara,lpara,maxw)
	         if (ier_num.ne.0) return
	         call ber_params(ianz,cpara,lpara,werte,maxw)
	         hklmin (1) = werte(1)
	         hklmin (2) = werte(2)
	         hklmin (3) = werte(3)
	         hklmax (1) = werte(4)
	         hklmax (2) = werte(5)
	         hklmax (3) = werte(6)
	         rmc_llim = sqrt(quad(hklmin,hklmin,cr_rten)/4.0)
	         rmc_ulim = sqrt(quad(hklmax,hklmax,cr_rten)/4.0)
	       else
	         ier_num = -6
	         ier_typ = ER_COMM
	       endif
c
c------ --- 'set scal': scalingfactor for all planes on/off
c
	     elseif (cpara(1)(1:3).eq.'SCA') then
	       call do_cap(cpara(2))
	       rmc_doskal = (cpara(2)(1:2).eq.'ON')
	       if (ianz.eq.3 .or. ianz.eq.4) then
	         call del_params (2,ianz,cpara,lpara,maxw)
	         if (ier_num.eq.0) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
	           if (ianz.eq.2) then
	             ip=nint(werte(2))
	             if (ip.gt.0 .and. ip.le.rmc_max_planes) then     
	               rmc_skal(ip) = 1.0 / werte(1)
	             else
  	               ier_num = -6
	               ier_typ = ER_COMM
	             endif
	           else
	             do i=1,rmc_max_planes
	               rmc_skal(i) = 1.0 / werte(1)
	             enddo
	           endif
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       else
	         do i=1,rmc_max_planes
	           rmc_skal(i) = 1.0 
	         enddo
	       endif
c
c------ --- 'set sigma': sets SIGMA for CHI2 calculation 
c
	     elseif(cpara(1)(1:2).eq.'SI') then
	       call del_params (1,ianz,cpara,lpara,maxw)
	       if (ier_num.eq.0) then
	         if (ianz.eq.1) then
	           call ber_params(ianz,cpara,lpara,werte,maxw)
	           rmc_sigma = werte(1)
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       endif
c
c------ --- 'set sym': include symmetry of measured data on/off
c
	    elseif (cpara(1)(1:3).eq.'SYM') then
	      call do_cap(cpara(2))
	      rmc_dosym  = (cpara(2)(1:2).eq.'ON')
	      rmc_calc_f = .true.
c
	      if (rmc_dosym .and. rmc_nosym) then
	        rmc_dosym = .false.
	        ier_num = -13
	        ier_typ = ER_RMC
	      endif
c
c------ --- unknown subcommand entered
c
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_set_move(move,sel_atom,ianz,cpara,werte,
     &	                        lpara,maxw)
c+
c	Sets RMC/MC maximum shift in 'shift' mode
c-
	implicit      none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'molecule.inc'
	include 	'rmc.inc'
	include 	'errlist.inc'
c
	integer		maxw,maxww
	parameter	(maxww = 4)
c
	character*200	cpara(maxw)
	real		move(3,0:MAXSCAT)
	real		werte(maxw)
	real		wwerte(maxww)
	integer		lpara(maxw)
	integer		ianz,ii,is,i,j
	logical		sel_atom
c
c------ Atoms
c
	if (sel_atom) then
	  if (ianz.eq.4) then
	    ii=ianz-3
	    call get_iscat (ii,cpara,lpara,werte,maxw,.false.)
	    if (ier_num.eq.0) then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      call ber_params (ianz,cpara,lpara,wwerte,maxw)
	      is=int(werte(1))
	      if (is.ne.-1) then
	        do i=1,ii
	          is=int(werte(i))
	          do j=1,3
	            move(j,is)=wwerte(j)
	          enddo
	        enddo
	      else
	        do i=1,cr_nscat
	          do j=1,3
	            move(j,i)=wwerte(j)
	          enddo
	        enddo
	      endif 
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ Molecules
c
	else
	  if (ianz.eq.4) then
	    if (cpara(1)(1:1).eq.'a') then
	      is = -1
	    else
	      call ber_params (1,cpara,lpara,werte,maxw)
	      is = nint(werte(1))
	      if (is.le.0 .or. is.gt.mole_num_type) then
	        ier_num = -64
	        ier_typ = ER_APPL
	        return
	      endif
c
	      call del_params (1,ianz,cpara,lpara,maxw)
	      call ber_params (ianz,cpara,lpara,wwerte,maxw)
	      if (is.ne.-1) then
	        do j=1,3
	          move(j,is)=wwerte(j)
	        enddo
	      else
	        do i=1,mole_num_type
	          do j=1,3
	            move(j,i)=wwerte(j)
	          enddo
	        enddo
	      endif 
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_set_mode(imode,ilocal,ianz,cpara,lpara,maxw)
c+
c	Sets RMC/MC mode
c-
	implicit      none
c
	include 	'config.inc'
	include 	'errlist.inc'
	include 	'rmc.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,imode,ilocal
c
	if (ianz.ge.1) then
	  call do_cap (cpara(1))
	  if (cpara(1)(1:3).eq.'SHI') then
	    imode = rmc_mode_shift
	  elseif (cpara(1)(1:3).eq.'SWD') then
	    imode = rmc_mode_swdisp
	  elseif (cpara(1)(1:3).eq.'SWC') then
	    imode = rmc_mode_swchem
	  elseif (cpara(1)(1:3).eq.'EXT') then
	    imode = rmc_mode_external
	  else
	    ier_typ = ER_RMC
	    ier_num = -9
	  endif
c
	  if (ianz.eq.2) then
	    call do_cap(cpara(2))
	    if (cpara(2)(1:1).eq.'A') then
	      ilocal = rmc_local_all
	    elseif (cpara(2)(1:1).eq.'L') then
	      ilocal = rmc_local_loc
	    elseif (cpara(2)(1:2).eq.'SL') then
	      ilocal = rmc_local_locsite
	    elseif (cpara(2)(1:2).eq.'SI') then
	      ilocal = rmc_local_site
	    else
	      ier_typ = ER_RMC
	      ier_num = -9
	    endif
	  else
	    ilocal = rmc_local_all
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_set_mindist(is,js,dist)
c+
c	Set minimal allowed atom distances
c-
c
	implicit      none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
	include 	'rmc.inc'
c
	integer		is,js,ii,jj
	real		dist
c

	if     (is.ne.-1 .and. js.ne.-1) then
	  rmc_mindist(is,js) = dist
	  rmc_mindist(js,is) = dist
	elseif (is.eq.-1 .and. js.ne.-1) then
	  do ii=1,cr_nscat
	    rmc_mindist(ii,js) = dist
	    rmc_mindist(js,ii) = dist
	  enddo  
	elseif (is.ne.-1 .and. js.eq.-1) then
	  do ii=1,cr_nscat
	    rmc_mindist(ii,is) = dist
	    rmc_mindist(is,ii) = dist
	  enddo  
	else
	  do ii=1,cr_nscat
	    do jj=1,cr_nscat
	      rmc_mindist(ii,jj) = dist
	      rmc_mindist(jj,ii) = dist
	    enddo
	  enddo  
	endif  
c
	end
c*****7*****************************************************************
	subroutine rmc_rvalue
c+
c	Calculate R-values
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'errlist.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	real		sumd,sumo,tsumd,tsumo
	real		cint,dint,oo,dd,r4
	integer		isym(rmc_max_planes)
	integer		ip,is,iq
c
	tsumd = 0.0
	tsumo = 0.0
c
	do ip=1,rmc_nplane
	  if (rmc_dosym) then
	    isym(ip) = rmc_nsym(ip)
	  else
	    isym(ip) = 1
	  endif
	enddo
c
	write (output_io,1000)
	do ip=1,rmc_nplane
	  do is=1,isym(ip)
	    sumo = 0.0
	    sumd = 0.0
	    call rmc_inten(ip,is,.false.)
	    do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	      cint =rmc_back(ip)+rmc_skal(ip)*dsi(iq)
	      dint =rmc_int(offq(ip)+iq)-cint
	      oo   =rmc_wic(offq(ip)+iq)*rmc_int(offq(ip)+iq)**2
	      dd   =rmc_wic(offq(ip)+iq)*dint**2
	      sumo =sumo + oo
	      sumd =sumd + dd
	      tsumo=tsumo+ oo
	      tsumd=tsumd+ dd
	    enddo
	    r4=0.0
	    if(sumo.ne.0.0) r4=sqrt(sumd/sumo)
	    write (output_io,1500) ip,rmc_nplane,is,isym(ip),r4*100.
	  enddo
	enddo
	r4=0.0
	if(tsumo.ne.0.0) r4=sqrt(tsumd/tsumo)
	write (output_io,1600) r4*100.
c
1000	format (' R-values for current RMC results : ')
1500	format ('    Plane ',i3,'/',i3,', sym. op. ',i2,'/',i2,' : ',
     &	        g12.6,' %')
1600	format ('                    Total R-value : ',g12.6,' %')
	end
c*****7*****************************************************************
	subroutine rmc_show (cmd)
c+
c	Show current RMC settings
c-
c
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
	include 	'diffuse.inc'
	include 	'molecule.inc'
	include 	'rmc.inc'
c
	character*(*)	cmd
	character*25	wic(8)
	character*9	at_lis(maxscat)
	character*8	ra
	integer		mol_lis(mole_max_type)
	integer		i,j,k,ie,len_str
c
	character*9	at_name_i,at_name_j
	character*9	at_name
c
	data 	wic	/'w(Int) = 1.0           ', 
     &			 'w(Int) = sqrt(Int)     ', 
     &			 'w(Int) = log(Int)      ',
     &	     		 'w(Int) = Int           ',   
     &			 'w(Int) = Int**2        ',    
     &			 'w(Int) = 1/Int         ',
     &			 'w(Int) = 1/sqrt(Int)   ', 
     &			 'w(Int) = read from file'/
c
	if (cmd(1:3).eq.'ALL' .or. cmd(1:3).eq.'MOD') then
	  if (rmc_mode.eq.rmc_mode_shift) then
	    write (output_io,1100) 'move atoms/molecules'
	  elseif (rmc_mode.eq.rmc_mode_swchem) then
	    write (output_io,1100) 'switch atoms/molecules'
	  elseif (rmc_mode.eq.rmc_mode_swdisp) then
	    write (output_io,1100) 'switch displacements'
	  elseif (rmc_mode.eq.rmc_mode_external) then
	    write (output_io,1100) 'defined in user supplied subroutine'
	  else
	    write (output_io,1100) '??????????'
	  endif
	  if (rmc_local.eq.rmc_local_all) then
	    write (output_io,1120) 'all'
	  elseif (rmc_local.eq.rmc_local_loc) then
	    write (output_io,1120) 'only local (+-1 unit cell)'
	  elseif (rmc_local.eq.rmc_local_locsite) then
	    write (output_io,1120) 'only local, same site'
	  elseif (rmc_local.eq.rmc_local_site) then
	    write (output_io,1120) 'all, same site'
	  else
	    write (output_io,1120) '??????????'
	  endif
	  if (rmc_doskal .and. rmc_doback) then
	    write (output_io,1150) 'scaling factor and background'
	  elseif (rmc_doskal .and. .not.rmc_doback) then
	    write (output_io,1150) 'scaling factor only'
	  elseif (rmc_doback .and. .not.rmc_doskal) then
	    write (output_io,1150) 'background only'
	  else
	    write (output_io,1150) 'none'
	  endif
c
	  if     (rmc_ilots.eq.LOT_OFF) then
	    write (output_io,1200)
	  elseif (rmc_ilots.eq.LOT_BOX) then
	    write (output_io,1210) rmc_nlots
	    write (output_io,1230) (ls_xyz(i),i=1,3),lperiod
	    if (rmc_ranloc) then
	      write (output_io,1235) 'random lots'
	    else
	      write (output_io,1235) rmc_lname(1:len_str(rmc_lname))
	    endif
	  elseif (rmc_ilots.eq.LOT_ELI) then
	    write (output_io,1220) rmc_nlots
	    write (output_io,1230) (ls_xyz(i),i=1,3),lperiod
	    if (rmc_ranloc) then
	      write (output_io,1235) 'random lots'
	    else
	      write (output_io,1235) rmc_lname(1:len_str(rmc_lname))
	    endif
	  endif
c
	  if (rmc_ave.gt.0.0) write (output_io,1240) rmc_ave*100.0
	  write (output_io,1250) rmc_maxcyc
	  write (output_io,1300) rmc_display,rmc_log
	  write (output_io,1400) rmc_sigma
	endif
c
	if (cmd(1:3).eq.'ALL' .or. cmd(1:3).eq.'DAT') then
	  if (rmc_data.eq.rmc_data_nipl) then
	    write (output_io,1500) 'NIPL'
	  elseif (rmc_data.eq.rmc_data_pgm) then
	    write (output_io,1500) 'PGM '
	  else
	    write (output_io,1500) '????'
	  endif
	  do i=1,rmc_nplane
	    write (output_io,1800) i,rmc_num(1,i),rmc_num(2,i),
     &	                   rmc_fname(i)(1:len_str(rmc_fname(i)))
	    ra = 'Neutrons'
	    if (rmc_lxray(i)) ra = 'X-rays'
	    if (rmc_lambda(i).eq.'    ') then
	      write (output_io,1850) rmc_rlambda(i),ra,rmc_ldbw(i),rmc_ano(i)
	    else
	      write (output_io,1860) rmc_lambda(i),ra,
     &	                             rmc_ldbw(i),rmc_ano(i)
	    endif
	    write (output_io,1900) wic(rmc_wic_typ(i))
	    if (rmc_constrain(i).ne.i) then
	      write (output_io,1950) rmc_constrain(i)
	    endif
	  enddo
	  write (output_io,1970) 2.0*rmc_qmin,2.0*rmc_qmax,
     &	                 2.0*rmc_llim,2.0*rmc_ulim
	endif
c
	if (cmd(1:3).eq.'ALL' .or. cmd(1:3).eq.'SYM') then
	  write (output_io,2000)
	  do i=1,rmc_nplane
	    ie = rmc_nsym(i)
	    if (.not. rmc_dosym) ie=1
	    do j=1,ie
	      write (output_io,2020) i,rmc_nplane,j,rmc_nsym(i)
	      write (output_io,2030) (rmc_eck(k,1,j,i),k=1,3)
	      write (output_io,2040) (rmc_eck(k,2,j,i),k=1,3)
	      write (output_io,2050) (rmc_eck(k,3,j,i),k=1,3)
	      write (output_io,2060) (rmc_vi (k,1,j,i),k=1,3)
	      write (output_io,2070) (rmc_vi (k,2,j,i),k=1,3)
	    enddo
	  enddo
	endif
c
	if (cmd(1:3).eq.'ALL' .or. cmd(1:3).eq.'ATO') then
	  write (output_io,2200)
	  do i=1,cr_nscat
	    do j=i,cr_nscat
	      at_name_i=at_name(i)
	      at_name_j=at_name(j)
	      write (output_io,2300) at_name_i,at_name_j,rmc_mindist(i,j)
	    enddo
	  enddo
c
	  write (output_io,2400)
	  if (rmc_sel_atom) then
	    do i=1,cr_nscat
	      at_name_i=at_name(i)
	      write (output_io,2500) at_name_i,(rmc_maxmove(j,i),j=1,3)
	    enddo
c
	    j=0
	    do i=0,cr_nscat
	      if (rmc_allowed(i)) then
	        j=j+1
	        at_lis(j)=at_name(i)
	      endif
	    enddo
	    write (output_io,2700) (at_lis(k),k=1,j)
c
	  else
	    do i=1,mole_num_type
	      write (output_io,2600) i,(rmc_maxmove(j,i),j=1,3)
	    enddo
c
	    j=0
	    do i=1,mole_num_type
	      if (rmc_allowed(i)) then
	        j=j+1
	        mol_lis(j)=i
	      endif
	    enddo
	    write (output_io,2800) (mol_lis(k),k=1,j)
c
	  endif
	endif
c
1100	format (  ' RMC running mode            : ',A)
1120	format (  '   Valid neighbours          : ',A)
1150	format (  '   Calc. scaling/backgr.     : ',A)
1200	format (  '   Fourier volume            : complete crystal')
1210	format (  '   Fourier volume            : ',I4,
     &	            ' box shaped lots')
1220	format (  '   Fourier volume            : ',I4,
     &	            ' ellipsoid shaped lots')
1230	format (  '   Lot size                  : ',I3,' x ',I3,' x ',I3,
     &	            ' unit cells (per. bound. = ',L1,')')
1235	format (  '   Lot mode (random / file)  : ',A)
1240	format (  '   Calc. of <F> based on     : ',F5.1,' % of crystal')
1250	format (  '   Max. number of RMC cycles : ',I8)
1300	format (  '   Output update intervall   : ',I8, 
     &	          '   (logging to rmc.log=',L1,')')
1400	format (  '   SIGMA for CHI calculation : ',F8.4)
1500	format (/,' Experimental data (',A4,')    : ')
1800	format (/,' Plane ',I2,' (',I3,' x ',I3,' points) : ',A)
1850	format (  '    Method (wavelen.: ',F5.3,'A): ',A8,
     &	          ' (Debye-Waller=',L1,',anomalous scat.=',L1,')')
1860	format (  '    Method (wavelen.: ',A6,'): ',A8,
     &	          ' (Debye-Waller=',L1,',anomalous scat.=',L1,')')
1900	format (  '       Weighting scheme used : ',A)
1950	format (  '    Scal. / back. from plane : ',I2)
1970	format (/,' Q-range of input data       : ',F6.3,' - ',F6.3,
     &	          ' used : ',F6.3,' - ',F6.3,' [1/A]')
2000	format (/,' Symmetry of input data      : ')
2020	format (  '       Reciprocal data layer : plane ',I2,'/',I2,
     &	          ', sym.op. ',I2,'/',I2)
2030	format (  '           lower left corner : ',3(F7.3,1X))
2040	format (  '          lower right corner : ',3(F7.3,1X))
2050	format (  '           upper left corner : ',3(F7.3,1X))
2060	format (  '              increment hor. : ',3(F7.3,1X))
2070	format (  '             increment vert. : ',3(F7.3,1X),/)
2200	format (  ' Minimal allowed distances   : ')
2300	format (  '     ',A9,' and ',A9,' : ',F7.3)
2400	format (/,' Sigma for created RMC moves : ')
2500	format (  '              Atom ',A9,' : ',3(F7.3,1X))
2600	format (  '     Molecule type ',I9,' : ',3(F7.3,1X))
2700	format (/,' Selected atoms for RMC      : ',100(A9,1X))
2800	format (/,' Selected molecules for RMC  : ',100(I4,1X))
	end
c*****7*****************************************************************
	subroutine rmc_save (zeile,lp)
c+
c	save scattering amplitudes / structure
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'rmc.inc'
c
	integer		maxw
	parameter	(maxw=10)
c
	character*(*)	zeile
	integer		i,il,lp
c
	character*200	cdummy,cpara(maxw)
	integer		ianz,lpara(maxw),ip,is
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if(ianz.ge.2) then
	  call do_cap(cpara(1))
	  cdummy = cpara(1)
	  if (ier_num.ne.0) return
c
c------ --save structure
c
	  if (cdummy(1:3).eq.'STR') then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	    write (output_io,1500) cpara(1)
	    call save_struc(cpara(1),lpara(1))
c
c------ --save lot origins
c
	  elseif (cdummy(1:3).eq.'LOT') then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	    write (output_io,1505) cpara(1)
	    call oeffne (57,cpara(1),'unknown',.false.)
	    if (ier_num.eq.0) then
	      do il=1,rmc_nlots
	        write (57,'(3(I4,1X))') (rmc_lots_orig(i,il),i=1,3)
	      enddo
	      close (57)
	    endif
c
c------ --save scattering intensities
c
	  elseif (cdummy(1:3).eq.'SCA') then
	    if (ianz.ge.3) then
	      call del_params (1,ianz,cpara,lpara,maxw)
              call ber_params(1,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      ip=nint(werte(1))
	      if (ip.le.0 .or. ip.gt.rmc_nplane) then
	        ier_num = -7
	        ier_typ = ER_RMC
	      else
	        call del_params (1,ianz,cpara,lpara,maxw)
	        call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	        if (ier_num.ne.0) return
	        write (output_io,1510) ip,cpara(1)
	        call rmc_writedat (ip,1,cpara(1))
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
c
c------ --save scattering intensities of sym. equivalent planes
c
	  elseif (cdummy(1:3).eq.'SYM') then
	    if (ianz.ge.4) then
	      call del_params (1,ianz,cpara,lpara,maxw)
              call ber_params(2,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      ip=nint(werte(1))
	      is=nint(werte(2))
	      if (ip.le.0 .or. ip.gt.rmc_nplane) then
	        ier_num = -7
	        ier_typ = ER_RMC
	      else
	        if (is.le.0 .or. is.gt.rmc_nsym(ip)) then
	          ier_num = -8
	          ier_typ = ER_RMC
	        else
	          call del_params (2,ianz,cpara,lpara,maxw)
	          call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	          if (ier_num.ne.0) return
	          write (output_io,1520) ip,is,cpara(1)
	          call rmc_writedat (ip,is,cpara(1))
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
c
c------ --unknown subcommand given
c
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1500	format (' Saving structure to file : ',A30)
1505	format (' Saving lot origins to file : ',A30)
1510	format (' Saving intensities of plane ',I2,' to file : ',A30)
1520	format (' Saving intensities (ip:',I2,'/is:',I2,
     &	        ') to file: ',A30)
1600	format (3(F7.3,1X),3X,F15.2)
	end
c*****7*****************************************************************
	subroutine rmc_writedat (ip,is,fname)
c+
c	Save scattering data to disk
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
	include 	'rmc.inc'
	include 	'diffuse.inc'
c
	character*(*)	fname
	integer		pgmmax,ip,is,i,j
c
	pgmmax = 255
c
c------ first convert to intensities and check for points w=0
c
	call rmc_inten(ip,is,.false.)
c
	do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	  dsi(i) = rmc_back(ip)+rmc_skal(ip)*dsi(i)
	  if (rmc_wic(offq(ip)+i).eq.0.0) then
	    if (rmc_data.eq.rmc_data_nipl) then
	      dsi(i) = -9999.0
	    elseif (rmc_data.eq.rmc_data_pgm) then
	      dsi(i) = 0.0
	    endif
	  endif
	enddo
c
	open (unit=44,file=fname,status='unknown')
c
c------ NIPL file
c
	if (rmc_data.eq.rmc_data_nipl) then
	  write (44,*) (rmc_num(i,ip),i=1,2)
	  write (44,*) (rmc_xy(i,ip),i=1,4)
	  do j=1,rmc_num(2,ip)
	    write (44,*) (dsi((i-1)*rmc_num(2,ip)+j),
     &	                  i=1,rmc_num(1,ip))
	  enddo
c
c------ PGM file
c
	elseif (rmc_data.eq.rmc_data_pgm) then
	  write (44,2000) 
	  write (44,2100) 'lower left ',(rmc_eck(i,1,is,ip),i=1,3)
	  write (44,2100) 'lower right',(rmc_eck(i,2,is,ip),i=1,3)
	  write (44,2100) 'upper left ',(rmc_eck(i,3,is,ip),i=1,3)
	  write (44,*) (rmc_num(i,ip),i=1,2)
	  write (44,*) pgmmax
	  do j=rmc_num(2,ip),1,-1
	    write (44,*) (nint(dsi((i-1)*rmc_num(2,ip)+j)),
     &	                  i=1,rmc_num(1,ip))
	  enddo
	endif
c
	close (44)
c
2000	format ('P2'/'# DISCUS RMC resulting plane',/,'#')
2100	format ('# ',A11,' : ',3(F7.3,1X))
	end
c*****7*****************************************************************
	subroutine rmc_readdata (zeile,lp)
c+
c	read experimental data for rmc fit
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
	include 	'debug.inc'
	include 	'rmc.inc'
c
	integer		maxw
	parameter	(maxw=13)
	integer		max_sym
	parameter	(max_sym=48)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw),cfile,cwic
	character*4	cdummy
	integer		lpara(maxw)
	integer		ianz,nsym,rsym
	integer		ip,nx,ny,wx,wy
	integer		i,j,k
	real		werte(maxw),d
	real		e1(3),e2(3),e3(3),vi1(3),vi2(3),z(3)
	real		qmin,qmax
	real		ee1(4,max_sym)
	real		ee2(4,max_sym)
	real		ee3(4,max_sym)
	real		zone(4,max_sym)
	real		mat(4,4,max_sym)
	logical		lexist
c
	real		rmc_dowic
c
c------ check if there is space for another plane
c
	if (rmc_nplane.eq.rmc_max_planes) then
	  ier_num = -1
	  ier_typ = ER_RMC
	  return
	endif
	ip = rmc_nplane+1
c
c------ work through all the parameters given
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
	if (ianz.ge.13) then
c
c------ - get exp. method (neutron/x-ray)
c
	  call do_cap(cpara(1))
	  if (cpara(1)(1:1).eq.'N') then
	    rmc_lxray(ip)=.false.
	  elseif (cpara(1)(1:1).eq.'X') then
	    rmc_lxray(ip)=.true.
	  else
	   ier_num = -5
	   ier_typ = ER_RMC
	   return
	  endif
c
c------ - get wavelength (cua1 or real number)
c
	  call do_cap(cpara(2))
	  if(ichar('A') .le. ichar(cpara(2)(1:1)) .and.
     &       ichar(cpara(2)(1:1)) .le. ichar('Z')      ) then
            rmc_lambda(ip)=cpara(2)
          else
            call ber_params(1,cpara(2),lpara(2),werte,maxw)
            rmc_rlambda(ip)=werte(1)
            rmc_lambda(ip)=' '
          endif
c
c------	- build file name
c
	  call do_build_name(ianz,cpara,lpara,werte,maxw,3)
	  if(ier_num.eq.0) then
	    cfile = cpara(3)
	  else
	    return
	  endif
	  cwic  = cpara(4)
	  call del_params(4,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  do i=1,3
	    e1(i)=werte(i)
	    e2(i)=werte(i+3)
	    e3(i)=werte(i+6)
	  enddo
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
	if (ier_num.ne.0) return
c
c------ check if specified file is present
c
	inquire (file=cfile,exist=lexist)
	if (.not. lexist) then
	  ier_num = -1
	  ier_typ = ER_IO
	  return
	endif
c
c------ get weighting scheme
c
	cdummy = cwic(1:4)
	call do_cap(cdummy)
	if     (cdummy.eq.'ONE ') then
	  rmc_wic_typ(ip) = rmc_wic_eins
	elseif (cdummy.eq.'SQRT') then
	  rmc_wic_typ(ip) = rmc_wic_sqrt
	elseif (cdummy.eq.'LOG ') then
	  rmc_wic_typ(ip) = rmc_wic_log
	elseif (cdummy.eq.'LIN ') then
	  rmc_wic_typ(ip) = rmc_wic_lin
	elseif (cdummy.eq.'SQUA') then
	  rmc_wic_typ(ip) = rmc_wic_qua
	elseif (cdummy.eq.'INV ') then
	  rmc_wic_typ(ip) = rmc_wic_inv
	elseif (cdummy.eq.'ISQ ') then
	  rmc_wic_typ(ip) = rmc_wic_isq
	else
	  inquire (file=cwic,exist=lexist)
	  if (lexist) then
	    rmc_wic_typ(ip) = rmc_wic_dat
	    call oeffne (18,cwic,'old',.false.)
	  else
	    ier_num = -16
	    ier_typ = ER_RMC
	    return
	  endif
	endif
c
c------ read data for given plane
c
	rmc_fname(ip)=cfile
	call oeffne (17,cfile,'old',.false.)
	if (ier_num.ne.0) return
c
c------ NIPL file
c
	if (rmc_data.eq.rmc_data_nipl) then
	  read (17,*,end=99,err=999) nx,ny
	  read (17,*,end=99,err=999) (rmc_xy(j,ip),j=1,4)
	  if (rmc_wic_typ(ip).eq.rmc_wic_dat) then
	    read (18,*,end=99,err=999) wx,wy
	    read (18,*,end=99,err=999) d,d,d,d
	    if (wx.ne.nx .or. wy.ne.ny) then
	      ier_num = -17
	      ier_typ = ER_RMC
	    endif
	  endif
	  if (nx*ny.gt.(rmc_max_q-offq(ip))) then
	    ier_num = -2
	    ier_typ = ER_RMC
	  endif
c
	  if (ier_num.ne.0) goto 98
c
	  do j=1,ny
	    read (17,*,end=99,err=999) 
     &	         (rmc_int(offq(ip)+(i-1)*ny+j),i=1,nx)
	    if (rmc_wic_typ(ip).eq.rmc_wic_dat) then
	      read (18,*,end=99,err=999) 
     &	           (rmc_wic(offq(ip)+(i-1)*ny+j),i=1,nx)
	    else
	      do i=1,nx
	        k = offq(ip)+(i-1)*ny + j
	        rmc_wic(k) = rmc_dowic(rmc_wic_typ(ip),rmc_int(k))
	      enddo
	    endif
	  enddo
c
c------ PGM file
c
	elseif (rmc_data.eq.rmc_data_pgm) then
	  call rmc_pgmheader (17,nx,ny,rmc_xy(1,ip),rmc_xy(2,ip),
     &	                      rmc_xy(3,ip),rmc_xy(4,ip))
	  if (rmc_wic_typ(ip).eq.rmc_wic_dat) then
	    call rmc_pgmheader (18,wx,wy,d,d,d,d)
	    if (wx.ne.nx .or. wy.ne.ny) then
	      ier_num = -17
	      ier_typ = ER_RMC
	    endif
	  endif
	  if (nx*ny.gt.(rmc_max_q-offq(ip))) then
	    ier_num = -2
	    ier_typ = ER_RMC
	  endif
c
	  if (ier_num.ne.0) goto 98
c
	  do j=ny,1,-1
	    read (17,*,end=99,err=999) 
     &	         (rmc_int(offq(ip)+(i-1)*ny+j),i=1,nx)
	    if (rmc_wic_typ(ip).eq.rmc_wic_dat) then
	      read (18,*,end=99,err=999) 
     &	           (rmc_wic(offq(ip)+(i-1)*ny+j),i=1,nx)
	    else
	      do i=1,nx
	        k = offq(ip) + (i-1)*ny + j
	        rmc_wic(k) = rmc_dowic(rmc_wic_typ(ip),rmc_int(k))
	      enddo
	    endif
	  enddo
	endif
c
98	continue
	close (17)
	if (rmc_wic_typ(ip).eq.rmc_wic_dat) close (18)
	if (ier_num.ne.0) return
c
c------ calculate increments and zone axis
c
	do i=1,3
	  vi1(i) = (e2(i)-e1(i))/float(max(1,nx-1))
	  vi2(i) = (e3(i)-e1(i))/float(max(1,ny-1))
	enddo
	call vekprod (vi1,vi2,z,cr_reps,cr_gten)
c
c------ perform symmetry operations
c
	do i=1,3
	  zone(i,1) =  z(i)
	  ee1 (i,1) = e1(i)
	  ee2 (i,1) = e2(i)
	  ee3 (i,1) = e3(i)
	enddo
	zone(4,1) = 0.0
	ee1 (4,1) = 0.0
	ee2 (4,1) = 0.0
	ee3 (4,1) = 0.0
c
	do i=1,4
	  do j=1,4
	    mat(i,j,1) = 0.0
	  enddo
	  mat(i,i,1) = 1.0
	enddo
c
	call rmc_symmetry (nsym,zone,mat,max_sym,.true.,cr_acentric)
	if (ier_num.ne.0) return
c
	if (nsym.gt.rmc_max_sym) then
	  rmc_dosym = .false.
	  rmc_nosym = .true.
	  rsym      = nsym
	  nsym      = 1
	else
	  rsym      = nsym
	  rmc_nosym = .false.
	endif
c
c------ do transform q=gSg* and transform corners of input plane
c
	call rmc_gsg (mat,nsym,max_sym)
	call rmc_trans (ee1,mat,nsym,max_sym)
	call rmc_trans (ee2,mat,nsym,max_sym)
	call rmc_trans (ee3,mat,nsym,max_sym)
c
c------ data accepted
c
	rmc_nplane=ip
	rmc_num(1,ip)=nx
	rmc_num(2,ip)=ny
	rmc_nsym(ip)=nsym
c
c------ set offq and offsq values 
c
	do i=2,nsym
	  offsq(ip,i)=offsq(ip,1)+(i-1)*nx*ny
	enddo
	offsq(ip+1,1)=offsq(ip,nsym)+nx*ny
	offq (ip+1)  =offq (ip)+nx*ny
c
	if (dbg) then
    	  do i=1,rmc_max_planes
    	    do j=1,rmc_max_sym
    	      write (output_io,9999) i,j,offq(i),offsq(i,j)
    	    enddo
    	  enddo
	endif
9999	format (' DBG : ip:',i3,' is:',i3,' offq:',i8,' offsq:',i8)
c
c------ store plane increments and corners
c
	do i=1,nsym
	  do j=1,3
	    rmc_vi(j,1,i,ip) = (ee2(j,i)-ee1(j,i))/float(max(1,nx-1))
	    rmc_vi(j,2,i,ip) = (ee3(j,i)-ee1(j,i))/float(max(1,ny-1))
	  enddo
	enddo
	do i=1,nsym
	  do j=1,3
	    rmc_eck (j,1,i,ip) = ee1(j,i)
	    rmc_eck (j,2,i,ip) = ee2(j,i)
	    rmc_eck (j,3,i,ip) = ee3(j,i)
	  enddo	
	enddo
c
c------ initial q range of data to be used is ALL data
c
	call rmc_layer(1,ip)
	call rmc_stltab(1,ip,.true.)
c
	qmin = ristl(offsq(ip,1)+1)*CFINC
	qmax = ristl(offsq(ip,1)+1)*CFINC
	do i=2,rmc_num(1,ip)*rmc_num(2,ip)
	  qmin = min(qmin,ristl(offsq(ip,1)+i)*CFINC)
	  qmax = max(qmax,ristl(offsq(ip,1)+i)*CFINC)
	enddo
c
	rmc_qmin = min(rmc_qmin,qmin)
	rmc_qmax = max(rmc_qmax,qmax)
	rmc_llim = rmc_qmin
	rmc_ulim = rmc_qmax
c
c------	write comment
c
	if (rmc_lxray(ip)) then
	  write (output_io,9000) 'x-ray',ip,rmc_num(1,ip),
     &	                 rmc_num(2,ip),rsym
	else
	  write (output_io,9000) 'neutron',ip,rmc_num(1,ip),
     &	                 rmc_num(2,ip),rsym
	endif
	return
c
99	continue
	ier_num = -6
	ier_typ = ER_IO
	return
c
999	continue
	ier_num = -3
	ier_typ = ER_IO
	return
c
9000	format (' Read ',A,'-data for plane ',I2,' (',I3,' x ',I3,
     &	        ' points / ',I2,' sym. equivalent planes)')
	end
c*****7*****************************************************************
	subroutine rmc_pgmheader (ifile,nx,ny,xmin,xmax,ymin,ymax)
c+
c	Reading PGM Header incl. possible comments
c-
	implicit	none
c
	include		'errlist.inc'
c
	character*132	zeile
	character*2	cmagic
	real		xmin,xmax,ymin,ymax
	integer		ifile,nx,ny,id
c
	read (ifile,1000,end=99,err=999) cmagic
	if (cmagic.ne.'P2') then
	  ier_num = -18
	  ier_typ = ER_RMC
	else
5	  continue
	  read (ifile,1000,end=99,err=999) zeile
	  if (zeile(1:1).eq.'#') goto 5
	  backspace(ifile)
	  read (ifile,*,end=99,err=999) nx,ny,id
c
	  xmin = 0.0
	  xmax = float(nx-1)
	  ymin = 0.0
	  ymax = float(ny-1)
	endif
	return
c
99	continue
	ier_num = -6
	ier_typ = ER_IO
	return
c
999	continue
	ier_num = -3
	ier_typ = ER_IO
	return
c
1000	format(a)
	end
c*****7*****************************************************************
        real function rmc_dowic (typ,inte)
c+
c	Calulates the weight for intensity int
c-
	implicit 	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'rmc.inc'
c
	real		inte
	integer		typ
c
	if (inte.le.0.0) then
	  rmc_dowic = 0.0
	else
	  if (typ.eq.rmc_wic_eins) then
	    rmc_dowic = 1.0
	  elseif (typ.eq.rmc_wic_sqrt) then
	    rmc_dowic = sqrt(inte)
	  elseif (typ.eq.rmc_wic_log ) then
	    rmc_dowic = log(inte)
	  elseif (typ.eq.rmc_wic_lin ) then
	    rmc_dowic = inte
	  elseif (typ.eq.rmc_wic_qua ) then
	    rmc_dowic = inte**2
	  elseif (typ.eq.rmc_wic_inv ) then
	    rmc_dowic = 1.0/inte
	  elseif (typ.eq.rmc_wic_isq ) then
	    rmc_dowic = 1.0/sqrt(inte)
	  endif
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_run
c+
c	main rmc loop - called by run command
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'random.inc'
	include 	'errlist.inc'
	include		'chem.inc'
	include		'param.inc'
	include 	'rmc.inc'
	include		'diffuse.inc'
c
	real*8		rmc_cc,rmc_c,rmc_ce
	real*8		rmc_e(rmc_max_planes)
	real*8		rmc_ee(rmc_max_planes)
	real		chi2_new,chi2_old,sig2
	real		prob,psum,p2sum,pave,psig,pmax,pn
	real		chi2(rmc_max_planes)
	real		skal(rmc_max_planes)
	real		back(rmc_max_planes)
	real		start,zeit,seknds
	real		p_new(3,rmc_max_atom)
	integer		i_new(rmc_max_atom)
	integer		isel(rmc_max_atom),natoms
	integer		imol(rmc_max_atom)
	integer		isym(rmc_max_planes)
	integer		zh,zm,zs
	integer		i,j,ip,iq,is,iii
	integer		igen,itry,iacc_good,iacc_bad
	logical		loop,laccept
c
	real		ran1
c
	igen      = 0
	itry      = 0
	iacc_good = 0
	iacc_bad  = 0
	loop      = .true.
	laccept   = .true.
c
	psum      = 0.0
	p2sum     = 0.0
	pmax      = 0.0
	pn        = 0.0
c
c------ check consistency of input data
c
	call rmc_check_input
	if (ier_num.ne.0) return
c
c------ sym ?
c
	do ip=1,rmc_nplane
	  if (rmc_dosym) then
	    isym(ip) = rmc_nsym(ip)
	  else
	    isym(ip) = 1
	  endif
	enddo
c
c------ Write some start information
c
	write (output_io,1000) (cr_icc(i),i=1,3),cr_natoms
c
c------ calculate scattering amplitudes 
c
	if (rmc_calc_f) then
	  write (output_io,1100) 
	  call rmc_calc_scat(isym)
	else
	  write (output_io,1150)
	endif
c
c------ Calculate maximal value in rmc_mindist array
c
	rmc_mindist_max = rmc_mindist(1,1)
	do i=1,cr_nscat
	  do j=1,cr_nscat
	    rmc_mindist_max = max (rmc_mindist_max,rmc_mindist(i,j))
	  enddo
	enddo
c
	if (rmc_log) then
	  open (34,file='rmc.log',status='unknown',access='append')
	  if (rmc_mode.eq.rmc_mode_swchem) then
	    write (34,2000) 'Mode SWCHEM'
	  elseif (rmc_mode.eq.rmc_mode_swdisp) then
	    write (34,2000) 'Mode SWDISP'
	  elseif (rmc_mode.eq.rmc_mode_shift) then
	    write (34,2000) 'Mode SHIFT'
	  endif
	  close (34)
	endif
c
c------ calculate sums from exp. data needed and initial chi2
c
	chi2_old = 0.0
	do ip=1,rmc_nplane
	  rmc_wtot (ip)=0.0
	  rmc_e    (ip)=0.0
	  rmc_ee   (ip)=0.0
	  rmc_c  = 0.0
	  rmc_cc = 0.0
	  rmc_ce = 0.0
c
	  do is=1,isym(ip)
c
	    call rmc_inten(ip,is,.false.)
c
	    do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	      if ((ristl(offsq(ip,1)+iq)*CFINC).ge.rmc_llim .and.
     &	          (ristl(offsq(ip,1)+iq)*CFINC).le.rmc_ulim     ) then
	         iii=offq(ip)+iq
	         rmc_wtot(ip)=rmc_wtot(ip)+rmc_wic(iii)
	         rmc_e (ip)=rmc_e (ip)+rmc_wic(iii)*rmc_int(iii)
	         rmc_ee(ip)=rmc_ee(ip)+rmc_wic(iii)*rmc_int(iii)**2
	         rmc_c =rmc_c +rmc_wic(iii)*dsi(iq)
	         rmc_cc=rmc_cc+rmc_wic(iii)*dsi(iq)**2
	         rmc_ce=rmc_ce+rmc_wic(iii)*dsi(iq)*rmc_int(iii)
	      endif
	    enddo
	  enddo
	  if (rmc_e(ip).gt.1E-06) then
	    call rmc_calcskal(ip,rmc_wtot(ip),rmc_c,rmc_cc,rmc_ce,
     &	                      rmc_e(ip),rmc_ee(ip),skal,back)
	    chi2(ip)=rmc_ee(ip)+skal(ip)**2*rmc_cc+back(ip)**2*
     &	             rmc_wtot(ip)+2.0*(skal(ip)*back(ip)*rmc_c-back(ip)*
     &	             rmc_e(ip)-skal(ip)*rmc_ce)	
	    chi2(ip)=chi2(ip)/rmc_wtot(ip)
	    chi2_old=chi2_old + chi2(ip)
	  endif
	enddo
c
	if (chi2_old.lt.1E-06) then
	  ier_num = -14
	  ier_typ = ER_RMC
	  return
	endif
c
c------ Here is the main loop
c
	start = seknds(0.0)
	if (rmc_mode.eq.rmc_mode_swchem) then
	  write (output_io,2000) 'Mode SWCHEM'
	elseif (rmc_mode.eq.rmc_mode_swdisp) then
	  write (output_io,2000) 'Mode SWDISP'
	elseif (rmc_mode.eq.rmc_mode_shift) then
	  write (output_io,2000) 'Mode SHIFT'
	elseif (rmc_mode.eq.rmc_mode_external) then
	  write (output_io,2000) 'external defined mode'
	endif
c
	sig2 = rmc_sigma**2/2.0
c
	do while (loop)
	  laccept = .true.
	  igen = igen + 1
c
c-------- generate move and check for limits
c
	  if (rmc_sel_atom) then
	    call rmc_genmove (laccept,natoms,p_new,i_new,isel)
	  else
	    call rmc_genmove_mol (laccept,natoms,p_new,i_new,isel,imol)
	  endif
	  if (ier_num.ne.0) return
c
c-------- calc new scattering amplitudes and chi2
c
	  if (laccept) then
	    itry = itry + 1
	    chi2_new = 0.0
	    do ip=1,rmc_nplane
	      rmc_c    = 0.0
	      rmc_cc   = 0.0
	      rmc_ce   = 0.0
	      call dlink(rmc_lxray(ip),rmc_ano(ip),
     &	                rmc_lambda(ip),rmc_rlambda(ip))
	      do is=1,isym(ip)
	        call rmc_fcalc (ip,is,natoms,i_new,p_new,isel)
	        call rmc_inten (ip,is,.true.)
	        do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	          if ((ristl(offsq(ip,is)+iq)*CFINC).ge.rmc_llim .and.
     &	              (ristl(offsq(ip,is)+iq)*CFINC).le.rmc_ulim ) then
	            iii=offq(ip)+iq
	            rmc_c =rmc_c +rmc_wic(iii)*dsi(iq)
	            rmc_cc=rmc_cc+rmc_wic(iii)*dsi(iq)**2
	            rmc_ce=rmc_ce+rmc_wic(iii)*dsi(iq)*rmc_int(iii)
	          endif
	        enddo
	      enddo
c
c------ ----- if plane contains no valid q points at all
c
	      if (rmc_e(ip).lt.1E-06) goto 1111
c
	      call rmc_calcskal(ip,rmc_wtot(ip),rmc_c,rmc_cc,rmc_ce,
     &	                        rmc_e(ip),rmc_ee(ip),skal,back)
	      chi2(ip)=rmc_ee(ip)+skal(ip)**2*rmc_cc+back(ip)**2*
     &	               rmc_wtot(ip)+2.0*(skal(ip)*back(ip)*rmc_c-
     &	               back(ip)*rmc_e(ip)-skal(ip)*rmc_ce)	
	      chi2(ip)=chi2(ip)/rmc_wtot(ip)
	      chi2_new=chi2_new + chi2(ip)
c
1111	      continue
	    enddo
c
c------	----Accept move ?
c
	    prob = chi2_new - chi2_old
c
	    if (prob.lt.0) then
	      laccept = .true.
	    else
	      if (sig2.gt.0.0) then
	        psum    = psum  + prob
	        p2sum   = p2sum  + prob**2
	        pmax    = max (pmax,prob)
	        pn      = pn + 1
	        prob    = exp(-prob/sig2)
	        laccept = (prob.gt.ran1(idum))
	      else
	        laccept = .false.
	      endif
	    endif
c
	    if (rmc_sigma.eq.-9999.) laccept = .true.
c
c------ ----if accepted make move	  
c
	    if (laccept) then
	      call  rmc_makemove (isym,natoms,i_new,p_new,isel,imol)
	      chi2_old = chi2_new
	      do i=1,rmc_nplane
	        rmc_chi2(i)=chi2(i)
	        rmc_back(i)=back(i)
	        rmc_skal(i)=skal(i)
	      enddo
	      if (prob.lt.0) then
	        iacc_good = iacc_good + 1
	      else
	        iacc_bad  = iacc_bad + 1
	      endif
	    endif
c
c------ --write info and terminate or loop again
c
	  endif
c
2222	  continue
c
	  loop = (itry.lt.rmc_maxcyc)
c
	  if (igen.gt.1000*rmc_display .and. itry.eq.0) then
	    ier_num = -20
	    ier_typ = ER_RMC
	    loop    = .false.
	  endif
c
	  if (mod(igen,rmc_display).eq.0 .or. .not.loop) then
	    if (.not.loop) write (output_io,1250)
	    write (output_io,1300) igen,itry,iacc_good,iacc_bad,
     &	                   chi2_old
	    do i=1,rmc_nplane
	      write (output_io,1400) i,1.0/rmc_skal(i),
     &	                             rmc_back(i),rmc_chi2(i)
	    enddo
	    if (rmc_log) then
	      open (34,file='rmc.log',status='unknown',access='append')
	      if (.not.loop) write (34,1250)
	      write (34,1300) igen,itry,iacc_good,iacc_bad,chi2_old
	      do i=1,rmc_nplane
	        write (34,1400) i,1.0/rmc_skal(i),rmc_back(i),rmc_chi2(i)
	      enddo
	      close (34)
	    endif
	  endif
	enddo
c
c------ write timing summary and "prob" statistics
c
	if (pn.gt.0) then
	  pave = psum / pn
	  psig = sqrt (p2sum/pn - (psum/pn)**2)
	  write (output_io,3000) pave,psig,pmax
	endif
c
	zeit = seknds(start)
	zh = int (zeit / 3600.)
	zm = int ((zeit - zh*3600.) / 60.)
	zs = int ( zeit - zh*3600 - zm*60.)
	write (output_io,4000) zh,zm,zs,zeit/itry
c	
	if (rmc_log) then
	  open (34,file='rmc.log',status='unknown',access='append')
	  write (34,3000) pave,psig,pmax
	  write (34,4000) zh,zm,zs,zeit/float(itry)
	  close (34)
	endif
c
c------ save some results to res[i] block
c
	res_para(0) = 8
c
	res_para(1) = chi2_old
	res_para(2) = float(itry)
	res_para(3) = float(iacc_good)
	res_para(4) = float(iacc_bad)
	res_para(5) = pave
	res_para(6) = psig
	res_para(7) = pmax
	res_para(8) = zeit/float(itry)
c
c------ Update crystal dimensions
c
        call update_cr_dim
c
c------ formats
c
1000	format (' Running RMC-Fit ...',//
     &	        ' Size of model crystal     : ',I3,' x ',I3,' x ',I3,
     &	        ' containing ',I9,' atoms')
1100	format (/,' Calculating scattering amplitudes ... ')
1150	format (/,' Using old scattering amplitudes ...',/
     &	          ' (Use command rese to force recalculation)'/)
1250	format (/,' ---- Final configuration ---- ')
1300	format (/,' Gen: ',I6,' try: ',I6,' acc: (good/bad): ',I6,
     &	          ' / ',I6,' s2x2: ',G15.8)
1400	format ('   Plane ',I2,': scal: ',G10.4,' / back: ',G10.4,
     &	        ' / s2x2: ',G22.8)
2000	format (/,' Starting main RMC loop ...',/,
     &	         ' (',A,') ',/)
3000	format (/,' Delta Chi    : ave: ',G15.4,' sig: ',G15.4,
     &	          ' max: ',G15.4)
4000	format (/,' Elapsed time : ',I4,' h ',I2,' min ',I2,' sec ',/
     &	          ' Time/cycle   : ',F9.3,' sec',/)
	end
c*****7*****************************************************************
	subroutine rmc_calc_scat (isym)
c+
c	This routine calculates initial Fourier transform
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'rmc.inc'
	include		'diffuse.inc'
	include		'errlist.inc'
c
	integer		isym(rmc_max_planes)
	integer		lbeg(3),ncell
	integer		ip,iscat,nlot,i,k,iii
c
	call four_cexpt
	call rmc_zero
	call four_csize(cr_icc,rmc_csize,lperiod,ls_xyz)
c
c------ Create random lots or read file
c
	if (rmc_ranloc) then
	  do nlot=1,rmc_nlots
	    call four_ranloc(rmc_csize,lbeg)
	    do i=1,3
	      rmc_lots_orig(i,nlot) = lbeg(i)
	    enddo
	  enddo
	else
	  call oeffne (57,rmc_lname,'old',.false.)
	  if (ier_num.ne.0) return
	  do nlot=1,rmc_nlots
	    read (57,*,end=99,err=999) (rmc_lots_orig(i,nlot),i=1,3)
	  enddo
	  close (57)
	endif
c
c------ Loop over all exp. data planes
c
  	do ip=1,rmc_nplane
	  call dlink(rmc_lxray(ip),rmc_ano(ip),
     &	            rmc_lambda(ip),rmc_rlambda(ip))
	  call rmc_formtab(ip,.true.)
c
c------ - Loop over all sym. equivalent planes
c
	  do k=1,isym(ip) 
	    do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	      acsf(i) = cmplx(0.0d0,0.0d0)
	    enddo
c
	    call rmc_layer(k,ip)
	    call rmc_stltab(k,ip,.true.)
	    call four_aver(rmc_ilots,rmc_ave)
c
	    iii = offsq(ip,k)
c
c------ --- Loop over all 'lots'
c 
	    do nlot=1,rmc_nlots
	      do i=1,3
	        lbeg(i) = rmc_lots_orig(i,nlot)
	      enddo
c
c------ ----- Loop over all atom types
c 
  	      do iscat=1,cr_nscat
	        call four_getatm(iscat,rmc_ilots,lbeg,rmc_csize,ncell)
	        call four_strucf(iscat,.true.)
	        do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	          rmc_csf(iii+i,nlot) = rmc_csf(iii+i,nlot) + tcsf(i)
	        enddo
	      enddo
c
	      do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	        rmc_csf(iii+i,nlot) = rmc_csf(iii+i,nlot) - acsf(i)
	      enddo
	    enddo
	    write (output_io,1200) ip,k
	  enddo
	enddo
c
c------ we don't need to calculate the initial Fourier again
c
	rmc_calc_f = .false.
	return
c
99	continue
	ier_num = -6
	ier_typ = ER_IO
	close(57)
	return
c
999	continue
	ier_num = -3
	ier_typ = ER_IO
	close(57)
	return
c
c
1200    format (' Done plane ',I2,', sym.op. ',I2,' ...')
	end
c*****7*****************************************************************
	subroutine rmc_inten (ip,is,lnew)
c+
c	This routine copies average intensity for given plane and sym
c	in array 'dsi' ...
c-
	implicit	none
c
	include		'config.inc'
	include		'diffuse.inc'
	include		'rmc.inc'
c
	integer		iq,is,ip,il
	logical		lnew
c
c------ We want to average rmc_csf_new ..
c
	if (lnew) then
c
c------ - First copy lot 1 to dsi ..
c
	  do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	    dsi(iq)=real(rmc_csf_new(offsq(ip,is)+iq,1)*
     &                   conjg(rmc_csf_new(offsq(ip,is)+iq,1)))
	  enddo
c
c------ - If there is more than 1 lot, add the rest ..
c
	  if (rmc_nlots.gt.1) then
	    do il=2,rmc_nlots
	      do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	        dsi(iq)=dsi(iq)+real(rmc_csf_new(offsq(ip,is)+iq,il)*
     &                          conjg(rmc_csf_new(offsq(ip,is)+iq,il)))
	      enddo
	    enddo
	  endif
c
c------ We want to average rmc_csf
c
	else
c
	  do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	    dsi(iq)=real(rmc_csf(offsq(ip,is)+iq,1)*
     &                   conjg(rmc_csf(offsq(ip,is)+iq,1)))
	  enddo
c
	  if (rmc_nlots.gt.1) then
	    do il=2,rmc_nlots
	      do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	        dsi(iq)=dsi(iq)+real(rmc_csf(offsq(ip,is)+iq,il)*
     &                          conjg(rmc_csf(offsq(ip,is)+iq,il)))
	      enddo
	    enddo
	  endif
c
	endif
	end
c*****7*****************************************************************
	subroutine rmc_check_input
c+
c	This routine checks if all input parameters are valid 
c	for a RMC run.
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'chem.inc'
	include		'rmc.inc'
	include		'molecule.inc'
	include		'errlist.inc'
c
	integer		i,j,ia
	logical		laccept
c
	ier_num = 0
	ier_typ = ER_NONE
c
c------ atoms present ?
c
	if (cr_natoms.eq.0) then
	  ier_typ = ER_RMC
	  ier_num = -3
	  return
	endif
c
c------ experimental data present ?
c
	if (rmc_nplane.eq.0) then
	  ier_num = -4
	  ier_typ = ER_RMC
	  return
	endif
c
c------ operational mode ok ?
c------ first count selected atoms
c
	ia=0
	do i=0,cr_nscat
	  if(rmc_allowed(i)) ia=ia+1
	enddo
c
c------ ALL modes need at least one selected atom typ
c
	if (ia.lt.1) then
	  ier_num = -10
	  ier_typ = ER_RMC
	  return
	endif
c
c------ SWCHEM mode needs at least two selected atoms
c
	if (rmc_mode.eq.rmc_mode_swchem .and. ia.lt.2) then
	  ier_num = -11
	  ier_typ = ER_RMC
	  return
	endif
c
c------ SWDISP needs initial displacements > 0.000001
c
	if (rmc_mode.eq.rmc_mode_swdisp) then
	  write (output_io,1050)
	  call chem_aver (.false.)
	  laccept=.false.
	  do i=1,cr_nscat
	    laccept = laccept                         .or.
     &	              (chem_ave_sig(1,i).gt.0.000001) .or.
     &	              (chem_ave_sig(2,i).gt.0.000001) .or.
     &	              (chem_ave_sig(3,i).gt.0.000001) 
	  enddo
	  if (.not. laccept) then
	    ier_num = -12
	    ier_typ = ER_RMC
	  endif
	endif
c
c------ Some checks in case we are using molecules
c
	if (.not. rmc_sel_atom) then
	  if (rmc_mode.ne.rmc_mode_shift) then
	    laccept = .true.
	    do i=1,mole_num_mole
	      do j=i,mole_num_mole
	        if (rmc_allowed(mole_type(i)) .and.
     &	            rmc_allowed(mole_type(j))      ) then
	          laccept = laccept .and. 
     &	                    (mole_len(i).eq.mole_len(j))
	        endif
	      enddo
	    enddo
	    if (.not. laccept) then
	      ier_num = -67
	      ier_typ = ER_APPL
	    endif
	  endif
c
	  ia = 0
	  do i=1,mole_num_mole
	    if (mole_len(i).gt.ia) ia = mole_len(i)
	  enddo
	  if (rmc_mode.ne.rmc_mode_shift) ia=2*ia
	  if (ia.gt.rmc_max_atom) then
	    ier_num = -6
	    ier_typ = ER_RMC
	  endif
	endif
c
c------ EXTERNAL will get the average structure too ..
c
	if (rmc_mode.eq.rmc_mode_external) then
	  write (output_io,1050)
	  call chem_aver (.false.)
	endif
c
1050	format (/,' Calculating average structure ... ')
	end
c*****7*****************************************************************
	subroutine rmc_calcskal(ip,wtot,c,cc,ce,se,see,sk,ba)
c+
c	Calculate scaling factor / background
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'rmc.inc'
c
	real*8		c,cc,ce,se,see
	real		sk(rmc_max_planes)
	real		ba(rmc_max_planes)
	real		wtot
	integer		ip
c
c------ calculate the values
c
	if (rmc_doskal .and. rmc_doback) then
	  sk(ip) = (wtot*ce - se*c) / (wtot*cc - c*c)
	  ba(ip) = (se-sk(ip)*c) / wtot
	elseif (rmc_doskal .and. .not. rmc_doback) then
	  sk(ip) = (ce - rmc_back(ip)*c) / cc
	  ba(ip) = rmc_back(ip)
	elseif (.not. rmc_doskal .and. rmc_doback) then
	  sk(ip) = rmc_skal(ip)
	  ba(ip) = (se-sk(ip)*c) / wtot
	else
	  sk(ip) = rmc_skal(ip)
	  ba(ip) = rmc_back(ip)
	endif
c
c------ possible constrains
c
	if (rmc_constrain(ip).ne.ip) then
	  sk(ip) = sk(rmc_constrain(ip))
	  ba(ip) = ba(rmc_constrain(ip))
	endif
c
c------ force the values to be positive
c
	if (sk(ip).le.0.0) sk(ip) = rmc_skal(ip)
	if (ba(ip).le.0.0) ba(ip) = rmc_back(ip)
c
	end
c*****7*****************************************************************
      	subroutine rmc_stltab (is,ip,lsave)
c+
c	Save or restore SIN(THETA)/LAMBDA for diffuse routines
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'rmc.inc'
	include		'diffuse.inc'
c
	integer		is,ip,i
	logical		lsave
c
c------ Calculate sin(theta)/lambda table for plan 'ip' and
c------ symmetry peration 'is' and store it
c
	if (lsave) then
	  call four_stltab
	  do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	    ristl(offsq(ip,is)+i) = istl(i)
	  enddo
c
c------ just restore sin(theta)/lambda table
c
	else
	  do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	    istl(i) = ristl(offsq(ip,is)+i)
	  enddo
	endif
c
	end
c*****7*****************************************************************
      	subroutine rmc_formtab (ip,lsave)
c+
c	Save or restore FORMTAB for diffuse routines
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include		'diffuse.inc'
	include 	'rmc.inc'
c
	integer		ip,i,j
	logical		lsave,bano,blxray,bldbw
c
c------ Calculate formfactor table for plan 'ip' and store it
c
	if (lsave) then
	  bano   = ano
	  blxray = lxray
	  bldbw  = ldbw
	  ano    = rmc_ano(ip)
	  lxray  = rmc_lxray(ip)
	  ldbw   = rmc_ldbw(ip)
c
	  call four_formtab
	  do i=1,cr_nscat
	    do j=0,CFPKT
	      rcfact(j,i,ip) = cfact(j,i)
	    enddo
	  enddo
c
	  ano    = bano
	  lxray  = blxray
	  ldbw   = bldbw
c
c------ Restore diffuse cfact array for given plane
c
	else
	  do i=1,cr_nscat
	    do j=0,CFPKT
	      cfact(j,i)=rcfact(j,i,ip)
	    enddo
	  enddo
	endif
c
	end
c*****7*****************************************************************
      	subroutine rmc_layer (is,ip)
c+
c	Set diffuse layer variables for current plane/isym
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	integer		ip,is,i
c
	do i=1,3
	  xm (i) = rmc_eck(i,1,is,ip)
	  uin(i) = rmc_vi (i,1,is,ip)
	  vin(i) = rmc_vi (i,2,is,ip)
	enddo
c
	do i=1,2
	  num(i) = rmc_num(i,ip)
	enddo
c
	end
c*****7*****************************************************************
      	subroutine rmc_zero
c+
c	Reset rmc_csf array
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	integer		ip,il
c
	do ip=1,rmc_max_sq
	  do il=1,rmc_nlots
	    rmc_csf(ip,il) = cmplx(0.0d0,0.0d0)
	  enddo
	enddo
c
	end
c*****7*****************************************************************
      	subroutine rmc_gsg (mat,nsym,n)
c+
c	Do transformation q = gSg*
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'rmc.inc'
c
	integer		nsym,i,j,k,n
	real		mat(4,4,n),a(3,3),b(3,3)
c
	do i=1,nsym
	  do j=1,3
	    do k=1,3
	      a(j,k) = mat(j,k,i)
	    enddo
	  enddo
c
	  call matmulx (b,a,cr_rten)
	  call matmulx (a,cr_gten,b)
	  do j=1,3
	    do k=1,3
	      mat(j,k,i) = a(j,k)
	    enddo
	    mat(j,4,i) = mat(j,4,i)
	    mat(4,j,i) = 0.0
	  enddo
	  mat(4,4,i) = 1.0
	enddo
c
	end
c*****7*****************************************************************
      	subroutine rmc_trans (v,mat,nsym,n)
c+
c	Compute symmetrically equivalent vectors of 'v' using 'mat'
c-
	implicit      	none
c
	include		'config.inc'
c
	integer		nsym,n
	real		v(4,n),mat(4,4,n)
c
	integer		i,j,isym
c
	do isym=2,nsym
	  do i=1,3
	    v(i,isym)=0.0
	    do j=1,3
	      v(i,isym) = v(i,isym) + mat(i,j,isym)*v(j,1)
	    enddo
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine rmc_fcalc (ip,is,natoms,i_new,p_new,isel)
c+
c	This routine calculates the change in structure factor
c	cause by the RMC move.
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	real		p_new(3,rmc_max_atom)
	real		p_old(3,rmc_max_atom)
	real		off(3)
	integer		i_new(rmc_max_atom)
	integer		i_old(rmc_max_atom)
	integer		isel(rmc_max_atom)
	integer		i,ip,is,il,natoms
c
	logical		rmc_inlot
c
	do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	  do il=1,rmc_nlots
	    rmc_csf_new(offsq(ip,is)+i,il) = rmc_csf(offsq(ip,is)+i,il)
	  enddo
	enddo
c
	do i=1,natoms
	  p_old(1,i) = cr_pos(1,isel(i))
	  p_old(2,i) = cr_pos(2,isel(i))
	  p_old(3,i) = cr_pos(3,isel(i))
	  i_old(i)   = cr_iscat(isel(i))
	enddo
c
	call rmc_formtab(ip,.false.)
	call rmc_layer(is,ip)
	call rmc_stltab(is,ip,.false.)
c
	do il=1,rmc_nlots
	  do i=1,natoms
	    if (rmc_inlot(isel(i),il,off)) then
	      call rmc_strucf (i_old(i),p_old,i,is,ip,il,off,.false.)
	      call rmc_strucf (i_new(i),p_new,i,is,ip,il,off,.true.)
	    endif
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine rmc_strucf (iscat,pos,isite,is,ip,il,off,lplus)
c+
c	Here is the real calculation :-)
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	real		pos(3,rmc_max_atom),off(3)
	integer		iscat,isite,is,ip,il,i
	logical		lplus
c
	if (iscat.eq.0) return
c
	nxat = 1
	xat (nxat,1) = pos(1,isite) - off(1)
	xat (nxat,2) = pos(2,isite) - off(2)
	xat (nxat,3) = pos(3,isite) - off(3)
c
	call four_strucf(iscat,.true.)
c
	if (lplus) then
	  do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	    rmc_csf_new(offsq(ip,is)+i,il)=
     &	            rmc_csf_new(offsq(ip,is)+i,il) + tcsf(i)
	  enddo
	else
	  do i=1,rmc_num(1,ip)*rmc_num(2,ip)
	    rmc_csf_new(offsq(ip,is)+i,il)=
     &	            rmc_csf_new(offsq(ip,is)+i,il) - tcsf(i)
	  enddo
	endif
c
	end
c*****7*****************************************************************
      	logical function rmc_inlot (ia,il,off)
c+
c	Checks if given atom 'ia' is in given lot 'il'
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	real		xtest(3),off(3),x0
	integer*4	cr_end
	integer		iz(3),izmin,izmax
	integer		ia,il,is,i
c
	cr_end = cr_ncatoms*cr_icc(1)*cr_icc(2)*cr_icc(3) + 1
c
c------ We are not using lots
c
	if (rmc_ilots.eq.LOT_OFF) then
	  rmc_inlot = .true.
	  do i=1,3
	    off(i) = 0.0
	  enddo
c
c------ Box shaped lot
c
        elseif (ilots.eq.LOT_BOX) then
          if (ia.lt.cr_end) then
            call indextocell (ia,iz,is)
          else
            do i=1,3
              iz(i) = int(cr_pos(i,ia) + cr_dim0(i,1)) - 1
            enddo
          endif
c
          rmc_inlot = .true.
c
	  do i=1,3
	    izmin  = rmc_lots_orig(i,il)
	    izmax  = rmc_lots_orig(i,il)+ls_xyz(i)-1
	    off(i) = cr_dim0(i,1)+rmc_lots_orig(i,il)-1
	    rmc_inlot = rmc_inlot .and. 
     &	                (iz(i).ge.izmin).and.(iz(i).le.izmax)
c
	    if (.not. rmc_inlot .and. (izmax.gt.cr_icc(i))) then
	      izmin  = 1
	      izmax  = izmax - cr_icc(i)
	      off(i) = off(i)-float(cr_icc(i))
	      rmc_inlot = rmc_inlot .and. 
     &	                  (iz(i).ge.izmin).and.(iz(i).le.izmax)
	    endif
	  enddo
c
c------ Ellipsoid shaped lot
c
	elseif (rmc_ilots.eq.LOT_ELI) then
          if (ia.lt.cr_end) then
            call indextocell (ia,iz,is)
          else
            do i=1,3
              iz(i) = int(cr_pos(i,ia) + cr_dim0(i,1)) - 1
            enddo
          endif
c
          rmc_inlot = .true.
c
	  do i=1,3
	    x0       = float(ls_xyz(i))/2.0
	    izmin    = rmc_lots_orig(i,il)
	    izmax    = rmc_lots_orig(i,il)+ls_xyz(i)-1
	    off(i)   = cr_dim0(i,1)+rmc_lots_orig(i,il)-1
	    xtest(i) = (iz(i)-izmin-x0+0.5)**2/x0**2
	    rmc_inlot = rmc_inlot .and. 
     &	                (iz(i).ge.izmin).and.(iz(i).le.izmax)
c
	    if (.not. rmc_inlot .and. (izmax.gt.cr_icc(i))) then
	      izmin  = 1
	      izmax  = izmax - cr_icc(i)
	      off(i) = off(i)-float(cr_icc(i))
	      rmc_inlot = rmc_inlot .and. 
     &	                  (iz(i).ge.izmin).and.(iz(i).le.izmax)
	    endif
	  enddo
c
	  rmc_inlot = rmc_inlot .and.
     &	              ((xtest(1)+xtest(2)+xtest(3)).le.1.0)
c
	endif
	end
c*****7*****************************************************************
      	subroutine rmc_makemove (isym,natoms,i_new,p_new,isel,imol)
c+
c	make accepted move
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'molecule.inc'
	include 	'rmc.inc'
c
	real		p_new(3,rmc_max_atom)
	integer		i_new(rmc_max_atom)
	integer		isel(rmc_max_atom)
	integer		imol(rmc_max_atom)
	integer		natoms
	integer		isym(rmc_max_planes)
c
	integer		i,j,ip,iq,is,il
c
	do ip=1,rmc_nplane
	  do is=1,isym(ip)
	    do iq=1,rmc_num(1,ip)*rmc_num(2,ip)
	      do il=1,rmc_nlots
	        rmc_csf(offsq(ip,is)+iq,il)=
     &	               rmc_csf_new(offsq(ip,is)+iq,il)
	      enddo
	    enddo
	  enddo
	enddo
c
	do i=1,natoms
	  cr_iscat(isel(i))=i_new(i)
	  do j=1,3
	    cr_pos(j,isel(i))=p_new(j,i)
	  enddo
	enddo
c
	if (.not. rmc_sel_atom .and. rmc_mode.eq.rmc_mode_swchem) then
	  is = mole_type(imol(1))
	  mole_type(imol(1)) = mole_type(imol(2))
	  mole_type(imol(2)) = is
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_check_dist (laccept,iatom,i_new,p_new)
c+
c	check distances
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include		'atom_env.inc'
	include		'param.inc'
	include		'chem.inc'
	include 	'rmc.inc'
c
	real		p_new(3,rmc_max_atom)
	integer		i_new(rmc_max_atom),iatom
	logical		laccept
c
	integer		i
	real		pos(3),werte
c
	werte = -1
	do i=1,3
	  pos(i)=p_new(i,iatom)
	enddo
c
c------ check distances to neighbouring atoms
c
	if (i_new(iatom).eq.0) return
c
	call do_find_env (1,werte,1,pos,0.1,rmc_mindist_max,chem_quick,
     &	                  chem_period)
	do i=1,atom_env(0)
	  if (cr_iscat(atom_env(i)).ne.0 .and. laccept) then
	    laccept = (res_para(i).ge.rmc_mindist(i_new(iatom),
     &	               cr_iscat(atom_env(i))))
	  endif
	enddo
c
	end
c*****7*****************************************************************
	subroutine rmc_select (imode,natoms,isel,iz1,iz2,is1,is2)
c+
c	Select two atom sites (global, local, ...)
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'random.inc'
	include 	'rmc.inc'
c
	integer		imode,natoms
	integer		isel(rmc_max_atom)
	integer		iz1(3),iz2(3)
	integer		i,is1,is2
	real		ran1
c
10	continue
	isel(1) = int(ran1(idum)*cr_natoms)+1
	if (isel(1).gt.cr_natoms .or. isel(1).lt.1) goto 10
c
20	continue
	isel(2) = int(ran1(idum)*cr_natoms)+1
	if (isel(2).gt.cr_natoms .or. isel(2).lt.1) goto 20
c
	call indextocell (isel(1),iz1,is1)
	call indextocell (isel(2),iz2,is2)
c
c------ Force to be same site
c
	if (imode.eq.rmc_local_site) then
	  is2 = is1
	  call celltoindex(iz2,is2,isel(2)) 
c
c------ only local +- 1 unit cell for second atom
c
	elseif (imode.eq.rmc_local_loc) then
	  do i=1,3
	    iz1(i)=max(2,iz1(i))
	    iz1(i)=min(cr_icc(i)-1,iz1(i))
	    iz2(i) = iz1(i) + nint(1.0-2.0*ran1(idum))
	  enddo
	  is2 = int(ran1(idum)*cr_ncatoms)+1
	  call celltoindex(iz1,is1,isel(1)) 
	  call celltoindex(iz2,is2,isel(2)) 
c
c------ only local +- 1 unit cell for second atom, same site
c
	elseif (imode.eq.rmc_local_locsite) then
	  do i=1,3
	    iz1(i)=max(2,iz1(i))
	    iz1(i)=min(cr_icc(i)-1,iz1(i))
	    iz2(i) = iz1(i) + nint(1.0-2.0*ran1(idum))
	  enddo
	  is2 = is1
	  call celltoindex(iz1,is1,isel(1)) 
	  call celltoindex(iz2,is2,isel(2)) 
	endif
c
	end
c*****7*****************************************************************
	subroutine rmc_genmove (laccept,natoms,p_new,i_new,isel)
c+
c	generate RMC move (for switch and relax. mode)
c
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'random.inc'
	include		'chem.inc'
	include 	'debug.inc'
	include 	'diffuse.inc'
	include 	'rmc.inc'
c
	real		p_new(3,rmc_max_atom)
	integer		i_new(rmc_max_atom)
	integer		isel(rmc_max_atom)
	integer		natoms
	logical		laccept
c
	integer		iz1(3),iz2(3)
	integer		i,j,is1,is2,il
	real		disp1,disp2
	real		ran1,gasdev
	real		dummy(3)
	logical		lflag,rmc_inlot
c
c------ Mode 'swchem': only switch atoms => pnew=pold
c
	if (rmc_mode.eq.rmc_mode_swchem) then
	  natoms  = 2
	  call rmc_select (rmc_local,natoms,isel,iz1,iz2,is1,is2)
	  laccept = rmc_allowed(cr_iscat(isel(1))) .and.
     &	            rmc_allowed(cr_iscat(isel(2))) .and.
     &	            cr_iscat(isel(1)).ne.cr_iscat(isel(2))
	  if (laccept) then
	    i_new(1) = cr_iscat(isel(2))
	    i_new(2) = cr_iscat(isel(1))
	    do j=1,3
	      p_new(j,1) = cr_pos(j,isel(1))
	      p_new(j,2) = cr_pos(j,isel(2))
	    enddo
	  endif
c
c------ Mode 'shift': shift atoms (gaussian distribution)
c
	elseif (rmc_mode.eq.rmc_mode_shift) then
	  natoms  = 1
10	  continue
	  isel(1) = int(ran1(idum)*cr_natoms)+1
	  if (isel(1).gt.cr_natoms .or. isel(1).lt.1) goto 10
	  laccept = rmc_allowed(cr_iscat(isel(1)))
	  if (laccept) then
	    i_new(1) = cr_iscat(isel(1))
	    do i=1,3
	      p_new(i,1) = cr_pos(i,isel(1)) + 
     &	                   gasdev(rmc_maxmove(i,i_new(1)))
	    enddo
	  endif
c
c------ Mode 'swdisp': switch displacements of two atoms
c
	elseif (rmc_mode.eq.rmc_mode_swdisp) then
	  natoms  = 2
	  call rmc_select (rmc_local,natoms,isel,iz1,iz2,is1,is2)
	  laccept = rmc_allowed(cr_iscat(isel(1))) .and.
     &	            rmc_allowed(cr_iscat(isel(2)))
	  if (laccept) then
	    i_new(1) = cr_iscat(isel(1))
	    i_new(2) = cr_iscat(isel(2))
	    do j=1,3
	      disp1 = cr_pos(j,isel(1))-chem_ave_pos(j,is1)-
     &	              float(iz1(j)-1)-cr_dim0(j,1)
	      disp2 = cr_pos(j,isel(2))-chem_ave_pos(j,is2)-
     &	              float(iz2(j)-1)-cr_dim0(j,1)
	      p_new(j,1) = cr_pos(j,isel(1)) - disp1 + disp2
	      p_new(j,2) = cr_pos(j,isel(2)) - disp2 + disp1
	    enddo
	  endif
c
c------ Mode 'external': Moves defined by external subroutine
c
	elseif (rmc_mode.eq.rmc_mode_external) then
          call rmc_genmove_ext (laccept,natoms,p_new,i_new,isel)
	endif
c
c------ Check if atoms are in any defined 'lots'
c
	if (laccept .and. (rmc_ilots.ne.LOT_OFF)) then
	  do i=1,natoms
	    lflag = .false.
	    do il=1,rmc_nlots
	      lflag = lflag .or. (rmc_inlot(isel(i),il,dummy))
	    enddo
	    laccept = laccept .and. lflag
	  enddo
	endif
c
c------ Check distances
c
	do i=1,natoms
	  if (laccept) then
	    call rmc_check_dist(laccept,i,i_new,p_new)
	  endif
	enddo
c
c------ Uncomment these lines to debug generated moves
c
	if (laccept .and. dbg) then
	  do j=1,natoms
	    write (output_io,11) j,isel(j),cr_at_lis(cr_iscat(isel(j))),
     &	                 cr_iscat(isel(j)),(cr_pos(i,isel(j)),i=1,3),
     &                   isel(j),cr_at_lis(i_new(j)),i_new(j),
     &	                 (p_new(i,j),i=1,3),
     &	                 ((p_new(i,j)-cr_pos(i,isel(j))),i=1,3)
	  enddo
	endif
c
11	format(' DBG: # ',I3,' old: atom ',I6,' (',A4,',#',I3,
     &	       ') : ',3(F9.3,1X),/,12X,'new: atom ',I6,' (',A4,
     &	       ',#',I3,') : ',3(F9.3,1X),/,35X,'shift : ',
     &	       3(F9.3,1X),/,72('-'))
c
	end
c**********************************************************************
	subroutine rmc_genmove_mol (laccept,natoms,p_new,i_new,
     &	                            isel,imol)
c+
c	Generate RMC move (for switch and relax. mode)
c	Version for molecules ..
c
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'random.inc'
	include		'chem.inc'
	include 	'diffuse.inc'
	include 	'molecule.inc'
	include 	'rmc.inc'
	include		'debug.inc'
	include		'errlist.inc'
c
	real		p_new(3,rmc_max_atom)
	integer		i_new(rmc_max_atom)
	integer		isel(rmc_max_atom)
	integer		imol(rmc_max_atom)
	integer		natoms
	logical		laccept
c
	integer		iz1(3),iz2(3)
	integer		i,j,k,is1,is2,il,i0,j0
	real		disp1(3),disp2(3)
	real		ran1,gasdev
	real		dummy(3)
	logical		lflag,rmc_inlot
c
	natoms = 0
c
c------ Mode 'swchem': only switch atoms => pnew=pold
c
	if (rmc_mode.eq.rmc_mode_swchem) then
	  imol(1) = int(ran1(idum)*mole_num_mole)+1
	  imol(2) = int(ran1(idum)*mole_num_mole)+1
	  laccept = rmc_allowed(mole_type(imol(1))) .and.
     &	            rmc_allowed(mole_type(imol(2))) .and.
     &	            mole_type(imol(1)).ne.mole_type(imol(2))
c
	  if (laccept) then
	    natoms = 2*mole_len(imol(1))
	    i0 = mole_cont(mole_off(imol(1))+1)
            j0 = mole_cont(mole_off(imol(2))+1)
	    do i=1,mole_len(imol(1))
	      j        = i + mole_len(imol(1))
	      isel(i)  = mole_cont(mole_off(imol(1))+i)
	      isel(j)  = mole_cont(mole_off(imol(2))+i)
	      i_new(i) = cr_iscat(isel(j))
	      i_new(j) = cr_iscat(isel(i))
	      do k=1,3
	        p_new(k,i) = cr_pos(k,isel(j)) -
     &	                     cr_pos(k,j0) + cr_pos(k,i0)
	        p_new(k,j) = cr_pos(k,isel(i)) -
     &	                     cr_pos(k,i0) + cr_pos(k,j0)
	      enddo
	    enddo
	  endif
c
c------ Mode 'shift': shift atoms (gaussian distribution)
c
	elseif (rmc_mode.eq.rmc_mode_shift) then
	  imol(1) = int(ran1(idum)*mole_num_mole)+1
	  laccept = rmc_allowed(mole_type(imol(1))) 
c
	  if (laccept) then
	    natoms = mole_len(imol(1))
	    do j=1,3
	      disp1(j) = gasdev(rmc_maxmove(j,mole_type(imol(1))))
	    enddo
c
	    do i=1,mole_len(imol(1))
	      isel(i)  = mole_cont(mole_off(imol(1))+i)
	      i_new(i) = cr_iscat(isel(i))
	      do j=1,3
	        p_new(j,i) = cr_pos(j,isel(i)) + disp1(j)
	      enddo
	    enddo
	  endif
c
c------ Mode 'swdisp': switch displacements of two atoms
c
	elseif (rmc_mode.eq.rmc_mode_swdisp) then
	  imol(1) = int(ran1(idum)*mole_num_mole)+1
	  imol(2) = int(ran1(idum)*mole_num_mole)+1
	  laccept = rmc_allowed(mole_type(imol(1))) .and.
     &	            rmc_allowed(mole_type(imol(2))) .and.
     &	            mole_type(imol(1)).ne.mole_type(imol(2))
c
	  if (laccept) then
	    natoms = 2*mole_len(imol(1))
	    call indextocell (mole_cont(mole_off(imol(1))+1),iz1,is1)
	    call indextocell (mole_cont(mole_off(imol(2))+1),iz2,is2)
	    do j=1,3
	      disp1(j) = cr_pos(j,mole_cont(mole_off(imol(1))+1))
     &	              -chem_ave_pos(j,is1)-float(iz1(j)-1)-cr_dim0(j,1)
	      disp2(j) = cr_pos(j,mole_cont(mole_off(imol(2))+1))
     &	              -chem_ave_pos(j,is2)-float(iz2(j)-1)-cr_dim0(j,1)
	    enddo
c
	    do i=1,mole_len(imol(1))
	      j        = i + mole_len(imol(1))
	      isel(i)  = mole_cont(mole_off(imol(1))+i)
	      isel(j)  = mole_cont(mole_off(imol(2))+i)
	      i_new(i) = cr_iscat(isel(i))
	      i_new(j) = cr_iscat(isel(j))
	      do k=1,3
	        p_new(k,i) = cr_pos(k,isel(i)) - disp1(k) + disp2(k)
	        p_new(k,j) = cr_pos(k,isel(j)) - disp2(k) + disp1(k)
	      enddo
	    enddo
	  endif
c
c------ Mode 'external': Moves defined by external subroutine
c
	elseif (rmc_mode.eq.rmc_mode_external) then
          call rmc_genmove_ext (laccept,natoms,p_new,i_new,isel)
	endif
c
c------ Check if atoms are in any defined 'lots'
c
	if (laccept .and. (rmc_ilots.ne.LOT_OFF)) then
	  do i=1,natoms
	    lflag = .false.
	    do il=1,rmc_nlots
	      lflag = lflag .or. (rmc_inlot(isel(i),il,dummy))
	    enddo
	    laccept = laccept .and. lflag
	  enddo
	endif
c
c------ Check distances
c
	do i=1,natoms
	  if (laccept) then
	    call rmc_check_dist(laccept,i,i_new,p_new)
	  endif
	enddo
c
c------ Uncomment these lines to debug generated moves
c
        if (laccept .and. dbg) then
    	  write (output_io,11) imol
          do j=1,natoms
            write (output_io,12) j,isel(j),cr_at_lis(cr_iscat(isel(j))),
     &	                 cr_iscat(isel(j)),(cr_pos(i,isel(j)),i=1,3),
     &	                 isel(j),cr_at_lis(i_new(j)),i_new(j),
     &	                 (p_new(i,j),i=1,3),
     &	                 ((p_new(i,j)-cr_pos(i,isel(j))),i=1,3)
          enddo
        endif
c
11	format(/,' DBG: molecules # ',10I3)
12	format(' DBG: # ',I3,' old: atom ',I6,' (',A4,',#',I3,
     &	       ') : ',3(F9.3,1X),/,12X,'new: atom ',I6,' (',A4,
     &	       ',#',I3,') : ',3(F9.3,1X),/,35X,'shift : ',
     &	       3(F9.3,1X),/,72('-'))
c
	end
c**********************************************************************
	subroutine rmc_symmetry (n_hkl,hkl,mat_hkl,max_hkl,
     %	                         lfriedel_remove,lacentric)
c-
c	Performs the space group symmetry on the given [hkl].
c	Altered from 'subroutine symmetry' in 'structur.f'.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'errlist.inc'
c
	integer		max_hkl,n_hkl
	real		hkl(4,max_hkl),mat_hkl(4,4,max_hkl)
	logical         lfriedel_remove
	logical         lacentric
c
	real          	 x(4),y(4),xmat(4,4),wmat(4,4),eps
	integer        	i,j,k,l,ii,iii,igs,igg,ipg,ia,iaa
c
	include 	'generate.inc'
c
	data           	eps/0.00001/
c
	n_hkl     = 1
	ii        = n_hkl
	lacentric = .true.
c
	if (cr_spcgrno.eq.0) cr_spcgrno=1
c
c------ Loop over all generators of spacegroup cr_spcgrno
c
	do igs = 1,generspcgr(0,cr_spcgrno)
	  igg = generspcgr(igs,cr_spcgrno)
	  iii = n_hkl
	  do i = 1,4
	    do j = 1,4
	      xmat(i,j)=0.0
	    enddo
	    xmat(i,i)=1.0
	  enddo
c
c------	--Loop over all powers of generator igg
c
	  do ipg = 1,generpower(igg)
	    do i = 1,4
	      do j = 1,4
	        wmat(i,j)=0.0
	        do k = 1,4
	          wmat(i,j)=wmat(i,j)+generators(i,k,igg)*xmat(k,j)
	        enddo
	      enddo
	    enddo
c
c------ ----remove translation part of generator
c
c	    do i=1,3
c	      wmat(i,4)=0.0
c	      wmat(4,i)=0.0
c	    enddo
c	    wmat(4,4)=1.0
c
c------	----Multiply first point with current generator
c	    Since x(4) = 0.0, removal of translational part is no longer
c	    necessary. The part is indeed needed for proper phase transferal
c
	    do i=1,4
	      x(i)=hkl(i,ii)
	    enddo
	    x(4)=0.0
	    call trans(x,wmat,y,4)
c
c------	----check for existence with all previously generated hkl's
c 
	    do ia=ii,iii
	      if( (abs( y(1)-hkl(1,ia)).lt. eps .and. 
     &	           abs( y(2)-hkl(2,ia)).lt. eps .and. 
     &	           abs( y(3)-hkl(3,ia)).lt. eps      )  ) goto 10
	      if( (abs(-y(1)-hkl(1,ia)).lt. eps .and.
     &	           abs(-y(2)-hkl(2,ia)).lt. eps .and. 
     &	           abs(-y(3)-hkl(3,ia)).lt. eps      )  ) then
	        lacentric = .false.
	        if(lfriedel_remove) then
	          goto 10
	        endif
	      endif
	    enddo
c
	    if(n_hkl.lt.max_hkl) then
	      n_hkl = n_hkl+1
	      do i=1,4
	        hkl(i,n_hkl) = y(i)
	        do j=1,4
	          mat_hkl(i,j,n_hkl)=wmat(i,j)
	        enddo
	      enddo
	    else
	      ier_num = -13
	      ier_typ = ER_RMC
	      return
	    endif
c
c------	----Multiply all other points with current generator
c
	    do iaa = ii+1,iii
	      do i=1,4
	        x(i)=hkl(i,iaa)
	      enddo
	      x(4)=0.0
	      call trans(x,wmat,y,4)
c
	      if(n_hkl.lt.max_hkl) then
	        n_hkl = n_hkl+1
	        do i=1,4
	          hkl(i,n_hkl) = y(i)
	          do j=1,4
	            mat_hkl(i,j,n_hkl)=0.0
	            do l=1,4
	              mat_hkl(i,j,n_hkl)=mat_hkl(i,j,n_hkl)+
     &	                                 mat_hkl(i,l,iaa)*wmat(l,j)
	            enddo
	          enddo
	        enddo
	      else
	        ier_num = -13
	        ier_typ = ER_RMC
	        return
	      endif
	    enddo
c
c------	--set xmat to current power of generator
c
	    do i = 1,4
	      do j = 1,4
	        xmat(i,j)=wmat(i,j)
	      enddo
	    enddo
	  enddo
10	  continue
	enddo
	end
c**********************************************************************
	subroutine errlist_rmc
c-
c	Displays error Messages for the error type RMC
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (iu=-21,io=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  'Number of LOTS exceeds maximum              ',      !-21
     &  'No valid move after 1000 disp. intervalls   ',      !-20
     &  'Invalid constrain entered                   ',      !-19
     &  'Data file is not an ASCII PGM file          ',      !-18
     &  'Data and weight file have different sizes   ',      !-17
     &  'Invalid weighting scheme or weighting file  ',      !-16
     &  'Invalid data type selected                  ',      !-15
     &  'No experimental data within given q limits  ',      !-14
     &  'Too many symmetrically equivalent planes    ',      !-13
     &  'Displacements too small for SWDISP mode     ',      !-12
     &  'Only ONE atom type present in SWCHEM mode   ',      !-11
     &  'No atom types selected for RMC run          ',      !-10
     &  'Invalid RMC/MC mode selected                ',      ! -9
     &  'Invalid symmetry number selected            ',      ! -8
     &  'Invalid plane selected                      ',      ! -7
     &  'Too many atoms per molecule for RMC         ',      ! -6
     &  'Invalid method (x,n) selected               ',      ! -5
     &  'No experimental data present                ',      ! -4
     &  'No atoms present in model crystal           ',      ! -3
     &  'Too many experimental data points           ',      ! -2 
     &  'Too many experimental data planes           ',      ! -1 
     &  ' '       !  0
     &	/
c
	call disp_error ('RMC ',error,iu,io)
	end
C*****7****************************************************************
