c*****7*****************************************************************
	subroutine matmulx(a,b,c)
c+
c	Matrixmultiplication a=b*c
c-
	implicit      none
c
	integer       i,j,k
	real          a(3,3),b(3,3),c(3,3)
c
	do i=1,3
	  do j=1,3
	    a(i,j)=0.0
	    do k=1,3
	      a(i,j)=a(i,j) + b(i,k)*c(k,j)
	    enddo
	  enddo
	enddo
	end
c*****7*****************************************************************
	subroutine matmul4(a,b,c)
c+
c	Matrixmultiplication a=b*c for (4x4) matrices.
c-
	implicit      none
c
	integer       i,j,k
	real          a(4,4),b(4,4),c(4,4)
c
	do i=1,4
	  do j=1,4
	    a(i,j)=0.0
	    do k=1,4
	      a(i,j)=a(i,j) + b(i,k)*c(k,j)
	    enddo
	  enddo
	enddo
	end
c*****7*****************************************************************
	subroutine transmat(mat,idim)
c-
c	Replaces a matrix by its transpose
c+
	implicit      none
c
	integer       idim,i,j
	real          mat(idim,idim),d
c
	do i=2,idim
	  do j=1,i-1
	    d        = mat(i,j)
	    mat(i,j) = mat(j,i)
	    mat(j,i) = d
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine invmat(imat,a)
c
c	calculates the inverse matrix "imat" to input matrix "a"
c 
	implicit      none
c
	include      'errlist.inc'
c
	real imat(3,3),a(3,3),det
c
	det= a(1,1)*(a(2,2)*a(3,3) - a(2,3)*a(3,2)) +
     &       a(2,1)*(a(3,2)*a(1,3) - a(1,2)*a(3,3)) +
     &       a(3,1)*(a(1,2)*a(2,3) - a(1,3)*a(2,2))

	if(abs(det).gt.0.0) then
	  ier_num = 0
	  ier_typ = ER_NONE
c
	  imat(1,1)= (a(2,2)*a(3,3) - a(2,3)*a(3,2) )/det
	  imat(1,2)=-(a(1,2)*a(3,3) - a(1,3)*a(3,2) )/det
	  imat(1,3)= (a(1,2)*a(2,3) - a(1,3)*a(2,2) )/det
c
	  imat(2,1)=-(a(2,1)*a(3,3) - a(3,1)*a(2,3) )/det
	  imat(2,2)= (a(1,1)*a(3,3) - a(1,3)*a(3,1) )/det
	  imat(2,3)=-(a(1,1)*a(2,3) - a(1,3)*a(2,1) )/det
c
	  imat(3,1)= (a(2,1)*a(3,2) - a(3,1)*a(2,2) )/det
	  imat(3,2)=-(a(1,1)*a(3,2) - a(1,2)*a(3,1) )/det
	  imat(3,3)= (a(1,1)*a(2,2) - a(1,2)*a(2,1) )/det
c
	else
	  ier_num = -1
	  ier_typ = ER_MATH
	endif
c
	end      
C*****7*****************************************************************
	subroutine invmat4(matrix)
c-
c	inverts a 4*4 Symmetry operation
c+
	implicit       none
c
	include        'errlist.inc'
c
	real           matrix(4,4)
	integer        i,j
	real           a(3,3),b(3,3),t(3)
c
	do i=1,3
	  do j=1,3
	    a(i,j) = matrix(i,j)
	  enddo
	  t(i) = matrix(i,4)
	enddo
c
	call invmat(b,a)
c
	if(ier_num.eq.0) then
	  do i=1,3
	    matrix(i,4) = 0.0
	    do j=1,3
	      matrix(i,j) = b(i,j)
	      matrix(i,4) = matrix(i,4) - b(i,j)*t(j)
	    enddo
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine lattice(a0,ar,eps,gten,reps,rten,win,wrez,vol,vr,
     &	                 lout)
c+
c     	Calculates lattice constants, metric and reciprocal metric
c     	tensor, permutation tensors and unit cell volume.
c	It's done quite some old fashioned way, rather than calculating
c	the direct metric tensor and its inverse.
c-
	implicit      	none
c
	include 	'prompt.inc'
	include 	'errlist.inc'
	include 	'wink.inc'
c
	integer     	i,i1,i2,j
	logical     	lout
      	real 		a0(3),ar(3),eps(3,3,3),gten(3,3)
	real		reps(3,3,3),rten(3,3)
	real 		win(3),wrez(3),vol,vr
	real		cosa,cosb,cosg,cos1,cos2,sin1,sin2
	real 		cosi,sind,cosd,acosd
c
	cosa=cosd(win(1))
	cosb=cosd(win(2))
	cosg=cosd(win(3))                        
	vol=1 - cosa*cosa - cosb*cosb - cosg*cosg
	vol=vol + 2*cosa*cosb*cosg
c
	if(vol.gt.0.0) then
	  vol=sqrt(vol)*a0(1)*a0(2)*a0(3)
	  vr=1./vol
c
c------ - calculate direct metric tensor
c
	  call tensor(gten,a0,win)
c
c------ - calculate reciprocal lattice constants
c
	  do i=1,3
	    i1=mod(i  ,3) + 1
	    i2=mod(i+1,3) + 1
	    ar(i)=a0(i1)*a0(i2)*sind(win(i))/vol
	    cos1=cosd(win(i1))
	    cos2=cosd(win(i2))
	    cosi=cosd(win(i))
	    sin1=sind(win(i1))
	    sin2=sind(win(i2))
	    wrez(i)=acosd((cos1*cos2-cosi)/(sin1*sin2))
	  enddo
c
c------ - calculate reciprocal tensor
c
	  call tensor(rten,ar,wrez)
c
c------ - calculate premutation tensors
c
	  eps(1,2,3)= vol
	  eps(2,3,1)= vol
	  eps(3,1,2)= vol
	  eps(1,3,2)=-vol
	  eps(3,2,1)=-vol
	  eps(2,1,3)=-vol
	  reps(1,2,3)= vr
	  reps(2,3,1)= vr
	  reps(3,1,2)= vr
	  reps(1,3,2)=-vr
	  reps(3,2,1)=-vr
	  reps(2,1,3)=-vr
c
c------ - output ?
c
	  if (lout) then
	    write(output_io,2001) (a0(i),i=1,3),(win(i),i=1,3),vol
	    write(output_io,2002) ((gten(i,j),j=1,3),i=1,3)
	    write(output_io,2003) (ar(i),i=1,3),(wrez(i),i=1,3),vr
	    write(output_io,2004) ((rten(i,j),j=1,3),i=1,3)
	  endif
c
	else
	  ier_num = -35
	  ier_typ = ER_APPL
	endif
c
2001 	format( ' Lattice constants :'/,6(2X,F9.5),2X,G12.6) 
2002 	format(/' Metric Tensor     :'/(3(' ',3(2X,F11.5)/)))
2003 	format( ' Reciprocal Lattice constants :'/,6(2X,F9.5),2X,G12.6)
2004 	format(/' Reciprocal metric tensor     : '/
     &            (3(' ',3(2X,F11.5)/)))
	end
c*****7*****************************************************************
	subroutine tensor(ten,vec,win)
c+
c	Calculates the metric tensor. Works both for direct and 
c	reciprocal metric tensor.
c-
	implicit      	none
c
	integer      	idim
	parameter	(idim=3)
c
	real 		ten(idim,idim),vec(idim),win(idim)
	real   		cosd
	integer 	i,j
c
	do i=1,idim 
	  do j=1,idim 
	    if(i .ne. j) then
	        ten(i,j)=vec(i)*vec(j)*cosd(win(6-(i+j)))
	    else
	        ten(i,j)=vec(i)*vec(j)
	    endif
	  enddo
	enddo
	end 
C*****7*****************************************************************
