c*****7*****************************************************************
      	subroutine ther_displ (line,ll)
c-
c     	Displaces atoms in a completely random fashion. The mean square
c     	displacement is given by the temperature factor. Molecules
c	are displaced according to the B value of the atom at the
c	origin.
c
c       Version  : 2.0
c       Date     : 31 October 1997
c
c       Author   : Th. Proffen (tproffen@lanl.gov)
c                  R.B. Neder  (reinhard.neder@mail.uni-wuerzburg.de)
c
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include		'molecule.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=2)
c
	character*(*)	line
	character*200	cpara(maxw)
	character*9	at_name,at_name_i
	integer		lpara(maxw)
	integer 	ll,i,j,k,is,ii,ianz
	integer		uc_n(0:maxscat)
      	real 		uc(3),up(3)
	real		uc_max(3,0:maxscat)
	real		uc_su2(3,0:maxscat)
	real		pi2,bfac,a
	real    	gasdev
	logical 	flag_all,flag_mol
c
c------ get parameters
c
	call get_params(line,ianz,cpara,lpara,maxw,ll)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  flag_all = .true.
	  flag_mol = .false.
	elseif (ianz.ge.1) then
	  flag_mol = (cpara(1)(1:1).eq.'m')
	  flag_all = .true.
	  if (flag_mol) then
	    if (ianz.eq.2) then
	      flag_all = (cpara(2)(1:1).ne.'2')
	    endif
    	  else
	    flag_all = (cpara(1)(1:1).ne.'2')
	  endif
	endif
c
c------ reset counters
c
	do i=0,maxscat
	  uc_n(i)=0
	  do j=1,3
	    uc_max(j,i)=0.0
	    uc_su2(j,i)=0.0
	  enddo
	enddo
c
      	pi2  = (4.0*atan(1.0))**2
      	bfac = 1.0/(8.0*pi2)
c
c------ Displace molecules
c
	if (flag_mol) then
      	  do i=1,mole_num_mole
	    is=mole_cont(mole_off(i)+1)
            a=sqrt(bfac*cr_dw(cr_iscat(is)))
	    is=mole_type(i)
	    do ii=1,3
	      if (flag_all .or. cr_icc(ii).ne.1) then
	        up(ii)=gasdev(a)
	      else
	        up(ii)=0.0
	      endif
	    enddo
            call trans(up,cr_gmat,uc,3)
            do j=1,3
	      do k=1,mole_len(i)
	        ii = mole_cont(mole_off(i)+k)
                cr_pos(j,ii)=cr_pos(j,ii)+uc(j)
	        uc_su2(j,is)=uc_su2(j,is)+up(j)**2 
	      enddo
	      uc_max(j,is)=max(uc_max(j,is),abs(up(j)))
	    enddo
	    uc_n(is) = uc_n(is)+1  
  	  enddo
c
c------ displace atoms
c
	else
      	  do i=1,cr_natoms
	    is=cr_iscat(i)
            a=sqrt(bfac*cr_dw(is))
	    do ii=1,3
	      if (flag_all .or. cr_icc(ii).ne.1) then
	        up(ii)=gasdev(a)
	      else
	        up(ii)=0.0
	      endif
	    enddo
            call trans(up,cr_gmat,uc,3)
            do j=1,3
              cr_pos(j,i )=cr_pos(j,i)+uc(j)
	      uc_max(j,is)=max(uc_max(j,is),abs(up(j)))
	      uc_su2(j,is)=uc_su2(j,is)+up(j)**2
	    enddo
	    uc_n(is) = uc_n(is)+1  
	  enddo
	endif
c
c------	Update crystal dimensions
c
	call update_cr_dim
c
c------ print statistics of 'therm' command
c
	if (flag_mol) then
	  write (output_io,1000) 'Molecule'
	  do i=1,mole_num_type
	    if (uc_n(i).ne.0) then
	      write (output_io,1050) i,cr_dw(i),(bfac*cr_dw(i)),
     &	                     ((uc_su2(j,i)/uc_n(i)),j=1,3),
     &	                     (uc_max(j,i),j=1,3)
	    endif
	  enddo
	else
	  write (output_io,1000) 'Atom    '
	  do i=0,cr_nscat
	    if (uc_n(i).ne.0) then
	      at_name_i = at_name(i)
	      write (output_io,1150) at_name_i,cr_dw(i),(bfac*cr_dw(i)),
     &	                     ((uc_su2(j,i)/uc_n(i)),j=1,3),
     &	                     (uc_max(j,i),j=1,3)
	    endif
	  enddo
	endif
c
1000	format (' Thermal displacements summary :',//,
     &	        17x,'Input',15x,'Achieved',13x,'Maximum displacement',/
     &	         3x,a8,4x,'B',3x,'<u**2>',5x,'<ux**2> <uy**2> <uz**2>',
     &	         8x,'x',6x,'y',6x,'z',/,3x,75('-'))
1050	format (3x,i5,4x,f5.2,2x,f6.4,6x,3(f6.4,2x),3x,3(f6.4,1x))
1150	format (3x,a9   ,f5.2,2x,f6.4,6x,3(f6.4,2x),3x,3(f6.4,1x))
      	end
