c*****7*****************************************************************
	subroutine waves
c-
c	calculates the displacement of the atoms due to a plane wave 
c	travelling through the crystal
c
c	Version  : 1.21
c	Date     : 14. January 1998
c
c	Author   : R.B. Neder  (reinhard.neder@mail.uni-wuerzburg.de)
c	           Th. Proffen (tproffen@lanl.gov)
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'doact.inc'
	include       'errlist.inc'
	include       'microsta.inc'
	include       'learn.inc'
	include       'macro.inc'
	include       'prompt.inc'
	include       'molecule.inc'
	include       'waves.inc'
c
	integer        maxw
	logical        lnew,lold
c
	parameter     (maxw=MAXSCAT)
	parameter     (lnew=.true.,lold=.false.)
c
	character*200  line,zeile,cpara(maxw)
	character*200  cdummy
	character*11   prom
	character*5    befehl
	real           werte(maxw)
	integer        lpara(maxw),lp,length,lbef,ldummy
	integer        indxg,ianz,is
	logical        lend
c
	logical        str_comp
c
	lend=.false.
	call no_error
c
	do while (.not. lend)
	  prom=prompt//'/wave'
	  call get_command (line,length,befehl,lbef,zeile,lp,prom)
	  if(ier_num.eq.0) then
	  if(line.ne.' ' .and. line(1:1).ne.'#') then
c
c------	--- search for "="
c
	    indxg=index(line,'=')
	    if(indxg.ne.0 
     &	           .and. .not.(str_comp(befehl,'echo',2,lbef,4))
     &	           .and. .not.(str_comp(befehl,'syst',2,lbef,4))
     &	           .and. .not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                       str_comp(befehl,'?   ',2,lbef,4)) )then
c
c------	----- evaluate an expression and assign value a variabble
c
	      call do_math(line,indxg,length)
	    else
c
c------ --- execute a macro file
c
            if(befehl(1:1).eq.'@') then
	      if(length.ge.2) then
	        call file_kdo(line(2:length),length-1)
	      else
	        ier_num = -13
	        ier_typ = ER_MAC
	      endif
c
c------	--- accoustic wave 'acco'
c
	    elseif(str_comp(befehl,'acco',2,lbef,4)) then
	      wv_lacoust = .true.
c
c------	--- amplitude of wave 'ampl'
c
	    elseif(str_comp(befehl,'ampl',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if (ier_num.eq.0) then
	        if (ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_amp = werte(1)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- list asymmetric unit 'asym'
c
	    elseif(str_comp(befehl,'asym',2,lbef,4)) then
	      call show_asym
c
c------	--- list atoms present in the crystal 'chem'
c
	    elseif(str_comp(befehl,'chem',2,lbef,4)) then
	      call show_chem
c
c------	--- continues a macro 'continue'
c
	    elseif(str_comp(befehl,'continue',2,lbef,8)) then
	      call macro_continue(zeile,lp)
c
c------	--- density wave 'dens'
c
	    elseif(str_comp(befehl,'dens',3,lbef,4)) then
	      wv_iwave = WV_DENS
c
c------	--- select/deselect molecules
c
	    elseif(str_comp(befehl,'msel',2,lbef,4) .or.
     &	           str_comp(befehl,'mdes',2,lbef,4)     ) then
c
	      call mole_select(zeile,lp,wv_latom,wv_repl,0,0,maxscat,
     &	                       wv_sel_atom,wv_md_s,wv_md_nr,lold,
     &	                       str_comp(befehl,'msel',2,lbef,4))
c
c------	--- select which atoms are copied to their image 'sele'
c
	    elseif(str_comp(befehl,'sele',2,lbef,4) .or.
     &	           str_comp(befehl,'dese',2,lbef,4)     ) then
c
	      call atom_select(zeile,lp,wv_latom,wv_repl,0,0,maxscat,
     &	                       wv_sel_atom,wv_md_s,wv_md_nr,lold,
     &	                       str_comp(befehl,'sele',2,lbef,4))
c
c------ --- Echo a string, just for interactive check in a macro 'echo'
c
	    elseif(str_comp(befehl,'echo',2,lbef,4)) then
	      call echo(zeile,lp)
c
c------	 -- Evaluate an expression, just for interactive check 'eval'
c
	    elseif(str_comp(befehl,'eval',2,lbef,4)) then
	      call do_eval(zeile,lp)
c
c------	--- exit 'exit'
c
	    elseif(str_comp(befehl,'exit',2,lbef,4)) then
	      lend=.true.
c
c------	--- wave function 'func'
c------ --- possible functions are 'box', 'sinusoidal', 'triangular'
c
	    elseif(str_comp(befehl,'func',1,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if (ianz.eq.1 .or. ianz.eq.2) then
	          if(str_comp(cpara(1),'box',3,lpara(1),3)) then
	            wv_func  = cpara(1)
	            wv_ifunc = WV_BOX
	            if (ianz.eq.2) then
	              call del_params(1,ianz,cpara,lpara,maxw)
	              call ber_params(ianz,cpara,lpara,werte,maxw)
	              if (werte(1).gt.0.0 .and. werte(1).lt.1.0) then
	                wv_asym = werte(1)
	              else
	                ier_num = -70
	                ier_typ = ER_APPL
	              endif
	            else
	              wv_asym = 0.5
	            endif
	          elseif(str_comp(cpara(1),'sinus',3,lpara(1),5)) then
	            wv_func  = cpara(1)
	            wv_ifunc = WV_SINUS
	          elseif(str_comp(cpara(1),'trian',3,lpara(1),5)) then
	            wv_func  = cpara(1)
	            wv_ifunc = WV_TRIANGLE
	          else
	            ier_num = -6
	            ier_typ = ER_COMM
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- help 'help','?'
c
	    elseif(str_comp(befehl,'help',2,lbef,4) .or.
     &	           str_comp(befehl,'?   ',1,lbef,4)     ) then
	      if(str_comp(zeile,'errors',2,lp,6) ) then
	        lp = lp + 7
	        call do_hel('discus '//zeile,lp)
	      else
	        lp = lp + 12
	        call do_hel('discus wave '//zeile,lp)
	      endif
c
c------	--- wave length 'leng'
c
	    elseif(str_comp(befehl,'leng',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_rlam = werte(1)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- longitudinal wave 'long'
c
	    elseif(str_comp(befehl,'long',2,lbef,4)) then
	      wv_iwave = WV_LONG
c
c------	----optical wave 'opti'
c
	    elseif(str_comp(befehl,'opti',2,lbef,4)) then
	      wv_lacoust = .false.
c
c------	--- direction of oscillation 'osci'
c
	    elseif(str_comp(befehl,'osci',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.3) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_swing(1) = werte(1)
	            wv_swing(2) = werte(2)
	            wv_swing(3) = werte(3)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- initial phase 'phas'
c
	    elseif(str_comp(befehl,'phas',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          if(str_comp(cpara(1),'rand',1,lpara(1),4)) then
	            wv_phase_typ = WV_RAND
	          else
	            wv_phase_typ = WV_FIX
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            if(ier_num.eq.0) then
	              wv_phase = werte(1)
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- upper limit of probability for density waves 'phig'
c
	    elseif(str_comp(befehl,'phig',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_phigh = min(1.0,abs(werte(1)))
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- lower limit of probability for density waves 'plow'
c
	    elseif(str_comp(befehl,'plow',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_plow = min(1.0,abs(werte(1)))
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- Select which molecules are repl. by density wave 'mrepl'
c
	    elseif(str_comp(befehl,'mrepl',2,lbef,5)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if (ier_num.eq.0) then
	        if (ianz.eq.2) then
	          cdummy = cpara(1)
	          ldummy = lpara(1)
	          call del_params(1,ianz,cpara,lpara,maxw)
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if (is.gt.0 .and. is.le.mole_num_type) then
	            is = nint(werte(1))
	            call mole_select(cdummy,ldummy,wv_latom,wv_repl,
     &	                 is,0,maxscat,wv_sel_atom,wv_md_s,wv_md_nr,
     &	                 lold,.true.)
	          else
	            ier_num = -64
	            ier_typ = ER_APPL
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif 
	      endif
c
c------	--- Select which atoms are repl. by density wave 'repl'
c
	    elseif(str_comp(befehl,'repl',2,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if (ier_num.eq.0) then
	        if (ianz.eq.2) then
	          cdummy = cpara(1)(1:lpara(1))
	          ldummy = lpara(1)
	          call del_params(1,ianz,cpara,lpara,maxw)
	          call get_iscat(ianz,cpara,lpara,werte,maxw,lnew)
	          if (ier_num.eq.0 .and. is.ne.-1) then
	            is = nint(werte(1))
	            call atom_select(cdummy,ldummy,wv_latom,wv_repl,
     &	                 is,0,maxscat,wv_sel_atom,wv_md_s,wv_md_nr,
     &	                 lold,.true.)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif 
	      endif
c
c------	--- select rotation mode for molecules
c
	    elseif(str_comp(befehl,'rot',2,lbef,3)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.3 .or. ianz.eq.6) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_iwave = WV_ROT
	            wv_rot_uvw(1) = werte(1)
	            wv_rot_uvw(2) = werte(2)
	            wv_rot_uvw(3) = werte(3)
	            if (ianz.eq.6) then
	              wv_rot_orig(1) = werte(4)
	              wv_rot_orig(2) = werte(5)
	              wv_rot_orig(3) = werte(6)
	            else
	              wv_rot_orig(1) = 0.0
	              wv_rot_orig(2) = 0.0
	              wv_rot_orig(3) = 0.0
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- run waves 'run'
c
	    elseif(str_comp(befehl,'run ',2,lbef,4)) then
	      call wave_run
c
c------	--- set constant shift 'shif'
c
	    elseif(str_comp(befehl,'shif',3,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_amp0 = werte(1)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	--- show current parameters 'show'
c
	    elseif(str_comp(befehl,'show',3,lbef,4)) then
	      call wave_show
c
c------	--- transversal wave 'tran'
c
	    elseif(str_comp(befehl,'tran',1,lbef,4)) then
	      wv_iwave = WV_TRANS
c
c------	--- direction of wave vector 'vect'
c
	    elseif(str_comp(befehl,'vect',1,lbef,4)) then
	      call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.3) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            wv_wave(1) = werte(1)
	            wv_wave(2) = werte(2)
	            wv_wave(3) = werte(3)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------ --- operating System Kommandos 'syst'
c
	    elseif(str_comp(befehl,'syst',2,lbef,4)) then
	      if(zeile.ne. ' ') then
	        call do_operating(zeile(1:lp),lp)
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ --- waiting for user input
c
	    elseif(str_comp(befehl,'wait',3,lbef,4)) then
	      call do_input(zeile,lp)
c
c------ --- End of command processing
c
	    else
	      ier_num = -8
	      ier_typ = ER_COMM
	    endif
	    endif
	  endif
	  endif
c
	  if(ier_num.ne.0) then
	    call errlist
	    if(lmakro) then
	      call macro_close
	      prompt_status = PROMPT_ON
	    endif
	    if(lblock) then
	      ier_num = -11
	      ier_typ = ER_COMM
	      return
	    endif
	    call no_error
	  endif
	enddo
c
	end
c*****7*********************************************************
	subroutine wave_show
c-
c	Shows current settiungs in WAVES segment
c+
	implicit      none
c
	include       'config.inc'
	include       'prompt.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microsta.inc'
	include       'molecule.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	character*15   cwave(3)
	character*9    at_lis(maxscat),at_name
	character*6    cphase(2)
	integer        mol_lis(maxscat)
	integer        i,j,k
c
	data cphase    /'random','fixed '/
	data cwave     /'sinusoidal wave',
     &	                'box shaped wave',
     &	                'triangular wave'/
c
	write (output_io,1000)
c
c------ Density wave selected
c
	if(wv_iwave.eq.WV_DENS) then
	  write (output_io,2000) 'density wave'
	  if (wv_ifunc.eq.WV_BOX) then
	    write (output_io,2100) cwave(wv_ifunc),wv_asym
	  else
	    write (output_io,2150) cwave(wv_ifunc)
	  endif
	  write (output_io,2200) (wv_wave(i),i=1,3)
	  write (output_io,2300) wv_rlam,wv_plow,wv_phigh,wv_phase,
     &	                cphase(wv_phase_typ)
	  write (output_io,2400) cmic_s(wv_md_s)
c
	  if(wv_md_s.eq.MD_S_IND) then
	    write (output_io,2500) wv_md_nr
	  endif
c
	  if (wv_sel_atom) then
	    write (output_io,2600)
	    do i=0,cr_nscat
	      if(wv_latom(i)) then
	        at_lis(1) = at_name(i)
	        at_lis(2) = at_name(wv_repl(i))
	        write (output_io,2650) i,at_lis(1),wv_repl(i),at_lis(2)
	      endif
	    enddo
	  else
	    write (output_io,2700)
	    do i=1,mole_num_type
	      write (output_io,2750) i,wv_repl(i)
	    enddo
	  endif
c
c------ Displacement or rotational wave 
c
	else
	  if(wv_lacoust) then
	    if (wv_iwave.eq.WV_TRANS) then
	      write (output_io,2000) 'transversal acoustic wave'
	    elseif (wv_iwave.eq.WV_LONG) then
	      write (output_io,2000) 'longitudinal acoustic wave'
	    elseif (wv_iwave.eq.WV_ROT) then
	      write (output_io,2000) 'rotational wave (molecules)'
	    endif
	  else
	    if (wv_iwave.eq.WV_TRANS) then
	      write (output_io,2000) 'transversal optical wave'
	    elseif (wv_iwave.eq.WV_LONG) then
	      write (output_io,2000) 'longitudinal optical wave'
	    elseif (wv_iwave.eq.WV_ROT) then
	      write (output_io,2000) 'rotational wave (molecules)'
	    endif
	  endif
	  if (wv_ifunc.eq.WV_BOX) then
	    write (output_io,2100) cwave(wv_ifunc),wv_asym
	  else
	    write (output_io,2150) cwave(wv_ifunc)
	  endif
	  write (output_io,2200) (wv_wave(i),i=1,3)
	  if(wv_iwave.eq.WV_TRANS) then
	    write (output_io,3100) (wv_swing(i),i=1,3)
	  endif
	  if(wv_iwave.eq.WV_ROT) then
	    write (output_io,3120) (wv_rot_uvw(i),i=1,3)
	    write (output_io,3140) (wv_rot_orig(i),i=1,3)
	  endif
	  write (output_io,3200) wv_rlam,wv_amp,wv_amp0,wv_phase,
     &	                cphase(wv_phase_typ)
	  write (output_io,2400) cmic_s(wv_md_s)
c
	  if(wv_md_s.eq.MD_S_IND) then
	    write (output_io,2500) wv_md_nr
	  endif
c
	  if (wv_sel_atom) then
	    j=0
	    do i=0,cr_nscat
	      if (wv_latom(i)) then
	        j=j+1
	        at_lis(j)=at_name(i)
	      endif
	    enddo
	    write (output_io,3300) (at_lis(k),k=1,j)
	  else
	    j=0
	    do i=0,mole_num_type
	      if (wv_latom(i)) then
	        j=j+1
	        mol_lis(j)=i
	      endif
	    enddo
	    write (output_io,3400) (mol_lis(k),k=1,j)
	  endif
	endif
c
1000	format (  ' Current settings for wave module : ')
c
2000	format (  '   Wave type           : ',2x,a)
2100	format (  '   Wave function       : ',2x,a,' - Delta : ',f4.2)
2150	format (  '   Wave function       : ',2x,a)
2200	format (  '   Wave vector         : ',3(2x,f9.4))
2300	format (  '   Wave length         : ',2x,f9.4/
     &	          '   Low  probabilty     : ',2x,f9.4/
     &	          '   High probabilty     : ',2x,f9.4/
     &	          '   Phase at 0,0,0      : ',2x,f9.4/
     &	          '   Phase status        : ',2x,a)
2400	format (/,'   Select status       : ',2x,a)
2500	format (  '   Selected MD type    : ',2x,i4)
2600	format (  '   Selected atoms      : ')
2650	format (  '     Atom typ ',i3,' (',a9,') to be replaced ',
     &                 'by atom type ',i3,' (',a9,')')
2700	format (  '   Selected molecules  : ')
2750	format (  '     Molecule typ ',i3,' to be replaced ',
     &                 'by molecule type ',i3)
c
3100	format (  '   Oscillation vector  : ',3(2x,f9.4))
3120	format (  '   Rotation axis       : ',3(2x,f9.4))
3140	format (  '   Rotation origin     : ',3(2x,f9.4))
3200	format (  '   Wave length         : ',2x,f9.4/
     &	          '   Amplitude           : ',2x,f9.4/
     &	          '   Constant shift      : ',2x,f9.4/
     &	          '   Phase at 0,0,0      : ',2x,f9.4/
     &	          '   Phase status        : ',2x,a   )
3300	format (  '   Sel. atom types     : ',2x,50(a9,1x))
3400	format (  '   Sel. molecule types : ',2x,50(i4,1x))
c
	end
c*****7*********************************************************
	subroutine wave_run
c-
c	Main waves routine	
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microsta.inc'
	include       'waves.inc'
	include       'symm.inc'
	include       'errlist.inc'
	include       'wink.inc'
c
	real           wavec(3),swingc(3)
	real           wavep(3),swingp(3)
	real           ww,rnn
	integer        i,lbef
c
	real           hkl(3),uvw(3),tran(3),orig(3),angle
	integer        start,end,power
	logical        latom(0:MAXSCAT)
	logical        pmult,mode,new,orig_mol,typ,sel_atom
c
	real           quad
	integer        len_str
	logical        str_comp
c
	real           sinus,box,triang
	external       sinus,box,triang
c
c------ Setup for rotational mode
c
	if (wv_iwave.eq.WV_ROT) then
	  if (wv_sel_atom) then
	    ier_num = -68
	    ier_typ = ER_APPL
	    return
	  endif
c
	  do i=1,3
	    hkl(i)   = sym_hkl(i)
	    uvw(i)   = sym_uvw(i)
	    tran(i)  = sym_trans(i)
	    orig(i)  = sym_orig(i)
	  enddo
	  angle    = sym_angle
	  start    = sym_start
	  end      = sym_end
	  power    = sym_power
	  mode     = sym_mode
	  new      = sym_new
	  orig_mol = sym_orig_mol
	  sel_atom = sym_sel_atom
	  typ      = sym_type
	  pmult    = sym_power_mult
	  do i=0,MAXSCAT
	    latom(i) = sym_latom(i)
	  enddo
c
	  sym_power      = 1
	  sym_power_mult = .false.
	  sym_type       = .true.
	  sym_new        = .false.
	  sym_mode       = .false.
	  sym_orig_mol   = .true.
	  sym_sel_atom   = .false.
	  do i=1,3
	    sym_uvw(i)   = wv_rot_uvw(i)
	    sym_orig(i)  = wv_rot_orig(i)
	    sym_trans(i) = 0.0
	  enddo
c
	  call trans(sym_uvw,cr_gten,sym_hkl,3)
	endif
c
c------	normalise wavevector to length 1 A,
c------	transform to normalised space
c
	ww=sqrt(quad(wv_wave,wv_wave,cr_gten))
	do i=1,3
	 wavec(i)=wv_wave(i)/ww
	enddo 
	call trans(wavec,cr_fmat,wavep,3)
c
c------	For density waves set correct values of amp and amp0 depending
c------	on wave function type
c
	if(wv_iwave.eq.WV_DENS) then
	  lbef = len_str(wv_func)
	  if    (str_comp(wv_func,'box',1,lbef,3)) then
	    wv_amp  =  wv_phigh-wv_plow
	    wv_amp0 =  wv_plow
	  elseif(str_comp(wv_func,'sinus',1,lbef,5)) then
	    wv_amp  = (wv_phigh-wv_plow)*0.5
	    wv_amp0 = (wv_phigh+wv_plow)*0.5
	  elseif(str_comp(wv_func,'trian',1,lbef,5)) then
	    wv_amp  =  wv_phigh-wv_plow
	    wv_amp0 =  wv_plow
	  endif
	else
c
c------	normalise oscillation vector to lenght 1 A,
c------	transform to normalised space
c
	  if(wv_iwave.eq.WV_TRANS) then
	    rnn=sqrt(quad(wv_swing,wv_swing,cr_gten))
	    do i=1,3
	      swingc(i)=wv_swing(i)/rnn
	    enddo
	  else
	    do i=1,3
	      swingc(i)=wavec(i)
	    enddo
	  endif
	endif
c
	call trans(swingc,cr_fmat,swingp,3)
c
c------ Working only inside microdomains
c
	if(wv_md_s.eq.MD_S_ALL .or.
     &	   wv_md_s.eq.MD_S_IND      ) then
	  if (wv_sel_atom) then
	    if    (wv_func(1:1).eq.'b') then
	      call wave_run_md(wavep,swingp,box)
	    elseif(wv_func(1:1).eq.'s') then
	      call wave_run_md(wavep,swingp,sinus)
	    elseif(wv_func(1:1).eq.'t') then
	      call wave_run_md(wavep,swingp,triang)
	    endif
	  else
	    if    (wv_func(1:1).eq.'b') then
	      call wave_run_md_mol(wavep,swingp,box)
	    elseif(wv_func(1:1).eq.'s') then
	      call wave_run_md_mol(wavep,swingp,sinus)
	    elseif(wv_func(1:1).eq.'t') then
	      call wave_run_md_mol(wavep,swingp,triang)
	    endif
	  endif
c
c------ or use all of the crystal (outside)
c
	elseif( wv_md_s.eq.MD_S_EVE .or.
     &	        wv_md_s.eq.MD_S_NON     ) then
	  if (wv_sel_atom) then
	    if    (wv_func(1:1).eq.'b') then
	      call wave_run_all(wavep,swingp,box)
	    elseif(wv_func(1:1).eq.'s') then
	      call wave_run_all(wavep,swingp,sinus)
	    elseif(wv_func(1:1).eq.'t') then
	      call wave_run_all(wavep,swingp,triang)
	    endif
	  else
	    if    (wv_func(1:1).eq.'b') then
	      call wave_run_all_mol(wavep,swingp,box)
	    elseif(wv_func(1:1).eq.'s') then
	      call wave_run_all_mol(wavep,swingp,sinus)
	    elseif(wv_func(1:1).eq.'t') then
	      call wave_run_all_mol(wavep,swingp,triang)
	    endif
	  endif
	endif
c
	call update_cr_dim
c
c------ Restore symm settings
c
	if (wv_iwave.eq.WV_ROT) then
	  do i=1,3
	    sym_hkl(i)   = hkl(i)
	    sym_uvw(i)   = uvw(i)
	    sym_trans(i) = tran(i)
	    sym_orig(i)  = orig(i)
	  enddo
	  sym_angle      = angle
	  sym_start      = start
	  sym_end        = end
	  sym_power      = power
	  sym_mode       = mode
	  sym_new        = new
	  sym_orig_mol   = orig_mol
	  sym_type       = typ
	  sym_sel_atom   = sel_atom
	  sym_power_mult = pmult
	  do i=0,MAXSCAT
	    sym_latom(i) = latom(i)
	  enddo
	endif
c
	end
c*****7*********************************************************
	subroutine wave_run_all(wavep,swingp,wave_func)
c-
c	Displaces all atoms by the corresponding
c	wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'random.inc'
	include       'microsta.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
	integer        i,j
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  wv_phase = ran1(idum) * 360.0
	endif
c
	do i=1,cr_natoms
c
c------ - Check if atom is a valid selection
c
	  if(wv_latom(cr_iscat(i))) then
	    if( wv_md_s      .eq.MD_S_EVE .or.
     &	        wv_md_s      .eq.MD_S_NON .and.
     &	        cr_md_next(i).ge.0             ) then
c
	      do j=1,3
	        uc(j)=cr_pos(j,i)
	      enddo
c
	      call trans(uc,cr_fmat,up,3)
	      arg = up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	      arg = arg + wv_phase*wv_rlam/360.
	      arg = amod(arg,wv_rlam)/wv_rlam
	      dis = wave_func(wv_amp,arg,wv_amp0)
c
	      if(wv_iwave.eq.WV_DENS) then
	        if(ran1(idum).gt.dis) then
	          cr_iscat(i) = wv_repl(cr_iscat(i))
	          if(cr_iscat(i).eq.0) then
	            cr_ndel = cr_ndel + 1
	          endif
	        endif
c
	      else
	        if (.not. wv_lacoust .and.
     &	           (index(cr_at_lis(cr_iscat(i)),'-').gt.0)) then
	          dis = -dis
	        endif
c
	        do j=1,3
	          disp(j) = dis*swingp(j)
	        enddo
	        call trans(disp,cr_gmat,disc,3)
	        do j=1,3
	          cr_pos(j,i) = cr_pos(j,i)+disc(j)
	        enddo
	      endif
	    endif
	  endif
	enddo
c
	end
c*****7*********************************************************
	subroutine wave_run_all_mol(wavep,swingp,wave_func)
c-
c	Displaces all molecules by the corresponding
c	wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'random.inc'
	include       'microsta.inc'
	include       'waves.inc'
	include       'symm.inc'
	include       'molecule.inc'
	include       'wink.inc'
c
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
	integer        i,j,ia,ityp,im,il
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  wv_phase = ran1(idum) * 360.0
	endif
c
	do i=1,mole_num_mole
	  ia = mole_cont(mole_off(i)+1)
c
c------ - Check if molecule is a valid selection
c
	  if(wv_latom(mole_type(i))) then
	    if( wv_md_s      .eq.MD_S_EVE .or.
     &	        wv_md_s      .eq.MD_S_NON .and.
     &	        cr_md_next(i).ge.0             ) then
c
	      do j=1,3
	        uc(j)=cr_pos(j,ia)
	      enddo
c
	      call trans(uc,cr_fmat,up,3)
	      arg = up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	      arg = arg + wv_phase*wv_rlam/360.
	      arg = amod(arg,wv_rlam)/wv_rlam
	      dis = wave_func(wv_amp,arg,wv_amp0)
c
c------- ---- Density wave
c
	      if(wv_iwave.eq.WV_DENS) then
	        if(ran1(idum).gt.dis) then
	          ityp = mole_type(i)
	          if (wv_repl(ityp).ne.0) then
	            call do_swap_mole(ityp,wv_repl(ityp),.false.)
	          else
	            mole_type(i) = 0
	            do j=1,mole_len(i)
	              cr_iscat(mole_cont(mole_off(i)+j)) = 0
	              cr_ndel = cr_ndel+1
	            enddo
	          endif
	        endif
c
c------- ---- Rotational wave
c
	      elseif(wv_iwave.eq.WV_ROT) then
	        sym_latom(mole_type(i)) = .true.
	        sym_angle = dis
	        sym_start = i
	        sym_end   = i
	        call symm_setup
	        call symm_mole_single
c
c------- ---- Displacement wave
c
	      else
	        if (.not. wv_lacoust .and.
     &	           (index(cr_at_lis(cr_iscat(ia)),'-').gt.0)) then
	          dis = -dis
	        endif
c
	        do j=1,3
	          disp(j) = dis*swingp(j)
	        enddo
	        call trans(disp,cr_gmat,disc,3)
	        do il=1,mole_len(i)
	          im = mole_cont(mole_off(i)+il)
	          do j=1,3
	            cr_pos(j,im) = cr_pos(j,im)+disc(j)
	          enddo
	        enddo
	      endif
	    endif
	  endif
	enddo
c
	end
c*****7*********************************************************
	subroutine wave_run_md(wavep,swingp,wave_func)
c-
c	Displaces all atoms inside a microdomain 
c	by the corresponding wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microdis.inc'
	include       'microsta.inc'
	include       'random.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
	real           phase_mc(0:MC_MAX_MICRO)
	integer        i,j
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  if(wv_md_s.eq.MD_S_ALL) then
	    do i=1,mc_num
	      phase_mc(i) = ran1(idum) * 360.0
	    enddo
	  elseif(wv_md_s.eq.MD_S_IND) then
	    phase_mc(wv_md_nr) = ran1(idum) * 360.0
	  endif
	endif
	      
c
c------	Select atom if:
c	  type has been selected and
c	       all microdomains have been selected               or
c	       a specific microdomain type has been selected
c
	do i=1,cr_natoms
	  if(wv_latom(cr_iscat(i)) .and. 
     &	     ( wv_md_s.eq.MD_S_ALL .and. cr_md_next(i).lt.0        .or.
     &	       wv_md_s.eq.MD_S_IND .and. cr_md_next(i).lt.0 .and.
     &         mc_type(-cr_md_next(i)).eq.wv_md_nr            ) ) then
c
	    if(wv_phase_typ.eq.WV_RAND) then
	      wv_phase = phase_mc(-cr_md_next(i))
	    endif
c
	    do j=1,3
	      uc(j)=cr_pos(J,I)-mc_orig(j,-cr_md_next(i))
	    enddo
c
	    call trans(uc,cr_fmat,up,3)
	    arg = up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	    arg = arg + wv_phase*wv_rlam/360.
	    arg = amod(arg,wv_rlam)/wv_rlam
	    dis = wave_func(wv_amp,arg,wv_amp0)
c
	    if(wv_iwave.eq.WV_DENS) then
	      if(ran1(idum).gt.dis) then
	        cr_iscat(i) = wv_repl(cr_iscat(i))
	        if(cr_iscat(i).eq.0) then
	          cr_ndel = cr_ndel + 1
	        endif
	      endif
c
	    else
	      if (.not. wv_lacoust .and.
     &	         (index(cr_at_lis(cr_iscat(i)),'-').gt.0)) then
	        dis = -dis
	      endif
c
	      do j=1,3
	        disp(j)=dis*swingp(j)
	      enddo
c
	      call trans(disp,cr_gmat,disc,3)
	      do j=1,3
	        cr_pos(j,i)=cr_pos(j,i)+disc(J)
	      enddo
	    endif
	  endif
	enddo
c
	end
c*****7*********************************************************
	subroutine wave_run_md_mol(wavep,swingp,wave_func)
c-
c	Displaces all inside a microdomain 
c	by the corresponding wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microdis.inc'
	include       'microsta.inc'
	include       'random.inc'
	include       'molecule.inc'
	include       'symm.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
	real           phase_mc(0:MC_MAX_MICRO)
	integer        i,j,ia,ityp,im,il
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  if(wv_md_s.eq.MD_S_ALL) then
	    do i=1,mc_num
	      phase_mc(i) = ran1(idum) * 360.0
	    enddo
	  elseif(wv_md_s.eq.MD_S_IND) then
	    phase_mc(wv_md_nr) = ran1(idum) * 360.0
	  endif
	endif
	      
c
c------	Select atom if:
c	  type has been selected and
c	       all microdomains have been selected               or
c	       a specific microdomain type has been selected
c
	do i=1,mole_num_mole
	  ia = mole_cont(mole_off(i)+1)
c
	  if(wv_latom(mole_type(i)) .and. 
     &	     ( wv_md_s.eq.MD_S_ALL .and. cr_md_next(i).lt.0        .or.
     &	       wv_md_s.eq.MD_S_IND .and. cr_md_next(i).lt.0 .and.
     &         mc_type(-cr_md_next(i)).eq.wv_md_nr            ) ) then
c
	    if(wv_phase_typ.eq.WV_RAND) then
	      wv_phase = phase_mc(-cr_md_next(i))
	    endif
c
	    do j=1,3
	      uc(j)=cr_pos(j,ia)-mc_orig(j,-cr_md_next(ia))
	    enddo
c
	    call trans(uc,cr_fmat,up,3)
	    arg = up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	    arg = arg + wv_phase*wv_rlam/360.
	    arg = amod(arg,wv_rlam)/wv_rlam
	    dis = wave_func(wv_amp,arg,wv_amp0)
c
c------- -- Density wave
c
	    if(wv_iwave.eq.WV_DENS) then
	      if(ran1(idum).gt.dis) then
	        ityp = mole_type(i)
	        call do_swap_mole(ityp,wv_repl(ityp),.false.)
	      endif
c
c------- -- Rotational wave
c
	    elseif(wv_iwave.eq.WV_ROT) then
	      sym_latom(mole_type(i)) = .true.
	      sym_angle = dis
	      sym_start = i
	      sym_end   = i
	      call symm_setup
	      call symm_mole_single
c
c------- -- Displacement wave
c
	    else
	      if (.not. wv_lacoust .and.
     &	         (index(cr_at_lis(cr_iscat(ia)),'-').gt.0)) then
	        dis = -dis
	      endif
c
	      do j=1,3
	        disp(j)=dis*swingp(j)
	      enddo
c
	      call trans(disp,cr_gmat,disc,3)
	      do il=1,mole_len(i)
	        im = mole_cont(mole_off(i)+il)
	        do j=1,3
	          cr_pos(j,im) = cr_pos(j,im)+disc(j)
	        enddo
	      enddo
	    endif
	  endif
	enddo
c
	end
c*****7*********************************************************
	real function sinus (amp,arg,amp0)
c-
c	Sinusoidal wave function
c+
	implicit      none
c
	include      'wink.inc'
c
	real          amp,arg,amp0
c
	arg   = arg*zpi
 	sinus = amp*cos(arg) + amp0
c
	end
c*****7*********************************************************
	real function box (amp,arg,amp0)
c-
c	box shaped wave function
c+
	implicit      none
c
	include      'config.inc'
	include      'wink.inc'
	include      'waves.inc'
c
	real          amp,arg,amp0
c
c------	We have a positive argument 'arg'.
c
	if(arg.ge.0.0 ) then
	  if(arg .lt. (      0.5*wv_asym) .or. 
     &	     arg .ge. (1.0 - 0.5*wv_asym)) then
	    box = amp0 
	  else
	    box = amp0 + amp
	  endif
c
c------	For negative argument the exclusions are reversed 
c------ to ensure continuity at arg=0.0
c
	else
	  if(-arg .le. (      0.5*wv_asym) .or. 
     &	     -arg .gt. (1.0 - 0.5*wv_asym)) then
	    box = amp0
	  else
	    box = amp0 + amp
	  endif
	endif
c
	end
c*****7*********************************************************
	real function triang (amp,arg,amp0)
c-
c	triangular shaped wave function
c+
	implicit      none
c
	include      'wink.inc'
c
	real          amp,arg,amp0
c
	if(arg .ge. 0) then
	  triang = amp0 + amp * arg
	else
	  triang = amp0 + amp * (1. + arg)
	endif
c
	end
c*****7*********************************************************
	subroutine get_iscat(ianz,cpara,lpara,werte,maxw,lnew)
c-
c	Determines the scattering type of the parameter
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'charact.inc'
	include       'errlist.inc'
c
	integer        maxw
c
	character*200  cpara(maxw),zeile
	integer        lpara(maxw)
	integer        i,j,l,ianz,jj,jp
	logical        lnew
	real           werte(maxw)
c
	real           berechne
c
c------	----Select which atoms are included in the wave
c
	ier_num = 0
	ier_typ = ER_NONE
	do i=1,maxw
	  werte(i) = 0.0
	enddo
	jj = 1
	jp = 0
	j  = 1
	do while( j.le.ianz .and. ier_num.eq.0 )
	  i  = ichar(cpara(j)(1:1))
	  if(cpara(j).eq.'all') then
	    werte(1) = -1
	    return
	  elseif(((a .le.i .and. i.le.z ).or. 
     &	          (aa.le.i .and. i.le.zz)   ) .and.
     &	          (index(cpara(j),'[').eq.0 )      )  then
	    call do_cap(cpara(j))
	    ier_num = -27
	    ier_typ = ER_APPL
	    do i=0,cr_nscat
	      if(cpara(j).eq.cr_at_lis(i)) then
	        werte(jj) = i
	        jj        = jj + 1
	        jp        = jp + 1
	        ier_num   = 0
	        ier_typ   = ER_NONE
	      endif
	    enddo
	    if(lnew) then
	      if(j.gt.1) then
	        if(cr_nscat.lt.MAXSCAT) then
	          cr_nscat             = cr_nscat + 1
	          cr_at_lis(cr_nscat)  = cpara(j)
	          cr_dw(cr_nscat)      = 0.0
	          werte(jj)            = cr_nscat
	          ier_num              = 0
	          ier_typ              = ER_NONE
	        else
	          ier_num              = -26
	          ier_typ              = ER_NONE
	        endif
	      endif
	    endif
	  else
	    zeile          = ' '
	    l              = lpara(j)
	    zeile(1  :1)   = '('
	    zeile(2  :l+1) = cpara(j)
	    zeile(l+2:l+2) = ')'
	    l              = l + 2
	    werte(jj)      = berechne(zeile,l)
	    if(ier_num.eq.0) then
	      if(0.le.nint(werte(jj)) .and. 
     &	         nint(werte(jj)).le.cr_nscat) then
	        jj      = jj + 1
	        ier_num = 0
	        ier_typ = ER_NONE
	      else
	        ier_num = -27
	        ier_typ = ER_APPL
	      endif
	    endif
	  endif
	  j = j + 1
	enddo
	ianz = max(ianz,jp)
c
	end
