c*****7*****************************************************************
c
c	Here are all the routines for color handling for the 
c	KUPLOT bitmaps.
c
c	Version : 4.0
c	Date    : 04. March 1999
c
c	Author  : Th. Proffen  (tproffen@lanl.gov )
c*****7*****************************************************************
	subroutine set_color (zeile,lp)
c+
c	Set colours for background & pens
c-
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=5)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,icol
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  call show_color
	elseif (ianz.eq.4) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  if (ier_num.ne.0) return
	  icol=nint(werte(1))
	  if (icol.lt.0 .or. icol.gt.18) then
	    ier_num = -6
	    ier_typ = ER_COMM
	  else
	    if (werte(2).le.1.0 .and. werte(2).ge.0.0  .and.
     &          werte(3).le.1.0 .and. werte(3).ge.0.0  .and.
     &          werte(4).le.1.0 .and. werte(4).ge.0.0       ) then
	      colour(iwin,icol,1) = werte(2)
	      colour(iwin,icol,2) = werte(3)
	      colour(iwin,icol,3) = werte(4)
	    else
	      ier_num = -18
	      ier_typ = ER_APPL
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine set_cmap (zeile,lp)
c
c	Set colour map
c
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw = 5)
c
	character*(*)	zeile
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz,lp
	logical		str_comp
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.ge.1) then
	  if     (str_comp(cpara(1),'gray',3,lpara(1),4)) then
	    call cmap_gray (.true.)
	  elseif (str_comp(cpara(1),'fire',3,lpara(1),4)) then
	    call cmap_fire (.true.)
	  elseif (str_comp(cpara(1),'kupl',3,lpara(1),4)) then
	    call cmap_kupl (.true.)
	  elseif (str_comp(cpara(1),'invert',3,lpara(1),6)) then
	    call cmap_invert (.true.)
	  elseif (str_comp(cpara(1),'read',3,lpara(1),4)) then
	    call do_build_name(ianz,cpara,lpara,werte,maxw,2)	
	    if (ier_num.eq.0) call cmap_read(cpara(2))
	  elseif (str_comp(cpara(1),'write',3,lpara(1),5)) then
	    call do_build_name(ianz,cpara,lpara,werte,maxw,2)	
	    if (ier_num.eq.0) call cmap_write(cpara(2))
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine cmap_invert(lout)
c
c	Setting colormap to gray
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	real		dummy(maxcol,3)
	integer		i
	logical		lout
c
	if (lout) write (output_io,1000)
c
	do i=1,maxcol
	  dummy(i,1) = col_map(iwin,i,1)
	  dummy(i,2) = col_map(iwin,i,2)
	  dummy(i,3) = col_map(iwin,i,3)
	enddo
c
	do i=1,maxcol
	  col_map(iwin,i,1) = dummy(maxcol-i+1,1)
	  col_map(iwin,i,2) = dummy(maxcol-i+1,2)
	  col_map(iwin,i,3) = dummy(maxcol-i+1,3)
	enddo
c
1000	format (' ------ > Inverting current colour map ... ')
c
	end
c*****7*****************************************************************
	subroutine cmap_gray (lout)
c
c	Setting colormap to gray
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	integer		i
	logical		lout
c
	if (lout) write (output_io,1000)
c
	do i=1,maxcol
	  col_map(iwin,i,1)=float(maxcol-i+1)/float(maxcol)
	  col_map(iwin,i,2)=float(maxcol-i+1)/float(maxcol)
	  col_map(iwin,i,3)=float(maxcol-i+1)/float(maxcol)
	enddo
c
1000	format (' ------ > Setting colour map to : gray ')
	end
c*****7*****************************************************************
	subroutine cmap_kupl (lout)
c
c	Setting colormap to original KUPL map
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	real		rh,rf,rq,rp,rt
	integer		i,ifarb
	logical		lout
c
	if (lout) write (output_io,1000)
c
	do ifarb=1,maxcol
	  rh = 0.1 + float (ifarb-1) / 284.0
	  rh = 6.0 * rh
	  i  = int (rh)
	  rf = rh - float(i)
	  rp = 0.0
	  rq = 1.0 - rf
	  rt = (1.0 - (1.0 - rf))
c
	  if (rt.gt.1.0) rt=1.0
	  if (rp.gt.1.0) rp=1.0
	  if (rq.gt.1.0) rq=1.0
c
	  if     (i.eq.3) then
	    col_map(iwin,ifarb,1)=1.
	    col_map(iwin,ifarb,2)=rt
	    col_map(iwin,ifarb,3)=rp
	  elseif (i.eq.4) then
            col_map(iwin,ifarb,1)=rq
            col_map(iwin,ifarb,2)=1.
            col_map(iwin,ifarb,3)=rp
          elseif (i.eq.5) then
            col_map(iwin,ifarb,1)=rp
            col_map(iwin,ifarb,2)=1.
            col_map(iwin,ifarb,3)=rt
          elseif (i.eq.0) then
            col_map(iwin,ifarb,1)=rp
            col_map(iwin,ifarb,2)=rq
            col_map(iwin,ifarb,3)=1.
          elseif (i.eq.1) then
            col_map(iwin,ifarb,1)=rt
            col_map(iwin,ifarb,2)=rp
            col_map(iwin,ifarb,3)=1.
          elseif (i.eq.2) then
            col_map(iwin,ifarb,1)=1.
            col_map(iwin,ifarb,2)=rp
            col_map(iwin,ifarb,3)=rq
	  endif
	enddo
c
1000	format (' ------ > Setting colour map to : kupl ')
	end
c*****7*****************************************************************
	subroutine cmap_fire (lout)
c
c	Setting colormap to fire
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	integer		i,ii,m
	logical		lout
c
	if (lout) write (output_io,1000)
c
	m=maxcol/3
	ii=1
c
	do i=1,m
	  col_map(iwin,ii,1)=float(3*i)/float(maxcol)
	  col_map(iwin,ii,2)=0.0
	  col_map(iwin,ii,3)=0.0
	  ii=ii+1
	enddo
c
	do i=1,m
	  col_map(iwin,ii,1)=1.0
	  col_map(iwin,ii,2)=float(3*i)/float(maxcol)
	  col_map(iwin,ii,3)=0.0
	  ii=ii+1
	enddo
c
	do i=1,m
	  col_map(iwin,ii,1)=1.0
	  col_map(iwin,ii,2)=1.0
	  col_map(iwin,ii,3)=float(3*i)/float(maxcol)
	  ii=ii+1
	enddo
c
1000	format (' ------ > Setting colour map to : fire ')
	end
c*****7*****************************************************************
	subroutine cmap_read (filname)
c
c	Reading colormap from file ..
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*(*)	filname
	integer		i,ir,ig,ib
c
	call oeffne (33,filname,'old',.false.)
	if (ier_num.eq.0) then
	  write (output_io,1000) filname
c
	  do i=1,maxcol
	    read(33,2000,err=100,end=200) ir,ig,ib 
	    col_map(iwin,i,1) = ir/255.
	    col_map(iwin,i,2) = ig/255.
	    col_map(iwin,i,3) = ib/255.
	  enddo
c
	  close(33)
	endif
	return
c
100	ier_num = -3
	ier_typ = ER_IO
	close(33)
	return
c
200	ier_num = -6
	ier_typ = ER_IO
	close(33)
	return
c
1000	format (' ------ > Reading colour map from file : ',A30)
2000	format (1x,3z2)
	end
c*****7*****************************************************************
	subroutine cmap_write (filname)
c
c	Writing colormap to file ..
c
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*(*)	filname
	integer		i,ir,ig,ib
c
	call oeffne (33,filname,'unknown',.false.)
	if (ier_num.eq.0) then
	  write (output_io,1000) filname
c
	  do i=1,maxcol
	    ir = int(col_map(iwin,i,1)*255.)
	    ig = int(col_map(iwin,i,2)*255.)
	    ib = int(col_map(iwin,i,3)*255.)
	    write(33,2000) ir,ig,ib 
	  enddo
c
	  close(33)
	endif
c
1000	format (' ------ > Writing colour map to file : ',A30)
2000	format ('#',3z2.2)
	end
