c*****7****************************************************************
c
c	Routines to allocate memory for data sets and load data sets
c	using different file formats.
c
c	Version : 4.1
c	Date    : 15. April 2000
c
c	Author  : Th. Proffen (tproffen@lanl.gov )
c
c*****7****************************************************************
	subroutine do_func (zeile,lp)
c
c	Create new data set from given function
c
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
	include		'param.inc'
c
	integer		maxw
	parameter	(maxw=7)
c
	character*(*)	zeile
	character*200	cpara(maxw),cfkt,cdummy
	integer		lp,lcfkt,lpara(maxw)
	integer		ianz,i,ii,jj
	integer		maxpkt,maxzz
	real		xstart,xend,xdelta
	real		ystart,yend,ydelta
	real		werte(maxw)
c
	real		berechne
c
c------ space left for additional data set ?
c
	if (iz.gt.maxkurvtot) then
	  ier_num = -1
	  ier_typ = ER_APPL
	  return
	endif
c
	maxpkt = maxarray - offxy(iz-1)
	maxzz  = maxarray - offz (iz-1)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ianz.lt.1) then
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	if (ier_num.ne.0) return
c
	cfkt  = cpara(1)(1:lpara(1))
	lcfkt = lpara(1)
	call del_params(1,ianz,cpara,lpara,maxw)
c
c------ 1D data set
c
	if (ianz.eq.3) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  xstart = werte(1)
	  xend   = werte(2)
	  xdelta = werte(3)
	  ii = nint((xend-xstart)/xdelta)+1
	  if(ii.gt.0 .and. ii.le.maxpkt) then
	    do i=1,ii
	       rpara(0) = xstart + (i-1)*xdelta
	       cdummy   = '('//cfkt(1:lcfkt)//')'
	       x(offxy(iz-1)+i) = rpara(0)
	       y(offxy(iz-1)+i) = berechne(cdummy,lcfkt+2)
	      dx(offxy(iz-1)+i) = 0.0
	      dy(offxy(iz-1)+i) = 0.0
	    enddo
	    fname(iz) = 'func.xy'
	    fform(iz) = 'XY'
	    lni(iz)   = .false.
	    len(iz)   = ii
	    offxy(iz) = offxy(iz-1) + len(iz)
	    offz(iz)  = offz (iz-1)
	    iz = iz+1
	    call show_data(iz-1)
	  else
	    ier_num = -6
	    ier_typ = ER_APPL
	  endif
c
c------ 2D data set
c
	elseif (ianz.eq.6) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  xstart = werte(1)
	  xend   = werte(2)
	  xdelta = werte(3)
	  ystart = werte(4)
	  yend   = werte(5)
	  ydelta = werte(6)
	  nx(iz) = nint((xend-xstart)/xdelta)+1
	  ny(iz) = nint((yend-ystart)/ydelta)+1
	  if(nx(iz).gt.0 .and. ny(iz).gt.0 .and. 
     &	     nx(iz)*ny(iz)     .le.maxzz   .and.
     &	     max(nx(iz),ny(iz)).le.maxpkt        ) then
	    do ii=1,nx(iz)
	      rpara(0) = xstart + (ii-1)*xdelta
	      do jj=1,ny(iz)
	        cdummy   = '('//cfkt(1:lcfkt)//')'
	        rpara(1) = ystart + (jj-1)*ydelta
	         z(offz (iz-1)+(ii-1)*ny(iz)+jj)=berechne(cdummy,lcfkt+2)
	         x(offxy(iz-1)+ii) = rpara(0)
	         y(offxy(iz-1)+jj) = rpara(1)
	        dx(offxy(iz-1)+ii) = 0.0
	        dy(offxy(iz-1)+jj) = 0.0
	      enddo
	    enddo
	    fname(iz) = 'func.nipl'
	    fform(iz) = 'NI'
	    lni(iz)   = .true.
	    len(iz)   = max(nx(iz),ny(iz))
	    offxy(iz) = offxy(iz-1) + len(iz)
	    offz(iz)  = offz (iz-1) + nx(iz)*ny(iz)
	    iz        = iz+1
	    call show_data(iz-1)
	  else
	    ier_num = -6
	    ier_typ = ER_APPL
	  endif
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM  
	endif
c
	end
c*****7****************************************************************
	subroutine do_allocate (zeile,lp)
c
c	Allocate space for new data set
c
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=3)
c
	character*(*)	zeile
	character*200	cpara(maxw)
	character*40	cdummy
	integer		lp,lpara(maxw)
	integer		ianz,i,ii,jj
	integer		maxpkt,maxzz
	real		werte(maxw)
c
	call no_error
c
c------ space left for additional data set ?
c
	if (iz.gt.maxkurvtot) then
	  ier_num = -1
	  ier_typ = ER_APPL
	  return
	endif
c
	maxpkt = maxarray - offxy(iz-1)
	maxzz  = maxarray - offz (iz-1)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.ne.2 .and. ianz.ne.3) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif

	cdummy=cpara(1)
	call del_params(1,ianz,cpara,lpara,maxw)
c
c------ 1D data set
c
	if (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  ii=nint(werte(1))
	  if(ii.gt.0 .and. ii.le.maxpkt) then
	    do i=1,ii
	       x(offxy(iz-1)+i) = 0.0
	       y(offxy(iz-1)+i) = 0.0
	      dx(offxy(iz-1)+i) = 0.0
	      dy(offxy(iz-1)+i) = 0.0
	    enddo
	    fname(iz) = cdummy
	    fform(iz) = 'XY'
	    lni(iz)   = .false.
	    len(iz)   = ii
	    offxy(iz) = offxy(iz-1) + len(iz)
	    offz(iz)  = offz (iz-1)
	    iz = iz+1
	    call show_data(iz-1)
	  else
	    ier_num = -6
	    ier_typ = ER_APPL
	  endif
c
c------ 2D data set
c
	elseif (ianz.eq.2) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  nx(iz)=nint(werte(1))
	  ny(iz)=nint(werte(2))
	  if(nx(iz).gt.0 .and. ny(iz).gt.0 .and. 
     &	     nx(iz)*ny(iz)     .le.maxzz   .and.
     &	     max(nx(iz),ny(iz)).le.maxpkt        ) then
	    do ii=1,nx(iz)
	      do jj=1,ny(iz)
	         z(offz (iz-1)+(ii-1)*ny(iz)+jj)=0.0
	         x(offxy(iz-1)+ii) = 0.0
	         y(offxy(iz-1)+jj) = 0.0
	        dx(offxy(iz-1)+ii) = 0.0
	        dy(offxy(iz-1)+jj) = 0.0
	      enddo
	    enddo
	    fname(iz) = cdummy
	    fform(iz) = 'NI'
	    lni(iz)   = .true.
	    len(iz)   = max(nx(iz),ny(iz))
	    offxy(iz) = offxy(iz-1) + len(iz)
	    offz(iz)  = offz (iz-1) + nx(iz)*ny(iz)
	    iz        = iz+1
	    call show_data(iz-1)
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM  
	endif
c
	end
c*****7****************************************************************
	subroutine do_load (string,laenge)
c
c	Load various file formats
c
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw,ifil,iwgb
	parameter	(maxw=10)
	parameter	(ifil=44)
	parameter	(iwgb=45)
c
	character*(*)	string
	character*200	cpara(maxw)
	character*80	wname
	character*2	unter
	real		werte(maxw)
	integer		laenge,lpara(maxw)
	integer		ianz,istr
	logical         zz_mod
c
	logical         str_comp
c
	istr = 1
	call no_error
c
c------ space left for additional data set ?
c
	if (iz.gt.maxkurvtot) then
	  ier_num = -1
	  ier_typ = ER_APPL
	  return
	endif
c
	call get_params(string,ianz,cpara,lpara,maxw,laenge)
	if (ier_num.ne.0) return
	if (ianz.ge.2) then
c
c------- -get file typ
c
	  unter = cpara(1)(1:2)
	  call do_cap(unter)
c
c------- -build first filename
c
	  call do_build_name(ianz,cpara,lpara,werte,maxw,2)
	  if (ier_num.ne.0) return
	  fname(iz) = cpara(2)
c
c------- -get second filename if present
c
	  if (ianz.eq.3 .and. unter.ne.'SP') then
	    call do_build_name(ianz,cpara,lpara,werte,maxw,3)
	    if (ier_num.ne.0) return
	    istr = 2
	    wname = cpara(3)
	  else
	    istr = 1
	    wname = ' '
	  endif
c
c------- -for format 'ZZ' or 'DE' get grid size 
c
	  if ((unter.eq.'ZZ' .or. unter.eq.'DE' .or.
     &	       unter.eq.'MP'                          )) then
	    if (ianz.eq.4 .or. ianz.eq.5 .or. ianz.eq.8) then
	      zz_mod = .false.
	      if (ianz.eq.5) then
	        if(str_comp(cpara(ianz),'mod',3,lpara(ianz),3)) then
	          zz_mod = .true.
	          ianz = 4
	        endif
	      endif
	      call del_params (2,ianz,cpara,lpara,maxw)
	      call ber_params (ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	      return
	    endif  
c
c------ - Other additional parameters
c
	  elseif (unter.eq.'MC') then
	    call del_params (2,ianz,cpara,lpara,maxw)
	    call ber_params (ianz,cpara,lpara,werte,maxw)
	    if (ier_num.ne.0) return
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ open the datafile(s)
c
	if(unter.ne.'MP') then
	  call oeffne(ifil,fname(iz),'old',.true.)
	else
	  open(ifil,file=fname(iz),status='unknown',access='direct',
     &	       form='unformatted',recl=16)
	endif
	if (ier_num.ne.0) return
	if (istr.eq.2 .and. unter.ne.'ZZ' .and. unter.ne.'DE' .and.
     &	                    unter.ne.'MP'                         ) then
	  call oeffne(iwgb,wname,'old',.true.)
	  if (ier_num.ne.0) then
	    istr=1
	    ier_num = 0
	    ier_typ = ER_NONE
	  endif
	endif
c
c------ Has the file a prepended History ?
c
	if(unter.ne.'MP' ) then
	  call extract_hist(ifil)
	endif
c
c------ Check file format
c	
	fform(iz)=unter
	if    (unter.eq. 'XY' .or. unter.eq.'YX') then
	  call read_xy (ifil,unter.eq.'XY')
	elseif(unter.eq. 'XX') then
	  call read_xx (ifil)
	elseif(unter.eq. 'CR') then
	  call read_crystal (ifil)
	elseif(unter.eq. 'DE') then
	  call read_xyz(ifil,ianz,werte,maxw,.true.,zz_mod)
	elseif(unter.eq. 'ZZ') then
	  call read_xyz(ifil,ianz,werte,maxw,.false.,zz_mod)
	elseif(unter.eq. 'MC') then
	  call read_mca_single(ifil,ianz,werte,maxw)
	elseif(unter.eq. 'MP') then
	  call read_ext(ifil,ianz,werte,maxw,.false.,zz_mod)
	elseif(unter.eq. 'Z1') then
	  call read_z(ifil,1)
	elseif(unter.eq. 'Z2') then
	  call read_z(ifil,2)
	elseif(unter.eq. 'MA') then
	  call read_marker (ifil)
	elseif(unter.eq. 'NI') then
	  call read_nipl (ifil,iwgb,istr,.false.)
	elseif(unter.eq. 'PG') then
	  call read_nipl (ifil,iwgb,istr,.true.)
	elseif(unter.eq. 'T1') then
	  call read_mess (ifil,4,8, '2-theta [deg.]','counts z1')
	elseif(unter.eq. 'T2') then
	  call read_mess (ifil,4,9, '2-theta [deg.]','counts z2')
	elseif(unter.eq. 'C1') then
	  call read_mess (ifil,5,8, 'chi [deg.]'    ,'counts z1')
	elseif(unter.eq. 'C2') then
	  call read_mess (ifil,5,9, 'chi [deg.]'    ,'counts z2')
	elseif(unter.eq. 'P1') then
	  call read_mess (ifil,6,8, 'phi [deg.]'    ,'counts z1')
	elseif(unter.eq. 'P2') then
	  call read_mess (ifil,6,9, 'phi [deg.]'    ,'counts z2')
	elseif(unter.eq. 'O1') then
	  call read_mess (ifil,7,8, 'omega [deg.]'  ,'counts z1')
	elseif(unter.eq. 'O2') then
	  call read_mess (ifil,7,9, 'omega [deg.]'  ,'counts z2')
	elseif(unter.eq. 'H1') then
	  call read_mess (ifil,1,8, 'h [r.l.u.]'    ,'counts z1')
	elseif(unter.eq. 'H2') then
	  call read_mess (ifil,1,9, 'h [r.l.u.]'    ,'counts z2')
	elseif(unter.eq. 'K1') then
	  call read_mess (ifil,2,8, 'k [r.l.u.]'    ,'counts z1')
	elseif(unter.eq. 'K2') then
	  call read_mess (ifil,2,9, 'k [r.l.u.]'    ,'counts z2')
	elseif(unter.eq. 'L1') then
	  call read_mess (ifil,3,8, 'l [r.l.u.]'    ,'counts z1')
	elseif(unter.eq. 'L2') then
	  call read_mess (ifil,3,9, 'l [r.l.u.]'    ,'counts z2')
	elseif(unter.eq. 'TE') then
	  call read_mess (ifil,0,11,'data point'    ,'temperature [k]')
	elseif(unter.eq. 'DM') then
	  call read_mess (ifil,0,12,'data point'    ,'value DMM')
	elseif(unter.eq. 'ZE') then
	  call read_mess (ifil,0,10,'data point'    ,'time [sec]')
	elseif(unter.eq. 'SC' .or. unter.eq. 'ST') then
	  call read_spec (ifil,ianz,cpara,lpara,werte,maxw,
     &	                  unter.eq.'SC')
	elseif(unter.eq. 'GS') then
	  call read_gsas (ifil,ianz,cpara,lpara,werte,maxw)
	else
	  ier_num = -2
	  ier_typ = ER_APPL
	endif
c
	close (ifil)
	if (istr.eq.2) close(iwgb)
	return
c
9000	continue
	close (ifil)
	if (istr.eq.2) close(iwgb)
c
	ier_num = -40
	ier_typ = ER_APPL
c
	end  
c*****7****************************************************************
	subroutine extract_hist(ifil)
c
c	Extract history information if present
c
	implicit	none
c
	include		'config.inc'
	include		'param.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	integer		ifil
c
	character*200	line
	integer		is,ie
c
	integer		len_str
c
	read(ifil,5000) line
	if (line(1:7).eq.'History') then
	  res_para(0)=0
	  write (output_io,1000)
22	  continue
	    read(ifil,5000) line
c
	    is = index(line,'runTitle=')
	    if (is.ne.0) then
	      titel(iwin,iframe,1) = line(is+9:len_str(line))
	    endif
c
	    call extract_key(line,'temp=')
	    call extract_key(line,'Qmax=')
	    if (line(1:16) .eq. '##### start data') goto 33
	  goto 22
	else
	  backspace(ifil)
	endif
c
33	continue
c
1000	format( ' History information found ...')
5000	format(a)
c
	end
c*****7*****************************************************************
	subroutine extract_key (line,key)
c+
c	Gets numbers from history part of data file
c-
	implicit      	none
c
	include		'config.inc'
	include		'param.inc'
c
	character*(*)	line,key
	integer		is,ie,ll,lk
c
	integer		len_str
c
	ll = len_str(line)
	lk = len_str(key)
c
	is = index(line,key(1:lk))
	if (is.ne.0) then
	  is = is+lk
	  do while (line(is:is).eq.' ')
	    is = is+1
	  enddo
	  ie = index(line(is:ll),' ')
	  if (ie.eq.0) ie = ll
	  res_para(0) = res_para(0)+1
	  read(line(is:is+ie),*) res_para(res_para(0))
	endif
c
	end
c*****7****************************************************************
	subroutine read_ext(ifil,ianz,werte,maxw,dens,zz_mod)
c+
c	Load MPAUS extract files ..
c-
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
c
	real		werte(maxw)
CDBG	real		xw,yw,zw
	real		xw,yw,   ow
	integer*4	zw
	real		deltax,deltay,dxx,dyy
	integer         irec
	integer		np(maxarray)
	integer		ifil,ixx,iyy,izeig,i
	integer		maxpkt,maxzz,ianz
	logical		dens,zz_mod
c
c------ get parameters
c            
	fform(iz)='NI'
c
	maxpkt = maxarray - offxy(iz-1)
	maxzz  = maxarray - offz (iz-1)
	deltax = werte(1)
	deltay = werte(2)
c
c------	get number of records
c
	read (ifil,rec=1) irec
c
c------ get dimensions of file
c
	if (ianz.eq.6) then
	  xmin(iz)=werte(3)
	  xmax(iz)=werte(4)
	  ymin(iz)=werte(5)
	  ymax(iz)=werte(6)
	else
	  read (ifil,rec=2) xw,yw,zw,ow
	  xmin(iz)=xw
	  xmax(iz)=xw
	  ymin(iz)=yw
	  ymax(iz)=yw
	  do i=3,irec
	    read (ifil,rec=i) xw,yw,zw,ow
	    xmin(iz)=min(xmin(iz),xw)
	    xmax(iz)=max(xmax(iz),xw)
	    ymin(iz)=min(ymin(iz),yw)
	    ymax(iz)=max(ymax(iz),yw)
	  enddo
	endif
c
c------	optionally round xmin... to integer multiples of delta
c
	if(zz_mod) then
	  xmin(iz) = float(nint(xmin(iz)/deltax))*deltax
	  xmax(iz) = float(nint(xmax(iz)/deltax))*deltax
	  ymin(iz) = float(nint(ymin(iz)/deltay))*deltay
	  ymax(iz) = float(nint(ymax(iz)/deltay))*deltay
	endif
c
c------ check dimensions
c
	if (xmin(iz).eq.xmax(iz) .or. ymin(iz).eq.ymax(iz)) then
	  ier_num = -29
	  ier_typ = ER_APPL
	  return
	endif
c
c------ get grid size 
c
	nx(iz)=nint((xmax(iz)-xmin(iz))/deltax)+1
	ny(iz)=nint((ymax(iz)-ymin(iz))/deltay)+1
c
c-------check array size
c
	if ((nx(iz)*ny(iz).gt.maxzz)       .or.
     &      (max(nx(iz),ny(iz)).gt.maxpkt)     ) then
	  ier_num = -6
	  ier_typ = ER_APPL
	  return
	endif
c
c------ zero some arrays
c
	do ixx=1,nx(iz)
	  do iyy=1,ny(iz)
	    izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	    z (izeig)=0.0
	    np(izeig)=0.0
	  enddo
	enddo
c
c------ read file and bin on specified grid
c
	dxx=deltax
	dyy=deltay
c
	do i=2,irec
	  read (ifil,rec=i) xw,yw,zw
	  if (zw.ne.-9999.0) then
	    ixx=nint((xw-xmin(iz))/dxx) + 1
	    iyy=nint((yw-ymin(iz))/dyy) + 1
	    if (ixx.gt.0 .and. ixx.le.nx(iz) .and.
     &	        iyy.gt.0 .and. iyy.le.ny(iz)      ) then
	      izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	      z (izeig)=z (izeig)+float(zw)
	      np(izeig)=np(izeig)+1
	    endif
	  endif
	enddo
40	continue
c
c------ normalise Z
c
	do ixx=1,nx(iz)
	  do iyy=1,ny(iz)
	    izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	    if (.not. dens) then
	      if (np(izeig).eq.0) then
	        z(izeig)=-9999.0
	      else
	        z(izeig)=z(izeig)/float(np(izeig))
	      endif
	    endif
	    zmax(iz)=max(z(izeig),zmax(iz))
	    zmin(iz)=min(z(izeig),zmin(iz))
	  enddo
	enddo
c
c------ fill X,Y arrays
c
	do i=1,nx(iz)
	  x(offxy(iz-1)+i)=xmin(iz)+(i-1)*dxx
	enddo
	do i=1,ny(iz)
	  y(offxy(iz-1)+i)=ymin(iz)+(i-1)*dyy
	enddo
c
c------ set other parameters
c
	lni(iz)   = .true.
	len(iz)   = max(nx(iz),ny(iz))
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1) + nx(iz)*ny(iz)
	iz=iz+1
	call show_data (iz-1)
c
	end
c*****7****************************************************************
	subroutine read_xyz(ifil,ianz,werte,maxw,dens,zz_mod)
c+
c	Load xyz files ..
c-
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
c
	real		werte(maxw)
	real		xw,yw,zw
	real		deltax,deltay,dxx,dyy
	integer		np(maxarray)
	integer		ifil,ixx,iyy,izeig,i
	integer		maxpkt,maxzz,ianz
	logical		dens,zz_mod
c
c------ get parameters
c            
	fform(iz)='NI'
c
	maxpkt = maxarray - offxy(iz-1)
	maxzz  = maxarray - offz (iz-1)
	deltax = werte(1)
	deltay = werte(2)
c
c------ get dimensions of file
c
	if (ianz.eq.6) then
	  xmin(iz)=werte(3)
	  xmax(iz)=werte(4)
	  ymin(iz)=werte(5)
	  ymax(iz)=werte(6)
	else
	  read (ifil,*,end=20) xw,yw,zw
	  xmin(iz)=xw
	  xmax(iz)=xw
	  ymin(iz)=yw
	  ymax(iz)=yw
10	  continue
	    read (ifil,*,end=20) xw,yw,zw
	    xmin(iz)=min(xmin(iz),xw)
	    xmax(iz)=max(xmax(iz),xw)
	    ymin(iz)=min(ymin(iz),yw)
	    ymax(iz)=max(ymax(iz),yw)
	  goto 10
20	  continue
	  rewind (ifil)
	endif
c
c------	optionally round xmin... to integer multiples of delta
c
	if(zz_mod) then
	  xmin(iz) = float(nint(xmin(iz)/deltax))*deltax
	  xmax(iz) = float(nint(xmax(iz)/deltax))*deltax
	  ymin(iz) = float(nint(ymin(iz)/deltay))*deltay
	  ymax(iz) = float(nint(ymax(iz)/deltay))*deltay
	endif
c
c------ check dimensions
c
	if (xmin(iz).eq.xmax(iz) .or. ymin(iz).eq.ymax(iz)) then
	  ier_num = -29
	  ier_typ = ER_APPL
	  return
	endif
c
c------ get grid size 
c
	nx(iz)=nint((xmax(iz)-xmin(iz))/deltax)+1
	ny(iz)=nint((ymax(iz)-ymin(iz))/deltay)+1
c
c-------check array size
c
	if ((nx(iz)*ny(iz).gt.maxzz)       .or.
     &      (max(nx(iz),ny(iz)).gt.maxpkt)     ) then
	  ier_num = -6
	  ier_typ = ER_APPL
	  return
	endif
c
c------ zero some arrays
c
	do ixx=1,nx(iz)
	  do iyy=1,ny(iz)
	    izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	    z (izeig)=0.0
	    np(izeig)=0.0
	  enddo
	enddo
c
c------ read file and bin on specified grid
c
	dxx=deltax
	dyy=deltay
30	continue
	  if (dens) then 
	    read (ifil,*,end=40) xw,yw
	    zw=1.0
	  else
	    read (ifil,*,end=40) xw,yw,zw
	  endif
	  if (zw.ne.-9999.0) then
	    ixx=nint((xw-xmin(iz))/dxx) + 1
	    iyy=nint((yw-ymin(iz))/dyy) + 1
	    if (ixx.gt.0 .and. ixx.le.nx(iz) .and.
     &	        iyy.gt.0 .and. iyy.le.ny(iz)      ) then
	      izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	      z (izeig)=z (izeig)+zw
	      np(izeig)=np(izeig)+1
	    endif
	  endif
	goto 30
40	continue
c
c------ normalise Z
c
	do ixx=1,nx(iz)
	  do iyy=1,ny(iz)
	    izeig=offz(iz-1)+(ixx-1)*ny(iz)+iyy
	    if (.not. dens) then
	      if (np(izeig).eq.0) then
	        z(izeig)=-9999.0
	      else
	        z(izeig)=z(izeig)/float(np(izeig))
	      endif
	    endif
	    zmax(iz)=max(z(izeig),zmax(iz))
	    zmin(iz)=min(z(izeig),zmin(iz))
	  enddo
	enddo
c
c------ fill X,Y arrays
c
	do i=1,nx(iz)
	  x(offxy(iz-1)+i)=xmin(iz)+(i-1)*dxx
	enddo
	do i=1,ny(iz)
	  y(offxy(iz-1)+i)=ymin(iz)+(i-1)*dyy
	enddo
c
c------ set other parameters
c
	lni(iz)   = .true.
	len(iz)   = max(nx(iz),ny(iz))
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1) + nx(iz)*ny(iz)
	iz=iz+1
	call show_data (iz-1)
c
	end
c*****7****************************************************************
	subroutine read_mess(ifil,ix,iy,str1,str2)
c+
c	Load MAN2 data files (MAN2 is a control program for the
c	neutron diffractometers at FRM II. For more information
c	contact the authors of KUPLOT.
c-                                
	implicit	none
c
	integer		maxw
	parameter	(maxw=20)
c
	character*(*)	str1,str2
	character*200	line
	real		werte(maxw)
	integer		ix,iy,ifil,nr,ianz,i,maxpp
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
c------ initial setup
c
	achse(iwin,iframe,1)=str1
	achse(iwin,iframe,2)=str2
c
c------ read data
c          
	nr=1
	maxpp=maxarray-offxy(iz-1)
	read(ifil,9999,end=20) line
	read(ifil,9999,end=20) line
10	continue
	  read(ifil,9999,end=20) line
	  call count_col (line,ianz)
	  if (ianz.eq.0) goto 10
	  if (ianz.gt.4) ianz=4
	  backspace(ifil)
15 	  continue
	  read (ifil,*,end=20) (werte(i),i=1,ianz)
	  if (ix.gt.0) then
	    x(offxy(iz-1)+nr)=werte(ix)
	  else
	    x(offxy(iz-1)+nr)=float(nr)
	  endif
	  y (offxy(iz-1)+nr)=werte(iy)
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 15
20	continue
	len(iz)=nr-1
	offxy(iz)=offxy(iz-1) + len(iz)
	offz(iz) =offz (iz-1)
	iz=iz+1
c
	call show_data(iz-1)
c                       
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine read_nipl (ifil,iwgb,istr,pgm)
c+
c	Read NIPL (KUPLOT own 2D format) and PGM (ASCII) files
c-
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		mrect,maxw
	parameter	(maxw =20)
	parameter	(mrect=50)
c
	character*80	line
	character*5	cmagic,kom
	real		rect(mrect,4)
	real		dxx,dyy
	integer		ifil,iwgb,istr,ir,i,j,zdummy
	integer		maxpkt,maxzz
	integer		nrect
	logical		pgm,inwg
c
c------ if there is a second file containing excluded regions, 
c------ read its contents
c
	if (istr.eq.2 .and. .not.pgm) then
	  read (iwgb,*,end=50,err=55) nrect
	  if (nrect.gt.mrect) then
	    ier_num = -9
	    ier_typ = ER_APPL
	    goto 999
	  endif
	  do i=1,nrect
	    read (iwgb,*,end=50,err=55) (rect(i,j),j=1,4)
	  enddo 
	endif
c
	maxpkt = maxarray - offxy(iz-1)
	maxzz  = maxarray - offz (iz-1)
c
c------ read main data
c------ NIPL file format
c
	if (.not.pgm) then
	  read (ifil,*,end=50,err=55) nx(iz),ny(iz)
c
c------ - check array size
c
	  if (max(nx(iz),ny(iz)).gt.maxzz .or.
     &        max(nx(iz),ny(iz)).gt.maxpkt    ) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    goto 999
	  endif
	  read (ifil,*,end=50,err=55) xmin(iz),xmax(iz),ymin(iz),ymax(iz) 
c
c------ - check dimensions
c
	  if (xmin(iz).eq.xmax(iz) .or. ymin(iz).eq.ymax(iz)) then
	    ier_num = -29
	    ier_typ = ER_APPL
	    return
	  endif
c
	  do j=1,ny(iz)
	    read (ifil,*,end=50,err=55) 
     &	         (z(offz(iz-1)+(i-1)*ny(iz)+j),i=1,nx(iz))
	  enddo
c
c------ PGM file format
c
	else
	  read (ifil,1000,end=50,err=55) cmagic
	  if (cmagic(1:2).ne.'P2') then
	    ier_num = -5
	    ier_typ = ER_APPL
	    goto 999
	  endif
c
5	  read (ifil,1000,end=50,err=55) kom
	  if (kom(1:1).eq.'#') then
	    backspace (ifil)
	    read (ifil,1000,end=50,err=55) line
	    goto 5
	  else
	    backspace (ifil)
	  endif 
	  read (ifil,*,end=50,err=55) nx(iz),ny(iz)
	  read (ifil,*,end=50,err=55) zdummy
c
c------ - check array size
c
	  if (max(nx(iz),ny(iz)).gt.maxzz .or.
     &        max(nx(iz),ny(iz)).gt.maxpkt    ) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    goto 999
	  endif
	  read (ifil,*,end=50,err=55) 
     &	       ((z(offz(iz-1)+(i-1)*ny(iz)+j),i=1,nx(iz)),j=ny(iz),1,-1)
	  xmin(iz)=0.0
	  xmax(iz)=float(nx(iz)-1)
	  ymin(iz)=0.0
	  ymax(iz)=float(ny(iz)-1)
	endif
c
c------ set values for X and Y
c
	dxx = (xmax(iz)-xmin(iz))/float(nx(iz)-1)
	dyy = (ymax(iz)-ymin(iz))/float(ny(iz)-1)
	do i=1,nx(iz)
	  x(offxy(iz-1)+i)=xmin(iz)+(i-1)*dxx
	enddo
	do i=1,ny(iz)
	  y(offxy(iz-1)+i)=ymin(iz)+(i-1)*dyy
	enddo
c
c------ check with excluded regions
c
	do i=1,nx(iz)
	  do j=1,ny(iz)
	    if (istr.eq.2 .and. .not.pgm) then
	      inwg = .false.
	      do ir=1,nrect
	        inwg = inwg .or. 
     &                 (x(offxy(iz-1)+i).ge.rect(ir,1) .and.
     &	                x(offxy(iz-1)+i).le.rect(ir,2) .and.
     &	                y(offxy(iz-1)+j).ge.rect(ir,3) .and.
     &                  y(offxy(iz-1)+j).le.rect(ir,4))
	      enddo
	      if (inwg) z(offz(iz-1)+(i-1)*ny(iz)+j) = -9999.0
	    endif
	  enddo
	enddo
c
c------ set remaining parameters
c
	lni(iz)  = .true.
	len(iz)  = max(nx(iz),ny(iz))
	offxy(iz)= offxy(iz-1) + len(iz)
	offz(iz) = offz (iz-1) + nx(iz)*ny(iz)
	iz=iz+1
c
	call show_data (iz-1)
	goto 999
c
50	continue
	ier_num = -6
	ier_typ = ER_IO
	goto 999
c
55	continue
	ier_num = -3
	ier_typ = ER_IO
	goto 999
c
999	continue
1000	format (a)
	end
c*****7****************************************************************
	subroutine read_spec (ifil,ianz,cpara,lpara,werte,maxw,lcount)
c+
c	Load SPEC scan files
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw,colm
c
	parameter	(colm = 25)
c
	character*200	cpara(maxw)
	character*200	line,mine,date,tit
	character*20	field(0:colm),input(colm)
	real		werte(maxw)
	real		col(0:colm),dummy
	integer		lpara(maxw),icell(5)
	integer		ifil,ianz,iianz,il,ipt
	integer		icol,iscan,istart,iend,i,j
	integer		nscan,nr,nf,maxpp
	logical		not_found,data_read,lcount,lsigma,lend,lall
c
	integer		len_str
	logical		str_comp
c
	lend = .false.
c
c------ No further parameters -> list scans in file
c
	if (ianz.eq.2) then
	  write(output_io,1000) fname(iz)(1:len_str(fname(iz)))
1	  continue
	    read(ifil,9999,end=2) line
	    if (line(1:2).eq.'#S') then
	      ipt = 0
11	      continue
	        read(ifil,9999,end=22) mine
	      if (mine(1:2).ne.'#L') goto 11
12	      continue
	        read(ifil,9999,end=22) mine
	        if (mine(1:2).eq.'#C') goto 12
	        read(mine,*,end=22,err=22) dummy
	        ipt = ipt+1
	      goto 12
22	      continue
	      if (ipt.gt.0) then
	        write(output_io,1100) ipt,line(3:min(70,len_str(line)))
	      endif
	      backspace(ifil)
	    endif
	  goto 1
2	  return
c
c------ Scan information
c
	elseif (ianz.eq.4) then
	  if(str_comp(cpara(4),'info',1,lpara(4),4)) then
	    call read_spec_info (ifil,ianz,cpara,lpara,maxw)
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	    return
	  endif
c
c------ Only scan number -> list scans in file
c
	elseif (ianz.eq.3 .or. (ianz.ge.5 .and. ianz.le.8)) then
	  data_read = ianz.gt.3
	  iianz     = 1
c
c------ - Search for scan number first
c
	  field(0) = 'ignored'
	  date     = 'none'
c
	  call get_scan_numbers (cpara(3),lpara(3),istart,iend,lall)
	  if(ier_num.ne.0) return
c
	  iscan = istart
c
	  call del_params (3,ianz,cpara,lpara,maxw)
	  do i=1,ianz
	    input(i) = cpara(i)(1:lpara(i))
	  enddo
c
1111	  continue
c
	  not_found = .true.
3	  continue
	    read(ifil,9999,end=4) line
	    if (line(1:2).eq.'#S') then
	      call get_col(line,field,nf,colm)
	      read (field(1),*) nscan
	      if (iscan.eq.nscan .or. lall) then
	        tit=line
33	        continue
	          read(ifil,9999,end=4) line
	          if (line(1:2).eq.'#D') then
	            date = line(3:len_str(line))
	          endif
	          if (line(1:2).eq.'#L') then
	            call get_col(line,field,nf,colm)
	            not_found = .false.
	            goto 4
	          endif
	        goto 33
44	        continue
	      endif
	    endif
	  goto 3
4	  continue
	  if (not_found) then
	    ier_num = -37
	    ier_typ = ER_APPL
	    return
	  endif
c
c------ - display info
c
	  if (date(1:4).eq.'none') then
	    write(output_io,2000) iscan,fname(iz)(1:len_str(fname(iz)))
	  else
	    write(output_io,2010) iscan,fname(iz)(1:len_str(fname(iz))),
     &	                          date(1:len_str(date))
	  endif
c
	  if (.not. data_read) then
	    write (output_io,*)
	    do i=1,nf
	      write(output_io,2020) i,field(i)(1:len_str(field(i)))
	    enddo
	    write (output_io,*)
	    return
	  endif
c
	  do j=1,5
	    icell(j) = 0
	  enddo
c
	  do i=1,nf
	    il   = len_str(field(i))
	    do j=1,ianz
	      if (field(i)(1:il).eq.input(j)(1:il)) then
	        icell(j) = i
	        cpara(j) = '0'
	        lpara(j) = 1
	      endif
	    enddo
	  enddo
c
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  not_found = .false.
	  do j=1,ianz
	    if (icell(j).eq.0) icell(j) = nint(werte(j))
	    if (icell(j).lt.0) icell(j) = nf + icell(j) + 1
	    not_found = not_found .or.
     &	               (icell(j).lt.0 .or. icell(j).gt.nf)
	  enddo
c
	  write(output_io,2100) 
     &	      'x',field(icell(1))(1:len_str(field(icell(1)))),icell(1)
	  write(output_io,2100) 
     &	      'y',field(icell(2))(1:len_str(field(icell(2)))),icell(2)
	  if (icell(3).ne.0) write(output_io,2110)
     &	          field(icell(3))(1:len_str(field(icell(3)))),icell(3)
	  if (icell(4).ne.0) write(output_io,2120)
     &	     'dx',field(icell(4))(1:len_str(field(icell(4)))),icell(4)
	  if (icell(5).ne.0) write(output_io,2120)
     &	     'dy',field(icell(5))(1:len_str(field(icell(5)))),icell(5)
	  write(output_io,*)
c
	  if (not_found) then
	    ier_num = -38
	    ier_typ = ER_APPL
	    return
	  endif
c
c------ - Finally reading of the data
c
	  nr=1
	  col(0)=0.0
	  lsigma = lcount .and. (icell(5).eq.0)
	  maxpp=maxarray-offxy(iz-1)
10	  continue
	    read(ifil,9999,end=20) mine
	    if (mine(1:2).eq.'#C') goto 10
	    read(mine,*,end=30,err=30) (col(i),i=1,nf)
	     x(offxy(iz-1)+nr) = col(icell(1))
	     y(offxy(iz-1)+nr) = col(icell(2))
	    dx(offxy(iz-1)+nr) = col(icell(4))
	    dy(offxy(iz-1)+nr) = col(icell(5))
	    if (lsigma .and. col(icell(2)).ge.0.0) then
	      dy(offxy(iz-1)+nr)=sqrt(col(icell(2)))
	    endif
	    if (icell(3).ne.0 .and. col(icell(3)).ne.0.0) then
	       y(offxy(iz-1)+nr)= y(offxy(iz-1)+nr)/col(icell(3))
	      dy(offxy(iz-1)+nr)=dy(offxy(iz-1)+nr)/col(icell(3))
	    endif
	    nr=nr+1
	    if (nr.gt.maxpp) then
	      ier_num = -6
	      ier_typ = ER_APPL
	      return
	    endif
	  goto 10
20	  continue
	  lend = .true.
30	  continue
c
c------ - Settings 
c
	  len(iz)   = nr-1
	  offxy(iz) = offxy(iz-1) + len(iz)
	  offz(iz)  = offz (iz-1)
	  iz=iz+1
c
	  achse(iwin,iframe,1) = 
     &	       field(icell(1))(1:len_str(field(icell(1))))
	  if (icell(3).eq.0) then
	    achse(iwin,iframe,2) = 
     &	         field(icell(2))(1:len_str(field(icell(2))))
	  else
	    achse(iwin,iframe,2) = 
     &	          field(icell(2))(1:len_str(field(icell(2))))
     &	          //'/'//field(icell(3))(1:len_str(field(icell(3))))
	  endif
	  titel(iwin,iframe,1)(1:len_str(tit))=tit(1:len_str(tit))
	  if (date(1:4).ne.'none') 
     &	        titel(iwin,iframe,2)(1:len_str(date)) = 
     &	             date(1:len_str(date))
c
	  call show_data(iz-1)
c
	  if (iscan.lt.iend .and. .not.lend) then
	    backspace(ifil)
c
	    iscan    = iscan + 1
	    fname(iz)= fname(iz-1)
	    goto 1111
	  endif
	endif
c                       
1000	format (' List of scans in file ',a,' :')
1100	format (i7,' pts --> #',a)
2000	format (1x,78('-'),/' Scan ',i4,' in file ',a,' :',/,1x,78('-'))
2010	format (1x,78('-'),/' Scan ',i4,' in file ',a,/,10x,
     &	        ' measured ',a,/,1x,78('-'))
2020	format ('   Data field ',i4,' : ',a)
2100	format ('   Axis ',a,' is       : ',a15,' (#',i4,')')
2110	format ('   Normalization   : ',a15,' (#',i4,')')
2120	format ('   Error column ',a,' : ',a15,' (#',i4,')')
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine read_spec_info (ifil,ianz,cpara,lpara,maxw)
c+
c	Read scan information from SPEC scan files
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw,maxr,maxc
c
	parameter	(maxr = 10)
	parameter	(maxc = 15)
c
	character*200	cpara(maxw)
	character*200	line,date,scan,energy
	character*20	field(0:maxc),field_t(0:maxr,maxc)
	character*20	out_t(5)
	real		field_v(0:maxr,maxc),out_v(5)
	integer		lpara(maxw)
	integer		ic,ir,nf,i,ii
	integer		ifil,ianz,istart,iend,iscan,nscan
	logical		lall,lend
c
	integer		len_str
	logical		str_comp
c
	date   = 'none'
	energy = 'none'
c
	do ir=0,maxr
	  do ic=1,maxc
	    field_t(ir,ic) = 'none'
	    field_v(ir,ic) = 0.0
	  enddo
	enddo
c
c------ Get header information
c
1	continue
	  read(ifil,9999,end=2) line
	  if (line(1:2).eq.'#S') then
	    backspace(ifil)
	    goto 2
	  elseif (line(1:2).eq.'#D') then
	    date = line(3:len_str(line))
	  elseif (line(1:6).eq.'#C E =') then
	    energy = line(3:len_str(line))
	  elseif (line(1:2).eq.'#O') then
	    read(line(3:4),*) ir
	    call get_col(line,field,nf,maxc)
	    do ic=1,nf
	      field_t(ir,ic) = field(ic)
	    enddo
	  endif
	goto 1	  
2	continue	
c
c------ Search for scan number first
c
	call get_scan_numbers (cpara(3),lpara(3),istart,iend,lall)
	if(ier_num.ne.0) return
c
	iscan = istart
c
1111	continue
	lend = .true.
c
3	continue
	  read(ifil,9999,end=20) line
	  if (line(1:2).eq.'#S') then
	    scan = line(3:len_str(line))
	    call get_col(line,field,nf,maxc)
	    read (field(1),*) nscan
	    lend = .false.
	    if (iscan.eq.nscan .or. lall) goto 4
	  endif
	goto 3
4	continue
c
c------ display info
c
	write(output_io,2000) iscan,fname(iz)(1:len_str(fname(iz)))
c
c------ read information
c
10	continue
	  read(ifil,9999,end=20) line
	  if (line(1:1).ne.'#') then
	    goto 30
	  elseif (line(1:2).eq.'#D') then
	    date = line(3:len_str(line))
	  elseif (line(1:2).eq.'#P') then
	    read(line(3:4),*) ir
	    call get_col(line,field,nf,maxc)
	    do ic=1,nf
	      read (field(ic),*) field_v(ir,ic)
	    enddo
	  endif
	goto 10
20	continue
	lend = .true.
30	continue
c
	if (lend) return
c
c------ display result
c
        write (output_io,3000) scan(1:len_str(scan))
	if (date(1:4)  .ne.'none') 
     &	    write (output_io,3010) date(1:len_str(date))
	if (energy(1:4).ne.'none') 
     &	    write (output_io,3020) energy(1:len_str(energy))
c
	ii=0
c
	write(output_io,4000)
	do ir=0,maxr
	  do ic=1,maxc
	    if (field_t(ir,ic).ne.'none') then
	      ii=ii+1
	      out_t(ii) = field_t(ir,ic)
	      out_v(ii) = field_v(ir,ic)
	    endif
	    if (ii.eq.5) then
	      write(output_io,4010) (out_t(i),i=1,5)
	      write(output_io,4020) (out_v(i),i=1,5)
	      ii=0
	    endif
	  enddo
	enddo
c
	if (ii.ne.0) then
	  write(output_io,4010) (out_t(i),i=1,ii)
	  write(output_io,4020) (out_v(i),i=1,ii)
	endif
c
c------ goto next scan
c
	if (iscan.lt.iend .and. .not.lend) then
	  backspace(ifil)
	  iscan = iscan + 1
	  goto 1111
	endif
c                       
2000	format (1x,78('-'),/' Scan ',i4,' in file ',a,' :',/,1x,78('-'))
3000	format (3x,'Scan       : ',a)
3010	format (3x,'Date       : ',a)
3020	format (3x,'Energy     : ',a)
4000	format (3x,'Parameters : ',/)
4010	format (3x,5(a12,3x))
4020	format (3x,5(g12.6,3x))
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine get_scan_numbers (str,len,istart,iend,lall)
c+
c	Analyse scan string (number or number>number)
c-                                
	implicit	none
c
	integer		maxw
	parameter	(maxw = 2)
c
	character*(*)	str
	integer		len,istart,iend
	logical		lall
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ipos,ianz
c
	integer		len_str
	logical		str_comp
c
	if(str_comp(str,'all',1,len,3)) then
	  istart = 1
	  iend   = 9999
	  lall   = .true.
	else
	  ipos = index(str(1:len),'>')
	  if (ipos.eq.0) then
	    cpara(1) = str
	    lpara(1) = len
	    cpara(2) = str
	    lpara(2) = len
	  else
	    cpara(1) = str(1:ipos-1)
	    lpara(1) = len_str(cpara(1))
	    cpara(2) = str(ipos+1:len)
	    lpara(2) = len_str(cpara(2))
	  endif
c
	  call ber_params(2,cpara,lpara,werte,maxw)
	  istart = nint(werte(1))
	  iend   = nint(werte(2))
	  lall   = .false.
	endif
c
	end
c*****7****************************************************************
	subroutine read_gsas (ifil,ianz,cpara,lpara,werte,maxw)
c+
c	Load GSAS files
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	integer		nbank
c
	parameter	(nbank=100)
c
	character*200	cpara(maxw)
	character*200	line
	character*80	iname,gsastit,filename
	character*1	units,cunits
	real*8		difc(nbank)
	real*8		difa(nbank)
	real*8		tzero(nbank)
	real		twotheta(nbank)
	real		werte(maxw)
	real		xval(maxarray)
	real		yval(maxarray)
	real		dyval(maxarray)
	integer		lpara(maxw)
	integer		bid(nbank)
	integer		ifil,ianz
	integer		iscan,istart,iend,i
	integer		ndat,maxpp
	logical		lend,lall,liparm,lunits
c
	integer		len_str
	integer		gsas_no_banks
c
	filename = fname(iz)
c
	lend   = .false.
	liparm = .false.
	lunits = .false.
c
	do i=1,nbank
	  bid(i)=i
	  twotheta(i)=-9999.
	enddo
c
	if (ianz.eq.2) then
c
c------ - Just print bank headers
c
55	  continue
	    read(ifil,'(a)',end=60) line
	    if (line(1:4).eq.'BANK') then
	      write(output_io,5000) line(1:len_str(line))
	    endif
	  goto 55
60	  continue
	elseif (ianz.ge.3) then
c
c------ - First get desired bank numbers
c
	  call get_scan_numbers (cpara(3),lpara(3),istart,iend,lall)
	  if(ier_num.ne.0) return
	  if(lall) iend=gsas_no_banks(ifil,bid,nbank)
c
c------ - Now check units parameter
c
	  if (ianz.gt.3) then
	    units=cpara(4)(1:1)
	    call do_cap(units)
	    lunits = .true.
	  endif
c
c------ - Now check for instrument parameter file and read
c
	  if (ianz.gt.4) then
	    call do_build_name(ianz,cpara,lpara,werte,maxw,5)
	    if (ier_num.ne.0) return
	    iname=cpara(5)
	    write(output_io,1000) iname(1:len_str(iname))
	    call read_iparm(iname,nbank,difc,difa,tzero,twotheta)
	    if (ier_num.ne.0) return
	    liparm=.true.
c
	  elseif (ianz.eq.4) then
	    call get_iparm_name(ifil,iname,liparm)
	    if (liparm) then
	      write(output_io,1000) iname(1:len_str(iname))
	      call read_iparm(iname,nbank,difc,difa,tzero,twotheta)
	      if (ier_num.ne.0) return
	    endif 
	  endif
c
c------ - Now Read the data
c
	  iscan=istart
c
1111	  continue
	  call read_gsas_data(ifil,iscan,xval,yval,dyval,ndat,
     &	                      bid,nbank,cunits,gsastit)
	  if (ier_num.ne.0) return
	  call convert_units(xval,yval,dyval,ndat,units,cunits,lunits,
     &                       difc(bid(iscan)),
     &	                     difa(bid(iscan)),
     &                       tzero(bid(iscan)),
     &	                     twotheta(bid(iscan)))
	  if (ier_num.ne.0) return
c
	  maxpp=maxarray-offxy(iz-1)
	  if (ndat.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
c
	  do i=1,ndat
	    x(offxy(iz-1)+i) = xval(i)
	    y(offxy(iz-1)+i) = yval(i)
	   dx(offxy(iz-1)+i) = 0.0
	   dy(offxy(iz-1)+i) = dyval(i)
	  enddo
c
c------ - Settings 
c
	  len(iz)   = i-1
	  offxy(iz) = offxy(iz-1) + len(iz)
	  offz(iz)  = offz (iz-1)
	  iz=iz+1
c
	  if (units.eq.'T') achse(iwin,iframe,1) = 
     &	                    'Time of flight (\\gmsec)'
	  if (units.eq.'Q') achse(iwin,iframe,1) = 'Q (\\A\\u-1\\d)'
	  if (units.eq.'D') achse(iwin,iframe,1) = 'd spacing (\\A)'
	  if (units.eq.'L') achse(iwin,iframe,1) = '\\gl (\\A)'
	  achse(iwin,iframe,2) = 'Intensity'
	  titel(iwin,iframe,1) = gsastit(1:len_str(gsastit))
	  if (bid(iscan).lt.10) then
	    write(fname(iz-1),2000) filename(1:len_str(filename)),
     &	                            bid(iscan)
	  else
	    write(fname(iz-1),2100) filename(1:len_str(filename)),
     &	                            bid(iscan)
	  endif
c
	  call show_data(iz-1)
c
	  if (iscan.lt.iend .and. .not.lend) then
	    backspace(ifil)
	    iscan    = iscan + 1
	    fname(iz)= filename
	    goto 1111
	  endif
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1000	format (' Reading GSAS instrument parameter file ',a, ' ..')
2000	format (a,'.Bank_',i1)
2100	format (a,'.Bank_',i2)
5000	format (1x,a)
	end	
c*****7****************************************************************
	subroutine convert_units(xval,yval,dyval,ndat,units,cunits,
     &	                         lunits,difc,difa,tzero,tth)
c+
c	Convert units from TOF to D,L,Q
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'debug.inc'
	include		'errlist.inc'
	include		'wink.inc'
c
	character*1	units,cunits
	real*8		difc,difa,tzero
	real		xval(maxarray)
	real		yval(maxarray)
	real		dyval(maxarray)
	real		tth,t,secondterm,d,tsint
	integer		ndat,j
	logical		lunits
c
	integer		len_str
c
c------ Keep units
c
	if (.not.lunits) then
	  units=cunits
	  return
	endif
c
c------ Selected same units
c
	if (units.eq.cunits) return
c
c------ ---------------------------------------------------------------
c------ Go from TOF to D,Q,L
c------ ---------------------------------------------------------------
c
	if (cunits.eq.'T') then
	  write(output_io,1100) units(1:len_str(units))
	  if (tth.eq.-9999.) then
	    write(output_io,1200)
	    units='T'
	    return
	  endif
c
c------ - D spacing
c------ - Use d = [-DIFC +/- sqrt(DIFC**2 - 4*DIFA*(TOF-TZERO))]/2*DIFA
c------ -     In case DIFA is 0.0 use d=(T-T0)/DIFC
c
	  if (units.eq.'D') then
            if (difa.ne.0.0) then
              do j=1,ndat
                t = xval(j) - tzero
                secondterm = dsqrt(4.0*difa*t + difc*difc)
                xval(j) = (-difc + secondterm)/2./difa
              enddo
            else
              do j=1,ndat
                xval(j) = (xval(j) - tzero)/difc
              enddo
            endif
c
c------ - Wavelength
c------ - Use l = 2d sin(theta)
c
	  elseif (units.eq.'L') then
            tsint = 2.*sin(abs(tth*pi)/360.)
            if (difa.ne.0.0) then
              do j=1,ndat
                t = xval(j) - tzero
                secondterm = dsqrt(4.0*difa*t + difc*difc)
                d = (-difc + secondterm)/2./difa
                xval(j) = d*tsint
              enddo
            else
              do j=1,ndat
                d = (xval(j) - tzero)/difc
                xval(j) = d*tsint
              enddo
            endif
c
c------ - Q
c------ - Use Q = 2 pi / d (and reverse arrays)
c
	  elseif (units.eq.'Q') then
            if (difa.ne.0.0) then
              do j=1,ndat
                t = xval(j) - tzero
                secondterm = dsqrt(4.0*difa*t + difc*difc)
                d = (-difc + secondterm)/2./difa
                xval(j) = 2*pi/d
              enddo
            else
              do j=1,ndat
                d = (xval(j) - tzero)/difc
                xval(j) = 2*pi/d
              enddo
            endif
c
            call reverse_array( xval,ndat)
            call reverse_array( yval,ndat)
            call reverse_array(dyval,ndat)
c
c------ - No valid unit
c
	  else
	    ier_num = -51
	    ier_typ = ER_APPL
	  endif
c
c------ ---------------------------------------------------------------
c------ Go from D to Q
c------ ---------------------------------------------------------------
c
	elseif (cunits.eq.'D') then
	   write(output_io,1100) units(1:len_str(units))
c
c------ - D to Q using Q=2 pi / D
c
	  if (units.eq.'Q') then
            do j=1,ndat
              xval(j) = 2*pi/xval(j)
            enddo
c
            call reverse_array( xval,ndat)
            call reverse_array( yval,ndat)
            call reverse_array(dyval,ndat)
c
c------ - No valid unit
c
	  else
	    ier_num = -51
	    ier_typ = ER_APPL
	  endif
c------ ---------------------------------------------------------------
c------ Go from Q to D
c------ ---------------------------------------------------------------
c
	elseif (cunits.eq.'Q') then
	   write(output_io,1100) units(1:len_str(units))
c
c------ - D to Q using Q=2 pi / D
c
	  if (units.eq.'D') then
            do j=1,ndat
              xval(j) = 2*pi/xval(j)
            enddo
c
            call reverse_array( xval,ndat)
            call reverse_array( yval,ndat)
            call reverse_array(dyval,ndat)
c
c------ - No valid unit
c
	  else
	    ier_num = -51
	    ier_typ = ER_APPL
	  endif
	endif
c
1100	format (' Converting to units ',a, ' ..')
1200	format (' Unit conversion requires instrument parameter',
     &	        ' file ..')
	end
c*****7****************************************************************
	subroutine reverse_array(dat,ndat)
c+
c	Reverse given array
c-                                
	implicit	none
c
	include		'config.inc'
c
	real		dat(maxarray)
	real		dummy(maxarray)
	integer		i,ndat
c
	do i=1,ndat
	  dummy(i)=dat(i)
	enddo
c
	do i=1,ndat
	  dat(i)=dummy(ndat-i+1)
	enddo
c
	end
c*****7****************************************************************
	subroutine get_iparm_name(ifil,iname,liparm)
c+
c	Read GSAS data
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'param.inc'
	include		'debug.inc'
	include		'errlist.inc'
c
	character*(*)	iname
	integer		ifil
	logical		liparm
c
	character*80  line,dummy
	integer       len_str
c
       liparm=.false.
       iname=''
c
       rewind(ifil)
c
c----- Search for Instrument parameter file line
c
10     continue
         read(ifil,'(a)',end=15) line
         dummy=line
         call do_cap(dummy)
         if (dummy(1:26).eq.'INSTRUMENT PARAMETER FILE:') then
           iname=line(27:len_str(dummy))
           liparm=.true.
         endif
       if (.not.liparm) goto 10
15     continue
c
c----- Check the file exists
c
       inquire(file=iname,exist=liparm)
       if (.not.liparm) write(output_io,1000) iname(1:len_str(iname))
c
1000   format(' Instrument parameter file ',a,' not found ..')
       end
c*****7****************************************************************
	subroutine read_gsas_data(ifil,ibank,xval,yval,dyval,ndat,
     &                            bid,nbank,cunits,gsastit)
c+
c	Read GSAS data
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'param.inc'
	include		'debug.inc'
	include		'errlist.inc'
c
	integer		nbank
c
	character*(*)	gsastit
	character*1	cunits
	real		xval(maxarray)
	real		yval(maxarray)
	real		dyval(maxarray)
	integer		bid(nbank)
	integer		ndat,ibank,ifil
c
	character*80  line,bank_line
	real          clckwdt,onetick
	real          c_min,c_d,monitor
	integer       tmap(3,maxarray),tmap3i,ntmax
	integer       i,j,ll,ib,nc,nr,iq,lt
	integer       idummy,itmap,icons
	integer       itmap1i,itmap2i,itmap3i
	integer       itmap1iplus1
	integer       i_tmap
	integer       maxmapu
	integer       it_no,it_nval,it_nrec,imax
	logical       ltmap,lbank,lmon
c
	integer       len_str
c
       ltmap=.false.
       lbank=.false.
       lmon =.false.
c
       rewind(ifil)
c
c----- Read title line
c
       read(ifil,'(a)',end=15,err=998) gsastit
c
c----- Search for Monitor count
c
10     continue
         read(ifil,'(a)',end=15,err=998) line
         if (line(1:8).eq.'Monitor:' .or.
     &       line(1:8).eq.'MONITOR:'     ) then
           read(line(9:ll),*,err=998) monitor
           lmon=.true.
         endif
       if (.not.lmon) goto 10

       if (lmon) then
         res_para(0)=1
         res_para(1)=monitor
         write(output_io,5000) monitor
       endif
c
c----- Now try to find the bank we are looking for
c
15     continue
       rewind(ifil)
20     continue
         read(ifil,'(a)',end=510,err=998) line
         ll=len_str(line)
         if (line(1:4).eq.'BANK') then
           itmap=index(line(5:ll),'TIME_MAP')
           icons=index(line(5:ll),'CON')
c
           if (index(line(5:ll),'TIME_MAP').ne.0) cunits='T'
c
           if (index(line(5:ll),'CONS').ne.0) cunits='T'
           if (index(line(5:ll),'CONQ').ne.0) cunits='Q'
           if (index(line(5:ll),'COND').ne.0) cunits='D'
c
c----- --- Type TIME_MAP

           if (itmap.gt.0) then
             read(line(5:itmap+3),*,err=998) ib,nc,nr
             lbank=(ib.eq.bid(ibank))
             if (lbank) then
               bank_line=line
               read(line(itmap+12:ll),*,err=998) i_tmap
               if (dbg) then
                 write(*,1000) i_tmap
                 write(*,1100) nc,nr
               endif
             endif
c
c----- --- Type CON*
c
           elseif (icons.gt.0) then
             read(line(5:icons+3),*,err=998) ib,nc,nr
             lbank=(ib.eq.bid(ibank))
             if (lbank) then
               bank_line=line
               read(line(icons+9:ll),*,err=998) c_min,c_d
               if (dbg) then
                 write(*,1200) c_min, c_d
                 write(*,1100) nc,nr
               endif
             endif
c
c----- --- Type unknown
c
           else
	     ier_num=-48
	     ier_typ=ER_APPL
	     return
           endif
         endif
       if (.not.lbank) goto 20
c
c----- Now we can read the data
c
       if (index(bank_line,'STD').ne.0) then
         read(ifil,'(10(i2,f6.0))') (idummy,yval(i),i=1,nc)
         do i=1,nc
           dyval(i)=sqrt(yval(i))
         enddo
         if (dbg) write(*,1300) 'STD'
       elseif (index(bank_line,'ESD').ne.0) then
         read(ifil,'(10f8))') (yval(i),dyval(i),i=1,nc)
         if (dbg) write(*,1300) 'ESD'
       else
         if (dbg) write(*,1300) '(default) STD'
         read(ifil,'(10(i2,f6.0))') (idummy,yval(i),i=1,nc)
         do i=1,nc
           dyval(i)=sqrt(yval(i))
         enddo
       endif      
c 
c----- Now we generate the TOF entry - ICONS
c
       if (icons.gt.0) then
         do i=1,nc
            xval(i) = c_min + (i-1)*c_d
            yval(i) =  yval(i)/c_d
           dyval(i) = dyval(i)/c_d
         enddo
c
c----- TIME_MAP
c
       elseif (itmap.gt.0) then
c
c----- we read the first time_map anyway (due to a bug in FSTBUSBIN)
c----- then try finding the matching one (HIPPO will have mult. TIME_MAPs)
c
         rewind (ifil)
50       continue
           read(ifil,'(a)',end=60,err=998) line
           ll=len_str(line)
           if (line(1:8).eq.'TIME_MAP') then
             lt=index(line(9:ll),'TIME_MAP')+8
             read(line(9:lt),*,err=998) it_no,it_nval,it_nrec
             read(line(lt+9:ll),*,err=998) clckwdt
             maxmapu=(it_nval-1)/3
             read(ifil,'(10i8)',err=998,end=998) 
     &           ((tmap(j,i),j=1,3),i=1,maxmapu),ntmax
             ltmap=.true.
           endif
         if (.not.ltmap .or. it_no.ne.i_tmap) goto 50
60       continue
c
         if (ltmap) then
           if (dbg) then
             write(*,1400) it_no,it_nval,it_nrec
             write(*,1500) clckwdt
           endif
           if (it_no.ne.i_tmap .and. dbg) write(*,1600)
         else
           ier_num=-49
	   ier_typ=ER_APPL
	   return
         endif
c
c----- - Now generate the tof entry from the tmap array
c
         maxmapu=(it_nval-1)/3
c         
         iq=0
         do i=1,maxmapu-1
           itmap3i = tmap(3,i)
            tmap3i = tmap(3,i)
           itmap2i = tmap(2,i)
           itmap1i = tmap(1,i)
           itmap1iplus1 = tmap(1,i+1)
           do j = itmap1i,itmap1iplus1-1
             iq = iq + 1
             xval(iq)  = itmap2i + itmap3i*(j-itmap1i) +  tmap3i/2
             yval(iq)  = yval(iq)/itmap3i
             dyval(iq) = dyval(iq)/itmap3i
           enddo
         enddo

         itmap3i = tmap(3,maxmapu)
          tmap3i = tmap(3,maxmapu)
         itmap2i = tmap(2,maxmapu)
         itmap1i = tmap(1,maxmapu)
         imax = (ntmax-itmap2i)/itmap3i + itmap1i
         do j = itmap1i,imax
           iq = iq + 1
           xval(iq)  = itmap2i + itmap3i*(j-itmap1i) +  tmap3i/2
           yval(iq)  = yval(iq)/itmap3i
           dyval(iq) = dyval(iq)/itmap3i
         enddo
         onetick = clckwdt/1000.
         do iq = 1,imax
           xval(iq) = xval(iq)*onetick
         enddo
       endif
       ndat=nc
       return
c
510    continue
       ier_num = -50
       ier_typ = ER_APPL
       return
c
998    continue
       ier_num = -3
       ier_typ = ER_IO
       return
c
1000	format(' debug > TOF binning found: TIME_MAP number ',i3)
1100	format(' debug > Bank has ',i6,' channels and ',i6,' records')
1200	format(' debug > TOF binning CONST found: TOF_MIN: ',f9.2,
     &	       ', TOF_D: ',f9.2)
1300	format(' debug > Data type ',a,' found')
1400	format(' debug > TIME_MAP found: no: ',i3,', nval: ',i6,
     &	       ', nrec: ',i6)
1500	format(' debug > TIME_MAP found: Clockwidth: ',f10.4)
1600	format(' debug > No matching TIME_MAP found, taking first !')
5000	format(' Monitor count found: ',f10.1)
c
	end
c*****7****************************************************************
	subroutine read_iparm(inam,nbank,difc,difa,tzero,twotheta)
c+
c	Read GSAS instrument parameter file
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'errlist.inc'
c
	integer		nbank
c
	character*(*)	inam
	character*80	line
	character*6	search
	real*8		difc(nbank)
	real*8		difa(nbank)
	real*8		tzero(nbank)
	real		twotheta(nbank)
	real		dummy
	integer		ibank,i,ll
c
	integer		len_str
c
	call oeffne(12,inam,'old',.true.)
	if (ier_num.ne.0) return
c
c------ Read diffractometer constants in lookup table
c
20      continue
	read (12,'(4x,i2,a6)',err=20,end=40) ibank,search
	if (search.ne.' ICONS') goto 20
	if (ibank.le.0 .or. ibank.gt.nbank) goto 997
	backspace(12)
	read (12,'(a)',err=998) line
	ll=len_str(line)
	read (line(13:ll),*,err=998,end=998) 
     &	      difc(ibank),difa(ibank),tzero(ibank)
c
30	continue
	read (12,'(4x,i2,a6)',err=30,end=40) ibank,search
	if (search.ne.'BNKPAR') goto 30
	backspace(12)
	read (12,'(a)',err=998) line
	ll=len_str(line)
	read (line(13:ll),*,err=998) 
     &	      dummy,twotheta(ibank)
	goto 20
c
40	continue
        close(12)
c
	write(output_io,1000)
	do i=1,nbank
	  if (twotheta(i).ne.-9999.) then
	    write(output_io,1100) i,twotheta(i),difc(i),difa(i),tzero(i)
	  endif
	enddo
	write(output_io,1200)
c
        return
c
997     continue
	ier_num=-46
	ier_typ=ER_APPL
	close(12)
	return
c
998     continue
	ier_num=-47
	ier_typ=ER_APPL
	close(12)
c
1000	format (' -------------------------------------------------',/,
     &	        ' Bank    TwoTheta     DifC         DifA      Tzero')
1100	format (1x,i3,5x,f7.2,3x,f9.2,3x,f8.2,3x,f8.2)
1200	format (' -------------------------------------------------')
        end
c*****7****************************************************************
	integer function gsas_no_banks(ifil,bid,nbank)
c+
c	Determined number of banks in GSAS file
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'debug.inc'
	include		'errlist.inc'
c
	integer		ifil,nbank
	integer		bid(nbank)
c
	character*80	line
	integer		ibank,ib
c
	integer		len_str
c
      ibank=0
10    continue
        read (ifil,'(a)', end=20) line
        if (line(1:5).eq.'BANK ') then
	  read(line(6:len_str(line)),*,err=998) ib
	  if (ib.ne.0) then
	    ibank=ibank+1
	    bid(ibank)=ib
	  endif
	endif
      goto 10
20    continue
c
      gsas_no_banks=ibank
      if (dbg) write(*,1000) ibank
c
      rewind(ifil)
      return
c
998   continue
      ier_num = -3
      ier_typ = ER_IO
      return
c
1000  format(' debug > Number of GSAS banks found: ',i3)
      end
c*****7****************************************************************
	subroutine read_xy (ifil,noswap)
c+
c	Load x,y or x,y,dx,dy files
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw = 4)
c
	character*200	line
	real		werte(maxw)
	integer		ifil,nr,iwex,iwey,iwdx,iwdy
	integer		ianz,i,maxpp
	logical		noswap
c
c------ xy or yx
c
	if (noswap) then
	  iwex=1
	  iwey=2
	  iwdx=3
	  iwdy=4
	else
	  iwex=2
          iwey=1
          iwdx=4
          iwdy=3
	endif
c
c------ read data
c      
	nr=1
	maxpp=maxarray-offxy(iz-1)
10	continue
	  read(ifil,9999,end=20) line
	  if (line(1:1).eq.'#') goto 10
	  call count_col (line,ianz)
	  if (ianz.eq.0) goto 10
	  if (ianz.gt.4) ianz=4
	  backspace (ifil)
15	  continue
	  read(ifil,*,end=20) (werte(i),i=1,ianz)
	  x(offxy(iz-1)+nr)=werte(iwex)
	  y(offxy(iz-1)+nr)=werte(iwey)
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  if(ianz.ge.3) dx(offxy(iz-1)+nr)=werte(iwdx)
	  if(ianz.ge.4) dy(offxy(iz-1)+nr)=werte(iwdy)
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 15
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	iz=iz+1
c
	call show_data(iz-1)
c                       
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine read_mca_single(ifil,ianz,werte,maxw)
c+
c	Load single MCA spectrum
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*200	line
	integer		maxw
	real		werte(maxw),xold,xnew,ynew
	integer		i,nr,ifil,ianz,iscan,nscan,maxpp
c
c------ check parameters
c
	if (ianz.ne.1) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ Move forward to correct scan
c
	iscan = nint(werte(1))
	nscan = 1
	xold  = -9999.
c
5	continue
	  if (nscan.eq.iscan) then
	    backspace(ifil)
	    goto 7
	  endif
	  read(ifil,*,end=90) xnew,ynew
	  if (xnew.lt.xold) nscan = nscan+1
	  xold = xnew
	goto 5
7	continue
c
c------ read data
c      
	nr=1
	xold=-9999.
c
	maxpp=maxarray-offxy(iz-1)
10	continue
	  read(ifil,*,end=20) xnew,ynew
	  if(xnew.lt.xold) goto 20
	  x(offxy(iz-1)+nr) =xnew
	  y(offxy(iz-1)+nr) =ynew
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  xold=xnew
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 10
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	fform(iz) = 'XY'
	iz=iz+1
c
	write(titel(iwin,iframe,2),1000) iscan
	achse(iwin,iframe,1) = 'Channel'
	achse(iwin,iframe,2) = 'Counts'
c
	call show_data(iz-1)
	return
c
90	continue
	ier_num = -41
	ier_typ = ER_APPL
c                       
1000	format ('MCA spectrum #',i5)
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine read_xx (ifil)
c+
c	Load files point no versus value
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw = 4)
c
	integer		ifil,nr
	integer		ianz,i,maxpp
c
	nr=1
	maxpp=maxarray-offxy(iz-1)
10	continue
	  read(ifil,*,end=20) y(offxy(iz-1)+nr)
	   x(offxy(iz-1)+nr)=float(nr)
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 10
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	fform(iz) = 'XY'
	iz=iz+1
c
	call show_data(iz-1)
c                       
	end	
c*****7****************************************************************
	subroutine read_crystal (ifil)
c+
c	Reads special crystal file format exported from DISCUS. 
c	Line per atom  : x,y,z, mtyp,mcol,msiz
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	real		rmsiz,zdummy
	integer		ifil,imcol,imtyp,nr,iii,maxpp
c
	nr=1
	maxpp=maxarray-offxy(iz-1)
10	continue
	  iii=offxy(iz-1)+nr
	  read(ifil,*,end=20) x(iii),y(iii),zdummy,
     &	                      imtyp,imcol,rmsiz
c
c-------  Put color / marker and size information in DX, DY
c
	  dx(iii)=float(imcol*1000+imtyp)
	  dy(iii)=rmsiz
c
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 10
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	iz=iz+1
c
	call show_data(iz-1)
c                       
9999	format (a)
c
	end	
c*****7****************************************************************
	subroutine read_marker (ifil)
c+
c	Read marker file (only x-values)
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*200	line
	integer		ifil,nr,maxpp
c
c------ read data
c          
	nr=1
	maxpp=maxarray-offxy(iz-1)
10	continue
	  read(ifil,9999,end=20) line
	  if (line(1:1).eq.'#') goto 10
	  backspace (ifil)
	  read(ifil,*,end=20) x(offxy(iz-1)+nr)
	   y(offxy(iz-1)+nr)=0.0
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 10
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	iz=iz+1
c
	call show_data(iz-1)
c
9999	format(a)
	end	
c*****7****************************************************************
	subroutine read_z(ifil,izaehl)
c+
c	Read powder scan files MAN1 program ..
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	real		dummy
	integer		ifil,izaehl,nr,maxpp
c
c------ setup
c
	achse(iwin,iframe,1)='2-theta'
	achse(iwin,iframe,2)='counts'
c
c------ read data
c          
	nr=1   
	maxpp=maxarray-offxy(iz-1)
c
        read(ifil,*,end=20)
        read(ifil,*,end=20)
        read(ifil,*,end=20)
10	continue
	  if (izaehl.eq.1) then
	    read(ifil,*,end=20) 
     &	         x(offxy(iz-1)+nr),y(offxy(iz-1)+nr)
	  else
	    read(ifil,*,end=20) 
     &	         x(offxy(iz-1)+nr),dummy,y(offxy(iz-1)+nr)
	  endif  
	  dx(offxy(iz-1)+nr)=0.0
	  dy(offxy(iz-1)+nr)=0.0
	  nr=nr+1
	  if (nr.gt.maxpp) then
	    ier_num = -6
	    ier_typ = ER_APPL
	    return
	  endif
	goto 10
20	continue
	len(iz)   = nr-1
	offxy(iz) = offxy(iz-1) + len(iz)
	offz(iz)  = offz (iz-1)
	iz=iz+1
c
	call show_data(iz-1)
c                       
	end	
c*****7****************************************************************
	subroutine get_extrema
c+
c	get max/min values of files
c-                                
	implicit	none
c
	include		'config.inc'
	include		'kuplot.inc'
c
	integer		ik
c
	do ik=1,iz-1
	  if(lni(ik)) then
	    call get_extrema_xy (x,ik,nx(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,ny(ik),ymin,ymax)
	    call get_extrema_z  (z,ik,nx(ik),ny(ik),zmin,zmax)
	  else
	    call get_extrema_xy (x,ik,len(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,len(ik),ymin,ymax)
	  endif
	enddo
c
	end  
c*****7****************************************************************
	subroutine get_col(zeile,field,nf,colm)
c+
c	This subroutine splits string at spaces
c-
	implicit	none
c
	integer		colm
c
	character*(*)	zeile,field(0:colm)
	integer		nf,i,is,ia
	logical		ein
c
	integer		len_str
c
	nf=0
	ein =.false.
c
	is=1
	do while(zeile(is:is).ne. ' ')
	  is=is+1
	enddo
c
	do i=is,len_str(zeile)
	  if (zeile(i:i).ne.' ') then
	    if (.not.ein) ia = i
	    ein = .true.
	  elseif (zeile(i:i).eq.' ' .and. ein) then
	    ein       = .false.
	    nf        = nf + 1
	    field(nf) = zeile(ia:i-1)
	  endif
	enddo
c
	if (ein) then
	  nf        = nf+1
	  field(nf) = zeile(ia:i-1)
	endif
c
	end
