c*****7*****************************************************************
c
c	This routine shows all settings of KUPLOT parameters. Some
c	parameter settings can also be displayed by entering the
c	corresponding command without parameters.
c
c	Version : 4.0
c	Date    : 06. March 1999
c
c	Author  : Th. Proffen  (tproffen@lanl.gov )
c
c*****7*****************************************************************
	subroutine do_show (line,lp)
c
c	Main show menu
c
	implicit	none
c
	integer		maxw
	parameter	(maxw=2)
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*(*)	line
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw),lp
	integer		ianz,iku
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
c
c------ - Output all information
c
	  call show_data(-1)
	  call show_plot
	  call show_style
	  call show_hlin
	  call show_annotation
	  call show_frames
	  call show_font
c
	elseif (ianz.eq.1 .or. ianz.eq.2) then
c
c------ - Contour line information
c
	  if(str_comp(cpara(1),'hlin',2,lpara(1),4)) then
	    call show_hlin
c
c------ - Data information
c
	  elseif(str_comp(cpara(1),'data',2,lpara(1),4)) then
	    if (ianz.eq.1) then
	      call show_data(-1)
	    else
	      call del_params(1,ianz,cpara,lpara,maxw)
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      iku=nint(werte(1))
	      call show_data(iku)
	    endif
c
c------ - Plot information
c
	  elseif(str_comp(cpara(1),'plot',2,lpara(1),4)) then
	    call show_plot
c
c------ - Style information (colors, ...)
c
	  elseif(str_comp(cpara(1),'style',2,lpara(1),5)) then
	    call show_style
c
c------ - Show annotations
c
	  elseif(str_comp(cpara(1),'annot',2,lpara(1),5)) then
	    call show_annotation
c
c------ - Current color map
c
	  elseif(str_comp(cpara(1),'cmap',3,lpara(1),4)) then
	    call show_cmap
c
c------ - Current color settings
c
	  elseif(str_comp(cpara(1),'color',3,lpara(1),5)) then
	    call show_color
c
c------ - Current frame setting
c
	  elseif(str_comp(cpara(1),'frame',2,lpara(1),5)) then
	    call show_frames
c
c------ - Configuration of KUPLOT
c
	  elseif(str_comp(cpara(1),'config',3,lpara(1),5)) then
	    call show_config
c
c------ - Current font settings
c
	  elseif(str_comp(cpara(1),'font',2,lpara(1),4)) then
	    call show_font
c
c------ - Error status setting
c
	  elseif(str_comp(cpara(1),'error',2,lpara(1),5)) then
	   if(ier_sta.eq.ER_S_CONT) then
	      write(output_io,2100)
	    else
	      write(output_io,2105)
	    endif
c
c------ - Result array
c
	  elseif(str_comp(cpara(1),'result',2,lpara(1),6)) then
	    call show_res
c
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif  
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
2100    format(' Program continues after display of error message')
2105    format(' Program terminates after display of error message')
	end
c*****7*****************************************************************
	subroutine show_config
c+
c	Show KUPLOT configuration
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
c
	write (output_io,1000)
	write (output_io,2000) MAXARRAY
	write (output_io,2010) MAXKURVTOT
	write (output_io,2020) MAXZ
	write (output_io,2030) MAXHL
	write (output_io,2040) MAXFRAME
	write (output_io,2045) MAXWIN
	write (output_io,2050) MAXSP
	write (output_io,2060) MAXAN
	write (output_io,2070) MAXBOND
	write (output_io,2080) MAXPARA
c
1000	format (' Current KUPLOT configuration :',/)
2000	format ('   Maximum total number of data points      : ',i8)
2010	format ('   Maximum number of data sets              : ',i8)
2020	format ('   Maximum bitmap size (maxz,maxz)          : ',i8)
2030	format ('   Maximum number of contour line sets      : ',i8)
2040	format ('   Maximum number of frames                 : ',i8)
2045	format ('   Maximum number of windows                : ',i8)
2050	format ('   Maximum number of points for step/spline : ',i8)
2060	format ('   Maximum number of annotations            : ',i8)
2070	format ('   Maximum number of different bonds        : ',i8)
2080	format ('   Maximum number of fit parameters         : ',i8,/)
	end
c*****7*****************************************************************
	subroutine show_hlin
c+
c	Show contour line information
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		j
c
	write (output_io,1000) iwin,iframe
	write (output_io,1010) 
	do j=1,iho(iwin,iframe)
	  write (output_io,1020) 
     &	  j,z_min(iwin,iframe,j),z_inc(iwin,iframe,j),nz(iwin,iframe,j)
	enddo
	write(output_io,*)
c
1000	format (' Contour line settings for window ',i3,
     &	        ' / frame ',i3,/)
1010	format (3x,'Cont.set    minimum       ',
     &	        'increment   # of contours',/,3x,56('-'))
1020	format (4x,i3,7x,g11.5,3x,g11.5,6x,i4)
	end
c*****7*****************************************************************
	subroutine show_data(iku)
c+
c	Print information about loaded data
c-                                
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		iku,ia,ie,ik
c
c------ check if we want ALL data sets
c
	if(iku.eq.-1) then
	  ia=1
	  ie=iz-1
	else
	  if(iku.gt.iz-1 .or. iku.lt.1) then
	    ier_num = -4
	    ier_typ = ER_APPL
	    return
	  endif
	  ia=iku
	  ie=iku
	endif
c
c------ Print information
c
	do ik=ia,ie
	  if(lni(ik)) then
	    call get_extrema_xy (x,ik,nx(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,ny(ik),ymin,ymax)
	    call get_extrema_z  (z,ik,nx(ik),ny(ik),zmin,zmax)
	    write(output_io,1000) 
     &	                  ik,fname(ik),nx(ik),ny(ik),xmin(ik),xmax(ik),
     &	                  ymin(ik),ymax(ik),zmin(ik),zmax(ik)
	  else
	    call get_extrema_xy (x,ik,len(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,len(ik),ymin,ymax)
	    write(output_io,1010) 
     &	                  ik,fname(ik),len(ik),xmin(ik),xmax(ik),
     & 	                  ymin(ik),ymax(ik)
	  endif
	enddo
c
1000	format (' Data set no.: ',i3,//,
     &            3x,'Filename : ',a35,'(',i4,' * ',i4,' pts )'/
     &            3x,'Range  x : ',g12.4,' to ',g12.4/  
     &            3x,'Range  y : ',g12.4,' to ',g12.4/  
     &            3x,'Range  z : ',g12.4,' to ',g12.4,/)
1010	format (' Data set no.: ',i3,//, 
     &            3x,'Filename : ',a35,'(',i6,' points )'/
     &            3x,'Range  x : ',g12.4,' to ',g12.4/  
     &            3x,'Range  y : ',g12.4,' to ',g12.4,/)
c
	end  
c*****7****************************************************************
	subroutine show_plot
c+
c	Show information about plot settings
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
	include 	'errlist.inc'
c
	write(output_io,1000) iwin,iframe,yskal_u(iwin,iframe),
     &	   shear(iwin,iframe),
     &	   ilinecol(iwin,iframe,0),linewid(iwin,iframe,0),
     &	   ibuf(iwin,iframe,1),ibuf(iwin,iframe,2),ibuf(iwin,iframe,3),
     &	   ibuf(iwin,iframe,4)
c
	if(ibox(iwin,iframe).eq.0) then
	  write(output_io,1005) 'no frame'
	elseif(abs(ibox(iwin,iframe)).eq.1) then
	  write(output_io,1005) 'only box around plot'
	elseif(abs(ibox(iwin,iframe)).eq.2) then
	  write(output_io,1005) 'box and tick marks / labels'
	elseif(abs(ibox(iwin,iframe)).eq.3) then
	  write(output_io,1005) 'box, tick marks and x,y-axis'
	endif
c
	if (ibox(iwin,iframe).ge.0) then
	  write (output_io,1007) 'LEFT'
	else
	  write (output_io,1007) 'RIGHT'
	endif
c
        if(ifname(iwin,iframe)) then
	  write(output_io,1010) 'on '
	else
	  write(output_io,1010) 'off'
	endif
        if(tot_frame(iwin)) then
	  write(output_io,1013) 'on '
	else
	  write(output_io,1013) 'off' 
	endif
        if(igrid(iwin,iframe)) then
	  write(output_io,1015) 'on '
	else
	  write(output_io,1015) 'off'
	endif
        if(iden(iwin)) then
	  write(output_io,1025) 'on '
	else
	  write(output_io,1025) 'off'
	endif
c
1000	format(' Parameter for window ',i3,' / frame ',i3,//
     &	        3x,'Ratio y/x-axis           : ',g11.5/
     &	        3x,'Angle between x&y axis   : ',f7.2/
     &	        3x,'Color of grid lines      : ',i7/
     &	        3x,'Line width of frame [cm] : ',f7.2/
     &	        3x,'Buffer around plot       : ',4(f4.2,1x))
1005	format( 3x,'Type of frame            : ',a)
1007	format( 3x,'Location of y-labels     : ',a)
1010	format( 3x,'Plot filenames           : ',a)
1013	format( 3x,'Plot frame               : ',a)
1015	format( 3x,'Plot grid                : ',a)
1025	format( 3x,'Plot user/date info.     : ',a/)
	end
c*****7****************************************************************
	subroutine show_frames
c+
c	Show current frame settings
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
c
	integer		i,j
c
	write (output_io,1000) iwin,iframe
	write (output_io,2000)
	do i=1,iaf(iwin)
	  write (output_io,2500) 
     &	         i,(frame(iwin,i,j),j=1,4),(frback(iwin,i,j),j=1,3)
	enddo
c
	write (output_io,3000)
	do i=1,iaf(iwin)
	  if(infra(iwin,i,1).ne.-1) then
	    write (output_io,3500) i,(infra(iwin,i,j),j=1,maxkurvtot)
	  else
	    write (output_io,3600) i,ftext(iwin,i)
	  endif
	enddo
	write (output_io,*)
c
1000	format (' Frame settings (window ',i3,' / frame ',i3,') :',/)
2000	format (3x,'Frame  xmin ymin xmax ymax  Backgr.  r   g   b',/
     &	        3x,'------------------------------------------------')
2500	format (3x,i3,4x,4(f4.2,1x),9x,3(f3.1,1x))
3000	format (/,3x,'Frame  data sets/text file',/,3x,67('-'))
3500	format (3x,i3,4x,20i3)
3600	format (3x,i3,4x,a40)
	end
c*****7****************************************************************
	subroutine show_font
c+
c	Show current font settings
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
c
	character*20    fon_name(4)
c
	data    fon_name  /'Standard','Roman',
     &	                   'Italic','Script'/
c
	write (output_io,1000) iwin,iframe,fonscal(iwin,iframe)
	write (output_io,1100) 
     &	       1,'Main title line    ',fonsize(iwin,iframe,1)/10.,
     &	       fon_name(fon_id(iwin,iframe,1)),fon_id(iwin,iframe,1),
     &	       foncol(iwin,iframe,1)
	write (output_io,1100) 
     &	      2,'Subtitle line      ',fonsize(iwin,iframe,2)/10.,
     &	      fon_name(fon_id(iwin,iframe,2)),fon_id(iwin,iframe,2),
     &	      foncol(iwin,iframe,2)
	write (output_io,1100) 
     &	      3,'Axis labels        ',fonsize(iwin,iframe,3)/10.,
     &	      fon_name(fon_id(iwin,iframe,3)),fon_id(iwin,iframe,3),
     &	      foncol(iwin,iframe,3)
	write (output_io,1100) 
     &	      4,'Numbers at axis    ',fonsize(iwin,iframe,4)/10.,
     &	      fon_name(fon_id(iwin,iframe,4)),fon_id(iwin,iframe,4),
     &	      foncol(iwin,iframe,4)
	write (output_io,1100) 
     &	      5,'Text in text frame ',fonsize(iwin,iframe,5)/10.,
     &	      fon_name(fon_id(iwin,iframe,5)),fon_id(iwin,iframe,5),
     &	      foncol(iwin,iframe,5)
	write (output_io,1100) 
     &	      6,'Filename & caption ',fonsize(iwin,iframe,6)/10.,
     &	      fon_name(fon_id(iwin,iframe,6)),fon_id(iwin,iframe,6),
     &	      foncol(iwin,iframe,6)
	write (output_io,*)
c
1000	format (' Font settings for window ',i3,' / frame ',i3,/,
     &	        3x,'Overall font scaling factor  : ',f6.2,//,
     &	        3x,'Font  where ?              size   fontname',
     &	        12x,'font-id   color',/,3x,69('-'))
1100	format (3x,i3,3x,a,2x,f4.0,2x,a,4x,i2,7x,i1)
	end
c*****7****************************************************************
	subroutine show_res
c+
c	Show result array
c-
	implicit	none
c
	include 	'prompt.inc'
	include 	'param.inc'
c
	integer		i,j,k,k1,k2
c
	if (res_para(0).eq.0) then
	  write(output_io,1000)
	else
          write(output_io,2000) nint(res_para(0))
          j = nint(res_para(0))/5
          if(mod(nint(res_para(0)),5).eq.0) then
            j = j-1
          endif
          do i=0,j
            k1 =     5*i+1
            k2 = min(5*i+5,nint(res_para(0)))
            write(output_io,2010) (res_para(k),k=k1,k2)
          enddo
	  write (output_io,*)
	endif
c
1000	format (' Result array is empty ')
2000	format (' Result array containing ',i4,' values :',/)
2010	format (3x,5(g13.7,2x))
	end
c*****7****************************************************************
	subroutine show_annotation
c+
c	Show annotations
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
c
	character*1	just
	integer		i,il,len_str
	logical		noann
c
	noann = .true.
c
	write (output_io,1000) iwin,iframe
	do i=1,maxan
	  if (antext(iwin,iframe,i)(1:3).ne.'OFF') then
	    if (anjust(iwin,iframe,i).eq.if_left)   just='L'
	    if (anjust(iwin,iframe,i).eq.if_right)  just='R'
	    if (anjust(iwin,iframe,i).eq.if_centre) just='C'
	    il=min(len_str(antext(iwin,iframe,i)),15)
	    write (output_io,2000) i,anx(iwin,iframe,i),
     &	                           any(iwin,iframe,i),
     &	                           anangle(iwin,iframe,i),just,
     &	                           antext(iwin,iframe,i)(1:il)
	    if (anx(iwin,iframe,i).ne.antx(iwin,iframe,i) .or.
     &          any(iwin,iframe,i).ne.anty(iwin,iframe,i)     ) then
	      write(output_io,2100) antx(iwin,iframe,i),
     &	                            anty(iwin,iframe,i)
	    endif
	    noann = .false.
	  endif
	enddo
c
	if (noann) write (output_io,3000)
	write (output_io,*)
c
1000	format (' Annotation for window ',i3,' / frame  ',i3,//
     &	        3x,'  #',6x,'x',13x,'y',8x,'angle  just  text',/,
     &	        3x,69('-'))
2000	format (3x,i3,2(2x,g12.6),2x,f5.1,2x,a1,4x,a)
2100	format (6x,2(2x,g12.6),14x,'Text position ..')
3000	format ('   *** no annotations set ***')
	end
c*****7****************************************************************
	subroutine show_cmap
c+
c	Show current color map
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
c
	integer		i,j
c
	write (output_io,1000) 
	write (output_io,2000) (i,(col_map(iwin,i,j),j=1,3),i=1,maxcol)
c
1000	format (' Current colour map values (RGB) : ',/)
2000	format (4(3x,i3,':',3(1x,f3.1)))
	end
c*****7****************************************************************
	subroutine show_color
c+
c	Show current colour settings
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
c
	integer		i,j
c
	write (output_io,1000) 
	write (output_io,2000) (colour(iwin,0,j),j=1,3)
	write (output_io,2050) (colour(iwin,6,j),j=1,3)
c
	do i=1,15
	  write (output_io,2100) i,(colour(iwin,i,j),j=1,3)
	enddo
c
1000	format (  ' Current colour settings (RGB) : ',/)
2000	format (  '   Background  :',3(1x,f4.2))
2050	format (  '   Foreground  :',3(1x,f4.2),' -> Pen 6 !'/)
2100	format (  '   Pen ',i2,'      :',3(1x,f4.2))
	end
c*****7****************************************************************
	subroutine show_style
c+
c	Show information about plot styles (color, ..)
c-
	implicit	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'kuplot.inc'
	include 	'errlist.inc'
c
	character*40	cdummy
	integer		i,j,ik
	logical		d2,d3
c
	integer		len_str
	logical		k_in_f
c
	d3 = .false.
	d2 = .false.
	do i=1,iz-1
	  d3 = d3 .or. (lni(i)      .and. k_in_f(i))
	  d2 = d2 .or. (.not.lni(i) .and. k_in_f(i))
	enddo
c
c------ 2D data set parameters
c
	if (d2) then
	  write (output_io,1000) iwin,iframe
	  do ik=1,iz-1
	    if (.not. lni(ik) .and. k_in_f(ik)) then
	      cdummy='off'
	      if (ilegend(iwin,iframe,ik).eq.1) 
     &	          cdummy=clegend(iwin,iframe,ik)
	      write(output_io,1100) 
     &	         ik,ilinecol(iwin,iframe,ik),ilinetyp(iwin,iframe,ik),
     &	         ilineart(iwin,iframe,ik),imarkcol(iwin,iframe,ik),
     &	         imarktyp(iwin,iframe,ik),imarkmax(iwin,iframe,ik),
     &	         ierr(iwin,iframe,ik),ierrcol(iwin,iframe,ik),
     &	         linewid(iwin,iframe,ik),sizemark(iwin,iframe,ik),
     &	         cdummy(1:min(len_str(cdummy),7))
	    endif
	  enddo 
	  write (output_io,*)
c
	  write (output_io,1500) iwin,iframe
	  do ik=1,iz-1
	    if (.not. lni(ik) .and. k_in_f(ik)) then
	      if(fillrange(iwin,iframe,ik,1).eq.-9999) then
	        write(output_io,1610) 
     &	         ik,ifillcol(iwin,iframe,ik),ifilltyp(iwin,iframe,ik)
	      elseif(fillrange(iwin,iframe,ik,3).eq.-9999) then
	        write(output_io,1620) 
     &	         ik,ifillcol(iwin,iframe,ik),ifilltyp(iwin,iframe,ik),
     &	         (fillrange(iwin,iframe,ik,i),i=1,2)
	      else
	        write(output_io,1600) 
     &	         ik,ifillcol(iwin,iframe,ik),ifilltyp(iwin,iframe,ik),
     &	         (fillrange(iwin,iframe,ik,i),i=1,2)
	      endif
	    endif
	  enddo 
	  write (output_io,*)
	endif
c
c------ 3D data set parameters
c
	if (d3) then
	  write (output_io,2000) iwin,iframe
	  do ik=1,iz-1
	    if (lni(ik) .and. k_in_f(ik)) then
	      cdummy='off'
	      if (ilegend(iwin,iframe,ik).eq.1) 
     &	          cdummy=clegend(iwin,iframe,ik)
	      write(output_io,2200) ik,hlineart(iwin,iframe,ik),
     &	                    (hlinecol(iwin,iframe,ik,j),j=1,maxhl),
     &	                    (hlinetyp(iwin,iframe,ik,j),j=1,maxhl),
     &	                    linewid(iwin,iframe,ik),
     &                      cdummy(1:min(len_str(cdummy),10))
	    endif
	  enddo
	  write (output_io,*)
	endif
c
1000	format(' Plot style for 2D data sets (Window ',i3,
     &	   ' / frame ',i3,') : ',//,
     &	   3x,'d.set  lcol  ltyp  lart  mcol  mtyp  ptyp  etyp  ecol',
     &	   '   lwid   msiz   caption',/,3x,77('-'))
1100	format( 3x,i3,1x,8(4x,i2),3x,f5.2,2x,f5.2,3x,a) 
1500	format(' Fill style for 2D data sets (Window ',i3,
     &	   ' / frame ',i3,') : ',//,
     &	   3x,'d.set  fcol  ftyp',10x,'range-x',19x,'range y',/,
     &	   3x,77('-'))
1600	format( 3x,i3,1x,2(4x,i2),3x,4(g12.4,1x))
1610	format( 3x,i3,1x,2(4x,i2),11x,'not set ..')
1620	format( 3x,i3,1x,2(4x,i2),3x,2(g12.4,1x),8x,'not set ..')
2000	format(' Plot style for 3D data sets (Window ',i3,
     &	   ' / frame ',i3,') : ',//,
     &     3x,'d.set',2x,'hart',5x,'hcol',9x,'htyp',8x,'lwid',
     &	   4x,'caption',/,3x,76('-'))
2200	format(3x,i3,5x,i2,4x,4(i1,','),i1,4x,4(i1,','),i1,3x,f5.2,5x,a)
c
	end
