	subroutine errlist
c*****7****************************************************************
c
c	Distributes the error codes into the special error routines.
c	Contains other error handling routines.
c
c	Version : 2.0
c	Date    : 16 Oct 96
c
c	Authors : R.B. Neder  (reinhard.neder@mail.uni-wuerzburg.de)
c	          Th. Proffen (proffen@pa.msu.edu)
c
c*****7****************************************************************
	implicit      none
c
	include      'errlist.inc'
c
	if    (ier_typ.eq.ER_NONE) then
	  call errlist_none
	elseif(ier_typ.eq.ER_COMM) then
	  call errlist_comm
	elseif(ier_typ.eq.ER_FORT) then
	  call errlist_fort
	elseif(ier_typ.eq.ER_IO  ) then
	  call errlist_io
	elseif(ier_typ.eq.ER_MAC ) then
	  call errlist_mac
	elseif(ier_typ.eq.ER_MATH) then
	  call errlist_math
	else
	  call errlist_appl
	endif
c
c------	Terminate program if an error occured and the 
c	error status is set to ER_S_EXIT
c
	if(ier_num.lt.0 .and. ier_sta.eq.ER_S_EXIT) then
	  write(*,1000) char(7)
	  stop
	endif
c
	call no_error
c
1000	format(' ****EXIT**** Program terminated by error status',
     &	'        ****',a1/)
	end
c*****7****************************************************************
	subroutine no_error
c+
c	Resets error variables
c-
	implicit	none
c
	include		'errlist.inc'
c
	ier_num = 0
	ier_typ = ER_NONE
c
	ier_msg(1) = ' '
	ier_msg(2) = ' '
	ier_msg(3) = ' '
c
	end
c*****7****************************************************************
	subroutine disp_error (typ,error,iu,io)
c-
c	Displays the error messages 
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       i,iu,io
	character*41  error(iu:io)
	character*4   typ
c
	if(iu.le.ier_num .and. ier_num.le.io .and. 
     &	   error(ier_num).ne.' ') then
	  write(*      ,1000) typ,error(ier_num),ier_num,char(7)
	  write(ier_out,1500) typ,error(ier_num)
	  do i=1,3
	    if (ier_msg(i)     .ne.' ') 
     &	      write(*,1500) typ,ier_msg(i),ier_num
	  enddo
	else
	  write(*,2000) ier_num,typ,char(7)
	endif
c
1000	format(' ****',a,'**** ',a41,' ****',i4,' ****',a1)
1500	format(' ****',a,'**** ',a41,' ****',i4,' ****')
2000    format(' !!!! No error message for error no.:',I8,' !!!!'/
     &	       '      Error type:    ',a,'            '/
     &	       '      Please document and report to the author',a1)
	end
c*****7****************************************************************
	subroutine errlist_none
c-
c	Displays error Messages for the error type NONE
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (iu=0,io=3)
c
	character*41  error(iu:io)
c
	DATA ERROR/
     &  'No error',                                 !  0  ! command
     &  'Done',                                     !  1  ! command
     &  ' ',                                        !  2  ! command
     &  'Task aborted'                              !  3  ! command
     &	/
c
	call disp_error ('NONE',error,iu,io)
	end
c*****7****************************************************************
	subroutine errlist_comm
c-
c	Displays error Messages for the error type COMMands
c+
	implicit      none
c
	include      'errlist.inc'
	include      'prompt.inc'
c
	integer       iu,io
	parameter    (IU=-17,IO=0)
c
	character*41  error(IU:IO)
c
	DATA ERROR/
     &  'Too many parameters',                      !-17  ! command
     &  ' ',                                        !-16  ! fortran
     &  ' ',                                        !-15  ! fortran
     &  ' ',                                        !-14  ! discus
     &  ' ',                                        !-13  ! macro
     &  ' ',                                        !-12  ! macro
     &  'Error in subroutine',                      !-11  ! command
     &  ' ',                                        !-10  ! discus
     &  ' ',                                        ! -9  ! io
     &  'Unknown command',                          ! -8  ! command
     &  ' ',                                        ! -7  ! discus
     &  'Missing or wrong parameters for command',  ! -6  ! command,fortran
     &  'Error in operating system command',        ! -5  ! command
     &  ' ',                                        ! -4  ! io
     &  ' ',                                        ! -3  ! io
     &  ' ',                                        ! -2  ! io
     &  '       directory not defined ',            ! -1  ! io
     &  ' '                                         !  0  ! command
     &	/
c
	ERROR(-1)(1:6) = pname_cap
c
	call disp_error ('COMM',error,iu,io)
	end
c*****7****************************************************************
	subroutine errlist_fort
c-
c	Displays error Messages for the error type FORTran
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU=-28,IO=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  'Too deeply leveled break command',         !-28  ! fortran
     &  ' ',                                        !-27  ! discus
     &  ' ',                                        !-26  ! discus
     &  ' ',                                        !-25  ! discus
     &  ' ',                                        !-24  ! discus
     &  ' ',                                        !-23  ! discus
     &  ' ',                                        !-22  ! discus
     &  ' ',                                        !-21  ! discus
     &  'Illigal argument for ln(x) function',      !-20  ! fortran
     &  'Illegal nesting of controll commands',     !-19  ! fortran
     &  'Unresolvable condition',                   !-18  ! fortran
     &  'Too many indizes for DO-loop counter',     !-17  ! fortran
     &  'Too deeply leveled (do,if) construction',  !-16  ! fortran
     &  'Too many commands',                        !-15  ! fortran
     &  'Index of DO-loop counter is missing',      !-14  ! fortran
     &  'Wrong number of indizes for array',        !-13  ! fortran
     &  'Expression between () is missing',         !-12  ! fortan
     &  'Number of parentheses is not matching',    !-11  ! fortan
     &  'Index for array element is missing',       !-10  ! fortan
     &  'Number of brackets is not matching',       ! -9  ! fortan
     &  'Index outside array limits',               ! -8  ! fortran
     &  'Argument for asin,acos greater 1',         ! -7  ! fortran
     &  'Missing or wrong Parameters for command',  ! -6  ! command,fortran
     &  'Square root of negative number',           ! -5  ! fortran
     &  'Division by zero',                         ! -4  ! fortran
     &  'Unknown intrinsic function',               ! -3  ! fortran
     &  'Unknown Variable',                         ! -2  ! fortran
     &  'Nonnumerical Parameters in expression',    ! -1  ! fortran
     &  ' '                                         !  0  ! command
     &	/
c
	call disp_error ('FORT',error,iu,io)
	end
c*****7****************************************************************
	subroutine errlist_io
c-
c	Displays error Messages for the error type IO
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU= -13,IO=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &	'Second parameter must be >= first Param.', ! -13 ! io
     &	'Error writing to file',                    ! -12 ! io
     &	'No IO stream open to close',               ! -11 ! io
     &	'IO stream already open',                   ! -10 ! io
     &	'Error reading user input',                 ! -9  ! io
     &  'Nothing learned - no macro written',       ! -8  ! io
     &  'Learning sequence already in progress',    ! -7  ! io
     &  'Unexpected end of file',                   ! -6  ! io
     &  'No such entry in online help',             ! -5  ! io
     &  'File already exists',                      ! -4  ! io
     &  'Error reading file',                       ! -3  ! io
     &  'Error opening file',                       ! -2  ! io
     &  'File does not exist',                      ! -1  ! io
     &  ' '                                         !  0  ! command
     &	/
c
	call disp_error ('I/O ',error,iu,io)
	end
c*****7****************************************************************
	subroutine errlist_mac
c-
c	Displays error Messages for the error type MACros
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU=-41,IO=3)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  'Not enough macro parameters given ',       !-41  ! macro
     &  ' ',                                        !-40  ! discus
     &  ' ',                                        !-39  ! discus
     &  ' ',                                        !-38  ! math
     &  ' ',                                        !-37  ! discus
     &  'Unexpected EOF in macro file',             !-36  ! macro
     &  'Too deeply leveled macros',                !-35  ! macro
     &  ' ',                                        !-34  ! discus
     &  ' ',                                        !-33  ! discus
     &  ' ',                                        !-32  ! discus
     &  ' ',                                        !-31  ! discus
     &  ' ',                                        !-30  ! discus
     &  ' ',                                        !-29  ! discus
     &  ' ',                                        !-28  ! fortran
     &  ' ',                                        !-27  ! discus
     &  ' ',                                        !-26  ! discus
     &  ' ',                                        !-25  ! discus
     &  ' ',                                        !-24  ! discus
     &  ' ',                                        !-23  ! discus
     &  ' ',                                        !-22  ! discus
     &  ' ',                                        !-21  ! discus
     &  ' ',                                        !-20  ! discus
     &  ' ',                                        !-19  ! fortran
     &  ' ',                                        !-18  ! fortran
     &  ' ',                                        !-17  ! command
     &  ' ',                                        !-16  ! fortran
     &  ' ',                                        !-15  ! fortran
     &  ' ',                                        !-14  ! discus
     &  'Macro name missing on command line',       !-13  ! macro
     &  'Macro not found',                          !-12  ! macro
     &  ' ',                                        !-11  ! command
     &  ' ',                                        !-10  ! discus
     &  ' ',                                        ! -9  ! io
     &  ' ',                                        ! -8  ! command
     &  ' ',                                        ! -7  ! discus
     &  ' ',                                        ! -6  ! command,fortran
     &  ' ',                                        ! -5  ! command
     &  ' ',                                        ! -4  ! io
     &  ' ',                                        ! -3  ! io
     &  ' ',                                        ! -2  ! io
     &  'Too many macro parameters given',          ! -1  ! macro
     &  ' ',                                        !  0  ! command
     &  ' ',                                        !  1  ! command
     &  ' ',                                        !  2  ! command
     &  ' '                                         !  3  ! command
     &	/
c
	call disp_error ('MAC ',error,iu,io)
	end
c*****7****************************************************************
	subroutine errlist_math
c-
c	Displays error Messages for the error type MATHematical errors
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU= -1,IO=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  'Singular Matrix',                          !- 1  ! math
     &  ' '                                         !  0  ! command
     &	/
c
	call disp_error ('MATH',error,iu,io)
	end
C*****7****************************************************************
