c*****7*****************************************************************
c
c	This file contains all routines dealing with the
c	least square refinement.
c
c	Version  : 1.0
c	Date     : 16. March 1999
c	Author   : Th. Proffen (tproffen@lanl.gov)
c 
c*****7*****************************************************************
	subroutine do_fit
c+
c	Main fit routine ..
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	write (output_io,1000)
c
c------ Here starts the fitting
c
	call fit_update
	call fit_run
	call fit_errors
c
1000	format (1x,'Starting refinement ...',/,1x,78('-'))
c
	end
c*****7*****************************************************************
	subroutine do_par (zeile,lp)
c-
c	Setting parameter definitions using the command 'par'
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'charact.inc'
	include 	'fit.inc'
	include 	'pdf.inc'
	include 	'debug.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=maxdpp+1)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw),ccpara(maxw)
	character*200	cz,cp,cv,ca
	integer       	lpara(maxw),llpara(maxw)
	integer		id(maxdpp),nid
	integer		jd(maxdpp),njd
	integer		ianz,iianz
	integer       	i,il,ipar,ieq,ikr,ikl,la,lz,ikk,ipp
	real          	dummy,werte(maxw),wwerte(maxw)
c
	logical         str_comp
	integer		bindex,len_str
	real		berechne
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
c------ Some initial checks of user input
c
	if (ianz.lt.1) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ Reset selected ??
c
	if (str_comp(cpara(1),'reset',3,lpara(1),5)) then
	  call fit_reset
	  write (output_io,100) 
	  return
100	  format(' ------ > Parameters reset ...')
	endif
c
c------ Get definition and analyse ...
c
	call rem_bl(cpara(1),lpara(1))
c
	ieq = index(cpara(1),'=')
	if (ieq.eq.0) then
	  ier_num = -107
	  ier_typ = ER_APPL
	  return
	endif
c
	cp  = cpara(1)(1:ieq-1)
	ikl =  index(cp,'[')
	ikr = bindex(cp,']')
	cv  = cp(1:ikl-1)
	ca  = cp(ikl+1:ikr-1)
	la  = ikr-ikl-1
c
c------ Analyse left side of definition first
c
	call get_params (ca,iianz,ccpara,llpara,maxw,la)
	call ber_params (iianz,ccpara,llpara,wwerte,maxw)
	if (ier_num.ne.0) return
c
	if     (cv(1:1).eq.'x'    .and. iianz.eq.1) then
	  ipar = (cr_ipha-1)*n_at*NMAX + 
     &	                    (nint(wwerte(1)-1)*n_at) + 1
	elseif (cv(1:1).eq.'y'    .and. iianz.eq.1) then
	  ipar = (cr_ipha-1)*n_at*NMAX + 
     &	                    (nint(wwerte(1))-1)*n_at + 2
	elseif (cv(1:1).eq.'z'    .and. iianz.eq.1) then
	  ipar = (cr_ipha-1)*n_at*NMAX + 
     &	                    (nint(wwerte(1))-1)*n_at + 3
	elseif (cv(1:1).eq.'u'    .and. iianz.eq.2) then
	  ipar = (cr_ipha-1)*n_at*NMAX + 
     &	                    (nint(wwerte(2))-1)*n_at + 
     &	                     nint(wwerte(1))         + 3 
	elseif (cv(1:1).eq.'o'    .and. iianz.eq.1) then
	  ipar = (cr_ipha-1)*n_at*NMAX + 
     &	                    (nint(wwerte(1))-1)*n_at + 10
c
	elseif (cv(1:3).eq.'lat'  .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + 
     &	         (cr_ipha-1)*n_st + nint(wwerte(1))
	elseif (cv(1:4).eq.'delt' .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + 
     &	         (nint(wwerte(1))-1)*n_st + 7
	elseif (cv(1:4).eq.'csca' .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + 
     &	         (nint(wwerte(1))-1)*n_st + 8
	elseif (cv(1:4).eq.'srat' .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + 
     &	         (nint(wwerte(1))-1)*n_st + 9
c
	elseif (cv(1:4).eq.'dsca' .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + MAXPHA*n_st +
     &	         (nint(wwerte(1))-1)*n_ex + 1
	elseif (cv(1:4).eq.'qsig' .and. iianz.eq.1) then
	  ipar = MAXPHA*n_at*NMAX + MAXPHA*n_st +
     &	         (nint(wwerte(1))-1)*n_ex + 2
c
	else
	  ier_num = -107
	  ier_typ = ER_APPL
	  return
	endif
c
c------ Replace variables in argument
c
	if     (iianz.eq.1) then
	  write(cz,1000) nint(wwerte(1))
	elseif (iianz.eq.2) then
	  write(cz,1010) nint(wwerte(1)),nint(wwerte(2))
	endif
	lz = len_str(cz)
	call rem_bl(cz,lz)
	cpara(1) = cpara(1)(1:ikl)//cz(1:lz)//cpara(1)(ikr:lpara(1))
	lpara(1) = len_str(cpara(1))
c
	if (dbg) then
	  write (output_io,8000) cpara(1)(1:lpara(1))
	endif
c
c------ Now look for p[i] in definition
c
	call par_replace(cpara(1),lpara(1),nid,id)
	if (ier_num.ne.0) return
c
	if (dbg) then
	  write (output_io,8000) cpara(1)(1:lpara(1))
	  write (output_io,8010) nid,(id(i),i=1,nid)
	  do i=1,fit_npar
	    if (fit_jd(i).eq.ipar) ipp = i
	  enddo
	  write (output_io,8020) ipar,ipp
	endif
c
c------ No p[i] found, so this value is fixed
c
	if(nid.eq.0) then
	  call do_math(cpara(1),ieq,lpara(1))
	  fit_const(ipar) = ' '
	  fit_ndpp(ipar)  = 0
c
c------ or we save the definition
c
	else
	  if (ianz.eq.(nid+1)) then
	    ieq = index(cpara(1),'=')
	    cp  = '('//cpara(1)(ieq+1:lpara(1))//')'
	    il  = len_str(cp)
	    dummy = berechne(cp,il)
	    if (ier_num.ne.0) return
	    fit_const(ipar) = cpara(1)(1:lpara(1))
c
	    do i=1,nid
	      call rem_bl(cpara(i+1),lpara(i+1))
	      call par_replace(cpara(i+1),lpara(i+1),njd,jd)
	      if (ier_num.ne.0) return
	      cp = '('//cpara(i+1)(1:lpara(i+1))//')'
	      il = lpara(i+1) + 2
	      dummy = berechne(cp,il)
	      if (ier_num.ne.0) return
	      fit_dconst(ipar,i) = cpara(i+1)(1:lpara(i+1))
	      fit_id(ipar,i)     = id(i)
	    enddo
c
	    fit_ndpp(ipar) = nid
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
1000	format (i8)
1010	format (i8,',',i8)
c
8000	format (' debug  > ',a)
8010	format (' debug  > nid  = ',i5,' npp  = ',50i4)
8020	format (' debug  > ipar = ',i5,' npar = ',i5)
	end
c*****7*****************************************************************
	subroutine par_replace (cpara,lpara,nid,id)
c-
c	Replaces variables in arguments and gets parameter id's
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'debug.inc'
	include 	'fit.inc'
	include 	'errlist.inc'
c
	character*(*)	cpara
	character*200	cc,ca
	integer       	lpara,lc,la
	integer		id(maxdpp),nid
	integer       	i,ieq,ipr,ipl,ikk,il,iv,ia,ioff
	logical		lflag
c
	integer		len_str
	real		berechne
c
	integer		idummy
c
	ia  = 1
	nid = 0
c
10	continue
	  ieq = index(cpara,'=')
	  cc  = cpara(ieq+ia:lpara)
	  lc  = len_str(cc)
	  ipl = index(cc,'[')
c
	  if (ipl.eq.0) goto 20
c
	  ikk = 1
	  il  = ipl+1
	  ipr = 0
	  do while (il.le.lc .and. ipr.eq.0)
	    if (cc(il:il).eq.'[') ikk=ikk+1
	    if (cc(il:il).eq.']') then
	      if (ikk.gt.1) then
	        ikk = ikk-1
	      else
	        ipr = il
	      endif
	    endif
	    il = il+1
	  enddo
c
	  if (ipr.gt.0) then
	    ca = '('//cc(ipl+1:ipr-1)//')'
	    la = len_str(ca)
	    iv = nint(berechne(ca,la))
	    if(ier_num.ne.0) goto 20
c
	    cpara = cpara(1:ieq+ia+ipl-1)//ca(1:la)//
     &	            cpara(ieq+ia+ipr-1:lpara)
	    lpara = len_str(cpara)
	    ioff = (ipr-ipl-1) - la
	    ipr  = ipr - ioff
	    lc   = lc  - ioff
c
c------ --- We have p[i] 
c
	    if (cc(ipl-1:ipl-1).eq.'p') then
	      if (iv.lt.1 .or. iv.gt.fit_npar) then
	        ier_num = -114
	        ier_typ = ER_APPL
	        return
	      endif
c
	      if (nid.eq.0) then
	        nid = nid+1
	        id(nid) = iv
	      else
	        lflag = .false.
	        do i=1,nid
	          lflag = lflag .or. (iv.eq.id(i))
	        enddo
	        if (.not.lflag) then
	          nid = nid+1
	          if (nid.gt.MAXDPP) then
	            ier_num = -108
	            ier_typ = ER_APPL
	            return
	          else
	            id(nid) = iv
	          endif
	        endif
	      endif
	    endif
c
	    ia = ia+ipr
ctep	    if (ia.gt.lc) goto 20
	    if (ia+ieq .gt. lpara) goto 20
c
	  else
	    ier_num = -107
	    ier_typ = ER_APPL
	    goto 20
	  endif
	goto 10
20	continue	  
c
c
	end
c*****7*****************************************************************
	subroutine do_ncycle (zeile,lp)
c-
c	Interpret 'ncycle' command
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'fit.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=3)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz
	real          	werte(maxw)
c
	logical         str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
c
	if     (ianz.eq.0) then
	  write (output_io,1000) fit_ncycle
1000	  format (1x,'------ > Maximum number of iterations : ',I4)
	elseif (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (nint(werte(1)).ge.0) then
	    fit_ncycle = nint(werte(1))
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine do_urf (zeile,lp)
c-
c	Interpret 'urf' command
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'fit.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=3)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz
	real          	werte(maxw)
c
	logical         str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
c
	if     (ianz.eq.0) then
	  write (output_io,1000) fit_urf
1000	  format (1x,'------ > Current URF setting : ',G14.6)
	elseif (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  fit_urf = werte(1)
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine fit_theory (ldiff,lout)
c+
c	Here we calculate PDF and derivatives during LS fit
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'debug.inc'
	include 	'fit.inc'
c
	character*85	cc
	integer		idummy,is,lc,ip,ieq
	logical		ldiff,lout
c
	integer		len_str
c
c------ First we compute all constraint equations and the PDF
c
	do ip=1,fit_npar
	  if (fit_ndpp(fit_jd(ip)).ne.0) then
	    idummy = cr_ipha
	    call fit_set_ipha(ip,cr_ipha)
	    lc  = len_str(fit_const(fit_jd(ip)))
	    cc  = fit_const(fit_jd(ip))(1:lc)
	    ieq = index(cc,'=')
	    call do_math(cc,ieq,lc)
	    cr_ipha = idummy
	  endif
	enddo
c
	if (ldiff) call fit_clear
c
	do ip=1,cr_npha
	  call lattice(ip,.false.)
	enddo
c
	do is=1,pdf_nset
	  call pdf_determine (is,ldiff,lout)
	enddo
c
c------ Now we compute the derivatives when required
c
	if (ldiff) then
	  call fit_setup_a
	  call fit_setup_c
	  call fit_calc_b
c
c------ - Print matrix A,B for data point 213 is debug is active
c
	  if (dbg) then
	    idummy = 213
	    write(output_io,8000) 'A'
	    write(output_io,8010) (ip,fit_a(idummy,1,ip),ip=1,fit_npar)
	    write(output_io,8000) 'B'
	    write(output_io,8010) (ip,fit_b(idummy,1,ip),ip=1,fit_npar)
	  endif
	endif
c
8000	format (/,' debug  > Matrix ',a1,' :')
8010	format (5(i4,':',g11.5))
	end
c*****7*****************************************************************
	subroutine fit_setup_a
c+
c	This routine sets up the matrix A with the derivatives 
c	d(PDF)/dx.
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'wink.inc'
	include 	'fit.inc'
c
	integer		ipar,mpar,ia,ip,is,i,j,k
	integer		na,ne,ncc
	real		r,bk,fac,facs,facp,ddrho
	real		ppp(MAXDAT)
c
c------ Loop over all data sets
c
	do is=1,pdf_nset
      	  do i=pdf_ncmin(is),pdf_ncmax(is)
c
c------ --- Some common variables
c
	    r = (i-1)*pdf_deltar(is) + pdf_rmin(is)
	    if (pdf_sigmaq(is).gt.0.0) then
	      bk = exp(-(r*pdf_sigmaq(is))**2/2.0)
	    else
  	      bk = 1.0
	    endif
c
c------ ----------------------------------------------------------------
c------     Derivatives per atom : x,y,z,u,o
c------ ----------------------------------------------------------------
c
	    do ip=1,cr_npha
	      facp = cr_skal(ip)*pdf_skal(is)*bk
              facs = 1.0 / (cr_np(ip)*r)
	      fac  = facs*facp
c
	      do ia=1,cr_natoms(ip)
c
c ----- ------- d/dx[ip,ia], d/du[j,ip,ia]
c
	        ipar = cr_off(ip-1)+(ia-1)*n_at
	        do j=1,9
	          mpar = fit_jd(ipar+j)
	          if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	            fit_a(i,is,ipar+j) = fac*fit_a(i,is,ipar+j)
	          endif
	        enddo
c
c ----- ------- d/do[ip,ia]
c
	        ipar = ipar + 10
	        mpar = fit_jd(ipar)
	        if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	          fit_a(i,is,ipar) = facp*
     &	                (facs*fit_a(i,is,ipar) - fpi*r/cr_v(ip))
	        endif
	      enddo
	    enddo
c
c------ ----------------------------------------------------------------
c------     Derivatives per phase : lat,delta,csca,srat
c------ ----------------------------------------------------------------
c
	    do ip=1,cr_npha
	      ipar = n_at*cr_total + (ip-1)*n_st
c
	      ncc  = cr_icc(1,ip)*cr_icc(2,ip)*cr_icc(3,ip)
	      facp = cr_skal(ip)*pdf_skal(is)*bk
              facs = 1.0 / (cr_np(ip)*r)
	      fac  = facs*facp
c
c ----- ----- d/d(lat[j] for j=1,2,3)
c
	      do j=1,3
	        mpar = fit_jd(ipar+j)
	        if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	          fit_a(i,is,ipar+j) = facp*(facs*fit_a(i,is,ipar+j) + 
     &	                 fpi*r*pdf_dnorm(ip)*cr_rho0(ip)/cr_a0(j,ip))
	        endif
	      enddo
c
c ----- ----- d/d(lat[4])
c
	      ipar = ipar + 4
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        ddrho = cr_a0(1,ip)**2*cr_a0(2,ip)**2*
     &	                cr_a0(3,ip)**2/cr_v(ip)**2*
     &	                (sina(ip)*cosa(ip) - sina(ip)*cosb(ip)*cosg(ip))
	        fit_a(i,is,ipar) = facp*(facs*fit_a(i,is,ipar) + 
     &	                fpi*r*pdf_dnorm(ip)*cr_rho0(ip)*ddrho)
	      endif
c
c ----- ----- d/d(lat[5])
c
	      ipar = ipar + 1
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        ddrho = cr_a0(1,ip)**2*cr_a0(2,ip)**2*
     &	                cr_a0(3,ip)**2/cr_v(ip)**2*
     &	                (sinb(ip)*cosb(ip) - sinb(ip)*cosa(ip)*cosg(ip))
	        fit_a(i,is,ipar) = facp*(facs*fit_a(i,is,ipar) + 
     &	                fpi*r*pdf_dnorm(ip)*cr_rho0(ip)*ddrho)
	      endif
c
c ----- ----- d/d(lat[6])
c
	      ipar = ipar + 1
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        ddrho = cr_a0(1,ip)**2*cr_a0(2,ip)**2*
     &	                cr_a0(3,ip)**2/cr_v(ip)**2*
     &	                (sing(ip)*cosg(ip) - sing(ip)*cosa(ip)*cosb(ip))
	        fit_a(i,is,ipar) = facp*(facs*fit_a(i,is,ipar) + 
     &	                fpi*r*pdf_dnorm(ip)*cr_rho0(ip)*ddrho)
	      endif
c
c ----- ----- d/d(delta[ip])
c
	      ipar = ipar + 1
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        fit_a(i,is,ipar) = - fac*fit_a(i,is,ipar)
	      endif
c
c ----- ----- d/d(csca[ip])
c
	      ipar = ipar + 1
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        fit_a(i,is,ipar) = pdf_calc(i,is,ip) * pdf_skal(is)
	      endif
c
c ----- ----- d/d(srat[ip])
c
	      ipar = ipar + 1
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	        fit_a(i,is,ipar) = fac*fit_a(i,is,ipar)
	      endif
	    enddo
c
c------ ----------------------------------------------------------------
c------     Derivatives per dataset : dsca,qsig
c------ ----------------------------------------------------------------
c
c ----- --- d/d(dsca[is])
c
	    ipar = n_at*cr_total + cr_npha*n_st + (is-1)*n_ex+1
	    mpar = fit_jd(ipar)
	    if (fit_ndpp(mpar).ne.0) then
	      fit_a(i,is,ipar) = pdf_tot(i,is) / pdf_skal(is)
	    endif
c
c ----- --- d/d(qsig[is])
c
	    ipar = ipar + 1
	    mpar = fit_jd(ipar)
	    if (fit_ndpp(mpar).ne.0 .and. pdf_psel(ip,is)) then
	      fit_a(i,is,ipar) = -r**2 * pdf_sigmaq(is) * pdf_tot(i,is) 
	    endif
	  enddo
	enddo
c
c------ Finally we need to convolute the derivatives with
c------ the SINC function
c
	do is=1,pdf_nset
	  if (pdf_qmax(is).gt.0.0) then
	    call pdf_setup_sinc(is,.false.)
	    do ip=1,fit_npar
	      mpar = fit_jd(ip)
	      if (fit_ndpp(mpar).ne.0) then
 	        do i=pdf_nfmin(is),pdf_nfmax(is)
                  ppp(i) = fit_a(i,is,ip)*(pdf_qmax(is)-pdf_sinc(2*i))
                  do k=pdf_nfmin(is),i-1
                    ppp(i) = ppp(i) + fit_a(k,is,ip)*
     &                               (pdf_sinc(i-k)-pdf_sinc(i+k))
                  enddo
                  do k=i+1,pdf_nfmax(is)
                    ppp(i) = ppp(i) + fit_a(k,is,ip)*
     &                               (pdf_sinc(k-i)-pdf_sinc(k+i))
                  enddo
                enddo
 	        do i=pdf_nfmin(is),pdf_nfmax(is)
  	          fit_a(i,is,ip) = ppp(i)*pdf_deltar(is)/pi
	        enddo
	      endif
	    enddo
	  endif
	enddo
c
	end
c*****7*****************************************************************
	subroutine fit_calc_b
c+
c	This routine calculates the needed derivatives for the
c	constrained model using B=AC.
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	integer		i,j,is,ip,na,ne,ppar,ipar,mpar
c
	do is=1,pdf_nset
      	  do ip = pdf_nfmin(is),pdf_nfmax(is)
	    do ppar=1,fit_npar
	      fit_b(ip,is,ppar) = 0.0
	      if (fit_lref(ppar)) then
	        do ipar=1,fit_npar
	          mpar = fit_jd(ipar)
	          if (fit_ndpp(mpar).ne.0) then
	            do j=1,fit_ndpp(mpar)
	              if (fit_id(mpar,j).eq.ppar) then
	                fit_b(ip,is,ppar) = fit_b(ip,is,ppar) +
     &	    	               fit_a(ip,is,ipar)*fit_c(ipar,j)
	              endif
	            enddo
	          endif
	        enddo
	      endif
	    enddo
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine fit_setup_c
c+
c	This routine sets up the constraint matrix
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	character*85	ca
	integer		la,ipar,mpar,idpp,idummy
	real		value
c
	integer		len_str
	real		berechne
c
	do ipar=1,fit_npar
	  mpar = fit_jd(ipar)
	  if (fit_ndpp(mpar).ne.0) then
	    idummy = cr_ipha
	    call fit_set_ipha(ipar,cr_ipha)
	    do idpp=1,fit_ndpp(mpar)
	      ca    = fit_dconst(mpar,idpp)
	      la    = len_str(ca)
	      ca    = '('//ca(1:la)//')'
	      la    = la+2
	      value = berechne(ca,la)
	      fit_c(ipar,idpp) = value
	    enddo
	    cr_ipha = idummy
	  endif
	enddo
c
	end
c*****7*****************************************************************
	subroutine fit_errors
c+
c	This routine calculates errors dx from dp ...
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	integer		ipar,ppar,mpar,ia,ip,is,i,j
	real		sum
c
	call fit_setup_c
c
c------ Loop over all data sets
c
	do is=1,pdf_nset
c
c------ ----------------------------------------------------------------
c------   Variables per atom : x,y,z,u,o
c------ ----------------------------------------------------------------
c
	  do ip=1,cr_npha
	    do ia=1,cr_natoms(ip)
c
c ----- ----- x[n],y[n],z[n]
c
	      ipar = cr_off(ip-1)+(ia-1)*n_at
	      do i=1,3
	        mpar = fit_jd(ipar+i)
	        if (fit_ndpp(mpar).ne.0) then
	          sum = 0.0
	          do j=1,fit_ndpp(mpar)
	            sum = sum + 
     &	                 (fit_c(ipar+i,j)*fit_dpar(fit_id(mpar,j)))**2
	          enddo
	          cr_dpos(i,ia,ip) = sqrt(sum)
	        else
	          cr_dpos(i,ia,ip) = 0.0
	        endif
	      enddo
c
c ----- ----- u[6,n]
c
	      ipar = ipar + 3
	      do i=1,6
	        mpar = fit_jd(ipar+i)
	        if (fit_ndpp(mpar).ne.0) then
	          sum = 0.0
	          do j=1,fit_ndpp(mpar)
	            sum = sum + 
     &	                 (fit_c(ipar+i,j)*fit_dpar(fit_id(mpar,j)))**2
	          enddo
	          cr_du(i,ia,ip) = sqrt(sum)
	        else
	          cr_du(i,ia,ip) = 0.0
	        endif
	      enddo
c
c ----- ----- o[n]
c
	      ipar = ipar + 7
	      mpar = fit_jd(ipar)
	      if (fit_ndpp(mpar).ne.0) then
	        sum = 0.0
	        do j=1,fit_ndpp(mpar)
	          sum = sum + 
     &	               (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	        enddo
	        cr_docc(ia,ip) = sqrt(sum)
	      else
	        cr_docc(ia,ip) = 0.0
	      endif
	    enddo
	  enddo
c
c------ ----------------------------------------------------------------
c------   Variables per phase : lat,delta
c------ ----------------------------------------------------------------
c
	  do ip=1,cr_npha
c
c ----- --- lat[j] for j=1,2,3
c
	    ipar = n_at*cr_total + (ip-1)*n_st
	    do i=1,3
	      mpar = fit_jd(ipar+i)
	      if (fit_ndpp(mpar).ne.0) then
	        sum = 0.0
	        do j=1,fit_ndpp(mpar)
	          sum = sum + 
     &	               (fit_c(ipar+i,j)*fit_dpar(fit_id(mpar,j)))**2
	        enddo
	        cr_da0(i,ip) = sqrt(sum)
	      else
	        cr_da0(i,ip) = 0.0
	      endif
	    enddo
c
c ----- --- lat[j] for j=4,5,6
c
	    ipar = ipar + 3
	    do i=1,3
	      mpar = fit_jd(ipar+i)
	      if (fit_ndpp(mpar).ne.0) then
	        sum = 0.0
	        do j=1,fit_ndpp(mpar)
	          sum = sum + 
     &	               (fit_c(ipar+i,j)*fit_dpar(fit_id(mpar,j)))**2
	        enddo
	        cr_dwin(i,ip) = sqrt(sum)
	      else
	        cr_dwin(i,ip) = 0.0
	      endif
	    enddo
c
c ----- --- delta[ip]
c
	    ipar = ipar + 4
	    mpar = fit_jd(ipar)
	    if (fit_ndpp(mpar).ne.0) then
	      sum = 0.0
	      do j=1,fit_ndpp(mpar)
	        sum = sum + 
     &	             (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	      enddo
	      pdf_ddelta(ip) = sqrt(sum)
	    else
	      pdf_ddelta(ip) = 0.0
	    endif
c
c ----- --- csca[ip]
c
	    ipar = ipar + 1
	    mpar = fit_jd(ipar)
	    if (fit_ndpp(mpar).ne.0) then
	      sum = 0.0
	      do j=1,fit_ndpp(mpar)
	        sum = sum + 
     &	             (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	      enddo
	      cr_dskal(ip) = sqrt(sum)
	    else
	      cr_dskal(ip) = 0.0
	    endif
c
c ----- --- srat[ip]
c
	    ipar = ipar + 1
	    mpar = fit_jd(ipar)
	    if (fit_ndpp(mpar).ne.0) then
	      sum = 0.0
	      do j=1,fit_ndpp(mpar)
	        sum = sum + 
     &	             (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	      enddo
	      pdf_dsrat(ip) = sqrt(sum)
	    else
	      pdf_dsrat(ip) = 0.0
	    endif
	  enddo
c
c------ ----------------------------------------------------------------
c------   Variables per dataset : dsca,qsig
c------ ----------------------------------------------------------------
c
c ----- - dsca
c
	  ipar = n_at*cr_total + cr_npha*n_st + (is-1)*n_ex+1
	  mpar = fit_jd(ipar)
	  if (fit_ndpp(mpar).ne.0) then
	    sum = 0.0
	    do j=1,fit_ndpp(mpar)
	      sum = sum + 
     &	           (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	    enddo
	    pdf_dskal(is) = sqrt(sum)
	  else
	    pdf_dskal(is) = 0.0
	  endif
c
c ----- - qsig
c
	  ipar = ipar + 1
	  mpar = fit_jd(ipar)
	  if (fit_ndpp(mpar).ne.0) then
	    sum = 0.0
	    do j=1,fit_ndpp(mpar)
	      sum = sum + 
     &	           (fit_c(ipar,j)*fit_dpar(fit_id(mpar,j)))**2
	    enddo
	    pdf_dsigmaq(is) = sqrt(sum)
	  else
	    pdf_dsigmaq(is) = 0.0
	  endif
	enddo
c
c------ Recalculate errors for metric tensor ..
c
	do ip=1,cr_npha
	  call lattice(ip,.false.)
	enddo
c
	end
c*****7*****************************************************************
	subroutine fit_reset
c+
c	Resets fit settings ..
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	integer		ip
c
	do ip=1,MAXP
	  fit_ndpp(ip) = 0
	  fit_lref(ip) = .true.
	enddo
c
	call fit_update
c
	end
c*****7*****************************************************************
	subroutine fit_set_ipha(ipar,ipha)
c+
c	This routine determines the phase number from the parameter
c	number.
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
	include 	'debug.inc'
c
	integer		ipar,ipha,ist,iat,ip
c
	iat  = n_at*cr_total
	ist  = iat + cr_npha*n_st
c
c------ Atom related parameter
c
	if (ipar.le.iat) then
	  do ip=1,cr_npha
	    if (ipar.gt.cr_off(ip-1) .and. ipar.le.cr_off(ip)) then
	      ipha = ip
	    endif
	  enddo
c
c------ Structural parameter
c
	elseif (ipar.le.ist) then
	  ipha = (ipar-iat)/(n_st) + 1
	endif
c
	if (dbg) then
	  write (output_io,8000) ipar,ipha
8000	  format (' debug > ipar = ',i5,' ipha = ',i5)
	endif
c
	end
c*****7*****************************************************************
	subroutine fit_clear
c+
c	This routine clears the LS matrices A,B and C ...
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
c
	integer		ip,ia,is,ii
c
	do is=1,pdf_nset
	  do ii=1,pdf_bin(is)
	    do ip=1,fit_npar
	      fit_a(ii,is,ip) = 0.0
	      fit_b(ii,is,ip) = 0.0
	    enddo
	  enddo
	enddo
c
	do ip=1,fit_npar
	  do ii=1,MAXDPP
	    fit_c(ip,ii) = 0.0
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine fit_update
c+
c	This routine calculates the actual maximum number of
c	total and fitting parameters.
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'debug.inc'
	include 	'fit.inc'
c
	integer		ip,ia,is,ii
c
	fit_npar = pdf_nset*n_ex + cr_npha*n_st+cr_total*n_at
c
c------ Computing array fit_jd
c
	do ip=1,cr_npha
	  do ia=1,cr_natoms(ip)
	    do ii=1,n_at
	      fit_jd(cr_off(ip-1)             +(ia-1)*n_at + ii) =
     &	             (ip-1)*n_at*NMAX         +(ia-1)*n_at + ii
	    enddo
	  enddo
	enddo
c
	do ip=1,cr_npha
	  do ii=1,n_st
	    fit_jd(cr_total*n_at         + (ip-1)*n_st + ii) =
     &	           MAXPHA*n_at*NMAX      + (ip-1)*n_st + ii
	  enddo
	enddo
c
	do is=1,pdf_nset
	  do ii=1,n_ex
	    fit_jd(cr_total*n_at + cr_npha*n_st + (is-1)*n_ex + ii) =
     &	          MAXPHA*n_at*NMAX + MAXPHA*n_st + (is-1)*n_ex + ii
	  enddo
	enddo
c
c------ Number of fitting parameters and constraint equations
c
	fit_nfpar  = 0
	do ip=1,fit_npar
	  fit_nfpar = fit_nfpar + fit_ndpp(fit_jd(ip))
	enddo
c
c------ Print out some debugging information if needed
c
	if (dbg) then
	  write (output_io,1000) fit_npar,fit_nfpar
	  write (output_io,1100) (cr_off(ip),ip=0,cr_npha)
	  write (output_io,1200) 
	  write (output_io,1300) (fit_jd(ip),ip=1,fit_npar)
c
1000	  format (' debug > fit_npar = ',i6,' fit_nfpar = ',i6)
1100	  format (' debug > cr_off(0..cr_npha)= ',10i6)
1200	  format (' debug > fit_jd : ')
1300	  format (' debug >  ',12i5)
	endif
c
	end
c*****7*******************************************************************
	subroutine fit_run
c+
c	This is the main LS routine (modified from FITTE) 
c-
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'fit.inc'
	include 	'errlist.inc'
c
	real		eps
	parameter	(eps = 1e-6)
c
	integer		idummy (MAXP)
	logical		ldiff,lpr
c
	n  = fit_npar
      	j  = 0
c
10    	continue
	zwert= 0.0
      	do k = 1,n
          fit_dz(k) = 0.0
          do l = k,n
            fit_cl(l,k) = 0.0
	  enddo
	enddo
	ldiff = .true.
        call fit_theory (ldiff,.false.)
	if (ier_num.ne.0) return
	do is = 1,pdf_nset
      	  do i = pdf_nfmin(is),pdf_nfmax(is)
            s = pdf_obs(i,is) - pdf_tot(i,is)
            zwert= zwert+ pdf_wic(i,is) * s*s
            do k = 1,n
	      if(abs(fit_b(i,is,k)).lt.eps) fit_b(i,is,k)=0.0
              ttt = pdf_wic(i,is) * fit_b(i,is,k)
              fit_dz(k) = fit_dz(k) - s * ttt
              do l = k,n
                fit_cl(l,k) = fit_cl(l,k) + ttt * fit_b(i,is,l)
	      enddo
	    enddo
	  enddo
	enddo
c
      	f = 1.0
      	if (fit_urf) 60,60,40
40    	continue
	h = 0.0
      	do k = 1,n
     	  if (fit_dz(k).ne.0.0) h=h+(fit_dz(k)*fit_dz(k))/
     &	                            (zwert*fit_cl(k,k))
	enddo
      	f = 1.000001 + fit_urf * h
60    	continue
c
	kk = 0
	do k=1,n
	  lpr = .false.
	  do iii=1,n
	    do jjj=1,fit_ndpp(fit_jd(iii))
	      lpr = lpr .or. (fit_id(fit_jd(iii),jjj).eq.k)
	    enddo
	  enddo
	  if(lpr .and. fit_lref(k)) then
	    kk = kk+1
	    idummy(kk) = k
	  endif
	enddo
c
	sum1 = 0.0
	sum2 = 0.0
	sum3 = 0.0
	sum4 = 0.0
c
	do is = 1,pdf_nset
      	  do i = pdf_nfmin(is),pdf_nfmax(is)
            di=pdf_obs(i,is) - pdf_tot(i,is)
            sum1  = sum1 + di*di
            sum2  = sum2 + pdf_obs(i,is) * pdf_obs(i,is)
            sum3  = sum3 + pdf_wic(i,is) * di*di
            sum4  = sum4 + pdf_wic(i,is) * pdf_obs(i,is)*pdf_obs(i,is)
	  enddo
	enddo
      	fit_r   = sqrt(sum1/sum2)
      	fit_rw  = sqrt(sum3/sum4)
c
	if (j.eq.0) then
  	  write (output_io,5010) j,zwert,f,fit_r,fit_rw
	else
  	  write (output_io,5015) j,zwert,f,fit_r,fit_rw,(fit_rw-fit_rwo)
	endif
c
	write (output_io,5020) (idummy(k),fit_par(idummy(k)),k=1,kk)
  	write (output_io,5030)
	fit_rwo  = fit_rw
	fit_lcyc = j
c
      	zalt=zl
      	zdif=zwert-zl
      	fend=f
      	if (j)    80,80,70
70    	continue
	if (abs(zdif).lt.eps) goto 300
	if (zdif) 80,300,300
80    	continue
	zl=zwert    
      	do k = 1,n
          fit_pl(k) = fit_par(k)
	enddo
      	do k =1,n
          do l = 1,k
            l1=l-1
            s=fit_cl(k,l)
            if (l-k) 100,140,100
100         if (l1)  130,130,110
110         do i = 1,l1
              s=s - fit_cl(i,l)*fit_cl(i,k)
	    enddo
130         fit_cl(l,k) = s
	  enddo
140       if (s)  145,142,145
142   	  fit_dl(k)=0.0
      	  goto 190
145       s = s*f
          if (l1)   170,170,150
150       do i=1,l1
            ttt = fit_cl(i,k)
            fit_cl(i,k) = ttt*fit_dl(i)
            s = s - ttt*fit_cl(i,k)  
	  enddo
170       if (s)    190,190,180
180       fit_dl(k)=1.0/s
190       continue 
	enddo
      	if (j-fit_ncycle)  200,300,300
200   	j=j+1
      	if (n-1)  230,230,210
210   	do l=2,n
          l1=l-1
          do k=1,l1
            fit_dz(l)=fit_dz(l)-fit_cl(k,l)*fit_dz(k)
	  enddo
	enddo
230   	fit_dz(n)=fit_dz(n)*fit_dl(n)
      	if (n-1)   260,260,240
240   	do nl=2,n
          l=n-nl+1
          l1=l+1
          fit_dz(l)=fit_dz(l)*fit_dl(l)
          do k=l1,n
            fit_dz(l)=fit_dz(l) - fit_cl(l,k)*fit_dz(k)
	  enddo
	enddo
260   	do k=1,n
          fit_par(k)=fit_par(k)-fit_dz(k)
	enddo
c
      	goto 10
c
300   	g=1.0
	nnn=0.0
	do is=1,pdf_nset
	  nnn = nnn + (pdf_nfmax(is)-pdf_nfmin(is))
	enddo
      	h=float((nnn+1)-n)
      	if (h*zl.gt.0.0)  g=zl/h
      	do k=1,n
          fit_dl(k)=fit_dl(k)*g
          fit_cl(k,k)=1.0 
	enddo
      	if (n-1)   350,350,320
320   	do l=2,n
          l1=l-1
          do k=1,l1
            s=0.0
            do i=k,l1
              s=s-fit_cl(i,l)*fit_cl(i,k)
	    enddo
            fit_cl(l,k)=s 
    	  enddo
	enddo
350   	do k=1,n
      	  do l=k,n
            s=0.0
            if (l-k)   380,360,380
360         do i=l,n
              ttt=fit_cl(i,k)
              fit_cl(i,k)=ttt*fit_dl(i)
              s=s+ttt*fit_cl(i,k)
	    enddo
            goto 400
380         do i=l,n
              s=s+fit_cl(i,l)*fit_cl(i,k) 
	    enddo
400	    continue
            fit_cl(l,k)=s
	  enddo
	enddo
      	do l=1,n
          if (fit_dl(l)) 502,501,502
501    	  fit_dpar(l)=1.0
          fit_dpar(l)=0.0      
          goto 570
502       f=sqrt(fit_cl(l,l))
          fit_dpar(l)=f
          do k= 1,l
            fit_dz(k)=fit_cl(l,k)/f
	  enddo
          do k=l,n
            fit_dz(k)=fit_cl(k,l)/f 
	  enddo
          do k=1,l
            if(fit_dpar(k).eq.0.0) then
              fit_cl(l,k)=fit_dz(k)
            else
              fit_cl(l,k)=fit_dz(k)/fit_dpar(k)
            endif      
	  enddo
570       continue   
	enddo  
c
      	do k=1,n
          fit_par(k)= fit_pl(k)
	enddo
	sum1 = 0.0
	sum2 = 0.0
      	sum3 = 0.0
      	sum4 = 0.0
      	nf=n
      	do i=1,n
     	  if(fit_b(1,1,i).eq.0.) nf=nf-1 
	enddo
	nnn=0.0
	do is=1,pdf_nset
	  nnn = nnn + (pdf_nfmax(is)-pdf_nfmin(is))
	enddo
	ldiff = .false.
        call fit_theory (ldiff,.false.)
	if (ier_num.ne.0) return
	do is = 1,pdf_nset
      	  do i = pdf_nfmin(is),pdf_nfmax(is)
            h=pdf_obs(i,is) - pdf_tot(i,is)
            sum1  = sum1 + h*h
            sum2  = sum2 + pdf_obs(i,is)*pdf_obs(i,is)
            sum3  = sum3 + pdf_wic(i,is) * h*h
            sum4  = sum4 + pdf_wic(i,is) * pdf_obs(i,is)*pdf_obs(i,is)
	  enddo
	enddo
      	fit_r  = sqrt(sum1/sum2)
      	fit_rw = sqrt(sum3/sum4)
      	fit_re = sqrt(((nnn+1)-nf)/sum4)
c
5010	format (1x,'Iteration  :',i4,2x,'Sum: ',g12.6,1x,
     &	                                'Urf: ',g12.6,/,
     &	        1x,18x,'R  : ',g12.6,1x,'Rw : ',g12.6,/,
     &	        1x,'Parameters :')
5015	format (1x,'Iteration  :',i4,2x,'Sum: ',g12.6,1x,
     &	                                'Urf: ',g12.6,/
     &	        1x,18x,'R  : ',g12.6,1x,'Rw : ',g12.6,1x,
     &	                                'DRw: ',g12.6,/
     &	        1x,'Parameters :')
5020	format (4(i4,': ',g12.6))
5030	format (1x,78('-'))
c
      	end
