c*****7*****************************************************************
c	Here are the routines needed for getting the scattering
c	lengths as in DISCUS.
c
c	Version : 1.0
c	Date :    26. November 1998
c	Author :  Thomas Proffen & Reinhard Neder
c*****7*****************************************************************
	subroutine do_scat (zeile,lp)
c+
c	Setting user defined scattering lengths
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'pdf.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=12)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw),ianz
	real		werte(maxw)
c
	integer		iat(MAXSCAT)
	integer		iianz,i,j,is
	logical		str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	iianz = 1
c
c------ Check for parameters
c
	if (ianz.gt.2) then
	  call ber_params(iianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  is = nint(werte(1))
	  if (is.lt.1 .or. is.gt.pdf_nset) then
	    ier_num = -106
	    ier_typ = ER_APPL
	    return
	  endif
c
c------ - Now get scattering lengths
c
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call get_iscat (iianz,cpara,lpara,werte,maxw,.false.)
	  if (ier_num.ne.0) return
c
	  if (werte(1).lt.0) then
	    do i=1,cr_nscat(cr_ipha)
	      iat(i)=i
	    enddo
	    iianz = cr_nscat(cr_ipha)
	  else
	    do i=1,iianz
	      iat(i)=nint(werte(i))
	    enddo
	  endif
c
c------ - User goes back to internal scattering lengths
c
	  if (str_comp(cpara(2),'internal',3,lpara(2),8)) then
	    do i=1,iianz
	      cr_scat_int(iat(i),is,cr_ipha) = .true.
	    enddo
c
c------ - User specifies new values
c
	  else
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if (ier_num.ne.0) return
c	  
	    if     (ianz.eq.1 .and. .not.pdf_lxray(is)) then
	      do i=1,iianz
	        cr_scat(1,iat(i),is,cr_ipha)   = werte(1)
	        cr_scat_int(iat(i),is,cr_ipha) = .false.
	      enddo
	    elseif (ianz.eq.9 .and.      pdf_lxray(is)) then
	      do i=1,iianz
	        cr_scat(1,iat(i),is,cr_ipha) = werte(9)
	        do j=2,9
	          cr_scat(j,iat(i),is,cr_ipha) = werte(j-1)
	        enddo
	        cr_scat_int(iat(i),is,cr_ipha) = .false.
	      enddo
	    else
	      ier_num = -6
	      ier_typ = ER_APPL
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine dlink(is)
c-
c     	This routine dlinks between the PDFFIT package and the routines
c     	that generate the atomic form factors. These routines have been
c     	adapted from the program LAZY.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'errlist.inc'
c
	common /anomal/ delfr,delfi
	common /neu/    fneu
	common /scat/   fa,fb,fc
c
	character*4   	line
	integer       	element,symwl,kodlp,i,j,is
	logical       	lxray
	real          	fa(4,8),fb(4,8),fc(8)
	real          	delfr,delfi,fneu
	real          	wave
c
	ier_num = 0
	ier_typ = ER_NONE
c
	write(line,100) '    '
	read (line,100) symwl
	if(.not. pdf_lxray(is)) then
	  kodlp=2
	else
	  kodlp=1
	endif
	if(cr_nscat(cr_ipha).gt.0) then
	  do i=1,cr_nscat(cr_ipha)
	    if(cr_scat_int(i,is,cr_ipha)) then
	      if(.not. pdf_lxray(is)) then
	        write(line ,100) cr_at_lis(i,cr_ipha)
	        line(3:4)='  '
	        read (line,100) element
	        call anoma(symwl,element,kodlp)
	        cr_scat(1,i,is,cr_ipha)=fneu
	        do j=2,9
	          cr_scat(j,i,is,cr_ipha)=0.0
	        enddo
	      else
	        write(line,100) cr_at_lis(i,cr_ipha)
	        read (line,100) element
	        call fsc(element,1)
	        cr_scat(1,i,is,cr_ipha)=fc(1)
	        do j=1,4
	          cr_scat(2*j  ,i,is,cr_ipha)=fa(j,1)
	          cr_scat(2*j+1,i,is,cr_ipha)=fb(j,1)
	        enddo
	      endif
c
	      if(element.eq.0) then
	        ier_num = -20
	        ier_typ = ER_APPL
	        goto 999
	      endif
	    endif
	  enddo
	else
	  ier_num = -21
	  ier_typ = ER_APPL
	endif
c
999	continue
c
100   	format(a4)
101   	format(a2)
      	end
c*****7*****************************************************************
	real function quad(h,k,rten)
c+
c     	Calculates the scalar product of h and k.
c     	1/d**2 = h(i)*k(j)*rten(i,j)
c-
	implicit	 none
c
	integer          idim
	parameter(idim=3)
c
	integer          i,j
	real             h(idim),k(idim),rten(idim,idim)
c
	quad=0.0
	do i=1,idim 
	  do j=1,idim 
	    quad=quad + h(i) * k(j) * rten(i,j)
	  enddo
	enddo
	end 
c*****7*****************************************************************
	real function form (ll,h2,is,ip)
c+
c 	calculates the form factor     
c-
	implicit	 none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'pdf.inc'
c
	integer	         ll,i,is,ip
	real             h2
c
	form = cr_scat(1,ll,is,ip)
	if (pdf_lxray(is)) then
	  do i=1,4
	    form = form + cr_scat(2*i,ll,is,ip)*
     &	                      exp(-cr_scat(2*i+1,ll,is,ip)*h2)
	  enddo
	endif
c
	end 
