c*****7*****************************************************************
	real function do_blen(lspace,u,v)
c-
c	Calculates the length of the vector v-u in the space defined by
c	lspace (.true. = real space , .false. = reciprocal space)
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
c
	logical        lspace
	real           u(3),v(3)
c
	integer        i
	real           w(3)
	real           skalpro
c
	do i=1,3
	  w(i) = v(i) - u(i)
	enddo
c
	if(lspace) then
	  do_blen = sqrt( skalpro(w,w,cr_gten,cr_ipha))
	else
	  do_blen = sqrt( skalpro(w,w,cr_rten,cr_ipha))
	endif
c
	end
c*****7*****************************************************************
	real function do_bang(lspace,u,v,w)
c-
c	Calculates the angle between vectors v-u and v-w in the space 
c	defined by lspace (.true. = real space , .false. = reciprocal space)
c+
	implicit       none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'errlist.inc'
	include       'wink.inc'
c
	logical        lspace
	real           u(3),v(3),w(3)
c
	integer        i
	real           xx,xy,yy,arg
	real           x(3),y(3)
	real           skalpro
c
	do i=1,3
	  x(i) = v(i) - u(i)
	  y(i) = v(i) - w(i)
	enddo
c
	if(lspace) then
	  xx = sqrt( skalpro(x,x,cr_gten,cr_ipha))
	  xy =       skalpro(x,y,cr_gten,cr_ipha)
	  yy = sqrt( skalpro(y,y,cr_gten,cr_ipha))
	else
	  xx = sqrt( skalpro(x,x,cr_rten,cr_ipha))
	  xy =       skalpro(x,y,cr_rten,cr_ipha)
	  yy = sqrt( skalpro(y,y,cr_rten,cr_ipha))
	endif
	if(xx.gt.0 .and. yy.gt.0) then
	  arg = xy/(xx*yy)
	  if (abs(arg).le.1.0) then
	    do_bang = acos(arg)/rad
	  else
	    do_bang = 0.0
	  endif
	else
	  ier_num = -32
	  ier_typ = ER_APPL
	  do_bang = 0.0
	endif
c
	end
c*****7*********************************************************
        real function skalpro(h,k,rten,ipha)
c-
c     	Calulates the SCALARPRODUCT of two vectors
c     	1/D**2 = H(I)*K(J)*RTEN(I,J)
c-
	implicit       none
c
	include        'config.inc'
c
	integer        idim,ipha
        parameter     (idim=3)
        real           h(idim),k(idim),rten(idim,idim,MAXPHA)
c
	integer      i,j
c
        skalpro=0.0
        do i=1,idim
          do j=1,idim
            skalpro=skalpro + h(i) * k(j) * rten(i,j,ipha)
	  enddo
	enddo
c
        end
c*****7*****************************************************************
	real function dskalpro(h,k,rten,dh,dk,drten,ipha)
c-
c	Calculates the error of scalar product
c-
	implicit       none
c
	include        'config.inc'
c
	integer		idim,ipha
	parameter	(idim=3)
	real  		h (idim), k(idim), rten(idim,idim,MAXPHA)
	real 		dh(idim),dk(idim),drten(idim,idim,MAXPHA)
c
	integer		i,j
c
	dskalpro=0.0
	do i=1,idim
	  do j=1,idim
	    dskalpro=dskalpro + abs(dh(i) *  k(j) *  rten(i,j,ipha))
     &	                      + abs( h(i) * dk(j) *  rten(i,j,ipha))
     &	                      + abs( h(i) *  k(j) * drten(i,j,ipha))
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine vekprod(u,v,ww,eps,rten,ipha)
c-
c	calculates the VECTORPRODUCT in general triclinic space
c	with  EPS and RTEN in direct space
c	with REPS and GTEN in reciprocal space
c+
	implicit       none
c
	include        'config.inc'
c
	integer        i,j,k,l,ipha
	real           u(3),v(3),ww(3)
	real           eps(3,3,3,MAXPHA),rten(3,3,MAXPHA)
c
	do i=1,3
	  ww(i)=0.0
	  do j=1,3
	    do k=1,3
	      do l=1,3
	        ww(i)=ww(i) + eps(j,k,l,ipha)*u(k)*v(l)*rten(j,i,ipha)
	      enddo
	    enddo
	  enddo
	enddo
c
	end
C*****7*****************************************************************
	subroutine matmulx(a,b,c)
c+
c	Matrixmultiplication c=a*b
c-
	implicit      none
c
	integer       i,j,k
	real          a(3,3),b(3,3),c(3,3)
c
	do i=1,3
	  do j=1,3
	    a(i,j)=0.0
	    do k=1,3
	      a(i,j)=a(i,j) + b(i,k)*c(k,j)
	    enddo
	  enddo
	enddo
	end
c*****7*****************************************************************
	subroutine transmat(mat,idim)
c-
c	Replaces a matrix by its transpose
c+
	implicit      none
c
	integer       idim,i,j
	real          mat(idim,idim),d
c
	do i=2,idim
	  do j=1,i-1
	    d        = mat(i,j)
	    mat(i,j) = mat(j,i)
	    mat(j,i) = d
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine invmat(imat,a)
c
c	calculates the inverse matrix "imat" to input matrix "a"
c 
	implicit      none
c
	include      'errlist.inc'
c
	real imat(3,3),a(3,3),det
c
	det= a(1,1)*(a(2,2)*a(3,3) - a(2,3)*a(3,2)) +
     &       a(2,1)*(a(3,2)*a(1,3) - a(1,2)*a(3,3)) +
     &       a(3,1)*(a(1,2)*a(2,3) - a(1,3)*a(2,2))

	if(abs(det).gt.0.0) then
	  ier_num = 0
	  ier_typ = ER_NONE
c
	  imat(1,1)= (a(2,2)*a(3,3) - a(2,3)*a(3,2) )/det
	  imat(1,2)=-(a(1,2)*a(3,3) - a(1,3)*a(3,2) )/det
	  imat(1,3)= (a(1,2)*a(2,3) - a(1,3)*a(2,2) )/det
c
	  imat(2,1)=-(a(2,1)*a(3,3) - a(3,1)*a(2,3) )/det
	  imat(2,2)= (a(1,1)*a(3,3) - a(1,3)*a(3,1) )/det
	  imat(2,3)=-(a(1,1)*a(2,3) - a(1,3)*a(2,1) )/det
c
	  imat(3,1)= (a(2,1)*a(3,2) - a(3,1)*a(2,2) )/det
	  imat(3,2)=-(a(1,1)*a(3,2) - a(1,2)*a(3,1) )/det
	  imat(3,3)= (a(1,1)*a(2,2) - a(1,2)*a(2,1) )/det
c
	else
	  ier_num = -1
	  ier_typ = ER_MATH
	endif
c
	end      
C*****7*****************************************************************
	subroutine invmat4(matrix)
c-
c	inverts a 4*4 Symmetry operation
c+
	implicit       none
c
	include        'errlist.inc'
c
	real           matrix(4,4)
	integer        i,j
	real           a(3,3),b(3,3),t(3)
c
	do i=1,3
	  do j=1,3
	    a(i,j) = matrix(i,j)
	  enddo
	  t(i) = matrix(i,4)
	enddo
c
	call invmat(b,a)
c
	if(ier_num.eq.0) then
	  do i=1,3
	    matrix(i,4) = 0.0
	    do j=1,3
	      matrix(i,j) = b(i,j)
	      matrix(i,4) = matrix(i,4) - b(i,j)*t(j)
	    enddo
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine lattice(ipha,lout)
c+
c     	Calculates lattice constants, metric and reciprocal metric
c     	tensor, permutation tensors and unit cell volume.
c	It's done quite some old fashioned way, rather than calculating
c	the direct metric tensor and its inverse.
c
c	This extended version calculates standard deviations as well
c-
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
	include 	'wink.inc'
c
	character*20	cc,cstr(9)
	integer     	ipha,i,i1,i2,j,k,ncc,lstr(9)
	logical     	lout
	real		cos1,cos2,sin1,sin2
	real		dcos1,dcos2,dsin1,dsin2
	real 		cosi,dcosi,sind,cosd,acosd
	real		abscosa,abscosb,abscosg,arg,darg
	real		dcosa,dcosb,dcosg,voll,dvoll
c
	cosa(ipha)=cosd(cr_win(1,ipha))
	cosb(ipha)=cosd(cr_win(2,ipha))
	cosg(ipha)=cosd(cr_win(3,ipha))                        
c
	sina(ipha)=sind(cr_win(1,ipha))
	sinb(ipha)=sind(cr_win(2,ipha))
	sing(ipha)=sind(cr_win(3,ipha))                        
c
	abscosa=abs(cosd(cr_win(1,ipha)))
	abscosb=abs(cosd(cr_win(2,ipha)))
	abscosg=abs(cosd(cr_win(3,ipha)))
c
	dcosa=abs(sind(cr_win(1,ipha))*rad*cr_dwin(1,ipha))
	dcosb=abs(sind(cr_win(2,ipha))*rad*cr_dwin(2,ipha))
	dcosg=abs(sind(cr_win(3,ipha))*rad*cr_dwin(3,ipha))
c
	voll = 1    - cosa(ipha)*cosa(ipha) - 
     &	              cosb(ipha)*cosb(ipha) - 
     &	              cosg(ipha)*cosg(ipha)
	voll = voll + 2*cosa(ipha)*cosb(ipha)*cosg(ipha)
c
	if(voll.gt.0.0) then
	  cr_v(ipha)=sqrt(voll)*
     &	         cr_a0(1,ipha)*cr_a0(2,ipha)*cr_a0(3,ipha)
c
	  dvoll=2.*abscosa*dcosa + 2.*abscosb*dcosb + 2.*abscosg*dcosg +
     &	     2.*(  dcosa*abscosb*abscosg + abscosa*dcosb*abscosg +
     &	         abscosa*abscosb*  dcosg)
	  cr_dv(ipha)=0.5/sqrt(voll)*dvoll*
     &	     cr_a0(1,ipha)*cr_a0(2,ipha)*cr_a0(3,ipha)+
     &	     sqrt(voll)*cr_da0(1,ipha)*cr_a0 (2,ipha)*cr_a0 (3,ipha)+
     &	     sqrt(voll)*cr_a0 (1,ipha)*cr_da0(2,ipha)*cr_a0 (3,ipha)+
     &	     sqrt(voll)*cr_a0 (1,ipha)*cr_a0 (2,ipha)*cr_da0(3,ipha)
c
	  cr_vr(ipha) =1./cr_v(ipha)
	  cr_dvr(ipha)=1./(cr_v(ipha)*cr_v(ipha))*cr_dv(ipha)
c
c------ - calculate direct metric tensor
c
	  call  tensor(cr_gten,cr_a0,cr_win,ipha)
	  call dtensor(cr_a0,cr_win,cr_dgten,cr_da0,cr_dwin,ipha)
c
c------ - calculate reciprocal lattice constants
c
	  do i=1,3
	    i1=mod(i  ,3) + 1
	    i2=mod(i+1,3) + 1
	    cr_ar(i,ipha)=cr_a0(i1,ipha)*cr_a0(i2,ipha)*
     &	                     sind(cr_win(i,ipha))/cr_v(ipha)
	    cos1=cosd(cr_win(i1,ipha))
	    cos2=cosd(cr_win(i2,ipha))
	    cosi=cosd(cr_win(i ,ipha))
	    sin1=sind(cr_win(i1,ipha))
	    sin2=sind(cr_win(i2,ipha))
	    cr_wrez(i,ipha)=acosd((cos1*cos2-cosi)/(sin1*sin2))
c
	    cr_dar(i,ipha)=
     &	      cr_da0(i1,ipha)*cr_a0 (i2,ipha)*
     &	      sind(cr_win(i,ipha))/cr_v(ipha)+
     &        cr_a0 (i1,ipha)*cr_da0(i2,ipha)*
     &	      sind(cr_win(i,ipha))/cr_v(ipha) +
     &        cr_a0 (i1,ipha)*cr_a0 (i2,ipha)*
     &	      cosd(cr_win(i,ipha))*rad*cr_dwin(i,ipha)/cr_v(ipha) +
     &        cr_a0 (i1,ipha)*cr_a0 (i2,ipha)*
     &	      sind(cr_win(i,ipha))/(cr_v(ipha)*cr_v(ipha))*cr_dv(ipha)
            dcos1=sind(cr_win(i1,ipha))*rad*cr_dwin(i1,ipha)
            dcos2=sind(cr_win(i2,ipha))*rad*cr_dwin(i2,ipha)
            dcosi=sind(cr_win(i ,ipha))*rad*cr_dwin(i ,ipha)
            dsin1=cosd(cr_win(i1,ipha))*rad*cr_dwin(i1,ipha)
            dsin2=cosd(cr_win(i2,ipha))*rad*cr_dwin(i2,ipha)
            arg=(cos1*cos2-cosi)/(sin1*sin2)
            darg=dcos1* cos2/(sin1*sin2) +
     &            cos1*dcos2/(sin1*sin2) +
     &                 dcosi/(sin1*sin2) +
     &           arg/sin1*dsin1 + arg/sin2*dsin2
            cr_dwrez(i,ipha)=1./sqrt(1-arg*arg)*darg/rad
	  enddo
c
c------ - calculate reciprocal tensor
c
	  call  tensor(cr_rten,cr_ar,cr_wrez,ipha)
	  call dtensor(cr_ar,cr_wrez,cr_drten,cr_dar,cr_dwrez,ipha)
c
c------ - calculate premutation tensors
c
	  do i=1,3
	    do j=1,3
	      do k=1,3
	         cr_eps(i,j,k,ipha)   = 0.0
	         cr_reps(i,j,k,ipha)  = 0.0
	         cr_deps(i,j,k,ipha)  = 0.0
	         cr_dreps(i,j,k,ipha) = 0.0
	      enddo
	    enddo
	  enddo
c
	  cr_eps(1,2,3,ipha)   =  cr_v(ipha)
	  cr_eps(2,3,1,ipha)   =  cr_v(ipha)
	  cr_eps(3,1,2,ipha)   =  cr_v(ipha)
	  cr_eps(1,3,2,ipha)   = -cr_v(ipha)
	  cr_eps(3,2,1,ipha)   = -cr_v(ipha)
	  cr_eps(2,1,3,ipha)   = -cr_v(ipha)
	  cr_reps(1,2,3,ipha)  =  cr_vr(ipha)
	  cr_reps(2,3,1,ipha)  =  cr_vr(ipha)
	  cr_reps(3,1,2,ipha)  =  cr_vr(ipha)
	  cr_reps(1,3,2,ipha)  = -cr_vr(ipha)
	  cr_reps(3,2,1,ipha)  = -cr_vr(ipha)
	  cr_reps(2,1,3,ipha)  = -cr_vr(ipha)
c
	  cr_deps(1,2,3,ipha)  =  cr_dv(ipha)
	  cr_deps(2,3,1,ipha)  =  cr_dv(ipha)
	  cr_deps(3,1,2,ipha)  =  cr_dv(ipha)
	  cr_deps(1,3,2,ipha)  = -cr_dv(ipha)
	  cr_deps(3,2,1,ipha)  = -cr_dv(ipha)
	  cr_deps(2,1,3,ipha)  = -cr_dv(ipha)
	  cr_dreps(1,2,3,ipha) =  cr_dvr(ipha)
	  cr_dreps(2,3,1,ipha) =  cr_dvr(ipha)
	  cr_dreps(3,1,2,ipha) =  cr_dvr(ipha)
	  cr_dreps(1,3,2,ipha) = -cr_dvr(ipha)
	  cr_dreps(3,2,1,ipha) = -cr_dvr(ipha)
	  cr_dreps(2,1,3,ipha) = -cr_dvr(ipha)
c
c------ - Calculate number density
c
	  cr_np (ipha) = 0.0
	  cr_dnp(ipha) = 0.0
	  do i=1,cr_natoms(ipha)
	    cr_np (ipha) = cr_np (ipha) + cr_occ (i,ipha)
	    cr_dnp(ipha) = cr_dnp(ipha) + cr_docc(i,ipha)
	  enddo
	  ncc = cr_icc(1,ipha)*cr_icc(2,ipha)*cr_icc(3,ipha)
	  cr_rho0(ipha)  = cr_np(ipha)/(ncc*cr_v(ipha))
	  cr_drho0(ipha) = (cr_dnp(ipha)/cr_v(ipha))**2 +
     &	                   (cr_dv(ipha)*cr_np(ipha)/cr_v(ipha)**2)**2
	  cr_drho0(ipha) = sqrt(cr_drho0(ipha))
c
c------ - output ?
c
	  if (lout) then
	    write(output_io,2001) ipha
	    write(output_io,2002) cr_name(ipha)
c
	    cstr(1) = cc(cr_a0 (1,ipha),cr_da0 (1,ipha),lstr(1))
	    cstr(2) = cc(cr_a0 (2,ipha),cr_da0 (2,ipha),lstr(2))
	    cstr(3) = cc(cr_a0 (3,ipha),cr_da0 (3,ipha),lstr(3))
	    cstr(4) = cc(cr_win(1,ipha),cr_dwin(1,ipha),lstr(4))
	    cstr(5) = cc(cr_win(2,ipha),cr_dwin(2,ipha),lstr(5))
	    cstr(6) = cc(cr_win(3,ipha),cr_dwin(3,ipha),lstr(6))
	    cstr(7) = cc(cr_v  (  ipha),cr_dv  (  ipha),lstr(7))
	    write(output_io,2003) (cstr(i)(1:lstr(i)),i=1,7)
c
	    cstr(1) = cc(cr_rho0(ipha),cr_drho0(ipha),lstr(1))
	    write(output_io,2004) cstr(1)(1:lstr(1))
c
	    do j=1,3
	      do i=1,3
	        k=(j-1)*3+i
	        cstr(k)=cc(cr_gten (i,j,ipha),
     &	                   cr_dgten(i,j,ipha),lstr(k))
	      enddo
	    enddo
	    write(output_io,2005) (cstr(i)(1:lstr(i)),i=1,9)
c
	    cstr(1) = cc(cr_ar  (1,ipha),cr_dar  (1,ipha),lstr(1))
	    cstr(2) = cc(cr_ar  (2,ipha),cr_dar  (2,ipha),lstr(2))
	    cstr(3) = cc(cr_ar  (3,ipha),cr_dar  (3,ipha),lstr(3))
	    cstr(4) = cc(cr_wrez(1,ipha),cr_dwrez(1,ipha),lstr(4))
	    cstr(5) = cc(cr_wrez(2,ipha),cr_dwrez(2,ipha),lstr(5))
	    cstr(6) = cc(cr_wrez(3,ipha),cr_dwrez(3,ipha),lstr(6))
	    cstr(7) = cc(cr_vr  (  ipha),cr_dvr  (  ipha),lstr(7))
	    write(output_io,2006) (cstr(i)(1:lstr(i)),i=1,7)
c
	    do j=1,3
	      do i=1,3
	        k=(j-1)*3+i
	        cstr(k)=cc(cr_rten (i,j,ipha),
     &	                   cr_drten(i,j,ipha),lstr(k))
	      enddo
	    enddo
	    write(output_io,2007) (cstr(i)(1:lstr(i)),i=1,9)
	  endif
c
	  cr_la = .false.
c
	else
	  ier_num = -35
	  ier_typ = ER_APPL
	endif
c
2001	format( ' Phase number           : ',I4)
2002	format( ' Phase title            : ',A50)
2003 	format(/' Lattice parameters     : ',3A15,/
     &	        '           & angles     : ',3A15,/
     &	        ' Unit cell volume       : ',A15)
2004	format( ' Number density         : ',A15)
2005 	format( ' Metric tensor          : ',3A15,2(/,26X,3A15),/)
2006 	format( ' Recip. lat. parameters : ',3A15,/
     &	        '               & angles : ',3A15,/
     &	        ' Recip. unit cell vol.  : ',A15)
2007 	format( ' Recip. metric tensor   : ',3A15,2(/,26X,3A15))
	end
c*****7*****************************************************************
	subroutine tensor(ten,vec,win,ipha)
c+
c	Calculates the metric tensor. Works both for direct and 
c	reciprocal metric tensor.
c-
	implicit      	none
c
	include		'config.inc'
c
	integer      	idim,ipha
	parameter	(idim=3)
c
	real 		ten(idim,idim,MAXPHA)
	real		vec(idim,MAXPHA),win(idim,MAXPHA)
	real   		cosd
	integer 	i,j
c
	do i=1,idim 
	  do j=1,idim 
	    if(i .ne. j) then
	      ten(i,j,ipha)=vec(i,ipha)*vec(j,ipha)*
     &	                    cosd(win(6-(i+j),ipha))
	      if (abs(ten(i,j,ipha)).lt.1e-6) ten(i,j,ipha) = 0.0
	    else
	      ten(i,j,ipha)=vec(i,ipha)*vec(j,ipha)
	    endif
	  enddo
	enddo
	end 
C*****7*****************************************************************
	subroutine dtensor(vec,win,dten,dvec,dwin,ipha)
c+
c	Calculates the standard devations for the metric tensor. 
c	Works both for direct and reciprocal metric tensor.
c-
	implicit      	none
c
	include		'config.inc'
	include		'wink.inc'
c
	integer      	idim,ipha
	parameter	(idim=3)
c
	real		vec(idim,MAXPHA),win(idim,MAXPHA)
	real 		dten(idim,idim,MAXPHA)
	real		dvec(idim,MAXPHA),dwin(idim,MAXPHA)
	real   		sind,cosd
	integer 	i,j
c
	do i=1,idim 
	  do j=1,idim 
	    if(i .ne. j) then
              dten(i,j,ipha)=dvec(i,ipha)* 
     &	           vec(j,ipha)*cosd(win(6-(i+j),ipha)) +
     &             vec(i,ipha)*dvec(j,ipha)*cosd(win(6-(i+j),ipha))+
     &             vec(i,ipha)* vec(j,ipha)*sind(win(6-(i+j),ipha))*
     &                                      rad*dwin(6-(i+j),ipha)
	      if (dten(i,j,ipha).lt.1e-6) dten(i,j,ipha) = 0.0
	    else
	      dten(i,j,ipha)=dvec(i,ipha)*vec(j,ipha)+
     &	                      vec(i,ipha)*dvec(j,ipha)
	    endif
	  enddo
	enddo
	end 
C*****7*****************************************************************
	subroutine trans(uc,gmat,ipha,up,idim)
c+
c	Transforms a point in the crystal space into plot space 
c	and vice versa
c-
	implicit	none
c
	include		'config.inc'
c
	integer	 i,j,idim,ipha
	real gmat(idim,idim,MAXPHA),uc(idim),up(idim)
c
	do i=1,idim
	  up(i)=0.0
	  do j=1,idim
	    up(i)=up(i)+gmat(i,j,ipha)*uc(j)
	  enddo
	enddo
c
	end
C*****7*****************************************************************
