c*****7*****************************************************************
c	This file contains routines to display information.
c
c	Version: 1.0
c	Date:    01 June 1999
c	Author:  Th. Proffen (tproffen@lanl.gov)
c*****7*****************************************************************
	subroutine do_show (zeile,lp)
c-
c	Main menu for show command.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=3)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz
	real          	werte(maxw)
c
	logical         str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
c
	if (ianz.eq.0) then
	  ier_num = -6
	  ier_typ = ER_COMM
	else
c
c-----    'atom' : show atom info
c
	  if(str_comp(cpara(1),'atom',2,lpara(1),4)) then
	    call do_show_atom (ianz,cpara,lpara,werte,maxw)
c
c-----    'config' : show various limits ...
c
	  elseif(str_comp(cpara(1),'config',4,lpara(1),6)) then
	    call do_show_config
c
c-----    'const' : show constraints
c
	  elseif(str_comp(cpara(1),'const',4,lpara(1),5)) then
	    call do_show_const (ianz,cpara,lpara,werte,maxw)
c
c-----    'elem' : show crystal info
c
	  elseif(str_comp(cpara(1),'elem',2,lpara(1),4)) then
	    call do_show_elem (ianz,cpara,lpara,werte,maxw)
c
c-----    'fit' : show fit info
c
	  elseif(str_comp(cpara(1),'fit',2,lpara(1),3)) then
	    call output_fit(6)
c
c-----    'metric' : show current metric ...
c
	  elseif(str_comp(cpara(1),'metric',2,lpara(1),6)) then
	    call lattice (cr_ipha,.true.)
c
c-----    'par' : show parameter info
c
	  elseif(str_comp(cpara(1),'par',2,lpara(1),3)) then
	    call output_par(6)
c
c-----    'pdf' : show PDF calculation parameters 
c
	  elseif(str_comp(cpara(1),'pdf',2,lpara(1),3)) then
	    call do_show_pdf (ianz,cpara,lpara,werte,maxw)
c
c-----    'phase' : show information for current phase
c
	  elseif(str_comp(cpara(1),'phase',2,lpara(1),5)) then
	    call output_phase(6,cr_ipha)
c
c-----    'scat' : show scattering lengths
c
	  elseif(str_comp(cpara(1),'scat',2,lpara(1),4)) then
	    call do_show_scat (ianz,cpara,lpara,werte,maxw)
c
c-----    Unknown command
c
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
c**********************************************************************
	subroutine do_show_config
c-
c	Shows information about PDFFIT configuration
c+
	implicit       none
c
	include       'config.inc'
	include       'prompt.inc'
	include       'crystal.inc'
	include       'pdf.inc'
	include       'errlist.inc'
c
	write (output_io,1000)
	write (output_io,2000) MAXPHA
	write (output_io,2010) NMAX
	write (output_io,2020) MAXSCAT
	write (output_io,2030) MAXDSET
	write (output_io,2040) MAXDAT 
	write (output_io,2050) MAXP
	write (output_io,2060) MAXDPP
	write (output_io,*)
c
1000	format (' Current PDFFIT configuration :',/)
2000	format ('   Maximum number of phases                 : ',i8)
2010	format ('   Maximum number of atoms per phase        : ',i8)
2020	format ('   Maximum number of atom types per phase   : ',i8)
2030	format ('   Maximum number of data sets              : ',i8)
2040	format ('   Maximum number of data points per file   : ',i8)
2050	format ('   Maximum number of parameters             : ',i8)
2060	format ('   Maximum number of par. in one constraint : ',i8)
c
	end
c**********************************************************************
	subroutine do_show_atom(ianz,cpara,lpara,werte,maxw)
c-
c	Shows information about an atom.
c+
	implicit       none
c
	include       'config.inc'
	include       'prompt.inc'
	include       'crystal.inc'
	include       'errlist.inc'
c
	integer        maxw
c
	character*200  cpara(maxw)
	integer        ianz
	integer        lpara(maxw)
	real           werte(maxw)
c
	character*20   cc,cc_o(6)
	character*4    at_name_d
	integer        lc(6),la
	integer        i,j,istart,iend
c
	integer        len_str
	logical        str_comp
c
	call del_params (1,ianz,cpara,lpara,maxw)
	if(str_comp(cpara(1),'all',1,lpara(1),3)) then
	  werte(1) = 1
	  werte(2) = cr_natoms(cr_ipha)
	  ianz     = 2
	elseif(str_comp(cpara(1),'last',1,lpara(1),4)) then
	  werte(1) = cr_natoms(cr_ipha)
	  werte(2) = cr_natoms(cr_ipha)
	  ianz     = 2
	else
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	endif
	if(ier_num.ne.0) return
c
	if(ianz.eq.1 .or. ianz.eq.2) then
	  istart = nint(werte(1))
	  if(ianz.eq.1) then
	    iend = istart
	  else
	    iend = nint(werte(2))
	  endif
	  iend = min(iend,cr_natoms(cr_ipha))
	  write(output_io,3000) cr_ipha
	  do i=istart,iend
	    at_name_d = cr_at_lis(cr_iscat(i,cr_ipha),cr_ipha)
	    la=len_str(at_name_d)
	    do j=1,3
	      cc_o(j) = cc(cr_pos (j,i,cr_ipha),
     &	                   cr_dpos(j,i,cr_ipha),lc(j))
	    enddo
	    cc_o(4) = cc(cr_occ(i,cr_ipha),cr_docc(i,cr_ipha),lc(4))
	    write (output_io,3010) 
     &	          at_name_d(1:la),(cc_o(j)(1:lc(j)),j=1,4)
	    do j=1,3
	      cc_o(j) = cc(cr_u (j,i,cr_ipha),
     &	                   cr_du(j,i,cr_ipha),lc(j))
	    enddo
	    write (output_io,3020) (cc_o(j)(1:lc(j)),j=1,3)
	    if (cr_u(4,i,cr_ipha).ne.0.0 .or. 
     &	        cr_u(5,i,cr_ipha).ne.0.0 .or.
     &	        cr_u(6,i,cr_ipha).ne.0.0     ) then
	      do j=4,6
	        cc_o(j) = cc(cr_u (j,i,cr_ipha),
     &	                     cr_du(j,i,cr_ipha),lc(j))
	      enddo
	      write (output_io,3030) (cc_o(j)(1:lc(j)),j=4,6)
	    endif
	  enddo
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
3000	format (' Atom parameters (phase ',i3,') :')
3010	format (3x,a4,' Pos:',3(a12,1x),4x,'Occ:',a10)
3020	format (  7x,' uii:',3(a12,1x))
3030	format (  7x,' uij:',3(a12,1x))
	end
c*****7*****************************************************************
	subroutine do_show_pdf (ianz,cpara,lpara,werte,maxw)
c+
c	Show information about PDF calculation settings
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'pdf.inc'
	include		'errlist.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	character*9	at_lis(MAXSCAT)
	integer		lpara(maxw),ianz
	real		werte(maxw)
c
	integer		i,j,k,ip,iset,jset
c
c------ Check for optional parameter
c
	iset = 1
	jset = pdf_nset
c
	if (ianz.gt.1) then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  if (nint(werte(1)).ge.1       .and. 
     &	      nint(werte(1)).le.pdf_nset     ) then
	    iset = nint(werte(1))
	    jset = nint(werte(1))
	  else
	    ier_num = -106
	    ier_typ = ER_APPL
	    return
	  endif
	endif
c
c------ Loop over all data sets
c
	do ip=iset,jset
	  call output_pdf(output_io,ip)
	enddo
c
	end
c*****7*****************************************************************
	subroutine do_show_const (ianz,cpara,lpara,werte,maxw)
c+
c	Show information about constraints
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'fit.inc'
	include		'errlist.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	integer		lpara(maxw),ianz
	real		werte(maxw)
c
	integer		ipar
c
c------ Check for optional parameter
c
	ipar = -1
c
	if (ianz.eq.2) then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call ber_params (ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  if (nint(werte(1)).ge.1       .and. 
     &	      nint(werte(1)).le.fit_npar     ) then
	    ipar = nint(werte(1))
	  else
	    ier_num = -110
	    ier_typ = ER_APPL
	    return
	  endif
	endif
c
	call output_const(output_io,ipar)
c
	end
c*****7*****************************************************************
	subroutine do_show_elem (ianz,cpara,lpara,werte,maxw)
c+
c	Show information about elements/rel. amounts within crystal
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'errlist.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	integer		lpara(maxw),ianz
	real		werte(maxw)
c
	character*4	at_name_i
	real		proz
	real		ratom(0:MAXSCAT)
	integer		natom(0:MAXSCAT)
	integer		i,ip,ipha,jpha
	logical		lout
c
c------ Check for optional parameter
c
	ipha = 1
	jpha = cr_npha
c
	if (ianz.gt.1) then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  if (nint(werte(1)).ge.1       .and. 
     &	      nint(werte(1)).le.cr_npha      ) then
	    ipha = nint(werte(1))
	    jpha = nint(werte(1))
	  else
	    ier_num = -101
	    ier_typ = ER_APPL
	    return
	  endif
	endif
c
c------ Loop over all phases
c
	do ip=ipha,jpha
	  do i=0,cr_nscat(ip)
	    natom(i)=0
	    ratom(i)=0.0
	  enddo
c
c------ - get size of model crystal, rel. amount of elements
c
	  do i=1,cr_natoms(ip)
	    ratom(cr_iscat(i,ip))=ratom(cr_iscat(i,ip))+cr_occ(i,ip)
	    natom(cr_iscat(i,ip))=natom(cr_iscat(i,ip))+1
	  enddo
c
c------   write output
c
	  write (output_io,1000) ip
	  write (output_io,1050) (cr_icc(i,ip),i=1,3)
	  write (output_io,1100) cr_natoms(ip),cr_ncatoms(ip),cr_nscat(ip)
	  do i=0,cr_nscat(ip)
	    proz=ratom(i)/float(cr_natoms(ip))
	    at_name_i = cr_at_lis(i,ip)
	    write (output_io,1200) at_name_i,proz,natom(i)
	  enddo
	  write (output_io,*)
	enddo
c
1000	format (' Structure phase number           : ',I6)
1050	format (' Size of the crystal (unit cells) : ',2(I4,' x '),I4)
1100	format (' Total number of atoms            : ',I6,/
     &	        ' Number of atoms per unit cell    : ',I6,/
     &	        ' Number of different atoms        : ',I6,/)
1200	format ('    Element : ',A4,' rel. abundance : ',F5.3,
     &	        '  (',I6,' atom sites)')
	end
c*****7*****************************************************************
	subroutine do_show_scat(ianz,cpara,lpara,werte,maxw)
c-
c	Shows the scattering curve of an atom type.
c+
	implicit       none
c
	include       'config.inc'
	include       'prompt.inc'
	include       'crystal.inc'
	include       'pdf.inc'
	include       'errlist.inc'
c
	integer        maxw
c
	character*200  cpara(maxw)
	integer        ianz
	integer        lpara(maxw)
	real           werte(maxw)
c
	character*9    at_name_d
	integer        isx(MAXDSET),isn(MAXDSET)
	integer        i,j,is,nx,nn
	logical        latom(0:MAXSCAT)
	logical	       str_comp
c
	do i=0,cr_nscat(cr_ipha)
	  latom(i) = .false.
	enddo
c
c------ Check what data sest are neutron or x-ray
c
	nx = 0
	nn = 0
c
	do i=1,pdf_nset
	  if (pdf_lxray(i)) then
	    nx     = nx+1
	    isx(i) = i
	  else
	    nn     = nn+1
	    isn(i) = i
	  endif
	enddo
c
c------ Get atom names
c
	if (ianz.gt.1) then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call get_iscat (ianz,cpara,lpara,werte,maxw,.false.)
	  if(ier_num.ne.0) return
	else
	  werte(1) = -1
	endif
c
	if(werte(1).eq.-1) then
	  do i=0,cr_nscat(cr_ipha)
	    latom(i) = .true.
	  enddo
	else
	  do i=1,ianz
	    latom(nint(werte(i))) = .true.
	  enddo
	endif
c
c------ Now write output for neutron and/or xray data sets
c
	write (output_io,3000) cr_ipha
c
	if (nx.gt.0) then
	  is = isx(1)
	  call dlink (is)
	  write (output_io,3010) 'X-ray',(isx(i),i=1,nx)
	  do i=1,cr_nscat(cr_ipha)
	    if(latom(i)) then
	      at_name_d = cr_at_lis(i,cr_ipha)
	      write(output_io,3020) at_name_d,
     &	                            (cr_scat(2*j  ,i,is,cr_ipha),j=1,4),
     &	                            (cr_scat(2*j+1,i,is,cr_ipha),j=1,4),
     &	                                        cr_scat(1,i,is,cr_ipha)
	    endif
	  enddo
	endif
c
	if (nn.gt.0) then
	  is = isn(1)
	  call dlink (is)
	  write (output_io,3010) 'Neutrons',(isn(i),i=1,nn)
	  do i=1,cr_nscat(cr_ipha)
	    if(latom(i)) then
	      at_name_d = cr_at_lis(i,cr_ipha)
	      write(output_io,3025) at_name_d,cr_scat(1,i,is,cr_ipha)
	    endif
	  enddo
	endif
c
3000	format (' Scattering lengths (phase : ',i3,') : ')
3010	format ('   Scattering technique  : ',a,/
     &	        '   PDF data sets         : ',50i3)
3020	format ('   Atom ',a4,'      : a(i) : ',4(f12.7,1x)/
     &	        '                    b(i) : ',4(f12.7,1x)/
     &	        '                       c : ',f12.7)
3025	format ('   Atom ',a4,' :         c : ',f12.7)
c
	end
c*****7*****************************************************************
	subroutine output_pdf (id,iset)
c+
c	Show information about PDF settings is data set 'iset'
c	and writes output to unit 'id'
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'pdf.inc'
	include		'errlist.inc'
c
	character*20	cc,cc_o
	character*4	at_lis(MAXSCAT)
	integer		pha_lis(MAXPHA)
	integer		i,j,k,lc,ll,id,iset
	integer		len_str
c
	call pdf_setup(cr_ipha,iset,.false.)
c
	ll = len_str(pdf_name(iset))
	write (id,1000) iset,pdf_name(iset)(1:ll)
	write (id,2000) pdf_rmin(iset),pdf_rmax(iset),pdf_deltar(iset)
	write (id,2010) pdf_rcmin(iset),pdf_rcmax(iset)
	write (id,2020) pdf_rfmin(iset),pdf_rfmax(iset),
     &	                pdf_nfmin(iset),pdf_nfmax(iset)
	if (pdf_lref(iset)) 
     &	   write(id,2030) pdf_rname(iset)(1:len_str(pdf_rname(iset)))
c
	write(id,2100)
	if (pdf_lxray(iset)) then
	  write (id,2110) 'X-Rays',pdf_xq
	else
	  write (id,2115) 'Neutrons'
	endif
	if (pdf_qmax(iset).eq.0.0) then
	  write (id,2120)
	else
	  write (id,2125) pdf_qmax(iset)
	endif
	if (pdf_sigmaq(iset).eq.0.0) then
	  write (id,2230)
	else
	  cc_o = cc(pdf_sigmaq(iset),pdf_dsigmaq(iset),lc)
	  write (id,2235) cc_o(1:lc)
	endif
	cc_o = cc(pdf_skal(iset),pdf_dskal(iset),lc)
	write(id,2240) cc_o(1:lc)
c
	write(id,3000)
	j=0
	do i=1,cr_nscat(cr_ipha)
	  if (pdf_allowed_i(i,iset)) then
	    j=j+1
	    at_lis(j)=cr_at_lis(i,cr_ipha)
	  endif
	enddo
	write (id,3010) (at_lis(k),k=1,j)
c
	j=0
	do i=1,cr_nscat(cr_ipha)
	  if (pdf_allowed_j(i,iset)) then
	    j=j+1
	    at_lis(j)=cr_at_lis(i,cr_ipha)
	  endif
	enddo
	write (id,3020) (at_lis(k),k=1,j)
c
	j=0
	do i=1,cr_npha
	  if (pdf_psel(i,iset)) then
	    j=j+1
	    pha_lis(j)=i
	  endif
	enddo
	write(id,4000) (pha_lis(k),k=1,j)
c
1000	format (1x,78('-'),/,' DATA SET : ',i3,' (',a,')',/,1x,78('-'))
2000	format (1x,'Date range in r [A]   : ',f8.4,' -> ',f8.4,
     &	        6x,'Step dr  : ',f8.4)
2010	format (1x,'Calculated range      : ',f8.4,' -> ',f8.4)
2020	format (1x,'Refinement r range    : ',f8.4,' -> ',f8.4,
     &	        6x,'Data pts : ',i5,' -> ',i5)
2030	format (1x,'Reference PDF file    : ',a)
2100	format (/,1x,'Experimental settings : ')
2110	format (3x,'Radiation           : ',a8,3x,'(at',f8.4,' A**-1)')
2115	format (3x,'Radiation           : ',a8)
2120	format (3x,'Termination at Qmax : not applied')
2125	format (3x,'Termination at Qmax : ',f15.4,' A**-1')
2230	format (3x,'Q resolution sigQ   : not applied')
2235	format (3x,'Q resolution sigQ   : ',a15,' A**-1')
2240	format (3x,'Scale factor        : ',a15)
3000	format (/,1x,'Selected atoms for PDF calculation : ')
3010    format (  3x,'Atoms (i)  : ',100(A4,1X))
3020    format (  3x,'Atoms (j)  : ',100(A4,1X))
4000	format (/,1x,'Selected phases for this data set : ',100(I3,1X))
c
	end
c*****7*****************************************************************
	subroutine output_par (id)
c-
c	Outputs parameter information on unit 'id'
c+
	implicit	none
c
	include		'config.inc'
	include		'fit.inc'
	include		'errlist.inc'
c
	character*20	cc,cc_o(4)
	integer		ipar(MAXP),lc(4)
	integer		i,j,k,ll,ie,id,npar
	logical		lnc
c
 	write (id,1000)
c
	npar = 0
c
	do i=1,fit_npar
	  lnc  = .true.
	  do j=1,fit_npar
	    do k=1,fit_ndpp(fit_jd(j))
	      if (lnc .and. fit_id(fit_jd(j),k).eq.i) then
	        npar       = npar+1
	        ipar(npar) = i
	        lnc        = .false.
	      endif
	    enddo
	  enddo
	enddo
	write (id,1010) fit_npar,npar
c
	if (npar.ne.0) then
	  write (id,2000)
	  do i=1,npar,3
	    ie = min(3,npar-i+1)
	    do j=0,ie-1
	      k = ipar(i+j)
	      cc_o(j+1) = cc(fit_par(k),fit_dpar(k),lc(j+1))
	    enddo
	    write (id,2010) (ipar(i+j-1),cc_o(j)(1:lc(j)),j=1,ie)
	  enddo
	endif
c
1000	format (1x,78('-'),/,' PARAMETER INFORMATION :',/,1x,78('-'))
1010	format (1x,'Maximum number of parameters : ',i5,/
     &	        1x,'Number of refined parameters : ',i5)
2000	format (/,1x,'Refinement parameters :')
2010	format (3x,3(i4,':',1x,a15,2x))
c
	end
c*****7*****************************************************************
	subroutine output_const (id,ipar)
c-
c	Outputs constraint information on unit 'id'
c+
	implicit	none
c
	include		'config.inc'
	include		'fit.inc'
	include		'errlist.inc'
c
	integer		ld(MAXDPP)
	integer		i,j,k,ll,id,ipar
	logical		lpar,lout
c
	integer		len_str
c
	lout = .false.
c
	write (id,1000) 
	do i=1,fit_npar
	  k = fit_jd(i)
	  if (fit_ndpp(k).ne.0) then
	    lpar = .false.
	    do j=1,fit_ndpp(k)
	      ld(j) = len_str(fit_dconst(k,j))
	      lpar  = lpar.or.(fit_id(k,j).eq.ipar).or.(ipar.eq.-1)
	    enddo
	    if (lpar) then
	      lout = .true.
	      ll = len_str(fit_const(k))
	      write (id,1010) fit_const(k)(1:ll)
	      write (id,1020) fit_id(k,1),
     &	                   fit_dconst(k,1)(1:ld(1))
	      if (fit_ndpp(k).gt.1) then
	        write (id,1030) (fit_id(k,j),
     &	                     fit_dconst(k,j)(1:ld(j)),j=2,fit_ndpp(k))
	      endif
	    endif
	  endif
	enddo
c
	if (.not.lout) then
	  write (id,2000)
	endif
c
1000	format (1x,78('-'),/,' PARAMETER DEFINITIONS :',/,1x,78('-'))
1010	format (/,1x,'Definition  : ',a)
1020	format (1x,'Derivatives : d/d(p[',i4,']) = ',a)
1030	format (1x,'              d/d(p[',i4,']) = ',a)
2000	format (1x,'No definitions found ...')
c
	end
c*****7*****************************************************************
	subroutine output_fit (id)
c-
c	Outputs fit information on unit 'id'
c+
	implicit	none
c
	include		'config.inc'
	include		'fit.inc'
	include		'errlist.inc'
c
	integer		ipar(MAXP)
	integer		id,i,j,npar
	logical		lkor
c
 	write (id,1000)
	write (id,1010) fit_ncycle,fit_lcyc,fit_urf
c
	npar = 0
	do i=1,fit_npar
	  if (fit_dpar(i).gt.0.0) then
	    npar = npar+1
	    ipar(npar) = i
	  endif
	enddo
c
	if (npar.gt.0) then
	  write (id,1020) fit_re,fit_rw,(fit_rw-fit_rwo)
	  write (id,2000)
	  lkor = .false.
	  do i=1,npar
	    do j=i+1,npar
	      if (abs(fit_cl(ipar(j),ipar(i))).ge.0.8) then
	        write (id,2100) ipar(j),ipar(i),fit_cl(ipar(j),ipar(i))
	        lkor = .true.
	      endif
	    enddo
	  enddo
	  if (.not.lkor) write (id,2110)
	endif
c
1000	format (1x,78('-'),/,' REFINEMENT INFORMATION :',/,1x,78('-'))
1010	format (1x,'Maximum # of iterations : ',i8,/,
     &	        1x,'Iterations carried out  : ',i8,/,
     &	        1x,'Magic number URF        : ',f15.8)
1020	format (1x,'R-values -     expected : ',f15.8,/,
     &	        1x,'               weighted : ',f15.8,/,
     &	        1x,'      change last cycle : ',f15.8)
2000	format (/,1x,'Correlations greater than 0.8 : ',/)
2100	format (3x,'Between p[',i5,'] and p[',i5,'] : ',f6.2)
2110	format (3x,'*** none ***')
c
	end
c*****7*****************************************************************
	subroutine output_phase (id,ipha)
c-
c	Outputs phase information for phase 'ipha' on file 'id'
c+
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'pdf.inc'
	include		'errlist.inc'
c
	character*20	cc, cc_o(6)
	character*4	at_name_o
	integer		ipha,id,ia,i,j,k,la,lc(6)
c
	integer		len_str
c
 	write (id,1000) ipha,cr_name(ipha)(1:len_str(cr_name(ipha)))
c
	cc_o(1) = cc(cr_skal(ipha),cr_dskal(ipha),lc(1))
	write (id,1010) cc_o(1)(1:lc(1))
	cc_o(1) = cc(pdf_delta(ipha),pdf_ddelta(ipha),lc(1))
	write (id,1020) cc_o(1)(1:lc(1))
	cc_o(1) = cc(pdf_srat(ipha),pdf_dsrat(ipha),lc(1))
	write (id,1025) cc_o(1)(1:lc(1)),pdf_rcut(ipha)
c
	do i=1,3
	  cc_o(i)   = cc(cr_a0(i,ipha), cr_da0(i,ipha), lc(i))
	enddo
	do i=1,3
	  cc_o(i+3) = cc(cr_win(i,ipha),cr_dwin(i,ipha),lc(i+3))
	enddo
	write (id,1030) (cc_o(i)(1:lc(i)),i=1,6)
c
	write (id,1040)
	do ia=1,cr_natoms(ipha)
	  at_name_o = cr_at_lis(cr_iscat(ia,ipha),ipha)
	  la = len_str(at_name_o)
	  do i=1,3
	    cc_o(i) = cc(cr_pos(i,ia,ipha),cr_dpos(i,ia,ipha),lc(i))
	  enddo
	  cc_o(4) = cc(cr_occ(ia,ipha),cr_docc(ia,ipha),lc(4))
	  write (id,1050) at_name_o(1:la),(cc_o(i)(1:lc(i)),i=1,4)
	enddo
c
	write (id,1060)
	do ia=1,cr_natoms(ipha)
	  at_name_o = cr_at_lis(cr_iscat(ia,ipha),ipha)
	  la = len_str(at_name_o)
	  do i=1,3
	    cc_o(i) = cc(cr_u(i,ia,ipha),cr_du(i,ia,ipha),lc(i))
	  enddo
	  write (id,1070) at_name_o(1:la),(cc_o(i)(1:lc(i)),i=1,3)
	  if (cr_u(4,ia,ipha).ne.0.0 .or. cr_u(5,ia,ipha).ne.0.0 .or.
     &	      cr_u(6,ia,ipha).ne.0.0) then
	    do i=4,6
	      cc_o(i) = cc(cr_u(i,ia,ipha),cr_du(i,ia,ipha),lc(i))
	    enddo
	    write (id,1080) (cc_o(i)(1:lc(i)),i=4,6)
	  endif
	enddo
c
c
1000	format (  1x,78('-'),/,1x,'PHASE ',i2,' : ',a,/,1x,78('-'))
1010	format (  1x,'Scale factor       : ',a15)
1020	format (  1x,'Dyn. corr. factor  : ',a15)
1025	format (  1x,'Low r sigma ratio  : ',a15,/,
     &	          1x,'R cutoff [A]       : ',f15.3)
1030	format (  1x,'Lattice parameters : ',2(a15,' '),a15,/,
     &	          1x,'          & angles : ',2(a15,' '),a15)
1040	format (/,1x,'Atom positions & occupancies    : ')
1050	format (  3x,a4,3(a15,' '),4x,a10)
1060	format (/,1x,'Anisotropic temparature factors : ')
1070	format (  3x,a4,3(a15,' '))
1080	format ( 12x,3(a15,' '))
c
	end
c*****7*****************************************************************
	character*20 function cc (x,dx,lc)
c
c	This routine creates the format x(dx) ..
c
	implicit	none
c
	character*40	cf,ce
	integer		il,ipow,inum,ierr,lc,lf,le
	real		x,dx
	logical		lnz
c
	integer		len_str
c
	if (dx.le.0.0) then
	  write (cc,'(f16.8)') x
	elseif (dx.ge.1.0) then
	  write (cf,'(f16.0)') x
	  write (ce,'(f16.0)') dx
	  lf = len_str(cf)
	  le = len_str(ce)
	  call rem_bl(cf,lf)
	  call rem_bl(ce,le)
	  cc = cf(1:lf)//'('//ce(1:le)//')'
	else
	  ipow = abs(int(log(dx+1.0E-10)/log(10.0)))+1
	  inum = abs(int(log(abs(x)    )/log(10.0)))+1
	  ierr = nint(dx*10**ipow)
	  if (ierr.le.2) then
	    ierr = nint(dx*10**(ipow+1))
	    write (cf,1000) ipow+inum+2,ipow+1
	    write (cc,cf) x
	    write (ce,'(i2)') ierr
	    cc = cc(1:ipow+inum+3)//'('//ce(1:2)//')'
	  else
	    write (cf,1000) ipow+inum+2,ipow
	    write (cc,cf) x
	    write (ce,'(i2)') ierr
	    cc = cc(1:ipow+inum+2)//'('//ce(1:2)//')'
	  endif
	endif
c
	lc  = len_str(cc)
	lnz = .false.
	do il=lc,1,-1
	  if (cc(il:il).eq.'0' .and. .not.lnz) then
	    cc(il:il) = ' '
	  else
	    lnz = .true.
	  endif
	enddo
	il = index(cc,'.')
	if (il.gt.1) then
	  if (cc(il-1:il-1).eq.' ') cc(il-1:il-1)='0'
	endif
	if (il.lt.lc) then
	  if (cc(il+1:il+1).eq.' ') cc(il+1:il+1)='0'
	endif
	call rem_bl(cc,lc)
c
1000	format ('(f',i2,'.',i2,')')
	end
c*****7***************************************************************
