c*****7****************************************************************
c	This file contains various structure related subroutines.
c
c	Version: 1.1
c	Date:    17 July 1999
c
c	Author:  Th. Proffen (tproffen@lanl.gov) 
c*****7****************************************************************
	subroutine calc_bang (zeile,lp)
c-
c	Calculate bond angles with errors
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'wink.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=3)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	character*20	cc,cstr
	character*4	at_name_i,at_name_j,at_name_k
	integer       	lpara(maxw),lstr
	integer       	ianz,i,ia,ja,ka
	real          	werte(maxw),x(3),y(3),dx(3),dy(3)
	real		xx,yy,xy,dxx,dyy,dxy,arg,darg,ang,dang
c
	real		acosd,skalpro,dskalpro
	integer		len_str
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.3) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  ia = nint(werte(1))
	  ja = nint(werte(2))
	  ka = nint(werte(3))
	  if (ia.lt.1 .or. ia.gt.cr_natoms(cr_ipha) .or.
     &	      ja.lt.1 .or. ja.gt.cr_natoms(cr_ipha) .or.
     &	      ka.lt.1 .or. ka.gt.cr_natoms(cr_ipha)     ) then
	    ier_num = -19
	    ier_typ = ER_APPL
	    return
	  endif
c
	  do i=1,3
	     x(i) = cr_pos (i,ja,cr_ipha) - cr_pos (i,ia,cr_ipha)
	     y(i) = cr_pos (i,ja,cr_ipha) - cr_pos (i,ka,cr_ipha)
	    dx(i) = cr_dpos(i,ja,cr_ipha) + cr_dpos(i,ia,cr_ipha)
	    dy(i) = cr_dpos(i,ja,cr_ipha) + cr_dpos(i,ka,cr_ipha)
	  enddo
c
	  xx = sqrt(skalpro(x,x,cr_gten,cr_ipha))
	  yy = sqrt(skalpro(y,y,cr_gten,cr_ipha))
	  xy =      skalpro(x,y,cr_gten,cr_ipha) 
c
	  if (xx.le.0.0 .or. yy.le.0.0) then
	    ier_num = -32
	    ier_typ = ER_APPL
	    return
	  endif
c
	  dxx=0.5/xx*dskalpro(x,x,cr_gten,dx,dx,cr_dgten,cr_ipha)
	  dyy=0.5/yy*dskalpro(y,y,cr_gten,dy,dy,cr_dgten,cr_ipha)
	  dxy=       dskalpro(x,y,cr_gten,dx,dy,cr_dgten,cr_ipha)
	  arg=xy/(xx*yy)
	  ang=acosd(arg)
	  darg=(1./(xx*yy)*dxy + arg/xx*dxx +arg/yy*dyy )
	  if (arg.ne.1.0) then
	    dang=abs(1./sqrt(1-arg*arg)/rad*darg)
	  else
	    dang=0.0
	  endif
c
	  at_name_i = cr_at_lis(cr_iscat(ia,cr_ipha),cr_ipha)
	  at_name_j = cr_at_lis(cr_iscat(ja,cr_ipha),cr_ipha)
	  at_name_k = cr_at_lis(cr_iscat(ka,cr_ipha),cr_ipha)
	  cstr      = cc(ang,dang,lstr)
	  write (output_io,1010) at_name_i(1:len_str(at_name_i)),ia,
     &	                         at_name_j(1:len_str(at_name_j)),ja,
     &	                         at_name_k(1:len_str(at_name_k)),ka,
     &	                         cstr(1:lstr)
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1010	format (3x,a4,' (#',i5,2(') - ',a4,' (#',i5),')   =   ',
     &	        a15,' degrees')
c
	end
c*****7****************************************************************
	subroutine calc_blen (zeile,lp)
c-
c	Calculate bond lengths with errors
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=4)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	character*20	cc,cstr
	character*4	at_name_i,at_name_j
	integer       	lpara(maxw),lstr
	integer		iii(3),istart(3),iend(3)
	integer		ioff(3),cell(3)
	integer		i,j,k,ii,jj
	integer       	ianz,is,js,ia,ja
	real          	werte(maxw),d(3),dd(3)
	real		dist,ddist,bmin,bmax
	logical		i_allowed(0:MAXSCAT)
	logical		j_allowed(0:MAXSCAT)
	logical		lfound
c
	real		skalpro,dskalpro
	integer		len_str
c
	lfound = .false.
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
c------ Simple distance between given atoms
c
	if (ianz.eq.2) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  ia = nint(werte(1))
	  ja = nint(werte(2))
	  if (ia.lt.1 .or. ia.gt.cr_natoms(cr_ipha) .or.
     &	      ja.lt.1 .or. ja.gt.cr_natoms(cr_ipha)     ) then
	    ier_num = -19
	    ier_typ = ER_APPL
	    return
	  endif
c
          do jj=1,3
	     d(jj) = cr_pos (jj,ia,cr_ipha) - cr_pos (jj,ja,cr_ipha) 
	    dd(jj) = cr_dpos(jj,ia,cr_ipha) + cr_dpos(jj,ja,cr_ipha)
	  enddo   
	   dist = sqrt(skalpro(d,d,cr_gten,cr_ipha))
	  ddist = 0.5/dist * dskalpro( d, d,cr_gten,
     &	                              dd,dd,cr_dgten,cr_ipha)
	  at_name_i = cr_at_lis(cr_iscat(ia,cr_ipha),cr_ipha)
	  at_name_j = cr_at_lis(cr_iscat(ja,cr_ipha),cr_ipha)
	  cstr      = cc(dist,ddist,lstr)
	  write (output_io,1010) at_name_i(1:len_str(at_name_i)),ia,
     &	                         at_name_j(1:len_str(at_name_j)),ja,
     &	                         cstr(1:lstr)
c
c------ Get all bonds in specified range
c
	elseif (ianz.eq.4) then
	  call get_atoms(cpara(1),lpara(1),i_allowed)
	  call get_atoms(cpara(2),lpara(2),j_allowed)
	  if (ier_num.ne.0) return
	  call del_params(2,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  write (output_io,1000)
c
	  bmin = werte(1)
	  bmax = werte(2)
	  call pdf_setup_bnd(cr_ipha)
c
c------ - Loop over all atoms within the crystal
c
	  do ia=1,cr_natoms(cr_ipha)
            is = cr_iscat(ia,cr_ipha)
	    if (.not. i_allowed(is)) goto 9999
	    do i=1,3
	      iii(i)    = int(cr_pos(i,ia,cr_ipha)-cr_dim0(i,1,cr_ipha))+1   
 	      istart(i) = iii(i)-1-int(bmax/cr_a0(i,cr_ipha))
	      iend(i)   = iii(i)+1+int(bmax/cr_a0(i,cr_ipha))
	    enddo
c
	    do k=istart(3),iend(3)
	    do j=istart(2),iend(2)
	    do i=istart(1),iend(1)
	      cell(1) = i
	      cell(2) = j
	      cell(3) = k
c  	 
c------ ----- Here we apply periodic boundaries
c------ ----- Modified to allow more than a single cycle !
c
	      do ii=1,3
	        iii(ii)  = cell(ii)
	        cell(ii) = pdf_bnd(ii,cell(ii))
	        ioff(ii) = iii(ii)-cell(ii)
	      enddo
c
c------ ----- Now look for neighbours in surrounding unit cells only
c
	      do js=1,cr_ncatoms(cr_ipha)
	        call celltoindex (cell,js,ja,cr_ipha)
	        if (j_allowed(cr_iscat(ja,cr_ipha))) then
                  do jj=1,3
	             d(jj) = cr_pos (jj,ia,cr_ipha) - 
     &	                     cr_pos (jj,ja,cr_ipha) + float(ioff(jj))
	            dd(jj) = cr_dpos(jj,ia,cr_ipha) +
     &	                     cr_dpos(jj,ja,cr_ipha)
	          enddo   
	          dist = sqrt(skalpro(d,d,cr_gten,cr_ipha))
	          if (dist.ge.bmin .and. dist.le.bmax) then
	            ddist = 0.5/dist * dskalpro( d, d,cr_gten,
     &	                                        dd,dd,cr_dgten,cr_ipha)
	            at_name_i = cr_at_lis(cr_iscat(ia,cr_ipha),cr_ipha)
	            at_name_j = cr_at_lis(cr_iscat(ja,cr_ipha),cr_ipha)
	            cstr      = cc(dist,ddist,lstr)
	            write (output_io,1010) 
     &	                   at_name_i(1:len_str(at_name_i)),ia,
     &	                   at_name_j(1:len_str(at_name_j)),ja,
     &	                   cstr(1:lstr)
	            lfound = .true.
	          endif
	        endif
	      enddo
	    enddo
	    enddo
	    enddo
	    write(output_io,*)
9999	    continue
	  enddo
c
	  if (.not.lfound) write (output_io,1020)
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1000	format (' Bond lengths found in current phase : ')
1010	format (3x,a4,' (#',i5,') - ',a4,' (#',i5,')   =   ',a15,' A')
1020	format (3x,'*** No pairs found ***')
	end
c*****7****************************************************************
	subroutine do_read (zeile,lp)
c-
c	Main menu for read command.
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=11)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz
	real          	werte(maxw)
c
	logical         str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
c
	if (ianz.eq.0) then
	  ier_num = -6
	  ier_typ = ER_COMM
	else
c
c-----    Reading a DISCUS structure file
c
	  if(str_comp(cpara(1),'stru',2,lpara(1),4)) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	    call do_read_stru (cpara(1))
	    call update_cr_dim
c
c-----    Reading a PDF data file
c
	  elseif(str_comp(cpara(1),'data',2,lpara(1),4)) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_read_data (ianz,cpara,lpara,werte,maxw,.false.)
c
c-----    Reading a PDF difference file
c
	  elseif(str_comp(cpara(1),'diff',2,lpara(1),4)) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_read_data (ianz,cpara,lpara,werte,maxw,.true.)
c
c-----    Reading a PDF output file
c
	  elseif(str_comp(cpara(1),'fit',2,lpara(1),3)) then
	    continue
c
c-----    Unknown command
c
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
c**********************************************************************
      	subroutine do_read_stru (strucfile)
c-
c     	This routine reads DISCUS type structure file. 
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	integer       	ist
	parameter	(ist=7)
c
	character*(*)  	strucfile
	real		tot
	integer       	i
	logical       	lread,lout,ldiscus
c
	if (cr_npha.lt.MAXPHA) then
	  cr_npha = cr_npha + 1
	  cr_ipha = cr_npha
	else
	  ier_num = -102
	  ier_typ = ER_APPL
	  return
	endif
c
	cr_natoms(cr_ipha) = 0
	lread = .true.
	lout  = .true.
c
	call oeffne(ist,strucfile,'old',lread)
	if(ier_num.ne.0) goto 999
	call struc_read_header(ist,ldiscus)
	if(ier_num.ne.0) goto 999
	call lattice(cr_ipha,.false.)
c
	if (ldiscus) then
	  write(output_io,1000)
	else
	  write(output_io,1100)
	endif
c
	call struc_read_atoms(ist,ldiscus)
	if(ier_num.ne.0) goto 999
c
	tot = cr_icc(1,cr_ipha)*cr_icc(2,cr_ipha)*cr_icc(3,cr_ipha)*
     &	      cr_ncatoms(cr_ipha)
	if (tot.ne.cr_natoms(cr_ipha)) then
	  ier_num = -109
	  ier_typ = ER_APPL
	  goto 999
	endif
c
	cr_total = cr_total + cr_natoms(cr_ipha)
	cr_off(cr_ipha) = cr_off(cr_ipha-1) + cr_natoms(cr_ipha)*n_at
	close(ist)
c
	call lattice(cr_ipha,lout)
	return
c
999	continue
	cr_npha = cr_npha-1
	cr_ipha = max(1,cr_npha)
	close(ist)
c
	if(ier_num.eq.-49 ) then
	  write(ier_msg(1),3000) cr_natoms(cr_ipha)+1
3000	  format('At atom number = ',i8)
	endif
c
1000	format (' Structure file format  : ',
     &	         'DISCUS (converting B -> Uij) ')
1100	format (' Structure file format  : PDFFIT')
c
	end
c**********************************************************************
	subroutine struc_read_header(ist,ldiscus)
c-
c	This subroutine reads the header of a structure file
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter	(maxw=12)
c
	character*200 	line,cpara(maxw)
	character*200   zeile
	character*6     befehl
	integer         ist,i,ll,j
	integer       	ianz
	integer       	spcgr_ianz
	integer       	lpara(maxw),lp
	integer         lbef,indxb
	real            spcgr_para
	real          	werte(maxw)
	logical		ldiscus
c
	integer       	len_str
	logical         str_comp
c
	befehl  = 'none'
	lbef    = 4
	ldiscus = .true.
c
	call no_error
	do while( .not. str_comp(befehl,'atoms',1,lbef,5))
	  read(ist,'(a)',end=999,err=999)  line
	  ll     = len_str(line)
c
c------	- The maximum number of significant characters depends on the 
c------	- length of the character constant befehl.
c
	  lbef   = len(befehl)
	  befehl = '    '
	  indxb  = index(line,' ')
	  lbef   = min(indxb-1,lbef)
	  befehl = line(1:lbef)
c
c------	- Command parameters start at the first character following 
c------ - the blank
c
	  zeile = ' '
	  lp    = 0
	  if(indxb+1 .le. ll) then
	    zeile = line(indxb+1:ll)
	    lp    = ll - indxb
	  endif
c
c------	- Commentary
c
	  if(line.eq.' ' .or. line(1:1).eq.'#') then
	    continue
c
c------ - Format
c
	  elseif(str_comp(befehl,'format',2,lbef,6) ) then
	    call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    ldiscus = .not.str_comp(cpara(1),'pdffit',3,lpara(1),6)
c
c------	- Scale factor (PDFFIT)
c
	  elseif(str_comp(befehl,'scale',2,lbef,5) ) then
            call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    if (ier_num.ne.0) return
	    if (ianz.eq.1) then
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      cr_skal (cr_ipha) = werte(1)
	      cr_dskal(cr_ipha) = 0.0
	    else
	      ier_num    = -46
	      ier_typ    = ER_APPL
	      ier_msg(1) = 'Error reading scale factor'
	      ier_num = -6
	    endif
c
c------	- Peak sharpening (PDFFIT)
c
	  elseif(str_comp(befehl,'sharpen',2,lbef,7) ) then
            call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    if (ier_num.ne.0) return
	    if (ianz.eq.3) then
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      pdf_delta (cr_ipha) = werte(1)
	      pdf_ddelta(cr_ipha) = 0.0
	      pdf_srat  (cr_ipha) = werte(2)
	      pdf_dsrat (cr_ipha) = 0.0
	      pdf_rcut  (cr_ipha) = werte(3)
	    else
	      ier_num    = -46
	      ier_typ    = ER_APPL
	      ier_msg(1) = 'Error reading sharpening parameters'
	      ier_num = -6
	    endif
c
c------ - Space group symbol (only to save it later for DISCUS use)
c
	  elseif(str_comp(befehl,'spcgr',1,lbef,5) ) then
	    call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    cr_spcgr(cr_ipha) = cpara(1)
c
c------	- Title / name for structure
c
	  elseif(str_comp(befehl,'title',1,lbef,5) ) then
	    cr_name(cr_ipha) = zeile
c
c------	- Cell constants
c
	  elseif(str_comp(befehl,'cell',1,lbef,4) ) then
            call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    if(ier_num.eq.0) then
	      if(ianz.eq.6) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          do i=1,3
	            cr_a0 (i,cr_ipha) = werte(i  )
	            cr_win(i,cr_ipha) = werte(i+3)
	          enddo
	        else
	          ier_num    = -46
	          ier_typ    = ER_APPL
	          ier_msg(1) = 'Error reading unit cell parameters'
	          return
	        endif
	      else
	        read (zeile,*,end=999,err=999)
     &               (cr_a0(i,cr_ipha),i=1,3),
     &	             (cr_win(i,cr_ipha),i=1,3)
	      endif
	    endif
c
c------	- Standard deviation of cell constants
c
	  elseif(str_comp(befehl,'dcell',2,lbef,5) ) then
            call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    if(ier_num.eq.0) then
	      if(ianz.eq.6) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          do i=1,3
	            cr_da0 (i,cr_ipha) = werte(i  )
	            cr_dwin(i,cr_ipha) = werte(i+3)
	          enddo
	        else
	          ier_num    = -46
	          ier_typ    = ER_APPL
	          ier_msg(1) = 'Error reading unit cell parameters'
	          return
	        endif
	      else
	        read (zeile,*,end=999,err=999)
     &               (cr_da0(i,cr_ipha),i=1,3),
     &	             (cr_dwin(i,cr_ipha),i=1,3)
	      endif
	    endif
c
c------	- Crystal dimensions and number of atoms per unit cell 'ncell'
c
	  elseif(str_comp(befehl,'ncell',1,lbef,5) ) then
            call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	    if(ier_num.eq.0) then
	      if(ianz.eq.4) then
                call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          do j=1,3
	            cr_icc(j,cr_ipha) = werte(j)
	          enddo
	          cr_ncatoms(cr_ipha) = werte(4)
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	        return
	      endif
	    endif
c
	  elseif(str_comp(befehl,'atoms',1,lbef,5) ) then
	    continue
c
	  else
	    write (*,1000) befehl,-99
1000	    format (' ****WARN**** Unknown keyword: ',a14,
     &	            ' (ignored) ****',i4,' ****')
	  endif
	enddo
c
999	continue
c
	end
c**********************************************************************
	subroutine struc_read_atoms(ist,ldiscus)
c-
c     	This subroutine reads the list of atoms into the crystal array
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'wink.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=4)
c
	character*200	line
	real          	werte(maxw)
	real		fac,dw
	integer       	i,j,ist,ibl
	logical		ldiscus
c
	fac = 1.0 / (8.0*pi**2)
c
1000	continue
	  read(ist,'(a)',end=2000,err=998) line
	  ibl = index(line,' ')
	  read(line(ibl:200),*,end=2000,err=998) (werte(j),j=1,4)
	  if(cr_natoms(cr_ipha).eq.nmax) then
	    ier_num = -10
	    ier_typ = ER_APPL
	    return
	  endif
c
	  cr_natoms(cr_ipha) = cr_natoms(cr_ipha) + 1
	  i  = cr_natoms(cr_ipha)
	  do j=1,3
	    cr_pos(j,i,cr_ipha) = werte(j)
	  enddo
	  dw = werte(4)
c
c------ - We have DISCUS format
c
	  if (ldiscus) then
	    cr_u(1,i,cr_ipha) = fac*dw
	    cr_u(2,i,cr_ipha) = fac*dw
	    cr_u(3,i,cr_ipha) = fac*dw
c
	    cr_u(4,i,cr_ipha) = 0.0
	    cr_u(5,i,cr_ipha) = 0.0
	    cr_u(6,i,cr_ipha) = 0.0
c
	    cr_occ (i,cr_ipha) = 1.0
	    cr_docc(i,cr_ipha) = 0.0
c
	    do j=1,3
	      cr_dpos(j,i,cr_ipha) = 0.0
	    enddo
c
	    do j=1,6
	      cr_du(j,i,cr_ipha) = 0.0
	    enddo
c
c------ - We have PDFFIT format
c
	  else
	    cr_occ(i,cr_ipha) = werte(4)
	    read(ist,*,end=2000,err=998) 
     &	        (cr_dpos(j,i,cr_ipha),j=1,3),cr_docc(i,cr_ipha),
     &	        (cr_u(j,i,cr_ipha),j=1,3),(cr_du(j,i,cr_ipha),j=1,3),
     &	        (cr_u(j,i,cr_ipha),j=4,6),(cr_du(j,i,cr_ipha),j=4,6)
	  endif
c
	  if(line(1:4).ne.'    ') then
	    ibl = ibl - 1
	    call do_cap(line(1:ibl))
	    do j=0,cr_nscat(cr_ipha)
              if(line(1:ibl).eq.cr_at_lis(j,cr_ipha)) then
                cr_iscat(i,cr_ipha)=j
                goto 1100
              endif
	    enddo
            cr_nscat(cr_ipha)   = cr_nscat(cr_ipha)+1
            cr_iscat(i,cr_ipha) = cr_nscat(cr_ipha)
            cr_at_lis(cr_nscat(cr_ipha),cr_ipha) = line(1:ibl)
	  endif
1100	  continue
	goto 1000
c
2000	continue
	close(ist)
	return
c
998	ier_num = -49
	ier_typ = ER_APPL
	close(ist)
c
	end
c**********************************************************************
	subroutine update_cr_dim
c-
c	Updates the crystal dimensions to the current values
c+
	implicit   none
c
	include    'config.inc'
	include    'crystal.inc'
c
	integer    i,j
c
c------	Set initial values
c
	if(cr_natoms(cr_ipha).gt.0) then
	  do j=1,3
	    cr_dim(j,1,cr_ipha) = cr_pos(j,1,cr_ipha)
	    cr_dim(j,2,cr_ipha) = cr_pos(j,1,cr_ipha)
	  enddo
	else
	  do j=1,3
	    cr_dim(j,1,cr_ipha) =  1.e10
	    cr_dim(j,1,cr_ipha) = -1.e10
	  enddo
	endif
c
c------	Update values from all atoms in crystal
c
	do i=1,cr_natoms(cr_ipha)
	  do j=1,3
            cr_dim(j,1,cr_ipha)=amin1(cr_dim(j,1,cr_ipha),
     &	                              cr_pos(j,i,cr_ipha))
            cr_dim(j,2,cr_ipha)=amax1(cr_dim(j,2,cr_ipha),
     &	                              cr_pos(j,i,cr_ipha))
	  enddo
	enddo
c
c------ Set cr_dim0 values
c
	do i=1,3
	  cr_dim0(i,1,cr_ipha) = float(nint(cr_dim(i,1,cr_ipha)))
	  cr_dim0(i,2,cr_ipha) = float(nint(cr_dim(i,2,cr_ipha)))
	enddo
c
	end
c*****7****************************************************************
	subroutine do_reset
c
c	resets the crystal structure to empty
c
	implicit 	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
c
	integer 	i,j
c
c------ Data sets
c
	pdf_nset = 0
	pdf_iset = 1
c
	do i=1,MAXDSET
	  pdf_rfmin(i) = -9999.
	enddo
c
	do j=0,MAXSCAT
	  do i=1,MAXDSET
	    pdf_allowed_i(j,i) = .true.
	    pdf_allowed_j(j,i) = .true.
	  enddo
	enddo
c
	do i=1,MAXPHA
	  do j=1,MAXDSET
	    pdf_psel(i,j) = .true.
	  enddo
	enddo
c
c------ Structure
c
	cr_total   = 0
	cr_npha    = 0
	cr_ipha    = 1
c
	do j=1,MAXPHA
	  cr_off(j)     = 0
	  cr_natoms(j)  = 0
	  cr_ncatoms(j) = 0
	  cr_nscat(j)   = 0
	  do i=1,maxscat
	    cr_at_lis(i,j) = ' '
	  enddo
	  do i=1,3
	    cr_dim(i,1,j) = 0.0
	    cr_dim(i,2,j) = 0.0
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine do_phase (zeile,lp)
c-
c	Interpret 'phase' command
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	integer       	maxw
	parameter    	(maxw=3)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz
	real          	werte(maxw)
c
	logical         str_comp
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
c
	if     (ianz.eq.0) then
	  write (output_io,1000) cr_ipha
1000	  format (1x,'------ > Active phase for input : ',I4)
	elseif (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (nint(werte(1)).ge.1       .and. 
     &	      nint(werte(1)).le.cr_npha      ) then
	    cr_ipha = nint(werte(1))
	  else
	    ier_num = -101
	    ier_typ = ER_APPL
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine do_occ (zeile,lp)
c-
c	Setting of the occupancies for atom/site
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
	include		'charact.inc'
c
	integer       	maxw
	parameter    	(maxw=20)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz,iianz,is,i,j,k
	real          	werte(maxw)
	real          	wwerte(maxw)
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if     (ianz.eq.2) then
	  iianz = ianz-1
	  call get_iscat(iianz,cpara,lpara,wwerte,maxw)
	  if(ier_num.ne.0) return
	  call del_params(1,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (werte(1).ge.0.0 .and. werte(1).le.1.0) then
	    if(wwerte(1).eq.-1) then
	      do i=1,cr_natoms(cr_ipha)
	        cr_occ(i,cr_ipha) = werte(1)
	      enddo
	    else
	      do i=1,iianz
	        is = nint(wwerte(i))
 	        do k=1,cr_natoms(cr_ipha)
	          if(cr_iscat(k,cr_ipha).eq.is) then
	            cr_occ(k,cr_ipha) = werte(1)
	          endif
	        enddo
	      enddo
	    endif
	  else
	    ier_num = -103
	    ier_typ = ER_APPL
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine do_temp (zeile,lp)
c-
c	Setting of the thermal factors for atom/site
c+
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
	include		'charact.inc'
c
	integer       	maxw
	parameter    	(maxw=20)
c
	character*(*)	zeile
	integer		lp
c
	character*200 	cpara(maxw)
	integer       	lpara(maxw),length
	integer       	ianz,iianz,is,i,j,k
	real          	werte(maxw)
	real          	wwerte(maxw)
c
	call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.2 .or. ianz.eq.4 .or. ianz.eq.7) then
	  iianz = ianz-1
	  call get_iscat(iianz,cpara,lpara,wwerte,maxw)
	  if(ier_num.ne.0) return
	  call del_params(1,ianz,cpara,lpara,maxw)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
c
	  if (ianz.eq.1) then
	    werte(2) = werte(1)
	    werte(3) = werte(1)
	    werte(4) = 0.0
	    werte(5) = 0.0
	    werte(6) = 0.0
	  elseif (ianz.eq.3) then
	    werte(4) = 0.0
	    werte(5) = 0.0
	    werte(6) = 0.0
	  endif
c
	  if(wwerte(1).eq.-1) then
	    do i=1,cr_natoms(cr_ipha)
	      cr_u(1,i,cr_ipha) = werte(1)
	      cr_u(2,i,cr_ipha) = werte(2)
	      cr_u(3,i,cr_ipha) = werte(3)
	      cr_u(4,i,cr_ipha) = werte(4)
	      cr_u(5,i,cr_ipha) = werte(5)
	      cr_u(6,i,cr_ipha) = werte(6)
	    enddo
	  else
	    do i=1,iianz
	      is = nint(wwerte(i))
 	      do k=1,cr_natoms(cr_ipha)
	        if(cr_iscat(k,cr_ipha).eq.is) then
	          cr_u(1,k,cr_ipha) = werte(1)
	          cr_u(2,k,cr_ipha) = werte(2)
	          cr_u(3,k,cr_ipha) = werte(3)
	          cr_u(4,k,cr_ipha) = werte(4)
	          cr_u(5,k,cr_ipha) = werte(5)
	          cr_u(6,k,cr_ipha) = werte(6)
	        endif
	      enddo
	    enddo
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine atom_select(zeile,lp,latom,lselect)
c+
c	This routine exectues the select command
c-
	implicit	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'pdf.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=MAXSCAT)
c
	character*(*)	zeile
	integer		lp
	logical		latom(0:MAXSCAT,MAXDSET),lselect
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		ianz,iset,i,is,lpara(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if(ier_num.ne.0) return
c
c------ Get data set number first
c
	if (ianz.le.1) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	else
	  call ber_params(1,cpara,lpara,werte,maxw) 
	  if(ier_num.ne.0) return
	  iset = nint(werte(1))
	  if (iset.lt.1 .or. iset.gt.pdf_nset) then
	    ier_num = -106
	    ier_typ = ER_APPL
	    return
	  endif
	  call del_params(1,ianz,cpara,lpara,maxw) 
	endif
c
c------ Now get selected atoms
c
	call get_iscat(ianz,cpara,lpara,werte,maxw)
	if(ier_num.ne.0) return
c
	if(werte(1).eq.-1) then
	  do i=0,cr_nscat(cr_ipha)
	    latom(i,iset) = lselect
	  enddo
	else
	  do i=1,ianz
	    is = nint(werte(i))
	    if (is.ge.0 .and. is.le.cr_nscat(cr_ipha)) then
	      latom(is,iset) = lselect
	    else
	      ier_num = -27
	      ier_typ = ER_APPL
	    endif
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine get_atoms (zeile,lp,latom)
c+
c	Gets the selected atoms from 'zeile'.
c-
	implicit	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=MAXSCAT+1)
c
	character*(*)	zeile
	integer		lp
	logical		latom(0:MAXSCAT)
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		ianz,i,is,lpara(maxw)
c
	ianz     = 1
	cpara(1) = zeile
	lpara(1) = lp
c
	do i=0,cr_nscat(cr_ipha)
	  latom(i) = .false.
	enddo
c
	call get_iscat(ianz,cpara,lpara,werte,maxw)
	if(ier_num.ne.0) return
c
	if(werte(1).eq.-1) then
	  do i=0,cr_nscat(cr_ipha)
	    latom(i) = .true.
	  enddo
	else
	  do i=1,ianz
	    is = nint(werte(i))
	    if (is.ge.0 .and. is.le.cr_nscat(cr_ipha)) then
	      latom(is) = .true.
	    else
	      ier_num = -27
	      ier_typ = ER_APPL
	    endif
	  enddo
	endif
c
	end
c**********************************************************************
	subroutine save_atoms(ipha,strucfile,lp)
c+
c	This subroutine saves the structure and/or the unit cell
c	onto a file. Format is for plotting with ATOMS.
c+
	implicit	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'errlist.inc'
c
	character*(*) 	strucfile
	integer       	ipha,lp,ist,i,j
	logical       	lread
c
	integer       	len_str
c
	data 	ist 	/17/
c
	lread = .false.
	call oeffne(ist,strucfile,'unknown',lread)
	if(ier_num.eq.0) then
c
	  write (ist, 500) cr_name(ipha)(1:len_str(cr_name(ipha)))
	  write (ist, 510) (cr_a0(i,ipha)*cr_icc(i,ipha),i=1,3),
     &	                   (cr_win(i,ipha),i=1,3)
	  write (ist, 520) 'P1'
	  write (ist, 530)
c
	  do i=1,cr_natoms(ipha)
	    write(ist,1000) cr_at_lis(cr_iscat(i,ipha),ipha), 
     &	                    cr_iscat(i,ipha),
     &	                   (cr_pos(j,i,ipha)/cr_icc(j,ipha),j=1,3),
     &	                   (cr_u(j,i,ipha),j=1,6)
	  enddo
	endif
	close(ist)
c
500	format ('TITL ',a)
510	format ('CELL ',6(f10.6,1x))
520	format ('SPGP ',a)
530	format ('FIELDS LAB TYP COO TFU')
1000	format (a4,1x,i4,3x,3(f11.6,1x),/,6(f8.6,1x))
	end
c**********************************************************************
	subroutine save_struc(ipha,strucfile,lp,ldis)
c+
c	This subroutine saves the structure and/or the unit cell
c	onto a file. The format uses keyword description.
c+
	implicit	none
c
	include 	'config.inc'
	include 	'pdf.inc'
	include 	'crystal.inc'
	include 	'wink.inc'
	include 	'errlist.inc'
c
	character*200 	strucfile
	integer       	ipha,lp,ist,i,j,k,j_end,l1,l2
	integer       	i_start,i_end
	real		fac,dw
	logical       	lread,ldis
c
	integer       	len_str
c
	data 	ist 	/17/
c
	fac = 8.0*pi*pi / 3.0
c
	lread = .false.
	call oeffne(ist,strucfile,'unknown',lread)
	if(ier_num.eq.0) then
c
c------- Write new type of structur file
c
	  j = len_str(cr_name(ipha))
	  if(j.eq.0) then
	    cr_name(ipha) = ' '
	    j             = 1
	  endif
	  write(ist,3000) cr_name(ipha)(1:j)
	  if(.not.ldis) then
	    write(ist,3005)
	    write(ist,3008) cr_skal(ipha)
	    write(ist,3009) pdf_delta(ipha),
     &	                    pdf_srat(ipha),pdf_rcut(ipha)
	  endif
	  write(ist,3010) cr_spcgr(ipha)
	  write(ist,3020) (cr_a0(i,ipha),i=1,3),
     &	                  (cr_win(i,ipha),i=1,3)
	  if (.not.ldis) 
     &	    write(ist,3025) (cr_da0(i,ipha),i=1,3),
     & 	                    (cr_dwin(i,ipha),i=1,3)
	  write(ist,3030) (cr_icc(i,ipha),i=1,3),cr_ncatoms(ipha)
	  write(ist,3900)
	  do i=1,cr_natoms(ipha)
	    if (ldis) then
	      dw = fac*(cr_u(1,i,ipha)+cr_u(2,i,ipha)+cr_u(3,i,ipha))
	      write(ist,4000) cr_at_lis(cr_iscat(i,ipha),ipha),
     &	                      (cr_pos (j,i,ipha),j=1,3),dw
	    else
	      write(ist,4010)  cr_at_lis(cr_iscat(i,ipha),ipha),
     &	                      (cr_pos (j,i,ipha),j=1,3),cr_occ (i,ipha),
     &	                      (cr_dpos(j,i,ipha),j=1,3),cr_docc(i,ipha),
     &	                      (cr_u   (j,i,ipha),j=1,3),
     &	                      (cr_du  (j,i,ipha),j=1,3),
     &	                      (cr_u   (j,i,ipha),j=4,6),
     &	                      (cr_du  (j,i,ipha),j=4,6)
	    endif
	  enddo
	endif
	close(ist)
c
3000	format('title  ',a)
3005	format('format pdffit')
3008	format('scale ',f10.6)
3009	format('sharp ',2(f10.6,','),f10.6)
3010	format('spcgr  ',a16)
3020	format('cell  ',5(f10.6,','),f10.6)
3025	format('dcell ',5(f10.6,','),f10.6)
3030	format('ncell ',3(i8,','),i10)
3900	format('atoms ')
4000	format(a4,3(2x,f16.8),1x,f12.4)
4010	format(a4,3(2x,f16.8),1x,f12.4,/,4x,3(2x,f16.8),1x,f12.4,
     &	       4(/,4x,3(2x,f16.8)))
	end
c*****7*****************************************************************
	subroutine get_iscat(ianz,cpara,lpara,werte,maxw)
c-
c	Determines the scattering type of the parameter
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'charact.inc'
	include       'errlist.inc'
c
	integer        maxw
c
	character*200  cpara(maxw),zeile
	integer        lpara(maxw)
	integer        i,j,l,ianz,jj,jp
	logical        lnew
	real           werte(maxw)
c
	real           berechne
c
	ier_num = 0
	ier_typ = ER_NONE
c
	do i=1,maxw
	  werte(i) = 0.0
	enddo
	jj = 1
	jp = 0
	j  = 1
	do while( j.le.ianz .and. ier_num.eq.0 )
	  i  = ichar(cpara(j)(1:1))
	  if(cpara(j).eq.'all') then
	    werte(1) = -1
	    return
	  elseif(((a .le.i .and. i.le.z ).or. 
     &	          (aa.le.i .and. i.le.zz)   ) .and.
     &	          (index(cpara(j),'[').eq.0 )      )  then
	    call do_cap(cpara(j))
	    ier_num = -27
	    ier_typ = ER_APPL
	    do i=0,cr_nscat(cr_ipha)
	      if(cpara(j).eq.cr_at_lis(i,cr_ipha)) then
	        werte(jj) = i
	        jj        = jj + 1
	        jp        = jp + 1
	        ier_num   = 0
	        ier_typ   = ER_NONE
	      endif
	    enddo
	  else
	    zeile          = '('//cpara(j)(1:lpara(j))//')'
	    werte(jj)      = berechne(zeile,lpara(j)+2)
	    if(ier_num.eq.0) then
	      if(0.le.nint(werte(jj)) .and. 
     &	         nint(werte(jj)).le.cr_nscat(cr_ipha)) then
	        jj      = jj + 1
	        ier_num = 0
	        ier_typ = ER_NONE
	      else
	        ier_num = -27
	        ier_typ = ER_APPL
	      endif
	    endif
	  endif
	  j = j + 1
	enddo
	ianz = max(ianz,jp)
c
	end
