c*****7*****************************************************************
	subroutine chem
c-
c	This sublevel contains all routines to analyse a structure
c	line calculation of bondlength distributions, the average
c	structure and correlations.
c
c	Note: Some variables are used in the MC section as well
c	      and settings might be overwritten.
c
c	Version  : 3.3
c	Date     : 17. July 1999
c	
c	Author   : Th. Proffen (proffen@pa.msu.edu)
c+
	implicit	none
c
	include		'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'doact.inc'
	include 	'macro.inc'
	include 	'errlist.inc'
	include 	'learn.inc'
	include 	'param.inc'
	include 	'prompt.inc'
	include 	'chem.inc'
c
	integer       	maxw
	parameter    	(maxw=20)
c
	character*5    	befehl
	character*11   	prom
	character*200  	line,zeile,cpara(maxw)
	real           	werte(maxw),wwerte(maxw),wwwerte(maxw)
	real            uwerte(maxw)
	integer		lbeg(3)
	integer        	lpara(maxw),lp,length
	integer        	indxg,ianz,lbef,i,ia,is,ic(3),iianz,jjanz
	integer         kkanz
	logical        	lout
c
	logical         str_comp
c
	call no_error
c
10	continue
c
	prom=prompt//'/chem'
	call get_command (line,length,befehl,lbef,zeile,lp,prom)
	if(ier_num.eq.0) then
	if(line.eq.' '.or.line(1:1).eq.'#') goto 10
c
c------ search for "="
c
	indxg=index(line,'=')
	if(indxg.ne.0.and..not.(str_comp(befehl,'echo',2,lbef,4))
     &	             .and..not.(str_comp(befehl,'syst',2,lbef,4))
     &	             .and..not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                        str_comp(befehl,'?   ',2,lbef,4)) ) then
          call do_math(line,indxg,length)
c
c------ execute a macro file
c
        elseif(befehl(1:1).eq.'@') then
          call file_kdo(line(2:length),length-1)
c
c------ Calculate average structure and sigmas 'aver'
c
	elseif(str_comp(befehl,'aver',2,lbef,4)) then
	  if (.not. chem_sel_atom) then
	    ier_num = -22
	    ier_typ = ER_CHEM
	  else
	    call chem_aver (.true.)
	  endif
c
c------ Calculate bond-angle distribution 'bang'
c
	elseif(str_comp(befehl,'bang',2,lbef,4)) then
	  if (.not. chem_sel_atom) then
	    ier_num = -22
	    ier_typ = ER_CHEM
	    return
	  endif
c
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.eq.0) then
	    if (ianz.ge.3) then
	      iianz=1
	      jjanz=1
	      kkanz=1
	      call get_iscat (iianz,cpara,lpara,werte,maxw,.false.)
	      call del_params (1,ianz,cpara,lpara,maxw)
	      if (ier_num.eq.0) then
	        call get_iscat (jjanz,cpara,lpara,wwerte,maxw,.false.)  
	        if (ier_num.eq.0) then
	          call del_params (1,ianz,cpara,lpara,maxw)
	          call get_iscat (kkanz,cpara,lpara,uwerte,maxw,.false.)  
	          if (ier_num.eq.0) then
	            call del_params (1,ianz,cpara,lpara,maxw)
	            call do_build_name (ianz,cpara,lpara,wwwerte,maxw,1)
	            chem_fname=cpara(1)
	            if (ier_num.eq.0) then
	              call chem_bang(iianz,jjanz,kkanz,werte,wwerte,
     &	                             uwerte,maxw)
	            endif
	          endif
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------ Calculate bond-length distribution 'blen'
c
	elseif(str_comp(befehl,'blen',2,lbef,4)) then
	  if (.not. chem_sel_atom) then
	    ier_num = -22
	    ier_typ = ER_CHEM
	    return
	  endif
c
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.eq.0) then
	    if (ianz.ge.2) then
	      iianz=1
	      jjanz=1
	      call get_iscat (iianz,cpara,lpara,werte,maxw,.false.)
	      call del_params (1,ianz,cpara,lpara,maxw)
	      if (ier_num.eq.0) then
	        call get_iscat (jjanz,cpara,lpara,wwerte,maxw,.false.)  
	        if (ier_num.eq.0) then
	          call del_params (1,ianz,cpara,lpara,maxw)
	          call do_build_name (ianz,cpara,lpara,wwwerte,maxw,1)
	          chem_fname=cpara(1)
	          if (ier_num.eq.0) then
	            call chem_blen(iianz,jjanz,werte,wwerte,maxw)
	          endif
	        endif
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------ 'bval' calculates the bond valence sum for a given atom
c
	elseif(str_comp(befehl,'bval',2,lbef,4)) then
	  call chem_bval(zeile,lp)
c
c------	continues a macro 'continue'
c
	elseif(str_comp(befehl,'continue',3,lbef,8)) then
	  call macro_continue(zeile,lp)
c
c------ Echo a string, just for interactive check in a macro 'echo'
c
	elseif(str_comp(befehl,'echo',2,lbef,4)) then
	  call echo(zeile,lp)
c
c------ Show relative amounts of elements 'elem'
c
	elseif(str_comp(befehl,'elem',2,lbef,4)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ianz.eq.1) then
	    call do_cap(cpara(1))
	    lout = (cpara(1)(1:2).eq.'ON')
	  else
	    lout = .true.
	  endif
c
	  if (chem_sel_atom) then
	    call chem_elem (lout)
	  else
	    call chem_mole (lout)
	  endif
c
c------ 'env' finds neighbours around given atom, site or position
c
	elseif(str_comp(befehl,'env',2,lbef,3)) then
	  call chem_env(zeile,lp)
c
c------ 'mode' toggles between working with molecules and atoms
c
	elseif(str_comp(befehl,'mode',2,lbef,4)) then
	  call chem_mode(zeile,lp)
c
c------ 'neig' finds neighbours according to given neighbour definition
c
	elseif(str_comp(befehl,'neig',2,lbef,4)) then
	  call chem_nei(zeile,lp)
c
c------ Evaluate an expression 'eval'
c
	elseif(str_comp(befehl,'eval',2,lbef,4)) then
	  call do_eval(zeile,lp)
c
c------	exit 'exit'
c
	elseif(str_comp(befehl,'exit',3,lbef,4)) then
	  goto 9999
c
c------ calculate correlation field
c
	elseif(str_comp(befehl,'field',2,lbef,5)) then
	  call chem_corr_field(zeile,lp)
c
c------ calculate neighbour probabilities 'corr'
c
	elseif(str_comp(befehl,'corr',3,lbef,4)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.ne.0) return
	  if (ianz.eq.3) then
	    if (chem_ctyp(1).eq.CHEM_NONE) then
	      ier_num = -11
	      ier_typ = ER_CHEM
	    else
	      if (str_comp(cpara(1),'occ',1,lpara(1),3)) then
                call del_params (1,ianz,cpara,lpara,maxw)
	        lbeg(1) = -1
	        lbeg(2) =  0
	        lbeg(3) =  0
c
	        if (chem_sel_atom) then
	          call chem_corr_occ(ianz,cpara,lpara,maxw,.true.,lbeg)
	        else
	          call chem_corr_occ_mol
     &	                    (ianz,cpara,lpara,maxw,.true.,lbeg)
	        endif
	      elseif (str_comp(cpara(1),'disp',1,lpara(1),4)) then
                call del_params (1,ianz,cpara,lpara,maxw)
	        if (chem_sel_atom) then
	          call chem_corr_dis(ianz,cpara,lpara,maxw,.true.,lbeg)
	        else
	          call chem_corr_dis_mol
     &	                    (ianz,cpara,lpara,maxw,.true.,lbeg)
	        endif
                call del_params (1,ianz,cpara,lpara,maxw)
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ calculate angles for given neighbours
c
	elseif(str_comp(befehl,'angle',3,lbef,4)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.ne.0) return
	  if (ianz.ge.2) then
	    if (chem_ctyp(1).eq.CHEM_NONE) then
	      ier_num = -11
	      ier_typ = ER_CHEM
	    else
	      if (ianz.gt.2) then
	        call do_build_name (ianz,cpara,lpara,werte,maxw,3)
	      endif
	      if (chem_sel_atom) then
	        call chem_angle(ianz,cpara,lpara,werte,wwerte,wwwerte
     &	                       ,maxw,.true.)
	      endif
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ calculate distortions for given neighbours
c
	elseif(str_comp(befehl,'disp',3,lbef,4)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  if (ier_num.ne.0) return
	  if (ianz.ge.2) then
	    if (chem_ctyp(1).eq.CHEM_NONE) then
	      ier_num = -11
	      ier_typ = ER_CHEM
	    else
	      if (ianz.gt.2) then
	        call do_build_name (ianz,cpara,lpara,werte,maxw,3)
	      endif
	      if (chem_sel_atom) then
	        call chem_disp(ianz,cpara,lpara,werte,maxw,.true.)
	      else
	        call chem_disp_mol(ianz,cpara,lpara,werte,maxw,.true.)
	      endif
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ 'homo' checks how homogeneous the crystal is
c
	elseif(str_comp(befehl,'homo',2,lbef,4)) then
	  call chem_homo(zeile,lp)
c
c------	help 'help','?'
c
	elseif(str_comp(befehl,'help',2,lbef,4) .or.
     &	       str_comp(befehl,'?   ',1,lbef,4)     ) then
	  if(str_comp(zeile,'errors',2,lp,6) ) then
	    lp = lp + 7
	    call do_hel('discus '//zeile,lp)
	  else
	    lp = lp + 12
	    call do_hel('discus chem '//zeile,lp)
	  endif
c
c------ set most parameters for 'chem' section: 'set'
c
	elseif(str_comp(befehl,'set',2,lbef,3)) then
	  call chem_set(zeile,lp)
c
c------ show parameters 'show'
c
	elseif(str_comp(befehl,'show',2,lbef,4)) then
	  zeile = 'all'
	  call chem_show(zeile)
c
c-------Operating System Kommandos 'syst'
c
	elseif(str_comp(befehl,'syst',2,lbef,4)) then
	  if(zeile.ne. ' ') then
	    call do_operating(zeile(1:lp),lp)
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ calc cell-atomindex and atomindex-cell
c
	elseif(str_comp(befehl,'trans',2,lbef,5)) then
	  call get_params (zeile,ianz,cpara,lpara,maxw,lp)
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if (ier_num.eq.0 .and. cr_natoms.ne.0) then
	    if (ianz.eq.1) then
	      ia=nint(werte(1))
	      if (ia.gt.0 .and. ia.le.cr_natoms) then
	        call indextocell (ia,ic,is)
	        write (output_io,3000) ia,ic,is
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
	    elseif (ianz.eq.4) then
	      is=nint(werte(4))
	      do i=1,3
	        ic(i)=nint(werte(i))
	      enddo
	      if (ic(1).gt.0 .and. ic(1).le.cr_icc(1) .and.
     &	          ic(2).gt.0 .and. ic(2).le.cr_icc(2) .and.
     &	          ic(3).gt.0 .and. ic(3).le.cr_icc(3) .and.
     &	          is   .gt.0 .and. is   .le.cr_ncatoms     ) then
	        call celltoindex (ic,is,ia)
	        write (output_io,3000) ia,ic,is
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif  
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif 
c
c------ Waiting for user input
c
	elseif(str_comp(befehl,'wait',3,lbef,4)) then
	  call do_input(zeile,lp)
c
c------ no command found
c
	else
	  ier_num = -8
	  ier_typ = ER_COMM
	endif
	endif
c
c------ any errors ?
c
	if(ier_num.ne.0) then
	  call errlist
	  if(lmakro) then
	    call macro_close
	    prompt_status = PROMPT_ON
	  endif
	  if(lblock) then
	    ier_num = -11
	    ier_typ = ER_COMM
	    return
	  endif
	  call no_error
	endif
	goto 10
c
9999	continue
c
3000	format(' Atomindex ',I6,' : Unitcell ',3(I4,1X),
     &	       ' / site ',I2)
	end
c*****7*****************************************************************
	subroutine chem_show (cmd)
c+
c	show current parameters
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'chem.inc'
	include		'diffuse.inc'
	include		'errlist.inc'
c
	character*200	cmd
	integer		i,j,ic
	logical		lnone
c
	call do_cap(cmd)
c
c------ General settings
c
	if (cmd(1:2).eq.'AL' .or. cmd(1:2).eq.'GE') then
	  write (output_io,1000) chem_quick,chem_period
	  if (chem_sel_atom) then
	    write (output_io,1050) 'atoms'
	  else
	    write (output_io,1050) 'molecules'
	  endif
	  if     (ilots.eq.LOT_OFF) then
	    write (output_io,1100)
	  elseif (ilots.eq.LOT_BOX) then
	    write (output_io,1110) nlots
	    write (output_io,1130) (ls_xyz(i),i=1,3),lperiod
	  elseif (ilots.eq.LOT_ELI) then
	    write (output_io,1120) nlots
	    write (output_io,1130) (ls_xyz(i),i=1,3),lperiod
	  endif
	endif
c
c------ Bond length distribution settings
c	
	if (cmd(1:2).eq.'AL' .or. cmd(1:2).eq.'BO') then
	  write (output_io,2000) chem_blen_cut
	  write (output_io,2100) chem_bin
	endif
c
c------ Correlation determination settings
c
	if (cmd(1:2).eq.'AL' .or. cmd(1:2).eq.'CO') then
	  write (output_io,3000)
	  write (output_io,3010)
c
	  do ic=1,chem_ncor
c
	    if     (chem_ctyp(ic).eq.CHEM_NONE) then
	      write (output_io,3050) 
c
	    elseif (chem_ctyp(ic).eq.CHEM_DIST) then
	      write (output_io,3100) ic,(chem_neig(i,1,ic),i=1,3),
     &	                     chem_freq_sigma(ic),
     &	                     chem_wink_sigma(ic),
     &	                     chem_rmin(ic),chem_rmax(ic),
     &	                     chem_cang(ic),chem_nnei(ic)
	    elseif (chem_ctyp(ic).eq.CHEM_VEC) then
	      write (output_io,3200) ic,(chem_use_vec(i,ic),
     &	                             i=1,chem_nvec(ic))
	    elseif (chem_ctyp(ic).eq.CHEM_ANG) then
	      write (output_io,3300) ic,(chem_use_win(i,ic),
     &	                             i=1,chem_nwin(ic))
	    elseif (chem_ctyp(ic).eq.CHEM_ENVIR) then
	      write (output_io,3400) ic,(chem_use_env(i,ic),
     &	                             i=1,chem_nenv(ic))
	    elseif (chem_ctyp(ic).eq.CHEM_RANGE) then
	      write (output_io,3500) ic,(chem_use_ran(i,ic),
     &	                             i=1,chem_nran(ic))
	    endif
	  enddo
c
c------ - Show defined interaction vectors
c
	  lnone = .true.
	  write (output_io,4000)
	  do i=1,CHEM_MAX_VEC
	    if (chem_cvec(1,i).ne.-9999) then
	      write (output_io,4010) i,(chem_cvec(j,i),j=1,5)
	      lnone = .false.
	    endif
	  enddo
	  if (lnone) write (output_io,3050)
c
c------ - Show defined interaction angles
c
	  lnone = .true.
	  write (output_io,4500)
	  do i=1,CHEM_MAX_ANG
	    if (chem_cwin(1,i).ne.-9999) then
	      write (output_io,4510) i,(chem_cwin(j,i),j=1,9)
	      lnone = .false.
	    endif
	  enddo
	  if (lnone) write (output_io,3050)
c
c------ - Show defined interaction environments
c
	  lnone = .true.
	  write (output_io,4600)
	  do i=1,CHEM_MAX_ENV
	    if (chem_cenv(0,i).ne.-9999) then
	      write (output_io,4610) i, chem_cenv(0,i),
     &	               chem_rmin_env(i),chem_rmax_env(i)
	      write (output_io,4620) (chem_cenv(j,i),j=1,
     &	                                chem_env_neig(i))
	      lnone = .false.
	    endif
	  enddo
	  if (lnone) write (output_io,3050)
c
c------ - Show defined interaction ranges
c
	  lnone = .true.
	  write (output_io,4700)
	  do i=1,CHEM_MAX_RAN
	    if (chem_cran_uvw(1,1,i).ne.-9999) then
	      write (output_io,4710) i,(chem_cran_uvw(j,1,i),j=1,3),
     &	                                chem_cran_sig(i),
     &	                                chem_cran_wsig(i)
	      if(chem_cran_cent(0,i).eq.-1) then
	        write (output_io,4720) 
	      else
	        write (output_io,4730)   (chem_cran_cent(j,i),j=1,
     &	                                  chem_cran_cent(0,i))
	      endif
	      if(chem_cran_neig(0,i).eq.-1) then
	        write (output_io,4740) 
	      else
	        write (output_io,4750)   (chem_cran_neig(j,i),j=1,
     &	                                  chem_cran_neig(0,i))
	      endif
	      if(chem_cran_short(i)) then
	        write (output_io,4760)    chem_cran_nshort(i)
	      endif
	      lnone = .false.
	    endif
	  enddo
	  if (lnone) write (output_io,3050)
c
c------ - Show defined displacement directions
c
	  lnone = .true.
	  write (output_io,5000)
	  do i=1,chem_ncor
	    if (chem_ldall(i)) then
	      write (output_io,5005) i
	      lnone = .false.
	    else
	      if (chem_dir(1,1,i).ne.-9999) then
	        write (output_io,5010) i,(chem_dir(j,1,i),j=1,3),
     &	                                 (chem_dir(j,2,i),j=1,3)
	        lnone = .false.
	      endif
	    endif
	  enddo
	  if (lnone) write (output_io,3050)
	endif
c
1000	format ('    Neighbour determination mode   : quick = ',L1,
     &	        ' / periodic boundaries(x,y,z) = ',3(L1,1x))
1050	format ('    Current operation mode         : ',A)
1100	format ('    Sample volume for homo-check   : complete crystal')
1110	format ('    Sample volume for homo-check   : ',I4,
     &	           ' box shaped lots')
1120	format ('    Sample volume for homo-check   : ',I4,
     &	           ' ellipsoid shaped lots')
1130	format ('    Lot size                       : ',I3,' x ',I3,
     &	        ' x ',I3,' unit cells ',/,
     &	        '    Periodic boundaries            : ',L1)
2000	format ('    Allowed bond length range      : ',
     &	        F6.3,' A to ',F6.3,' A')
2100	format ('    # points for histogramms       : ',I6,/)
3000	format ('    Neighbour definitions          : ',/)
3010 	format ('        #  Mode  Neighbour (or vec.)  fsig  wsig',
     &	        '   rmin[A] rmax[A] ang sym',/,7x,67('-'))
3050	format (7x,'** none defined **')
3100	format (7x,i2,'  neig',2x,3(f5.2,1x),2x,2(f5.2,1x),
     &	        1x,2(f7.3,1x),2x,l1,2x,i2)
3200	format (7x,i2,'  vec ',2x,50i4)
3300	format (7x,i2,'  ang ',2x,50i4)
3400	format (7x,i2,'  env ',2x,50i4)
3500	format (7x,i2,'  ran ',2x,50i4)
4000	format (/,'    Defined correlation vectors    : ',/)
4010	format ('       Correlation vector ',I3,'      : ',
     &	           'Site',I3,' -> site',I3,', neig =',3I3)
4500	format (/,'    Defined correlation angles     : ',/)
4510	format ('       Correlation angle  ',I3,'      : ',
     &	           'Site',I3,' -> site',I3,', neig =',3I3,/,
     &	                 44x,' -> site',I3,', neig =',3I3)
4600	format (/,'    Defined correlation environments: ',/)
4610	format ('       Correlation envir  ',I3,'      : ',
     &	           'Zentral Atom ',I4,/
     &	           37x,'r min, r max : ',f8.3,2x,f8.3)
4620	format (   37x,'neighbours   : ',15I4,/)
4700	format (/,'    Defined correlation ranges: ',/)
4710	format ('       Correlation range ',i3,': ',
     &	           3(F7.3,2x),'+-',F7.3,' A  +-',F7.3,'')
4720	format ('            central atom    ',' ALL')
4730	format ('            central atom    ',50I4)
4740	format ('            neigh.  atom    ',' ALL')
4750	format ('            neigh.  atom    ',50I4)
4760	format ('            Only the nearest',  I4,
     &	        ' atoms are considered neighbours')
5000	format (/,'    Defined displacement direc.    : ',/)
5005	format ('       Displacement corr. #',I3,'     : ',
     &	           'all directions')
5010	format ('       Displacement corr. #',I3,'     : ',
     &	           'A:',3(f5.2,1x),' B:',3(f5.2,1x))
	end
c*****7*****************************************************************
	subroutine chem_set (zeile,lp)
c+
c	sets most parameters for 'chem' section
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'diffuse.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw
	parameter	(maxw=200)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz
	integer		indxx,indxy,indxz
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.eq.0) then
	  if (ianz.ge.2) then
	    call do_cap(cpara(1))
c
c------ --- 'set neig': setting correlation determination method
c
             if(cpara(1)(1:2).eq.'NE') then
               call del_params (1,ianz,cpara,lpara,maxw)
	       call chem_set_neig (ianz,cpara,lpara,werte,maxw)
c
c------ --- 'set cryst': setting crystal dimensions
c
             elseif(cpara(1)(1:2).eq.'CR') then
               call del_params (1,ianz,cpara,lpara,maxw)
               if (ier_num.eq.0) then
                 if (ianz.eq.4) then
                   call ber_params(ianz,cpara,lpara,werte,maxw)
                   cr_icc(1)  = nint(werte(1))
                   cr_icc(2)  = nint(werte(2))
                   cr_icc(3)  = nint(werte(3))
                   cr_ncatoms = nint(werte(4))
                 else
                   ier_num = -6
                   ier_typ = ER_COMM
                 endif
               endif
c
c------	--- 'set lots': sample vloume (lots)
c
	     elseif(cpara(1)(1:3).eq.'LOT') then
               call del_params (1,ianz,cpara,lpara,maxw)
               if (ier_num.ne.0) return
	       if (ianz.eq.1) then
	         call do_cap (cpara(1))
	         if (cpara(1)(1:1).eq.'O') then
	           ilots = LOT_OFF
	           nlots = 1
	         else
	           ier_num = -6
	           ier_typ = ER_COMM
	         endif
	       elseif (ianz.eq.6) then
	         call do_cap (cpara(1))
	         if (cpara(1)(1:1).eq.'B') then
	           ilots = LOT_BOX
	         elseif (cpara(1)(1:1).eq.'E') then
	           ilots = LOT_ELI
	         else
	           ier_num = -2
	           ier_typ = ER_FOUR
	         endif
	         if (ier_num.eq.0) then
	           call del_params (1,ianz,cpara,lpara,maxw)
	           call ber_params(ianz-1,cpara,lpara,werte,maxw)
	           if (ier_num.eq.0) then
	             ls_xyz(1) = nint(werte(1))
	             ls_xyz(2) = nint(werte(2))
	             ls_xyz(3) = nint(werte(3))
	             nlots     = nint(werte(4))
	             call do_cap(cpara(5))
	             lperiod = (cpara(5)(1:1).eq.'Y')
	           endif
	         endif
	       else
	         ier_num = -6
	         ier_typ = ER_COMM
	       endif
c
c------ --- set mode: set calculation mode to quick/exact
c
	    elseif (cpara(1)(1:2).eq.'MO') then
	      call do_cap(cpara(2))
	      chem_quick = (cpara(2)(1:3).eq.'QUI')
	      if (ianz.ge.3) then
	        call do_cap(cpara(3))
	        if(cpara(3)(1:3).eq.'PER') then
	          if (ianz.eq.4) then
	            call do_cap(cpara(4))
	            indxx = index(cpara(4),'X')
	            indxy = index(cpara(4),'Y')
	            indxz = index(cpara(4),'Z')
	            chem_period(1) = indxx.gt.0
	            chem_period(2) = indxy.gt.0
	            chem_period(3) = indxz.gt.0
	          else
	            chem_period(1) = .true.
	            chem_period(2) = .true.
	            chem_period(3) = .true.
	          endif
	        else
	          chem_period(1) = .false.
	          chem_period(2) = .false.
	          chem_period(3) = .false.
	        endif
	      endif
c
c------ --- set bin: set number of points for histogramm binning
c
	    elseif (cpara(1)(1:2).eq.'BI') then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      call ber_params (ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      if (ianz.eq.1) then
	        if (nint(werte(1)).lt.chem_max_bin) then
	          chem_bin = nint(werte(1))
	        else
	          ier_num = -1
	          ier_typ = ER_CHEM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ --- set blen: set allowed range for bond-length histogramm
c
	    elseif (cpara(1)(1:2).eq.'BL') then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      if (ianz.eq.2) then
	        if (werte(1).gt.0.01 .and. werte(2).gt.werte(1)) then
	          chem_blen_cut(1)=werte(1)
	          chem_blen_cut(2)=werte(2)
	        else
	          ier_num = -3
	          ier_typ = ER_CHEM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ --- set bang: set allowed range for bond-angle histogramm
c
	    elseif (cpara(1)(1:2).eq.'BA') then
	      call del_params (1,ianz,cpara,lpara,maxw)
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      if (ianz.eq.2) then
	        if (werte(1).gt.0.00 .and. werte(2).gt.werte(1)) then
	          chem_bang_cut(1)=werte(1)
	          chem_bang_cut(2)=werte(2)
	        else
	          ier_num = -25
	          ier_typ = ER_CHEM
	        endif
	      else
	        ier_num = -6
	        ier_typ = ER_COMM
	      endif
c
c------ --- set ang: sets correlation angles
c
	    elseif (cpara(1)(1:2).eq.'AN') then
              call del_params (1,ianz,cpara,lpara,maxw)
	      call chem_set_angle (ianz,cpara,lpara,werte,maxw)
c
c------ --- set vec: sets correlation vectors
c
	    elseif (cpara(1)(1:2).eq.'VE') then
              call del_params (1,ianz,cpara,lpara,maxw)
	      call chem_set_vec (ianz,cpara,lpara,werte,maxw)
c
c------ --- set env: sets correlation environment
c
	    elseif (cpara(1)(1:2).eq.'EN') then
              call del_params (1,ianz,cpara,lpara,maxw)
	      call chem_set_envir (ianz,cpara,lpara,werte,maxw)
c
c------ --- unknown subcommand entered
c
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_set_vec (ianz,cpara,lpara,werte,maxw)
c+
c	'set vec' is processed here
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz,is1,is2,iv,i,j
c
	logical		str_comp
c
	if(str_comp(cpara(1),'rese',2,lpara(1),4)) then
	  do i=1,CHEM_MAX_VEC
	    chem_cvec(1,i) = -9999
	    do j=2,5
	      chem_cvec(j,i) = 0
	    enddo
	    do j=1,CHEM_MAX_COR
	      chem_use_vec(i,j) = 1
	    enddo
	  enddo
	  return
	endif
c
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
	if (ianz.eq.6) then
	  iv  = nint(werte(1))
	  is1 = nint(werte(2))
	  is2 = nint(werte(3))
	  if (iv.le.0 .or. iv.gt.CHEM_MAX_VEC) then
	    ier_num = -9
	    ier_typ = ER_CHEM
	  else
	    if (is1.le.0 .or. is1.gt.cr_ncatoms .or.
     &	        is2.le.0 .or. is2.gt.cr_ncatoms     ) then
	      ier_num = -10
	      ier_typ = ER_CHEM
	    else
	      chem_cvec(1,iv) = werte(2)
	      chem_cvec(2,iv) = werte(3)
	      chem_cvec(3,iv) = werte(4)
	      chem_cvec(4,iv) = werte(5)
	      chem_cvec(5,iv) = werte(6)
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_set_ranges (ianz,cpara,lpara,werte,maxw)
c+
c	'set range' is processed here
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		max_uvw
	parameter	(max_uvw=48)
	integer		maxw
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz,is1,is2,iv,i,j,k
	integer		janz
	logical		lold
	logical		lacentric
	parameter	(lold = .true.)
c
	real		u(3),v(3)
	real		uvw(4,max_uvw)
	real		uvw_mat(4,4,max_uvw)
c
	logical		str_comp
	real		do_blen
c
	if(str_comp(cpara(1),'rese',2,lpara(1),4)) then
	  do i=1,CHEM_MAX_RAN
	    do k=1,48
	      do j=1,3
	        chem_cran_uvw(j,k,i) = 0
	      enddo
	    enddo
	    chem_cran_uvw(1,1,i) = -9999
	    chem_cran_sig(i)  = 0.00
	    chem_cran_wsig(i) = 0.00
	    do j=1,CHEM_MAX_COR
	      chem_use_ran(i,j) = 1
	    enddo
	    chem_cran_cent(0,i) = 0
	    chem_cran_neig(0,i) = 0
	  enddo
	  return
	elseif(str_comp(cpara(1),'direc',2,lpara(1),5)) then
c
c-----	--Define the direction, sigma of direction and sigma of angle
c
	  call del_params (1,ianz,cpara,lpara,maxw)
	  call ber_params (ianz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (ianz.gt.3) then
	    iv  = nint(werte(1))
c
	    chem_cran_cang(iv) = .false.
	    chem_cran_lsym(iv) = .false.
c
c------	----Is last parameter "sym" or "nosym" ??
c
	    if(str_comp(cpara(ianz),'sym',3,lpara(ianz),3)) then
	      chem_cran_lsym(iv) = .true.
	      ianz = ianz - 1
	    elseif(str_comp(cpara(ianz),'nosym',3,lpara(ianz),5)) then
	      chem_cran_lsym(iv) = .false.
	      ianz = ianz - 1
	    endif
c
	    chem_cran_uvw(1,1,iv) = werte(2)
	    chem_cran_uvw(2,1,iv) = werte(3)
	    chem_cran_uvw(3,1,iv) = werte(4)
	    chem_cran_sig(iv)     = werte(5)
	    if(ianz.eq.6) then
	      chem_cran_wsig(iv) = werte(6)
	      chem_cran_cang(iv) = .true.
	    endif
c
	    do i=1,3
	      u(i) = 0.0
	      v(i) = chem_cran_uvw(i,1,iv)
	    enddo
	    chem_cran_rmax(iv) = do_blen(.true.,u,v)
	    chem_cran_rmin(iv) = chem_cran_rmax(iv) - 
     &	                         chem_cran_sig(iv)/2.0
	    chem_cran_rmax(iv) = chem_cran_rmax(iv) + 
     &	                         chem_cran_sig(iv)/2.0
c
c------ - get symmetrically equivalent directions if needed
c
	    if (chem_cran_lsym(iv)) then
	      do i=1,3
	        uvw(i,1)=chem_cran_uvw(i,1,iv)
	      enddo
              uvw(4,1) = 0.0
              do i=1,4
                do j=1,4
                  uvw_mat(i,j,1)=0.0
                enddo
                uvw_mat(i,i,1)=1.0
              enddo
c
              call rmc_symmetry (chem_cran_nuvw(iv),uvw,uvw_mat,
     &	                         max_uvw,.true.,lacentric)
c
	      do i=1,chem_cran_nuvw(iv)
	        do j=1,3
	          chem_cran_uvw(j,i,iv) = uvw(j,i)
	        enddo
	      enddo
c
	    else
	      chem_cran_nuvw(iv) = 1
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	elseif(str_comp(cpara(1),'central',2,lpara(1),7)) then
c
c------	--Define the central atom(s)
c
	  call del_params (1,ianz,cpara,lpara,maxw)
	  janz = 1
	  call ber_params (janz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (ianz.ge.2) then
	    iv  = nint(werte(1))
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call get_iscat(ianz,cpara,lpara,werte,maxw,lold)
	    if(ier_num.ne.0) return
	    if(werte(1).eq.-1) then
	      chem_cran_cent(0,iv) = -1
	    else
	      chem_cran_cent(0,iv) = ianz
	      do i=1,ianz
	        chem_cran_cent(i,iv) = nint(werte(i))
	      enddo
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	elseif(str_comp(cpara(1),'neig',2,lpara(1),4)) then
c
c------	--Define the neighboring atom(s)
c
	  call del_params (1,ianz,cpara,lpara,maxw)
	  janz = 1
	  call ber_params (janz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (ianz.ge.2) then
	    iv  = nint(werte(1))
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call get_iscat(ianz,cpara,lpara,werte,maxw,lold)
	    if(ier_num.ne.0) return
	    if(werte(1).eq.-1) then
	      chem_cran_neig(0,iv) = -1
	    else
	      chem_cran_neig(0,iv) = ianz
	      do i=1,ianz
	        chem_cran_neig(i,iv) = nint(werte(i))
	      enddo
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	elseif(str_comp(cpara(1),'short',2,lpara(1),5)) then
c
c------	--Define the number of the closest atoms to be considered
c
	  call del_params (1,ianz,cpara,lpara,maxw)
	  janz = 1
	  call ber_params (janz,cpara,lpara,werte,maxw)
	  if (ier_num.ne.0) return
	  if (ianz.eq.2) then
	    iv  = nint(werte(1))
	    if(str_comp(cpara(2),'none',2,lpara(1),4)) then
	      chem_cran_short(iv)  = .false.
	      chem_cran_nshort(iv) = -1
	    else
	      call ber_params (ianz,cpara,lpara,werte,maxw)
	      if (ier_num.ne.0) return
	      chem_cran_short(iv)  = .true.
	      chem_cran_nshort(iv) = nint(werte(ianz))
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	  ier_msg(1) = cpara(1)
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_set_angle (ianz,cpara,lpara,werte,maxw)
c+
c	'set angle' is processed here
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz,is1,is2,is3,iv,i,j
c
	logical		str_comp
c
	if(str_comp(cpara(1),'rese',2,lpara(1),4)) then
	  do i=1,CHEM_MAX_ANG
	    chem_cwin(1,i) = -9999
	    do j=2,5
	      chem_cwin(j,i) = 0
	    enddo
	    do j=1,CHEM_MAX_COR
	      chem_use_win(i,j) = 1
	    enddo
	  enddo
	  return
	endif
c
c
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
	if (ianz.eq.10) then
	  iv  = nint(werte(1))
	  is1 = nint(werte(2))
	  is2 = nint(werte(3))
	  is3 = nint(werte(7))
	  if (iv.le.0 .or. iv.gt.CHEM_MAX_ANG) then
	    ier_num = -24
	    ier_typ = ER_CHEM
	  else
	    if (is1.le.0 .or. is1.gt.cr_ncatoms .or.
     &	        is2.le.0 .or. is2.gt.cr_ncatoms .or.
     &	        is3.le.0 .or. is3.gt.cr_ncatoms     ) then
	      ier_num = -10
	      ier_typ = ER_CHEM
	    else
	      chem_cwin(1,iv) = werte(2)
	      chem_cwin(2,iv) = werte(3)
	      chem_cwin(3,iv) = werte(4)
	      chem_cwin(4,iv) = werte(5)
	      chem_cwin(5,iv) = werte(6)
	      chem_cwin(6,iv) = werte(7)
	      chem_cwin(7,iv) = werte(8)
	      chem_cwin(8,iv) = werte(9)
	      chem_cwin(9,iv) = werte(10)
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_set_envir (ianz,cpara,lpara,werte,maxw)
c+
c	'set environment' is processed here
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz,janz,iv,i,j,k
c
	logical		str_comp
c
	if(str_comp(cpara(1),'rese',2,lpara(1),4)) then
	  do i=1,CHEM_MAX_ENV
	    chem_cenv(0,i) = -9999
	    do j=1,MAX_ATOM_ENV
	      chem_cenv(j,i) = 0
	    enddo
	    do j=1,CHEM_MAX_COR
	      chem_use_env(i,j) = 1
	      chem_env_neig(j)  = 0
	    enddo
	  enddo
	  return
	endif
c
c
	janz = 3
	call ber_params (janz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
	iv  = nint(werte(1))
	if (iv.le.0 .or. iv.gt.CHEM_MAX_ENV) then
	  ier_num = -26
	  ier_typ = ER_CHEM
	  return
	endif
c
	chem_rmin_env(iv) = werte(2)
	chem_rmax_env(iv) = werte(3)
	call del_params(3,ianz,cpara,lpara,maxw)
	chem_env_neig(iv) = ianz - 1
	janz = ianz
	do i=1,janz
	  j = 1
	  call get_iscat(j,cpara,lpara,werte,maxw,.false.)
	  chem_cenv(i-1,iv) = nint(werte(1))
	  call del_params(1,ianz,cpara,lpara,maxw)
	enddo
c
	end
c*****7*****************************************************************
	subroutine chem_set_neig (ianz,cpara,lpara,werte,maxw)
c+
c	Command 'set neig' processed here
c-
	implicit	none
c
	include		'config.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		max_uvw
	parameter	(max_uvw=48)
c
	integer		maxw
c
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw)
	integer		ianz
	integer		i,j,iv
	real		u(3),v(3)
	real		uvw(4,max_uvw)
	real		uvw_mat(4,4,max_uvw)
	logical		lacentric,csym
c
	real		do_blen
c
	call do_cap(cpara(1))
c
c------ set neig,add : Add neighbour definition to list
c
	if (cpara(1)(1:2).eq.'AD' .and. ianz.eq.1) then
	  if (chem_ncor.lt.CHEM_MAX_COR) then
            chem_ncor = chem_ncor+1
	  else
	    ier_num = -12
	    ier_typ = ER_CHEM
	  endif
c
c------ set neig,rese : Reset neighbour list
c
	elseif (cpara(1)(1:2).eq.'RE') then
	  chem_ncor=1
	  chem_nvec(1)=0
	  chem_nwin(1)=0
	  chem_nran(1)=0
c
c------ set neig,vec : Define neighbour via vectors
c
	elseif (cpara(1)(1:2).eq.'VE' .and. ianz.gt.1) then
          call del_params (1,ianz,cpara,lpara,maxw)
          call ber_params (ianz,cpara,lpara,werte,maxw)
          if (ier_num.ne.0) return
	  do i=1,ianz
	    iv = nint(werte(i))
	    if (iv.gt.0 .and. iv.le.CHEM_MAX_VEC) then
	      if (chem_cvec(1,iv).ne.-9999) then
	        chem_use_vec(i,chem_ncor)=iv
	      else
	        ier_num = -9
	        ier_typ = ER_CHEM
	      endif
	    else
	      ier_num = -9
	      ier_typ = ER_CHEM
	    endif
	  enddo
	  if (ier_num.ne.0) return
	  chem_nvec(chem_ncor)=ianz
	  chem_ctyp(chem_ncor)=CHEM_VEC
c
c------ set neig,ran : Define neighbour via ranges
c
	elseif (cpara(1)(1:2).eq.'RA' .and. ianz.gt.1) then
          call del_params (1,ianz,cpara,lpara,maxw)
          call ber_params (ianz,cpara,lpara,werte,maxw)
          if (ier_num.ne.0) return
	  do i=1,ianz
	    iv = nint(werte(i))
	    if (iv.gt.0 .and. iv.le.CHEM_MAX_RAN) then
	      if (chem_cran_uvw(1,1,iv).ne.-9999) then
	        chem_use_ran(i,chem_ncor)=iv
	      else
	        ier_num = -9
	        ier_typ = ER_CHEM
	      endif
	    else
	      ier_num = -9
	      ier_typ = ER_CHEM
	    endif
	  enddo
	  if (ier_num.ne.0) return
	  chem_nran(chem_ncor)=ianz
	  chem_ctyp(chem_ncor)=CHEM_RANGE
c
c------ set neig,ang : Define neighbour via angles
c
	elseif (cpara(1)(1:2).eq.'AN' .and. ianz.gt.1) then
          call del_params (1,ianz,cpara,lpara,maxw)
          call ber_params (ianz,cpara,lpara,werte,maxw)
          if (ier_num.ne.0) return
	  do i=1,ianz
	    iv = nint(werte(i))
	    if (iv.gt.0 .and. iv.le.CHEM_MAX_ANG) then
	      if (chem_cwin(1,iv).ne.-9999) then
	        chem_use_win(i,chem_ncor)=iv
	      else
	        ier_num = -9
	        ier_typ = ER_CHEM
	      endif
	    else
	      ier_num = -9
	      ier_typ = ER_CHEM
	    endif
	  enddo
	  if (ier_num.ne.0) return
	  chem_nwin(chem_ncor)=ianz
	  chem_ctyp(chem_ncor)=CHEM_ANG
c
c------ set neig,env : Define neighbour via environment
c	XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	elseif (cpara(1)(1:3).eq.'ENV' .and. ianz.gt.1) then
          call del_params (1,ianz,cpara,lpara,maxw)
          call ber_params (ianz,cpara,lpara,werte,maxw)
          if (ier_num.ne.0) return
	  do i=1,ianz
	    iv = nint(werte(i))
	    if (iv.gt.0 .and. iv.le.CHEM_MAX_ENV) then
	      if (chem_cenv(0,iv).ne.-9999) then
	        chem_use_env(i,chem_ncor)=iv
	      else
	        ier_num = -9
	        ier_typ = ER_CHEM
	      endif
	    else
	      ier_num = -9
	      ier_typ = ER_CHEM
	    endif
	  enddo
	  if (ier_num.ne.0) return
	  chem_nenv(chem_ncor)=ianz
	  chem_ctyp(chem_ncor)=CHEM_ENVIR
c
c------ set neig,dis : Define neighbour via distance
c
	elseif (cpara(1)(1:3).eq.'DIS' .and. ianz.ge.5) then
          call del_params (1,ianz,cpara,lpara,maxw)
          if (ier_num.ne.0) return
c
	  chem_cang(chem_ncor) = .false.
	  chem_ctyp(chem_ncor) = CHEM_DIST
c
	  csym = .false.
	  if (ianz.eq.5 .or. ianz.eq.6) then
	    call do_cap(cpara(ianz))
	    csym     = (cpara(ianz)(1:1).eq.'S')
	    ianz     = ianz - 1
	  endif
c
          call ber_params (ianz,cpara,lpara,werte,maxw)
c
	  chem_neig(1,1,chem_ncor)   = werte(1)
	  chem_neig(2,1,chem_ncor)   = werte(2)
	  chem_neig(3,1,chem_ncor)   = werte(3)
	  chem_freq_sigma(chem_ncor) = werte(4)
c
	  if (ianz.eq.5) then
	    chem_cang(chem_ncor)       = .true.
	    chem_wink_sigma(chem_ncor) = werte(5)
	  endif
c
	  do i=1,3
	    u(i) = 0.0
	    v(i) = chem_neig(i,1,chem_ncor)
	  enddo
	  chem_rmax(chem_ncor) = do_blen(.true.,u,v)
	  chem_rmin(chem_ncor) = chem_rmax(chem_ncor) - 
     &	                         chem_freq_sigma(chem_ncor)/2.0
	  chem_rmax(chem_ncor) = chem_rmax(chem_ncor) + 
     &	                         chem_freq_sigma(chem_ncor)/2.0
c
c------ - get symmetrically equivalent directions if needed
c
	  if (csym) then
	    do i=1,3
	      uvw(i,1)=chem_neig(i,1,chem_ncor)
	    enddo
            uvw(4,1) = 0.0
            do i=1,4
              do j=1,4
                uvw_mat(i,j,1)=0.0
              enddo
              uvw_mat(i,i,1)=1.0
            enddo
c
            call rmc_symmetry (chem_nnei(chem_ncor),uvw,uvw_mat,
     &	                       max_uvw,.true.,lacentric)
c
	    do i=1,chem_nnei(chem_ncor)
	      do j=1,3
	        chem_neig(j,i,chem_ncor) = uvw(j,i)
	      enddo
	    enddo
c
	  else
	    chem_nnei(chem_ncor) = 1
	  endif
c
c------ set neig,dir: sets directions for disp. correlations 
c
	elseif (cpara(1)(1:3).eq.'DIR') then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  if (cpara(1)(1:3).eq.'all') then
	    chem_ldall(chem_ncor) = .true.
	  else
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if (ier_num.ne.0) return
c
	    if (ianz.eq.3) then
	      chem_ldall(chem_ncor)   = .false.
	      chem_dir(1,1,chem_ncor) = werte(1)
	      chem_dir(2,1,chem_ncor) = werte(2)
	      chem_dir(3,1,chem_ncor) = werte(3)
	      chem_dir(1,2,chem_ncor) = werte(1)
	      chem_dir(2,2,chem_ncor) = werte(2)
	      chem_dir(3,2,chem_ncor) = werte(3)
	    elseif (ianz.eq.6) then
	      chem_ldall(chem_ncor)   = .false.
	      chem_dir(1,1,chem_ncor) = werte(1)
	      chem_dir(2,1,chem_ncor) = werte(2)
	      chem_dir(3,1,chem_ncor) = werte(3)
	      chem_dir(1,2,chem_ncor) = werte(4)
	      chem_dir(2,2,chem_ncor) = werte(5)
	      chem_dir(3,2,chem_ncor) = werte(6)
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
	  endif
c
c------ unknown subcommand
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_env(line,laenge)
c-
c	Finds the environment around an atom, site or position
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'chem.inc'
	include 	'param.inc'
	include 	'atom_env.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=6)
c
	character*(*)	line
	character*200	cpara(maxw)
	character*9	at_name,at_name_i
	integer		lpara(maxw)
	integer		i,ianz,laenge
	integer		iatom,isite,icell(3)
	real		werte(maxw),dummy
	real		radius,pos(3)
	logical		latom
c
	logical		str_comp
c
	latom = .false.
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.ne.0) return
c
c------ Given atom index
c
	if(str_comp(cpara(1),'atom',1,lpara(1),4)) then
	  if (ianz.eq.3) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    iatom = nint(werte(1))
	    if (iatom.gt.0 .and. iatom.le.cr_natoms) then
	      radius = werte(2)
	      call indextocell (iatom,icell,isite)
	      do i=1,3
	        pos(i) = cr_pos(i,iatom)
	      enddo
	      latom = .true.
	    else
	      ier_num = -10
	      ier_typ = ER_CHEM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ Given unit cell and site
c
	elseif(str_comp(cpara(1),'site',1,lpara(1),4)) then
	  if (ianz.eq.6) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    do i=1,3
	      icell(i) = nint(werte(i))
	    enddo
	    isite = nint(werte(4))
	    call celltoindex (icell,isite,iatom)
	    if (iatom.gt.0 .and. iatom.le.cr_natoms) then
	      radius = werte(5)
	      do i=1,3
	        pos(i) = cr_pos(i,iatom)
	      enddo
	      latom = .true.
	    else
	      ier_num = -10
	      ier_typ = ER_CHEM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ Given position
c
	elseif(str_comp(cpara(1),'pos',1,lpara(1),3)) then
	  if (ianz.eq.5) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    do i=1,3
	      pos(i) = werte(i)
	    enddo
	    radius = werte(4)
	    latom = .false.
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ No valid subcommand
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
c---------------------------------------------------------------------
c------ Find neighbours
c---------------------------------------------------------------------
c
	if (ier_num.ne.0) return
c
	dummy = -1
	call do_find_env(1,dummy,1,pos,0.01,radius,
     &	                 chem_quick,chem_period)
	if (ier_num.ne.0) return
c
	if (atom_env(0).gt.0) then
	  write (output_io,1000) chem_quick,chem_period
	  write (output_io,1100) (pos(i),i=1,3),radius
	  if (latom) then
	    at_name_i = at_name (cr_iscat(iatom))
     	    write (output_io,1200) (icell(i),i=1,3),isite,iatom,at_name_i
	  endif
	  write (output_io,1300)
	  do ianz=1,atom_env(0)
	    at_name_i = at_name (cr_iscat(atom_env(ianz)))
	    call indextocell (atom_env(ianz),icell,isite)
	    write (output_io,1400) ianz,atom_env(ianz),at_name_i,
     &	           res_para(ianz),(cr_pos(i,atom_env(ianz)),i=1,3),
     &	           icell,isite
	    res_para(ianz)=float(atom_env(ianz))
	  enddo
	  res_para(0) = atom_env(0)
	endif
c
1000	format (  ' Found neighbours (quick more = ',L1,
     &	          ' periodic boundaries (x,y,z) = ',3(L1,1x),') ')
1100	format (  '   Position of search center   : ',3(F9.3,2X),/
     &	          '   Search radius [A]           : ',F9.3)
1200	format (  '   Unit cell and site          : ',3(I4,1X),'/',I4,/
     &	          '   Atom index                  : ',I9,' (',A9,')')
1300	format (/,4X,'#',3X,'atom',4X,'name',5X,'dist [A]',12x,'pos',
     &	             16x,'cell',5X,'site',/,3X,74('-'))
1400	format (3X,I3,1X,I6,3X,A9,1X,F7.2,1X,3(F7.2,1X),
     &	        1X,3(I3,1X),1X,I3)
c
	end
c*****7*****************************************************************
	subroutine chem_bval(line,laenge)
c-
c	Calculates the bond valence sum for specified site/atom
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'chem.inc'
	include 	'param.inc'
	include 	'atom_env.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=6)
c
	character*(*)	line
	character*200	cpara(maxw)
	character*9	at_name,at_name_i
	integer		lpara(maxw)
	integer		i,iii,ianz,laenge
	integer		iatom,isite,icell(3)
	integer		elem_1,elem_2,ie_1,ie_2
	real		werte(maxw),dummy
	real		radius,bval,pos(3)
	logical		latom
c
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.ne.0) return
c
c------ Given atom index
c
	if(str_comp(cpara(1),'atom',1,lpara(1),4)) then
	  if (ianz.eq.3) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    iatom = nint(werte(1))
	    if (iatom.gt.0 .and. iatom.le.cr_natoms) then
	      radius = werte(2)
	      call indextocell (iatom,icell,isite)
	      do i=1,3
	        pos(i) = cr_pos(i,iatom)
	      enddo
	      latom = .true.
	    else
	      ier_typ = -10
	      ier_num = ER_CHEM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ Given unit cell and site
c
	elseif(str_comp(cpara(1),'site',1,lpara(1),4)) then
	  if (ianz.eq.6) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    do i=1,3
	      icell(i) = nint(werte(i))
	    enddo
	    isite = nint(werte(4))
	    call celltoindex (icell,isite,iatom)
	    if (iatom.gt.0 .and. iatom.le.cr_natoms) then
	      radius = werte(5)
	      do i=1,3
	        pos(i) = cr_pos(i,iatom)
	      enddo
	      latom = .true.
	    else
	      ier_typ = -10
	      ier_num = ER_CHEM
	    endif
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
c------ No valid subcommand
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
c---------------------------------------------------------------------
c------ Find neighbours and calculate BV sum
c---------------------------------------------------------------------
c
  	read(cr_at_lis(cr_iscat(iatom)),2000) elem_1
	call symbf(elem_1,ie_1)
	if (ie_1.eq.0) then
	  ier_num = -75
	  ier_typ = ER_APPL
	endif
	if (ier_num.ne.0) return
c
	dummy = -1
	call do_find_env(1,dummy,1,pos,0.01,radius,
     &	                 chem_quick,chem_period)
	if (ier_num.ne.0) return
c
	if (atom_env(0).gt.0) then
	  bval = 0.0
	  do ianz=1,atom_env(0)
  	    read(cr_at_lis(cr_iscat(atom_env(ianz))),2000) elem_2
	    call symbf(elem_2,ie_2)
	    if (ie_2.eq.0 .or. bv_index(ie_1,ie_2).eq.0) then
	      ier_num = -75
	      ier_typ = ER_APPL
	      return
	    else
	      iii  = bv_index(ie_1,ie_2)
	      bval = bval + exp((bv_r0(iii)-res_para(ianz))/bv_b(iii))
	    endif
	  enddo
	  res_para(0) = 1 
	  res_para(1) = bval
	  at_name_i   = at_name (cr_iscat(iatom))
	  write (output_io,1000) at_name_i,bval
	else
	  ier_num = -8
	  ier_typ = ER_CHEM
	endif
c
1000	format(' Bond valence sum for ',a9,' : ',f7.4)
2000	format(a4)
c
	end
c*****7*****************************************************************
	subroutine chem_mode(line,laenge)
c-
c	Sets molecule/atom mode
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'chem.inc'
	include 	'molecule.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw = 2)
c
	character*(*)	line
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,laenge
c
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.ne.0) return
c
	if    (str_comp(cpara(1),'mol', 1,lpara(1),3)) then
	  if (mole_num_mole.gt.0) then
	    chem_sel_atom = .false.
	    write (output_io,1000) 'molecules'
	  else
	    ier_num = -20
	    ier_typ = ER_CHEM
	  endif
c
	elseif(str_comp(cpara(1),'atom',1,lpara(1),4)) then
	  chem_sel_atom = .true.
	  write (output_io,1000) 'atoms'
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1000	format ( ' New operation mode : ',a,' ...')
	end
c*****7*****************************************************************
	subroutine chem_nei(line,laenge)
c-
c	Finds the environment for given neighbour definition
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'chem.inc'
	include 	'molecule.inc'
	include 	'param.inc'
	include 	'errlist.inc'
c
	integer		maxw,maxatom
	parameter	(maxw    = 3)
	parameter	(maxatom = chem_max_neig)
c
	character*(*)	line
	character*200	cpara(maxw)
	character*9	at_name,at_name_i
	real		werte(maxw)
	real		pos(3,0:maxatom)
	integer		lpara(maxw)
	integer		ind(0:maxatom),iatom,imol
	integer		i,j,n,ic,ianz,laenge
c
	call get_params(line,ianz,cpara,lpara,maxw,laenge)
	if(ier_num.ne.0) return
c
	if (ianz.eq.2) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)
	  if(ier_num.ne.0) return
c
c------ - Atoms
c
	  if (chem_sel_atom) then
	    iatom = nint(werte(1))
	    if (iatom.le.0 .or. iatom.gt.cr_natoms) then
	      ier_num = -10
	      ier_typ = ER_CHEM
	    endif
c
	    ic = nint(werte(2))
	    if (ic.le.0 .or. ic.gt.chem_ncor) then
	      ier_num = -14
	      ier_typ = ER_CHEM
	    endif
c
	    if (chem_ctyp(1).eq.CHEM_NONE) then
	      ier_num = -11
	      ier_typ = ER_CHEM
	    endif
	    if(ier_num.ne.0) return
c
	    call chem_neighbour(iatom,ic,ind,pos,n,maxatom)
c
	    if (n.gt.0) then
	      at_name_i = at_name(cr_iscat(iatom))
	      write (output_io,1000) at_name_i,
     &	                          (cr_pos(i,iatom),i=1,3),iatom,ic
	      write (output_io,1100)
	      do i=1,n
	        at_name_i = at_name(cr_iscat(ind(i)))
	        write (output_io,1200) i,at_name_i,ind(i),
     &	                              (cr_pos(j,ind(i)),j=1,3)
	      enddo
c
	    else
	      ier_num = -8
	      ier_typ = ER_CHEM
	    endif
c
c------ - Molecules
c
	  else
	    imol = nint(werte(1))
	    if (imol.le.0 .or. imol.gt.mole_num_mole) then
	      ier_num = -63
	      ier_typ = ER_APPL
	    endif
c
	    ic = nint(werte(2))
	    if (ic.le.0 .or. ic.gt.chem_ncor) then
	      ier_num = -14
	      ier_typ = ER_CHEM
	    endif
c
	    if (chem_ctyp(1).eq.CHEM_NONE) then
	      ier_num = -11
	      ier_typ = ER_CHEM
	    endif
	    if(ier_num.ne.0) return
c
	    call chem_neighbour_mol(imol,ic,ind,n,maxatom)
c
	    if (n.gt.0) then
	      write (output_io,2000) 
     &	           (cr_pos(i,mole_cont(mole_off(imol)+1)),i=1,3),
     &	            imol,mole_cont(mole_off(imol)+1),ic
	      write (output_io,2100)
	      do i=1,n
	        write (output_io,2200) i,mole_type(ind(i)),ind(i),
     &	                      cr_pos(1,mole_cont(mole_off(ind(i))+1)),
     &	                      cr_pos(2,mole_cont(mole_off(ind(i))+1)),
     &	                      cr_pos(3,mole_cont(mole_off(ind(i))+1))
	      enddo
	    else
	      ier_num = -19
	      ier_typ = ER_CHEM
	    endif
	  endif
c
c------ Wrong number of parameters
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
c------ store results in res_para
c
	if (n.gt.maxpar_res) then
	  ier_typ = ER_CHEM
	  ier_num = -2
	else
	  res_para(0)=n
	  do i=1,n
	    res_para(i) = float(ind(i))
	  enddo 
	endif
c
1000	format (  ' Neighbours found in defined direction ',/
     &	          '   Center atom name            : ',a9,/
     &	          '   Position of center atom     : ',3(f8.3,2x),/
     &	          '   Center atom index           : ',i7,/
     &	          '   Neighbour definition no.    : ',i7,/)
1100	format (  4x,'#',6x,'name',7x,'index',16x,'position',/
     &	          3x,(58('-')))
1200	format (  2x,i3,3x,a9,3x,i9,4x,3(f8.3,2x))
c
2000	format (  ' Neighbours found in defined direction ',/
     &	          '   Origin of center molecule   : ',3(f8.3,2x),/
     &	          '   Center molecule index       : ',i7,/
     &	          '   Atom index on origin of mol.: ',i7,/
     &	          '   Neighbour definition no.    : ',i7,/)
2100	format (  4x,'#',6x,'type',7x,'index',17x,'origin',/
     &	          3x,(58('-')))
2200	format (  2x,i3,3x,i9,3x,i9,4x,3(f8.3,2x))
	end
c*****7*****************************************************************
	subroutine chem_aver (lout)
c+
c	Calculate average structure and standard deviation
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'param.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	real		p(3),ez(3)
	integer		i,j,k,ii,jj,kk,ia,is
	logical 	flag,lout
c
	character*9	at_name_i
	character*9	at_name
c
c------ reset counters
c
	if(cr_ncatoms.gt.MAXAT_CELL) then
	  ier_num = -26
	  ier_typ = ER_APPL
	  return
	endif
	do i=1,cr_ncatoms
	  chem_ave_n(i)=0
	  do k=1,chem_max_atom
	    chem_ave_bese(i,k)=0.0
	  enddo
	  do j=1,3
	    chem_ave_pos(j,i)=0.0
	    chem_ave_sig(j,i)=0.0
	  enddo
	enddo
c
c------ loop over all unit cells ans atoms within unit cell
c
	do k=1,cr_icc(3)
	do j=1,cr_icc(2)
	do i=1,cr_icc(1)
	  ez(1)=cr_dim0(1,1)+float(i-1)
	  ez(2)=cr_dim0(2,1)+float(j-1)
	  ez(3)=cr_dim0(3,1)+float(k-1)
	  do ii=1,cr_ncatoms
	    ia=((k-1)*cr_icc(1)*cr_icc(2) +
     &	        (j-1)*cr_icc(1)           + (i-1))*cr_ncatoms+ii
	    do jj=1,3
	      p(jj)=cr_pos(jj,ia)-ez(jj)
	      chem_ave_pos(jj,ii)=chem_ave_pos(jj,ii)+p(jj)
	      chem_ave_sig(jj,ii)=chem_ave_sig(jj,ii)+p(jj)**2
	    enddo
c
c------ --- Calculate occupancies ..
c
	    if (chem_ave_n(ii).eq.0) then
	      chem_ave_n(ii)=1
	      chem_ave_iscat(ii,chem_ave_n(ii))=cr_iscat(ia)
	      is=1
	    else
	      flag=.true.
	      do kk=1,chem_ave_n(ii)
	        if (cr_iscat(ia).eq.chem_ave_iscat(ii,kk)) then
	          is=kk
	          flag=.false.
	        endif
	      enddo
	      if (flag) then
	        chem_ave_n(ii)=chem_ave_n(ii)+1
	        is=chem_ave_n(ii)
	        if (chem_ave_n(ii).gt.chem_max_atom) then
	          ier_typ = ER_CHEM
	          ier_num = -5
	          return
	        endif
	        chem_ave_iscat(ii,chem_ave_n(ii))=cr_iscat(ia)
	      endif
	    endif
	    chem_ave_bese(ii,is)=chem_ave_bese(ii,is)+1
	  enddo
	enddo
	enddo
	enddo
c
c------ output of average and sigma
c
	if (lout) write (output_io,1000)
	ia=cr_icc(1)*cr_icc(2)*cr_icc(3)
	do i=1,cr_ncatoms
	  do j=1,3
	    chem_ave_pos(j,i)=chem_ave_pos(j,i)/float(ia)
            chem_ave_sig(j,i)=chem_ave_sig(j,i)/float(ia)-
     &                                          chem_ave_pos(j,i)**2
	    if (chem_ave_sig(j,i).gt.0.0) then
              chem_ave_sig(j,i)=sqrt(chem_ave_sig(j,i))
            else
              chem_ave_sig(j,i)=0.0
            endif
	  enddo
	  if (lout) then
	    do k=1,chem_ave_n(i)
	      at_name_i =         at_name(chem_ave_iscat(i,k))
	      write (output_io,1100) i,at_name_i,
     &	                          (chem_ave_pos(ii,i),ii=1,3),
     &	                          (chem_ave_sig(ii,i),ii=1,3),
     &	                          chem_ave_bese(i,k)/ia
	    enddo
	  endif
	enddo
c
c------ store results in res_para
c
	if ((6*cr_ncatoms).gt.maxpar_res) then
	  ier_typ = ER_CHEM
	  ier_num = -2
	else
	  res_para(0)=6*cr_ncatoms
	  do i=1,cr_ncatoms
	    do j=1,3
	      res_para((i-1)*3+j  )=chem_ave_pos(j,i)
	    enddo 
	    do j=1,3
	      res_para((i-1)*3+j+3)=chem_ave_sig(j,i)
	    enddo 
	  enddo
	endif
c
1000	format (' Average structure : ',//,
     &	        3x,'Site',2x,'atom',11x,'average position',8x,
     &	           'standard deviation',3x,'occupancy',/,3x,75('-'))
1100	format (3x,i3,2x,a9,2x,3(f7.4,1x),1x,3(f7.4,1x),1x,f7.4)
	end
c*****7*****************************************************************
	subroutine chem_homo (line,lp)
c-
c	Check homogeniety of crstal
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'chem.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=15)
c
	character*(*)	line
	character*200	cpara(maxw),catom(2)
	real		werte(maxw),wwerte(maxw)
	integer		lpara(maxw),latom(2)
	integer		lp,ianz,iianz
	logical		locc
c
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,lp)
	if(ier_num.ne.0) return
c
	if (ianz.lt.3) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ Check concentration
c
	if    (str_comp(cpara(1),'occ',1,lpara(1),3)) then
	  call del_params (1,ianz,cpara,lpara,maxw)
	  iianz=1
	  if (chem_sel_atom) then
            call get_iscat (iianz,cpara,lpara,werte,maxw,.false.)
	  else
	    iianz = 1
	    call ber_params (1,cpara,lpara,werte,maxw)
	  endif
	  if (ier_num.ne.0) return
	  if (werte(1).ge.0.) then
	    call del_params (1,ianz,cpara,lpara,maxw)
	    call do_build_name (ianz,cpara,lpara,wwerte,maxw,1)
	    call chem_homo_occ(cpara(1),iianz,werte,maxw)
	  else
	    ier_num = -6
	    ier_typ = ER_CHEM
	  endif
c
c------ Check correlations
c
	elseif(str_comp(cpara(1),'cor',1,lpara(1),3)) then
	  if (ianz.ge.5) then
	    call do_cap(cpara(2))
	    locc = (cpara(2)(1:3).eq.'OCC')
	    catom(1) = cpara(3)
	    latom(1) = lpara(3)
	    catom(2) = cpara(4)
	    latom(2) = lpara(4)
	    call del_params (4,ianz,cpara,lpara,maxw)
	    call do_build_name (ianz,cpara,lpara,wwerte,maxw,1)
	    call chem_homo_corr(cpara(1),catom,latom,locc)
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_homo_occ(fname,ianz,werte,maxw)
c-
c	Calculates concentration distribution for given atom typ
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'chem.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'diffuse.inc'
	include 	'molecule.inc'
	include 	'errlist.inc'
c
	character*(*)	fname
	integer		ianz,maxw
	real		werte(maxw)
c
	integer*4	itot,ico
	integer		csize(3),lbeg(3)
	integer		i,imol,ibin,il,ia,iup
	real		c,sc,scc,ave_c,sig_c
c
	integer		len_str
	logical		atom_allowed,chem_inlot
c
c------ Some setup
c
	write (output_io,500)
	if     (ilots.eq.LOT_OFF) then
	  write (output_io,600)
	elseif (ilots.eq.LOT_BOX) then
	  write (output_io,610) nlots
	  write (output_io,630) (ls_xyz(i),i=1,3),lperiod
	elseif (ilots.eq.LOT_ELI) then
	  write (output_io,620) nlots
	  write (output_io,630) (ls_xyz(i),i=1,3),lperiod
	endif
c
	call four_csize(cr_icc,csize,lperiod,ls_xyz)
c
	do i=1,chem_max_bin
	  chem_hist(i)=0
	enddo
c
	c   = 0.0
	sc  = 0.0
	scc = 0.0
c
	iup = nlots/10
c
c------ Loop over all lots
c
	do il=1,nlots
	  call four_ranloc(csize,lbeg)
c
c------ - Get concentration in selected lot
c
	  itot = 0
	  ico  = 0
c
c------ - Atom mode
c
	  if (chem_sel_atom) then
	    do ia=1,cr_natoms
	      if (chem_inlot(ia,lbeg)) then
	        itot = itot + 1
	        if(atom_allowed(ia,werte,ianz,maxw)) ico = ico + 1
	      endif
	    enddo
c
c------ - Molecule mode
c
	  else
	    do ia=1,mole_num_mole
	      imol = mole_cont(mole_off(ia)+1)
	      if (chem_inlot(imol,lbeg)) then
	        itot = itot + 1
	        if(mole_type(ia).eq.nint(werte(1))) ico = ico + 1
	      endif
	    enddo
	  endif
c
	  if (itot.ne.0) then
	    c   = float(ico)/float(itot) 
	    sc  = sc + c
	    scc = scc + c*c
	  endif
c
	  ibin = int((chem_bin-1)*c) + 1
	  chem_hist(ibin) = chem_hist(ibin) + 1
c
	  if (nlots.gt.20) then
	    if (mod(il,iup).eq.0) write (output_io,900) il
	  endif
	enddo
c
c------ Write histogram file
c
	write (output_io,1000) fname(1:len_str(fname))
c
	call oeffne (37,fname,'unknown',.false.)
	if (ier_num.ne.0) return
	do i=1,chem_bin
	  write(37,2000) float(i-1)/float(chem_bin-1),chem_hist(i)
	enddo
	close (37)
c
c------ Output average concentration and sigma
c
	ave_c =  sc / float(nlots)
	sig_c = scc / float(nlots) - ave_c*ave_c
c
	write (output_io,1200) ave_c,sig_c
c
 500	format(  ' Computing concentration distribution ...')
 600	format ( '   Sample volume            : complete crystal')
 610	format ( '   Sample volume            : ',I4,
     &	           ' box shaped lots')
 620	format ( '   Sample volume            : ',I4,
     &	           ' ellipsoid shaped lots')
 630	format ( '   Lot size                 : ',I3,' x ',I3,
     &	         ' x ',I3,' unit cells ',/,
     &	         '   Periodic boundaries      : ',L1,/)
 900	format(  '   Finished lot #',i5,' ...')
1000	format(/,'   Saving histogram to file : ',a,/)
1200	format(  '   Average concentration    : ',f6.4,' +- ',f6.4)
2000	format(f8.3,1x,i12)
c
	end
c*****7*****************************************************************
	subroutine chem_homo_corr(fname,catom,latom,locc)
c-
c	Calculates correlation distribution 
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'chem.inc'
	include 	'mc.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'diffuse.inc'
	include 	'errlist.inc'
c
	character*(*)	fname,catom(2)
	integer		latom(2)
	logical		locc
c
	integer		csize(3),lbeg(3)
	integer		i,ibin,il,iup,ic,l
	real		sc(chem_max_cor),scc(chem_max_cor)
	real		ave_c,sig_c
c
	integer		len_str
c
c------ Some setup
c
	write (output_io,500)
	if     (ilots.eq.LOT_OFF) then
	  write (output_io,600)
	elseif (ilots.eq.LOT_BOX) then
	  write (output_io,610) nlots
	  write (output_io,630) (ls_xyz(i),i=1,3),lperiod
	elseif (ilots.eq.LOT_ELI) then
	  write (output_io,620) nlots
	  write (output_io,630) (ls_xyz(i),i=1,3),lperiod
	endif
c
	call four_csize(cr_icc,csize,lperiod,ls_xyz)
c
	do i=1,chem_max_bin
	  chem_hist(i)=0
	enddo
c
	do i=1,chem_max_cor
	  sc(i)  = 0.0
	  scc(i) = 0.0
	enddo
c
	iup = nlots/10
c
c------ Loop over all lots
c
	do il=1,nlots
	  call four_ranloc(csize,lbeg)
c
c------ - Get correlation in selected lot
c
	  if (locc) then
	    l = 2
	    if (chem_sel_atom) then
	      call chem_corr_occ (l,catom,latom,2,.false.,lbeg)
	    else
	      call chem_corr_occ_mol (l,catom,latom,2,.false.,lbeg)
	    endif
	  else
	    l = 2
	    if (chem_sel_atom) then
	      call chem_corr_dis (l,catom,latom,2,.false.,lbeg)
	    else
	      call chem_corr_dis_mol (l,catom,latom,2,.false.,lbeg)
	    endif
	  endif
	  if (ier_num.ne.0) return
c
	  do ic=1,chem_ncor
	    sc(ic)  = sc(ic)  + mo_ach_corr(ic)
	    scc(ic) = scc(ic) + mo_ach_corr(ic)**2
c
	    ibin = int((mo_ach_corr(ic)+1)*chem_bin / 2.0) + 1
	    chem_hist(ibin) = chem_hist(ibin) + 1
	  enddo
c
	  if (nlots.gt.20) then
	    if (mod(il,iup).eq.0) write (output_io,900) il
	  endif
	enddo
c
c------ Write histogram file
c
	write (output_io,1000) fname(1:len_str(fname))
c
	call oeffne (37,fname,'unknown',.false.)
	if (ier_num.ne.0) return
	do i=1,chem_bin
	  write(37,2000) (2.*float(i-1)/float(chem_bin-1))-1.,chem_hist(i)
	enddo
	close (37)
c
c------ Output average correlations and sigma
c
	do ic=1,chem_ncor
	  ave_c =  sc(ic) / float(nlots)
	  sig_c = scc(ic) / float(nlots) - ave_c*ave_c
	  write (output_io,1200) ic,ave_c,sig_c
	enddo
c
 500	format(  ' Computing correlation distribution ...')
 600	format ( '   Sample volume            : complete crystal')
 610	format ( '   Sample volume            : ',I4,
     &	           ' box shaped lots')
 620	format ( '   Sample volume            : ',I4,
     &	           ' ellipsoid shaped lots')
 630	format ( '   Lot size                 : ',I3,' x ',I3,
     &	         ' x ',I3,' unit cells ',/,
     &	         '   Periodic boundaries      : ',L1,/)
 900	format(  '   Finished lot #',i5,' ...')
1000	format(/,'   Saving histogram to file : ',a,/)
1200	format(  '   Average correlation #',i3,' : ',f6.4,' +- ',f6.4)
2000	format(f8.3,1x,i12)
c
	end
c*****7*****************************************************************
      	logical function chem_inlot (ia,lbeg)
c+
c	Checks if given atom 'ia' is in given lot 'lbeg'
c
	implicit      	none
c
	include 	'config.inc'
	include 	'crystal.inc'
	include 	'crystal.cmm'
	include 	'diffuse.inc'
c
	real		xtest(3),x0
	integer*4	cr_end
	integer		lbeg(3),iz(3),izmin,izmax
	integer		ia,is,i
c
	cr_end = cr_ncatoms*cr_icc(1)*cr_icc(2)*cr_icc(3) + 1
c
c------ We are not using lots
c
	if (ilots.eq.LOT_OFF) then
	  chem_inlot = .true.
c
c------ Box shaped lot
c
	elseif (ilots.eq.LOT_BOX) then
	  if (ia.lt.cr_end) then
	    call indextocell (ia,iz,is)
	  else
	    do i=1,3
	      iz(i) = int(cr_pos(i,ia) - cr_dim0(i,1)) + 1
	    enddo
	  endif
c
	  chem_inlot = .true.
c
	  do i=1,3
	    izmin  = lbeg(i)
	    izmax  = lbeg(i)+ls_xyz(i)-1
	    chem_inlot = chem_inlot .and. 
     &	                 (iz(i).ge.izmin).and.(iz(i).le.izmax)
c
	    if (.not. chem_inlot .and. (izmax.gt.cr_icc(i))) then
	      izmin  = 1
	      izmax  = izmax - cr_icc(i)
	      chem_inlot = chem_inlot .and. 
     &	                   (iz(i).ge.izmin).and.(iz(i).le.izmax)
	    endif
	  enddo
c
c------ Ellipsoid shaped lot
c
	elseif (ilots.eq.LOT_ELI) then
	  if (ia.lt.cr_end) then
	    call indextocell (ia,iz,is)
	  else
	    do i=1,3
	      iz(i) = int(cr_pos(i,ia) + cr_dim0(i,1)) - 1
	    enddo
	  endif
c
	  chem_inlot = .true.
c
	  do i=1,3
	    x0       = float(ls_xyz(i))/2.0
	    izmin    = lbeg(i)
	    izmax    = lbeg(i)+ls_xyz(i)-1
	    xtest(i) = (iz(i)-izmin-x0+0.5)**2/x0**2
	    chem_inlot = chem_inlot .and. 
     &	                 (iz(i).ge.izmin).and.(iz(i).le.izmax)
c
	    if (.not. chem_inlot .and. (izmax.gt.cr_icc(i))) then
	      izmin  = 1
	      izmax  = izmax - cr_icc(i)
	      chem_inlot = chem_inlot .and. 
     &	                   (iz(i).ge.izmin).and.(iz(i).le.izmax)
	    endif
	  enddo
c
	  chem_inlot = chem_inlot .and.
     &	               ((xtest(1)+xtest(2)+xtest(3)).le.1.0)
c
	endif
	end
c*****7*****************************************************************
	subroutine chem_corr_field(line,lp)
c-
c	Calculates correlation field
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'prompt.inc'
	include 	'chem.inc'
	include 	'mc.inc'
	include 	'errlist.inc'
c
	integer		maxw
	parameter	(maxw=15)
	integer		maxval
	parameter	(maxval=1000)
c
	character*(*)	line
	character*200	cpara(maxw)
	character*200	fname,catom(2)
	integer		back_cvec(5,chem_max_vec)
	integer		lpara(maxw)
	integer		latom(2),lbeg(3),lname
	integer		xmin,xmax,ymin,ymax
	integer		l,i,ix,iy,ianz,lp
	real		werte(maxw),nv(3)
	real		cval(maxval)
	real		back_neig(3,48,chem_max_cor)
	real		back_rmin(chem_max_cor)
	real		back_rmax(chem_max_cor)
	logical		locc
c
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,lp)
	if(ier_num.ne.0) return
c
	if(ianz.lt.6) then
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ Get correlation mode
c
	if    (str_comp(cpara(1),'occ',1,lpara(1),3)) then
	  locc = .true.
	elseif(str_comp(cpara(1),'dis',1,lpara(1),3)) then
	  locc = .false.
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	  return
	endif
c
c------ Save atom names
c
	catom(1) = cpara(2)(1:lpara(2))
	catom(2) = cpara(3)(1:lpara(3))
	latom(1) = lpara(2)
	latom(2) = lpara(3)
c
c------ Build filename
c
	call del_params (3,ianz,cpara,lpara,maxw)
	call do_build_name (ianz,cpara,lpara,werte,maxw,1)
	fname = cpara(1)
	lname = lpara(1)
c
c------ Get calculation range
c
	call del_params (1,ianz,cpara,lpara,maxw)
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ianz.eq.2) then
	  xmin = nint(werte(1))
	  xmax = nint(werte(2))
	  ymin = 0
	  ymax = 0
	elseif (ianz.eq.4) then
	  xmin = nint(werte(1))
	  xmax = nint(werte(2))
	  ymin = nint(werte(3))
	  ymax = nint(werte(4))
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	if (ier_num.ne.0) return
c
c-------------------------------------------------------------------
c------ Here starts the calculation of the correlation field
c-------------------------------------------------------------------
c
	call oeffne (37,fname,'unknown',.false.)
	if (ier_num.ne.0) return
	write (output_io,1000) fname(1:lname),catom(1)(1:latom(1)),
     &	                              catom(2)(1:latom(2))
c
c------ Save current 'neig' settings
c
	call chem_save_neig(back_cvec,back_neig,back_rmin,back_rmax,
     &	                    chem_cvec,chem_neig,chem_rmin,chem_rmax)
c
c------ We have a 1D file
c
	if (ymin.eq.0 .and. ymax.eq.0) then
	  call chem_inc_check(1)
	  if (ier_num.ne.0) goto 9999
	  lbeg(1) = -1
	  lbeg(2) =  0
	  lbeg(3) =  0
c
	  do ix=xmin,xmax
	    call chem_inc_neig(ix,0,back_cvec,back_neig,nv)
	    if (locc) then
	      l = 2
	      if (chem_sel_atom) then
	        call chem_corr_occ(l,catom,latom,2,.false.,lbeg)
	      else
	        call chem_corr_occ_mol(l,catom,latom,2,.false.,lbeg)
	      endif
	    else
	      l = 2
	      if (chem_sel_atom) then
	        call chem_corr_dis(l,catom,latom,2,.false.,lbeg)
	      else
	        call chem_corr_dis_mol(l,catom,latom,2,.false.,lbeg)
	      endif
	    endif
	    if (ier_num.ne.0) goto 9999
	    write (output_io,1100) ix,0,nv,mo_ach_corr(1)
	    write (37,*) ix,mo_ach_corr(1)
	  enddo
c	
c------ 2D (NIPL) file
c
	else
	  call chem_inc_check(2)
	  if (ier_num.ne.0) goto 9999
	  lbeg(1) = -1
	  lbeg(2) =  0
	  lbeg(3) =  0
c
	  write (37,*) (xmax-xmin+1),(ymax-ymin+1)
	  write (37,*) float(xmin),float(xmax),float(ymin),float(ymax)
	  do iy=ymin,ymax
	    do ix=xmin,xmax
	      call chem_inc_neig(ix,iy,back_cvec,back_neig,nv)
	      if (locc) then
	        l = 2
	        if (chem_sel_atom) then
	          call chem_corr_occ(l,catom,latom,2,.false.,lbeg)
	        else
	          call chem_corr_occ_mol(l,catom,latom,2,.false.,lbeg)
	        endif
	      else
	        l = 2
	        if (chem_sel_atom) then
	          call chem_corr_dis(l,catom,latom,2,.false.,lbeg)
	        else
	          call chem_corr_dis_mol(l,catom,latom,2,.false.,lbeg)
	        endif
	      endif
	      if (ier_num.ne.0) goto 9999
	      write (output_io,1100) ix,iy,nv,mo_ach_corr(1)
	      cval(ix-xmin+1) = mo_ach_corr(1)
	    enddo
	    write (37,*) (cval(i),i=1,xmax-xmin+1)
	  enddo
	endif
c
9999	continue
c
c------ Restore 'neig' settings
c
	call chem_save_neig(chem_cvec,chem_neig,chem_rmin,chem_rmax,
     &	                    back_cvec,back_neig,back_rmin,back_rmax)
c
	close (37)
c
1000	format (' Calculating correlation field ... ',/
     &	        '   Outputfile           : ',a,/,
     &	        '   Atoms/Molecules used : ',a4,' - ',a4,//,
     &	        6x,'x',7x,'y',16x,'Neighbours',12x,'Correlation',/,
     &	        3x,64('-'))
1100	format (3x,i5,3x,i5,4x,3(f8.2,2x),3x,f8.4)
	end
c*****7*****************************************************************
	subroutine chem_save_neig(a_cvec,a_neig,a_rmin,a_rmax,
     &	                          b_cvec,b_neig,b_rmin,b_rmax)
c-
c	Saves/restores current CHEM neighbour settings
c+
	implicit     	none
c
	include		'config.inc'
c
	integer		a_cvec(5,chem_max_vec)
	integer		b_cvec(5,chem_max_vec)
	real		a_neig(3,48,chem_max_cor)
	real		b_neig(3,48,chem_max_cor)
	real		a_rmin(chem_max_cor),a_rmax(chem_max_cor)
	real		b_rmin(chem_max_cor),b_rmax(chem_max_cor)
c
	integer		i,j,k
c
	do i=1,chem_max_vec
	  do j=1,5
	    a_cvec(j,i) = b_cvec(j,i)
	  enddo
	enddo
c
	do i=1,chem_max_cor
	  a_rmin(i) = b_rmin(i)
	  a_rmax(i) = b_rmax(i)
c
	  do j=1,3
	    do k=1,48
	      a_neig(j,k,i) = b_neig(j,k,i)
	    enddo
	  enddo
	enddo
c
	end
c*****7*****************************************************************
	subroutine chem_inc_neig(ix,iy,cvec,neig,nv)
c-
c	Updates neigbouring distances for correlation field
c	calculations.
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'chem.inc'
	include 	'errlist.inc'
c
	integer		i,j,ix,iy,iv
	integer		cvec(5,chem_max_vec)
	real		neig(3,48,chem_max_cor)
	real		nv(3),u(3),v(3)
c
	real		do_blen
c
c------ Mode VECTOR
c
	if (chem_ctyp(1).eq.CHEM_VEC) then
	  do i=1,3
	    do iv=1,chem_nvec(1)
	      chem_cvec(i+2,chem_use_vec(iv,1)) = 
     &	                ix*cvec(i+2,chem_use_vec(iv,1)) +
     &	                iy*cvec(i+2,chem_use_vec(iv,2))
	    enddo
	    nv(i) = chem_cvec(i+2,chem_use_vec(1,1))
	  enddo
c
c------ Mode DISTANCE
c
	elseif (chem_ctyp(1).eq.CHEM_DIST) then
	  do i=1,3
	    do j=1,chem_nnei(1)
	      chem_neig(i,j,1) = float(ix) * neig(i,j,1) +
     &	                         float(iy) * neig(i,j,2)
	    enddo
	  enddo
	  do i=1,3
	    u(i)  = 0.0
	    v(i)  = chem_neig(i,1,1)
	    nv(i) = chem_neig(i,1,1)
	  enddo
	  chem_rmax(1) = do_blen(.true.,u,v)
	  chem_rmin(1) = chem_rmax(1) - chem_freq_sigma(1)/2.0
	  chem_rmax(1) = chem_rmax(1) + chem_freq_sigma(1)/2.0
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_inc_check(ic)
c-
c	Checks input for correlation field
c+
	implicit     	none
c
	include 	'config.inc'
	include 	'chem.inc'
	include 	'errlist.inc'
c
	integer		i,ic
c
	do i=1,ic
	  if (chem_ctyp(i).eq.CHEM_NONE) then
	    ier_num = -11
	    ier_typ = ER_CHEM
	    return
	  endif
	enddo
c
	if (ic.eq.2 .and. chem_ctyp(1).ne.chem_ctyp(2)) then
	  ier_num = -17
	  ier_typ = ER_CHEM
	  return
	endif
c
	if (ic.eq.2 .and. chem_nvec(1).ne.chem_nvec(2)) then
	  ier_num = -18
	  ier_typ = ER_CHEM
	  return
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_disp (ianz,cpara,lpara,werte,maxw,lout)
c+
c	Calculates distortions within the crystal 
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'errlist.inc'
c
	integer		maxatom
c
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	logical		lout
c
	character*200	fname
	character*9	at_name_i,at_name_j
	integer		atom(0:maxatom),natom
	integer		iianz,jjanz,i,j,k,is,js,ic
	integer		bl_anz(0:maxscat,0:maxscat)
	real		bl_sum(0:maxscat,0:maxscat)
	real		bl_s2 (0:maxscat,0:maxscat)
	real		patom(3,0:maxatom)
	real		u(3),v(3),d(3),di
	real		wwerte(maxw)
	logical		lfile
c
	character*9	at_name
	real		do_blen
	logical		atom_allowed
c
	fname=cpara(3)(1:lpara(3))
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) cpara(1)(1:lpara(1)),cpara(2)(1:lpara(2))
	  if (ianz.gt.2) then
	    call oeffne (37,fname,'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.2)
c
	iianz=1
	jjanz=1
	call get_iscat(iianz,cpara,lpara, werte,maxw,.false.)
	call del_params(1,ianz,cpara,lpara,maxw)
	call get_iscat(jjanz,cpara,lpara,wwerte,maxw,.false.)
	if (ier_num.ne.0) return
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------ - reset counters, vectors, ..
c
	  do i=0,cr_nscat
	    do j=0,cr_nscat
	      bl_sum(i,j) = 0.0
	      bl_s2 (i,j) = 0.0
	      bl_anz(i,j) = 0
	    enddo
	  enddo
c
c------ - calculate distortions
c
	  do i=1,cr_natoms
	    if (atom_allowed(i,werte,ianz,maxw)) then
	      call chem_neighbour(i,ic,atom,patom,natom,maxatom)
	      if (natom.gt.0) then
	        do j=1,natom
	          if (atom_allowed(atom(j),wwerte,ianz,maxw)) then
	            do k=1,3
	              u(k) = cr_pos(k,i)
	              v(k) = patom (k,j)
	              d(k) = v(k) - u(k)
	            enddo
	            di = do_blen(.true.,u,v)
	            is = cr_iscat(i)
	            js = cr_iscat(atom(j))
	            if (lfile) write(37,3000) d,is,js
	            bl_sum(is,js) = bl_sum(is,js) + di
	            bl_s2 (is,js) = bl_s2 (is,js) + di**2
	            bl_anz(is,js) = bl_anz(is,js) + 1
	          endif
	        enddo
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	    do i=0,cr_nscat
	      do j=i,cr_nscat
	      if (bl_anz(i,j).ne.0 .or. bl_anz(j,i).ne.0 ) then
	        chem_disp_ave(ic,i,j)=(bl_sum(i,j)+bl_sum(j,i))/
     &	                              (bl_anz(i,j)+bl_anz(j,i))
	        chem_disp_sig(ic,i,j)=(bl_s2 (i,j)+bl_s2 (j,i))/
     &	                              (bl_anz(i,j)+bl_anz(j,i))
	        chem_disp_sig(ic,i,j)=(chem_disp_sig(ic,i,j)-
     &	                              (chem_disp_ave(ic,i,j)**2))
	        if (chem_disp_sig(ic,i,j).gt.0) then
	          chem_disp_sig(ic,i,j)=sqrt(chem_disp_sig(ic,i,j))
	        else
	          chem_disp_sig(ic,i,j)=0.0
	        endif
	        if (lout .and. bl_anz(i,j).ne.0) then
	          at_name_i = at_name(i)
	          at_name_j = at_name(j)
	          write (output_io,2000) ic,at_name_i,at_name_j,
     &	                              chem_disp_ave(ic,i,j),
     &	                              chem_disp_sig(ic,i,j),
     &	                              bl_anz(i,j)+bl_anz(j,i)
	        endif
	      endif
	    enddo
	  enddo
	  if (ier_num.ne.0) return
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating distortions ',/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &            '    Neig.  Atom A      Atom B       distance',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,a9,3x,a9,5x,f7.3,3x,f7.3,3x,i8)
3000	format (3(f12.5,1x),3x,2(i3,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_disp_multi (ianz,cpara,lpara,werte,maxw,lout)
c+
c	Calculates distortions within the crystal 
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include 	'mmc.inc'
	include		'errlist.inc'
c
	integer		maxatom
c
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	logical		lout
c
	character*9	at_name_i,at_name_j
	integer		atom(0:maxatom,CHEM_MAX_CENT)
	integer		natom(CHEM_MAX_CENT)
	integer		iianz,i,j,k,is,js,ic
	integer		ja,je
	integer		icent,ncent
	integer		bl_anz(0:maxscat,0:maxscat)
	real		bl_sum(0:maxscat,0:maxscat)
	real		bl_s2 (0:maxscat,0:maxscat)
	real		patom(3,0:maxatom,CHEM_MAX_CENT)
	real		u(3),v(3),d(3),di
	logical		lfile
c
	character*9	at_name
	real		do_blen
	logical		atom_allowed
c
	iianz = 2
	call get_iscat(iianz,cpara,lpara,werte,maxw,.false.)
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) cpara(1)(1:lpara(1)),cpara(2)(1:lpara(2))
	  if (ianz.gt.2) then
	    call oeffne (37,cpara(3),'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.2)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------	--reset the average and sigma values. Necessary when the correlations or
c	  periodicity has been changed
c
	  do i=0,cr_nscat
	    do j=i,cr_nscat
	      chem_disp_ave(ic,i,j)= 0.0
	      chem_disp_sig(ic,i,j)= 0.0
	    enddo
	  enddo
	  if(chem_ctyp(ic).eq.CHEM_VEC .or.
     &	     chem_ctyp(ic).eq.CHEM_ENVIR .or.
     &	     chem_ctyp(ic).eq.CHEM_RANGE .or.
     &	     chem_ctyp(ic).eq.CHEM_DIST     ) then
c
c------ --- reset counters, vectors, ..
c
	    do i=0,cr_nscat
	      do j=0,cr_nscat
	        bl_sum(i,j) = 0.0
	        bl_s2 (i,j) = 0.0
	        bl_anz(i,j) = 0
	      enddo
	    enddo
c
c------ --- calculate distortions
c
	    do i=1,cr_natoms
	      if (atom_allowed(i,werte,ianz,maxw)) then
CDBG
CDBG	if(ic.eq.93 .and. cr_iscat(i).eq.9) then
CDBG	write(*,*) ' selected atom ', i, cr_iscat(i)
CDBG	endif
	        call chem_neighbour_multi(i,ic,atom,patom,natom,ncent,
     &	                                  maxatom)
	        do icent=1,ncent
	          if (natom(icent).gt.0) then
	            if(i.eq.atom(0,icent) ) then
c
c------	--------- The selected atom is the central atom, check all atoms
c
	              ja = 1
	              je = natom(icent)
	              do k=1,3
	                u(k)=patom(k,0,icent)
	              enddo
	              is = cr_iscat(i)
	            else
c
c------	--------- The selected atom is a neighbour, check central atom only
c
	              ja = 0
	              je = 0
	              do k=1,3
	                u(k)=cr_pos(k,i)
	              enddo
	              is = cr_iscat(i)
	            endif
CDBG
CDBG	write(*,*) ' Limits        ',ja,je
	            do j=ja,je
	              if (atom_allowed(atom(j,icent),werte,ianz,maxw))
     &	                                                            then
CDBG
CDBG	write(*,*) ' Neighbor Nr.  ',j,cr_iscat(atom(j,icent))
CDBG	write(*,*) ' at            ',(patom (k,j,icent),k=1,3)
	                do k=1,3
	                  v(k) = patom (k,j,icent)
	                  d(k) = v(k) - u(k)
	                enddo
	                di = do_blen(.true.,u,v)
	                js = cr_iscat(atom(j,icent))
	                if (lfile) write(37,3000) d,is,js
	                bl_sum(is,js) = bl_sum(is,js) + di
	                bl_s2 (is,js) = bl_s2 (is,js) + di**2
	                bl_anz(is,js) = bl_anz(is,js) + 1
CDBG
CDBG	if(cr_iscat(i).eq. 8 .and. js.eq.15 .or. 
CDBG     &	   cr_iscat(i).eq.15 .and. js.eq.8       ) then
CDBG	  write(*,*) ' Atom, neigh, v,dist ',i,cr_iscat(i),is,j,
CDBG     &	  atom(j,icent),js
CDBG	  write(*,*) ' v,dist              ',v,di
CDBG	  write(*,*) ' bl_sum(8,15)        ',bl_sum(8,15),bl_sum(15,8)
CDBG	  write(*,*) ' bl_anz(8,15)        ',bl_anz(8,15),bl_anz(15,8)
CDBG	endif
	              endif
	            enddo
	          endif
	        enddo
	      endif
	    enddo
c
c------ - write results and save to res_para block
c
	      do i=0,cr_nscat
	        do j=i,cr_nscat
	        if (bl_anz(i,j).ne.0 .or. bl_anz(j,i).ne.0 ) then
	          chem_disp_ave(ic,i,j)=(bl_sum(i,j)+bl_sum(j,i))/
     &	                                (bl_anz(i,j)+bl_anz(j,i))
	          chem_disp_sig(ic,i,j)=(bl_s2 (i,j)+bl_s2 (j,i))/
     &	                                (bl_anz(i,j)+bl_anz(j,i))
	          chem_disp_sig(ic,i,j)=(chem_disp_sig(ic,i,j)-
     &	                                (chem_disp_ave(ic,i,j)**2))
	          if (chem_disp_sig(ic,i,j).gt.0) then
	            chem_disp_sig(ic,i,j)=sqrt(chem_disp_sig(ic,i,j))
	          else
	            chem_disp_sig(ic,i,j)=0.0
	          endif
	          if (lout) then
CDBG	if(ic.eq.93 .and. i.eq.9 .and. j.eq.17) then
	            at_name_i = at_name(i)
	            at_name_j = at_name(j)
	            write (output_io,2000) ic,at_name_i,at_name_j,
     &	                                chem_disp_ave(ic,i,j),
     &	                                chem_disp_sig(ic,i,j),
     &	                                bl_anz(i,j)+bl_anz(j,i)
	          endif
	        endif
	    enddo
	    enddo
	    if (ier_num.ne.0) return
	  endif
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating distortions ',/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &            '    Neig.  Atom A      Atom B       distance',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,a9,3x,a9,5x,f7.3,3x,f7.3,3x,i8)
3000	format (3(f12.5,1x),3x,2(i3,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_vector_multi (ianz,cpara,lpara,werte,maxw,lout)
c+
c	Calculates vector distortions within the crystal 
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include 	'mmc.inc'
	include		'errlist.inc'
	include		'param.inc'
c
	integer		maxatom
c
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	logical		lout
c
	character*9	at_name_i,at_name_j
	integer		atom(0:maxatom,CHEM_MAX_CENT)
	integer		natom(CHEM_MAX_CENT)
	integer		iianz,i,j,k,is,js,ic
	integer		ja,je,nv
	integer		icent,ncent
	integer		ia
	integer		isel(CHEM_MAX_ATOM)
	integer		bl_anz(  12,0:maxscat,0:maxscat)
	real		bl_sum(3,12,0:maxscat,0:maxscat)
	real		bl_s2 (3,12,0:maxscat,0:maxscat)
	real		patom(3,0:maxatom,CHEM_MAX_CENT)
	real		u(3),v(3),w(3),null(3)
	real		an
	real		e_tot,e_alt
	logical		lfile
CDBG
	integer		nerg
	
c
	character*9	at_name
	real		mmc_energy_vec 
	logical		atom_allowed
	real		do_bang
c
	data null/0.0,0.0,0.0/
	data nerg/0/
c
	save nerg
c
	e_tot = 0.
	e_alt = 0.
	iianz = 2
	call get_iscat(iianz,cpara,lpara,werte,maxw,.false.)
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) cpara(1)(1:lpara(1)),cpara(2)(1:lpara(2))
	  if (ianz.gt.2) then
	    call oeffne (37,cpara(3),'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.2)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
	  if(chem_ctyp(ic).eq.CHEM_ENVIR    ) then
c
c------ --- reset counters, vectors, ..
c
	    do i=0,cr_nscat
	      do j=0,cr_nscat
	        do nv=1,12
	          do k=1,3
	            bl_sum(k,nv,i,j) = 0.0
	            bl_s2 (k,nv,i,j) = 0.0
	          enddo
	          bl_anz(nv,i,j) = 0
	        enddo
	      enddo
	    enddo
c
c------ --- calculate distortions
c
	    do i=1,cr_natoms
	      if (atom_allowed(i,werte,ianz,maxw)) then
	        call chem_neighbour_multi(i,ic,atom,patom,natom,ncent,
     &	                                  maxatom)
	        do icent=1,ncent
	          if (natom(icent).gt.0) then
CDBG
CDBG	write(*,*) ' atom              ',(atom(k,icent),k=0,natom(icent))
	            if(i.eq.atom(0,icent) ) then
c
c------	--------- The selected atom is the central atom, check all atoms
c
	              ja = 1
	              je = natom(icent)
	              do k=1,3
	                u(k)=patom(k,0,icent)
	              enddo
	              is = cr_iscat(i)
	            else
c
c------	--------- The selected atom is a neighbour, check central atom only
c
	              ja = 0
	              je = 0
	              do k=1,3
	                u(k)=cr_pos(k,i)
	              enddo
	              is = cr_iscat(i)
	            endif
	            do j=ja,je
	              if (atom_allowed(atom(j,icent),werte,ianz,maxw))
     &	                                                            then
	                js = cr_iscat(atom(j,icent))
	                do nv=1,mmc_nvec(ic,is,js)
	                do k=1,3
	                  v(k) = patom (k,j,icent) - u(k)
	                  w(k) = mmc_vec(k,nv,ic,is,js)
	                enddo
	                an = do_bang(.true.,v,null,w)
	                if (lfile) write(37,3000) w,is,js
	                if(an.lt. mmc_vec(4,nv,ic,is,js) ) then
	                  do k=1,3
	                    bl_sum(k,nv,is,js)=bl_sum(k,nv,is,js)+v(k)
	                    bl_s2 (k,nv,is,js)=bl_s2(k,nv,is,js)+v(k)**2
	                  enddo
	                  bl_anz(nv,is,js) = bl_anz(nv,is,js) + 1
	                endif
	                enddo
	              endif
	            enddo
c
c------	--------- Calculate grand Energy
c
	            isel(1) = i 
	            ia      = 1
	if(ier_num.ne.0) then
	  write(*,*) ' 1 ier_num,ier_typ ',ier_num,ier_typ
	endif
	            e_tot = e_tot +
     &	            mmc_energy_vec(isel,ia,ic,atom,patom,icent,natom)
	if(ier_num.ne.0) then
	  write(*,*) ' 2 ier_num,ier_typ ',ier_num,ier_typ
	endif
	          endif
	        enddo
	      endif
	    enddo
c
CDBG	if(ier_num.ne.0) then
CDBG	  write(*,*) ' ier_num,ier_typ ',ier_num,ier_typ
CDBG	endif
CDBG	write(*,*) ' Total Energy : ', e_tot
CDBG	write(*,*) ' bl_anz(1,2,2)  ',bl_anz(1,2,2)
CDBG	write(*,*) ' bl_anz(2,2,2)  ',bl_anz(2,2,2)
CDBG	write(*,*) ' nerg           ',nerg
CDBG	rpara(nerg) = e_tot
CDBG	write(*,*) ' rpara(nerg)    ',rpara(nerg)
CDBG	nerg = nerg + 1
	  
	
c
c------ - write results and save to res_para block
c
	      do i=0,cr_nscat
	        do j=i,cr_nscat
	        do nv=1,mmc_nvec(ic,i,j)
	        if (bl_anz(nv,i,j).ne.0 .or. bl_anz(nv,j,i).ne.0 ) then
	          do k=1,3
	            chem_vect_ave(k,nv,ic,i,j)=
     &	                    (bl_sum(k,nv,i,j)+bl_sum(k,nv,j,i))/
     &	                    (bl_anz(  nv,i,j)+bl_anz(  nv,j,i))
	            chem_vect_sig(k,nv,ic,i,j)=
     &	                    (bl_s2 (k,nv,i,j)+bl_s2 (k,nv,j,i))/
     &	                    (bl_anz(  nv,i,j)+bl_anz(  nv,j,i))
	            chem_vect_sig(k,nv,ic,i,j)=
     &	                    (chem_vect_sig(k,nv,ic,i,j)-
     &	                    (chem_vect_ave(k,nv,ic,i,j)**2))
	            if (chem_vect_sig(k,nv,ic,i,j).gt.0) then
	              chem_vect_sig(k,nv,ic,i,j)=
     &	                    sqrt(chem_vect_sig(k,nv,ic,i,j))
	            else
	              chem_vect_sig(k,nv,ic,i,j)=0.0
	            endif
	          enddo
	          if (lout) then
	            at_name_i = at_name(i)
	            at_name_j = at_name(j)
	            write (output_io,2000) ic,at_name_i,at_name_j,
     &	                                chem_vect_ave(1,nv,ic,i,j),
     &	                                chem_vect_sig(1,nv,ic,i,j),
     &	                                bl_anz(nv,i,j)+bl_anz(nv,j,i)
	            write (output_io,2010) chem_vect_ave(2,nv,ic,i,j),
     &	                                chem_vect_sig(2,nv,ic,i,j)
	            write (output_io,2010) chem_vect_ave(3,nv,ic,i,j),
     &	                                chem_vect_sig(3,nv,ic,i,j)
	          endif
	        endif
	      enddo
	      enddo
	    enddo
	    if (ier_num.ne.0) return
	  endif
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating distortions ',/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &            '    Neig.  Atom A      Atom B       vector  ',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,a9,3x,a9,4x,'(',f7.3,') (',f7.3,')',2x,i8)
2010	format (35x                 ,'(',f7.3,') (',f7.3,')',2x,i8)
3000	format (3(f12.5,1x),3x,2(i3,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_disp_mol (ianz,cpara,lpara,werte,maxw,lout)
c+
c	Calculates distortions within the crystal 
c	Molecules version ..
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'molecule.inc'
	include		'errlist.inc'
c
	integer		maxmol
c
	parameter	(maxmol  = 48)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	logical		lout
c
	integer		imol,jmol,i,j,k,it1,it2,is,js,ic
	integer		mol(0:maxmol),nmol 
	integer		bl_anz(mole_max_type,mole_max_type)
	real		bl_sum(mole_max_type,mole_max_type)
	real		bl_s2 (mole_max_type,mole_max_type)
	real		u(3),v(3),d(3),di
	logical		lfile
c
	real		do_blen
c
c
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
c
c------ check input
c
	it1=nint(werte(1))
	it2=nint(werte(2))
c
	if (it1.le.0 .or. it1.gt.mole_num_type .or.
     &	    it2.le.0 .or. it2.gt.mole_num_type     ) then
	  ier_num = -64
	  ier_typ = ER_APPL
	endif
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) it1,it2
	  if (ianz.gt.2) then
	    call oeffne (37,cpara(3),'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.2)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------ - reset counters, vectors, ..
c
	  do i=1,mole_num_type
	    do j=1,mole_num_type
	      bl_sum(i,j) = 0.0
	      bl_s2 (i,j) = 0.0
	      bl_anz(i,j) = 0
	    enddo
	  enddo
c
c------ - calculate distortions
c
	  do i=1,mole_num_mole
	    imol = mole_cont(mole_off(i)+1)
	    if (it1.eq.mole_type(i) .or. it2.eq.mole_type(i)) then
	      call chem_neighbour_mol (i,ic,mol,nmol,maxmol)
	      if (nmol.gt.0) then
	        do j=1,nmol
	          jmol = mole_cont(mole_off(mol(j))+1)
	          if (it1.eq.mole_type(mol(j)) .or. 
     &	              it2.eq.mole_type(mol(j))     ) then
	            do k=1,3
	              u(k) = cr_pos(k,imol)
	              v(k) = cr_pos(k,jmol)
	              d(k) = u(k)-v(k)
	            enddo
	            if (lfile) write(37,3000) d
	            di = do_blen(.true.,u,v)
	            is = mole_type(i)
	            js = mole_type(mol(j))
	            bl_sum(is,js) = bl_sum(is,js) + di
	            bl_s2 (is,js) = bl_s2 (is,js) + di**2
	            bl_anz(is,js) = bl_anz(is,js) + 1
	          endif
	        enddo
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	    do i=1,mole_num_type
	      do j=i,mole_num_type
	      if (bl_anz(i,j).ne.0 .or. bl_anz(j,i).ne.0 ) then
	        chem_disp_ave(ic,i,j)=(bl_sum(i,j)+bl_sum(j,i))/
     &	                              (bl_anz(i,j)+bl_anz(j,i))
	        chem_disp_sig(ic,i,j)=(bl_s2 (i,j)+bl_s2 (j,i))/
     &	                              (bl_anz(i,j)+bl_anz(j,i))
	        chem_disp_sig(ic,i,j)=(chem_disp_sig(ic,i,j)-
     &	                              (chem_disp_ave(ic,i,j)**2))
	        if (chem_disp_sig(ic,i,j).gt.0) then
	          chem_disp_sig(ic,i,j)=sqrt(chem_disp_sig(ic,i,j))
	        else
	          chem_disp_sig(ic,i,j)=0.0
	        endif
	        if (lout) then
	          write (output_io,2000) ic,i,j,
     &	                              chem_disp_ave(ic,i,j),
     &	                              chem_disp_sig(ic,i,j),
     &	                              bl_anz(i,j)+bl_anz(j,i)
	        endif
	      endif
	    enddo
	  enddo
	  if (ier_num.ne.0) return
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating distortions ',/,
     &	          '    Molecule types : A = ',I4,' and B = ',I4,' ',//,
     &            '    Neig.  mole A      mole B       distance',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,i9,3x,i9,5x,f7.3,3x,f7.3,3x,i8)
3000	format (3(f12.5,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_angle (ianz,cpara,lpara,werte,uerte,verte,
     &	                       maxw,lout)
c+
c	Calculates angular distortions within the crystal 
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'errlist.inc'
c
	integer		maxatom
c
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	real		uerte(maxw)
	real		verte(maxw)
	logical		lout
c
	character*9	at_name_i,at_name_j
	character*9     name_1,name_2,name_3
	integer		atom(0:maxatom),natom
	integer		iianz,i,j,k,l,is,js,ic
	integer		jjanz,kkanz
	integer         lname_1,lname_2,lname_3
	integer		ba_anz(0:maxscat,0:maxscat)
	real		ba_sum(0:maxscat,0:maxscat)
	real		ba_s2 (0:maxscat,0:maxscat)
	real		patom(3,0:maxatom)
	real		u(3),v(3),w(3),wi
	logical		lfile
c
	character*9	at_name
	real		do_bang
	logical		atom_allowed
c
	iianz = 1
	call get_iscat(iianz,cpara,lpara,werte,maxw,.false.)
	name_1  = cpara(1)
	lname_1 = lpara(1)
	call del_params (1,ianz,cpara,lpara,maxw)
	jjanz = 1
	call get_iscat(jjanz,cpara,lpara,uerte,maxw,.false.)
	name_2  = cpara(1)
	lname_2 = lpara(1)
	call del_params (1,ianz,cpara,lpara,maxw)
	kkanz = 1
	call get_iscat(kkanz,cpara,lpara,verte,maxw,.false.)
	name_3  = cpara(1)
	lname_3 = lpara(1)
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) name_1(1:lname_1),
     &	                         name_2(1:lname_2),
     &	                         name_3(1:lname_3)
	  if (ianz.gt.1) then
	    call oeffne (37,cpara(2),'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.1)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------ - reset counters, vectors, ..
c
	  do i=0,cr_nscat
	    do j=0,cr_nscat
	      ba_sum(i,j) = 0.0
	      ba_s2 (i,j) = 0.0
	      ba_anz(i,j) = 0
	    enddo
	  enddo
c
c------ - calculate angles
c
	  do i=1,cr_natoms
	    if (atom_allowed(i,werte,iianz,maxw)) then
	      call chem_neighbour(i,ic,atom,patom,natom,maxatom)
	      if (natom.gt.1) then
	        do k=1,3
	          u(k) = patom (k,1)
	        enddo
	        do j=2,natom-1,2
	          if (atom(j).ne.i) then
	            if (atom_allowed(atom(j),uerte,jjanz,maxw)) then
	              do k=1,3
	                v(k) = patom (k,j)
	              enddo
	                if (atom(j+1).ne.atom(j)) then
	                  if(atom_allowed(atom(j+1),verte,kkanz,maxw)) then
	                    do k=1,3
	                      w(k) = patom (k,j+1)
	                    enddo
	                    wi = do_bang(.true.,v,u,w)
	                    is = cr_iscat(atom(j))
	                    js = cr_iscat(atom(j+1))
	                    if (lfile) write(37,3000) u,is,js
	                    ba_sum(is,js) = ba_sum(is,js) + wi
	                    ba_s2 (is,js) = ba_s2 (is,js) + wi**2
	                    ba_anz(is,js) = ba_anz(is,js) + 1
	                  endif
	                endif
	            endif
	          endif
	        enddo
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	    do i=0,cr_nscat
	      do j=i,cr_nscat
	      if (ba_anz(i,j).ne.0 .or. ba_anz(j,i).ne.0 ) then
	        chem_disp_ave(ic,i,j)=(ba_sum(i,j)+ba_sum(j,i))/
     &	                              (ba_anz(i,j)+ba_anz(j,i))
	        chem_disp_sig(ic,i,j)=(ba_s2 (i,j)+ba_s2 (j,i))/
     &	                              (ba_anz(i,j)+ba_anz(j,i))
	        chem_disp_sig(ic,i,j)=(chem_disp_sig(ic,i,j)-
     &	                              (chem_disp_ave(ic,i,j)**2))
	        if (chem_disp_sig(ic,i,j).gt.0) then
	          chem_disp_sig(ic,i,j)=sqrt(chem_disp_sig(ic,i,j))
	        else
	          chem_disp_sig(ic,i,j)=0.0
	        endif
	        if (lout) then
	          at_name_i = at_name(i)
	          at_name_j = at_name(j)
	          write (output_io,2000) ic,at_name_i,at_name_j,
     &	                              chem_disp_ave(ic,i,j),
     &	                              chem_disp_sig(ic,i,j),
     &	                              ba_anz(i,j)+ba_anz(j,i)
	        endif
	      endif
	    enddo
	  enddo
	  if (ier_num.ne.0) return
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating angles ',/,
     &	          '    Zentral atom   = ',A4,/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &            '    Neig.  Atom A      Atom B       angle   ',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,a9,3x,a9,5x,f7.3,3x,f7.3,3x,i8)
3000	format (3(f12.5,1x),3x,2(i3,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_angle_multi (ianz,cpara,lpara,werte,uerte,verte,
     &	                       maxw,lout)
c+
c	Calculates angular distortions within the crystal 
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'errlist.inc'
c
	integer		maxatom
c
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw)
	real		werte(maxw)
	real		uerte(maxw)
	real		verte(maxw)
	logical		lout
c
	character*9	at_name_i,at_name_j
	character*9     name_1,name_2,name_3
	integer		atom(0:maxatom,CHEM_MAX_CENT)
	integer		natom(CHEM_MAX_CENT)
	integer		iianz,i,j,k,l,is,js,ic
	integer		jj
	integer		jjanz,kkanz
	integer		icent,ncent
	integer         lname_1,lname_2,lname_3
	integer		ba_anz(0:maxscat,0:maxscat)
	real		ba_sum(0:maxscat,0:maxscat)
	real		ba_s2 (0:maxscat,0:maxscat)
	real		patom(3,0:maxatom,CHEM_MAX_CENT)
	real		u(3),v(3),w(3),wi
	logical		lfile
c
	character*9	at_name
	real		do_bang
	logical		atom_allowed
c
	iianz = 1
	call get_iscat(iianz,cpara,lpara,werte,maxw,.false.)
	name_1  = cpara(1)
	lname_1 = lpara(1)
	call del_params (1,ianz,cpara,lpara,maxw)
	jjanz = 1
	call get_iscat(jjanz,cpara,lpara,uerte,maxw,.false.)
	name_2  = cpara(1)
	lname_2 = lpara(1)
	call del_params (1,ianz,cpara,lpara,maxw)
	kkanz = 1
	call get_iscat(kkanz,cpara,lpara,verte,maxw,.false.)
	name_3  = cpara(1)
	lname_3 = lpara(1)
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) name_1(1:lname_1),
     &	                         name_2(1:lname_2),
     &	                         name_3(1:lname_3)
	  if (ianz.gt.1) then
	    call oeffne (37,cpara(2),'unknown',.false.)
	  endif
	endif
c
	lfile = lout .and. (ianz.gt.1)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
	  if(chem_ctyp(ic).eq.CHEM_ANG .or. 
     &	     chem_ctyp(ic).eq.CHEM_ENVIR    ) then
c
c------ - reset counters, vectors, ..
c
	    do i=0,cr_nscat
	      do j=0,cr_nscat
	        ba_sum(i,j) = 0.0
	        ba_s2 (i,j) = 0.0
	        ba_anz(i,j) = 0
	      enddo
	    enddo
c
c------ - calculate angles
c
	    do i=1,cr_natoms
	      if (atom_allowed(i,werte,iianz,maxw)) then
	        call chem_neighbour_multi(i,ic,atom,patom,natom,ncent,
     &	                                  maxatom)
	        do icent=1,ncent
	          if (natom(icent).gt.1) then
	            do k=1,3
	              u(k) = patom (k,0,icent)
	            enddo
	            do j =1,natom(icent)-1
	              do jj=j+1,natom(icent)
	                if(atom_allowed(atom(j ,icent),uerte,jjanz,
     &	                                                     maxw).and.
     &	                   atom_allowed(atom(jj,icent),verte,kkanz,
     &	                                                     maxw) .or.
     &	                   atom_allowed(atom(jj,icent),uerte,jjanz,
     &	                                                     maxw) .and.
     &	                   atom_allowed(atom(j ,icent),verte,kkanz,
     &	                                                     maxw)) then
	                  do k=1,3
	                    v(k) = patom (k,j ,icent)
	                  enddo
	                  do k=1,3
	                    w(k) = patom (k,jj,icent)
	                  enddo
	                  wi = do_bang(.true.,v,u,w)
	                  is = cr_iscat(atom(j ,icent))
	                  js = cr_iscat(atom(jj,icent))
	                  if (lfile) write(37,3000) u,is,js
	                  ba_sum(is,js) = ba_sum(is,js) + wi
	                  ba_s2 (is,js) = ba_s2 (is,js) + wi**2
	                  ba_anz(is,js) = ba_anz(is,js) + 1
	                endif
	              enddo
	            enddo
	          endif
	        enddo
	      endif
	    enddo
c
c------ - write results and save to res_para block
c
	      do i=0,cr_nscat
	        do j=i,cr_nscat
	        if (ba_anz(i,j).ne.0 .or. ba_anz(j,i).ne.0 ) then
	          chem_disp_ave(ic,i,j)=(ba_sum(i,j)+ba_sum(j,i))/
     &	                                (ba_anz(i,j)+ba_anz(j,i))
	          chem_disp_sig(ic,i,j)=(ba_s2 (i,j)+ba_s2 (j,i))/
     &	                                (ba_anz(i,j)+ba_anz(j,i))
	          chem_disp_sig(ic,i,j)=(chem_disp_sig(ic,i,j)-
     &	                                (chem_disp_ave(ic,i,j)**2))
	          if (chem_disp_sig(ic,i,j).gt.0) then
	            chem_disp_sig(ic,i,j)=sqrt(chem_disp_sig(ic,i,j))
	          else
	            chem_disp_sig(ic,i,j)=0.0
	          endif
	          if (lout) then
	            at_name_i = at_name(i)
	            at_name_j = at_name(j)
	            write (output_io,2000) ic,at_name_i,at_name_j,
     &	                                chem_disp_ave(ic,i,j),
     &	                                chem_disp_sig(ic,i,j),
     &	                                ba_anz(i,j)+ba_anz(j,i)
	          endif
	        endif
	      enddo
	    enddo
	    if (ier_num.ne.0) return
	  endif
	enddo
c
	if (lfile) close(37)
c
1000	format (  ' Calculating angles ',/,
     &	          '    Zentral atom   = ',A4,/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &            '    Neig.  Atom A      Atom B       angle   ',
     &	          '   sigma     # pairs',/,4x,60('-'))
2000	format (4x,i3,3x,a9,3x,a9,5x,f7.3,3x,f7.3,3x,i8)
3000	format (3(f12.5,1x),3x,2(i3,1x))
c
	end
c*****7*****************************************************************
	subroutine chem_corr_dis (ianz,cpara,lpara,maxw,lout,lbeg)
c+
c	Calculates displacement correlations within the crystal 
c 	according to: cij = <x(i)x(j)>/sqrt(<x(i)**2><x(j)**2>)
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'param.inc'
	include		'errlist.inc'
c
	integer		maxww,maxatom
c
	parameter	(maxww   = MAXSCAT)
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw),lbeg(3)
	logical		lout
c
	integer		atom(0:maxatom),natom
	integer		icc(3),jcc(3)
	integer		i,j,ii,is,js,ic,iianz,jjanz,nn
	real		patom(3,0:maxatom)
	real		werte(maxww),wwerte(maxww)
	real		idir(3),jdir(3),di(3),dj(3)
	real		rdi,rdj,dpi,dpj
	real		xij,xi2,xj2
	logical		lvalid
c
	real		skalpro
	logical		atom_allowed,chem_inlot
c
c------ writing output line
c
	if (lout) then
	  write (output_io,1000) cpara(1)(1:lpara(1)),cpara(2)(1:lpara(2))
	endif
c
	iianz=1
	jjanz=1
	call get_iscat(iianz,cpara,lpara, werte,maxww,.false.)
	call del_params(1,ianz,cpara,lpara,maxw)
	call get_iscat(jjanz,cpara,lpara,wwerte,maxww,.false.)
	if (ier_num.ne.0) return
c
	call chem_aver(.false.)
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
	  if (chem_dir(1,1,ic).eq.-9999. .and. 
     &	      .not.chem_ldall(ic)             ) then
	    ier_num = -15
	    ier_typ = ER_CHEM
	    return
	  endif
c
	  nn  = 0
	  xij = 0.0
	  xi2 = 0.0
	  xj2 = 0.0
	  do i=1,3
	    idir(i) = chem_dir(i,1,ic)
	    jdir(i) = chem_dir(i,2,ic)
	  enddo
c
c------ - calculate correlations
c
	  rdi=skalpro(idir,idir,cr_gten)
	  rdj=skalpro(jdir,jdir,cr_gten)
	  if (rdi.gt.0.0) rdi = sqrt(rdi)
	  if (rdj.gt.0.0) rdj = sqrt(rdj)
c
	  do i=1,cr_natoms
c
c------ --- Check if selected atom is valid
c
	    lvalid = atom_allowed(i,werte,iianz,maxww)
	    if (lbeg(1).gt.0) then
	      lvalid = lvalid .and. chem_inlot(i,lbeg)
	    endif
c
	    if (lvalid) then
	      call chem_neighbour(i,ic,atom,patom,natom,maxatom)
	      if (natom.gt.0) then
	        call indextocell (i,icc,is)
	        do j=1,3
	          di(j) = cr_pos(j,i) - chem_ave_pos(j,is) -
     &	                  float(icc(j)-1)-cr_dim0(j,1)
	        enddo
c
	        if (chem_ldall(ic)) then
	          do j=1,3
	            jdir(j) = di(j)
	          enddo
	          rdj=skalpro(jdir,jdir,cr_gten)
	          if (rdj.gt.0.0) then
	            rdj = sqrt(rdj)
	          else
	            rdj = 1.0
	          endif
	          dpi = 1.0
	        else
	          dpi = skalpro(di,idir,cr_gten)/rdi
	        endif
c
	        do j=1,natom
	          if (atom_allowed(atom(j),wwerte,jjanz,maxww)) then
	            call indextocell (atom(j),jcc,js)
	            do ii=1,3
	              dj(ii)=cr_pos(ii,atom(j))-chem_ave_pos(ii,js) -
     &	                     float(jcc(ii)-1)-cr_dim0(ii,1)
	            enddo
	            dpj = skalpro(dj,jdir,cr_gten)/rdj
	            xij = xij + dpi*dpj
	            xi2 = xi2 + dpi**2
	            xj2 = xj2 + dpj**2
	            nn  = nn + 1
	          endif
	        enddo
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	  if (nn.ne.0) then
	    xij = xij/float(nn)
	    xi2 = xi2/float(nn)
	    xj2 = xj2/float(nn)
c
	    if (xi2.ne.0 .and. xj2.ne.0.0) then
	      mo_ach_corr(ic)=xij/sqrt(xi2*xj2)
	    else
	      mo_ach_corr(ic)=0.0
	    endif
c
	    if (lout) then
	      if (chem_ldall(ic)) then
	        write (output_io,1100) ic,nn,mo_ach_corr(ic)
	      else
	        write (output_io,1110) 
     &	                   ic,idir,jdir,nn,mo_ach_corr(ic)
	      endif
	    endif
c
	  else
	    ier_num = -8
	    ier_typ = ER_CHEM
	  endif
	  if (ier_num.ne.0) return
	enddo
c
c------ Save results to res[i]
c
	res_para(0)=chem_ncor
	do ic=1,chem_ncor
	  if (res_para(0).lt.maxpar_res) then
	    res_para(ic)=mo_ach_corr(ic)
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (  ' Calculating correlations ',/,
     &	          '    Atom type: A = ',A4,' B = ',A4,
     &	        //,4x,'Neig.',5x,'Displacement A',6x,'Displacement B',
     &	           6x,'# pairs',5x,'correlation',/,4x,73('-'))
1100	format (5x,i3,16x,'all directions',15x,i8,5x,f7.4)
1110	format (5x,i3,4x,3(f5.2,1x),2x,3(f5.2,1x),3x,i8,5x,f7.4)
c
	end
c*****7*****************************************************************
	subroutine chem_corr_dis_mol (ianz,cpara,lpara,maxw,lout,lbeg)
c+
c	Calculates displacement correlations within the crystal 
c 	according to: cij = <x(i)x(j)>/sqrt(<x(i)**2><x(j)**2>)
c	Molecule version ...
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'param.inc'
	include		'molecule.inc'
	include		'errlist.inc'
c
	integer		maxww,maxmol
c
	parameter	(maxww  =  3)
	parameter	(maxmol = 48)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw),lbeg(3)
	logical		lout
c
	integer		mol(0:maxmol),nmol
	integer		icc(3),jcc(3)
	integer		i,j,ii,is,js,ic,nn
	integer		it1,it2,imol,jmol
	real		werte(maxww)
	real		idir(3),jdir(3),di(3),dj(3)
	real		rdi,rdj,dpi,dpj
	real		xij,xi2,xj2
	logical		lvalid
c
	real		skalpro
	logical		chem_inlot
c
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
c
c------ check input
c
	it1=nint(werte(1))
	it2=nint(werte(2))
c
	if (it1.le.0 .or. it1.gt.mole_num_type .or.
     &	    it2.le.0 .or. it2.gt.mole_num_type     ) then
	  ier_num = -64
	  ier_typ = ER_APPL
	endif
c
	call chem_aver(.false.)
c
c------ writing output line
c
	if (lout) then
	  write (output_io,1000) it1,it2
	endif
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
	  if (chem_dir(1,1,ic).eq.-9999. .and. 
     &	      .not.chem_ldall(ic)             ) then
	    ier_num = -15
	    ier_typ = ER_CHEM
	    return
	  endif
c
	  nn  = 0
	  xij = 0.0
	  xi2 = 0.0
	  xj2 = 0.0
	  do i=1,3
	    idir(i) = chem_dir(i,1,ic)
	    jdir(i) = chem_dir(i,2,ic)
	  enddo
c
c------ - calculate correlations
c
	  rdi=skalpro(idir,idir,cr_gten)
	  rdj=skalpro(jdir,jdir,cr_gten)
	  if (rdi.gt.0.0) rdi = sqrt(rdi)
	  if (rdj.gt.0.0) rdj = sqrt(rdj)
c
	  do i=1,mole_num_mole
	    imol = mole_cont(mole_off(i)+1)
c
c------ --- Check if selected atom is valid
c
	    lvalid = mole_type(i).eq.it1 .or. mole_type(i).eq.it2
	    if (lbeg(1).gt.0) then
	      lvalid = lvalid .and. chem_inlot(imol,lbeg)
	    endif
c
	    if (lvalid) then
	      call chem_neighbour_mol(i,ic,mol,nmol,maxmol)
	      if (nmol.gt.0) then
	        call indextocell (imol,icc,is)
	        do j=1,3
	          di(j) = cr_pos(j,imol) - chem_ave_pos(j,is) -
     &	                  float(icc(j)-1)-cr_dim0(j,1)
	        enddo
c
	        if (chem_ldall(ic)) then
	          do j=1,3
	            jdir(j) = di(j)
	          enddo
	          rdj=skalpro(jdir,jdir,cr_gten)
	          if (rdj.gt.0.0) then
	            rdj = sqrt(rdj)
	          else
	            rdj = 1.0
	          endif
	          dpi = 1.0
	        else
	          dpi = skalpro(di,idir,cr_gten)/rdi
	        endif
c
	        do j=1,nmol
	          if (mole_type(mol(j)).eq.it1 .or. 
     &	              mole_type(mol(j)).eq.it2) then
	            jmol = mole_cont(mole_off(mol(j))+1)
	            call indextocell (jmol,jcc,js)
	            do ii=1,3
	              dj(ii)=cr_pos(ii,jmol)-chem_ave_pos(ii,js) -
     &	                     float(jcc(ii)-1)-cr_dim0(ii,1)
	            enddo
	            dpj = skalpro(dj,jdir,cr_gten)/rdj
	            xij = xij + dpi*dpj
	            xi2 = xi2 + dpi**2
	            xj2 = xj2 + dpj**2
	            nn  = nn + 1
	          endif
	        enddo
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	  if (nn.ne.0) then
	    xij = xij/float(nn)
	    xi2 = xi2/float(nn)
	    xj2 = xj2/float(nn)
c
	    if (xi2.ne.0 .and. xj2.ne.0.0) then
	      mo_ach_corr(ic)=xij/sqrt(xi2*xj2)
	    else
	      mo_ach_corr(ic)=0.0
	    endif
c
	    if (lout) then
	      if (chem_ldall(ic)) then
	        write (output_io,1100) ic,nn,mo_ach_corr(ic)
	      else
	        write (output_io,1110) 
     &	                   ic,idir,jdir,nn,mo_ach_corr(ic)
	      endif
	    endif
c
	  else
	    ier_num = -8
	    ier_typ = ER_CHEM
	  endif
	  if (ier_num.ne.0) return
	enddo
c
c------ Save results to res[i]
c
	res_para(0)=chem_ncor
	do ic=1,chem_ncor
	  if (res_para(0).lt.maxpar_res) then
	    res_para(ic)=mo_ach_corr(ic)
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (  ' Calculating correlations ',/,
     &	          '    Molecule type: A = ',I4,' B = ',I4,
     &	        //,4x,'Neig.',5x,'Displacement A',6x,'Displacement B',
     &	           6x,'# pairs',5x,'correlation',/,4x,73('-'))
1100	format (5x,i3,16x,'all directions',15x,i8,5x,f7.4)
1110	format (5x,i3,4x,3(f5.2,1x),2x,3(f5.2,1x),3x,i8,5x,f7.4)
c
	end
c*****7*****************************************************************
	subroutine chem_corr_occ (ianz,cpara,lpara,maxw,lout,lbeg)
c+
c	Calculates occupational correlations within the crystal 
c 	according to: cij = (Pij-T**2)/T(1-T).
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'param.inc'
	include		'errlist.inc'
c
	integer		maxww,maxatom
c
	parameter	(maxww   = MAXSCAT)
	parameter	(maxatom = chem_max_neig)
c
	integer		ianz,maxw
	character*200	cpara(maxw),ccpara(MAXWW)
	integer		lpara(maxw),lbeg(3),llpara(MAXWW)
	logical		lout
c
	integer		atom(0:maxatom),natom
	integer		i,j,is,js,ic,iianz,jjanz
	integer		nneig
	real		patom(3,0:maxatom)
	real		werte(maxww),wwerte(maxww)
	real		pneig(2,2)
	real		pro00,pro01,pro11
	real		thet
	logical		lvalid
c
	logical		atom_allowed,chem_inlot
c
	iianz=1
	ccpara(1) = cpara(1)
	llpara(1) = lpara(1)
	call get_iscat(iianz,ccpara,llpara, werte,maxww,.false.)
c
	jjanz=1
	ccpara(1) = cpara(2)
	llpara(1) = lpara(2)
	call get_iscat(jjanz,ccpara,llpara,wwerte,maxww,.false.)
	if (ier_num.ne.0) return
c
c------ check input
c
	if (int(werte(1)).eq.-1 .or. int(wwerte(1)).eq.-1) then
	  ier_num = -6
	  ier_typ = ER_CHEM
	  return
	endif
c
	if (int(werte(1)).eq.int(wwerte(1))) then
	  ier_num = -7
	  ier_typ = ER_CHEM
	  return
	endif
c
c------ write output
c
	is=int( werte(1))
	js=int(wwerte(1))
c
	if (lout) then
	  write (output_io,1000) cr_at_lis(is),cr_at_lis(js)
	endif
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------ - reset counters, vectors, ..
c
	  do i=1,2
	    do j=1,2
	      pneig(i,j) = 0.0
	    enddo
	  enddo
c
c------ - calculate correlations
c
	  do i=1,cr_natoms
c
c------ --- Check if selected atom is valid
c
	    lvalid = .true.
	    if (lbeg(1).gt.0) then
	      lvalid = lvalid .and. chem_inlot(i,lbeg)
	    endif
c
	    if (lvalid) then
	      is = -1
	      if (atom_allowed(i, werte,iianz,maxww)) is = 1
	      if (atom_allowed(i,wwerte,jjanz,maxww)) is = 2
	      if (is.gt.0) then
	        call chem_neighbour(i,ic,atom,patom,natom,maxatom)
	        if (natom.gt.0) then
	          do j=1,natom
	            js = -1
	            if (atom_allowed(atom(j), werte,iianz,maxww)) js = 1
	            if (atom_allowed(atom(j),wwerte,jjanz,maxww)) js = 2
	            if (js.gt.0) then
	              pneig(is,js) = pneig(is,js) + 1.0
	            endif
	          enddo
	        endif
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	  nneig=nint(pneig(1,1)+pneig(2,2)+pneig(1,2)+pneig(2,1))
	  if (nneig.ne.0) then
	    pro00=pneig(1,1)/float(nneig)
	    pro01=(pneig(1,2)+pneig(2,1))/float(nneig)
	    pro11=pneig(2,2)/float(nneig)
	    thet=0.5*(2.0*pneig(1,1)+pneig(1,2)+
     &	                  pneig(2,1))/float(nneig)
	    if (thet.ne.0 .and. thet.ne.1.0) then
	      mo_ach_corr(ic)=(pro00-thet**2)/(thet*(1-thet))
	    else
	      mo_ach_corr(ic)=0.0
	    endif
c
	    if (lout) write (output_io,1100) ic,100.*pro00,100.*pro01,
     &	                          100.*pro11,nneig,mo_ach_corr(ic)
c
	  else
	    ier_num = -8
	    ier_typ = ER_CHEM
	  endif
	  if (ier_num.ne.0) return
	enddo
c
c------ Save results to res[i]
c
	res_para(0)=chem_ncor
	do ic=1,chem_ncor
	  if (res_para(0).lt.maxpar_res) then
	    res_para(ic)=mo_ach_corr(ic)
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (  ' Calculating correlations ',/,
     &	          '    Atom types : A = ',A4,' and B = ',A4,' ',//,
     &	          4x,'Neig.',5x,'AA',9x,'AB',9x,'BB',9x,
     &	          '# pairs    correlation',/,4x,65('-'))
1100	format (5x,i3,3x,3(f6.2,' % ',2x),1x,i8,6x,f7.4)
c
	end
c*****7*****************************************************************
	subroutine chem_corr_occ_mol (ianz,cpara,lpara,maxw,lout,lbeg)
c+
c	Calculates occupational correlations within the crystal 
c 	according to: cij = (Pij-T**2)/T(1-T).
c	Molecules version ...
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include 	'chem.inc'
	include 	'mc.inc'
	include		'param.inc'
	include		'errlist.inc'
	include		'molecule.inc'
c
	integer		maxmol,maxww
	parameter	(maxmol = 48)
	parameter	(maxww  =  3)
c
	integer		ianz,maxw
	character*200	cpara(maxw)
	integer		lpara(maxw),lbeg(3)
	logical		lout
c
	integer		mol(0:maxmol),nmol
	integer		i,j,is,js,ic
	integer		imol,it1,it2
	integer		nneig
	real		werte(maxww)
	real		pneig(2,2)
	real		pro00,pro01,pro11
	real		thet
	logical		lvalid
c
	logical		chem_inlot
c
	call ber_params (ianz,cpara,lpara,werte,maxw)
	if (ier_num.ne.0) return
c
c------ check input
c
	if (ianz.eq.2) then
	  it1=nint(werte(1))
	  it2=nint(werte(2))
c
	  if (it1.le.0 .or. it1.gt.mole_num_type .or.
     &	      it2.le.0 .or. it2.gt.mole_num_type     ) then
	    ier_num = -64
	    ier_typ = ER_APPL
	  endif
c
	  if (it1.eq.it2) then
	    ier_num = -21
	    ier_typ = ER_CHEM
	  endif
c
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
	if (ier_num.ne.0) return
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) it1,it2
	endif
c
c------ loop over all defined correlations
c
	do ic=1,chem_ncor
c
c------ - reset counters, vectors, ..
c
	  do i=1,2
	    do j=1,2
	      pneig(i,j) = 0.0
	    enddo
	  enddo
c
c------ - calculate correlations
c
	  do i=1,mole_num_mole
	    imol = mole_cont(mole_off(i)+1)
c
c------ --- Check if selected atom is valid
c
	    lvalid = .true.
	    if (lbeg(1).gt.0) then
	      lvalid = lvalid .and. chem_inlot(imol,lbeg)
	    endif
c
	    if (lvalid) then
	      is = -1
	      if (mole_type(i).eq.it1) is = 1
	      if (mole_type(i).eq.it2) is = 2
	      if (is.gt.0) then
	        call chem_neighbour_mol(i,ic,mol,nmol,maxmol)
	        if (nmol.gt.0) then
	          do j=1,nmol
	            js = -1
	            if (mole_type(mol(j)).eq.it1) js = 1
	            if (mole_type(mol(j)).eq.it2) js = 2
	            if (js.gt.0) then
	              pneig(is,js) = pneig(is,js) + 1.0
	            endif
	          enddo
	        endif
	      endif
	    endif
	  enddo
c
c------ - write results and save to res_para block
c
	  nneig=nint(pneig(1,1)+pneig(2,2)+pneig(1,2)+pneig(2,1))
	  if (nneig.ne.0) then
	    pro00=pneig(1,1)/float(nneig)
	    pro01=(pneig(1,2)+pneig(2,1))/float(nneig)
	    pro11=pneig(2,2)/float(nneig)
	    thet=0.5*(2.0*pneig(1,1)+pneig(1,2)+
     &	                  pneig(2,1))/float(nneig)
	    if (thet.ne.0 .and. thet.ne.1.0) then
	      mo_ach_corr(ic)=(pro00-thet**2)/(thet*(1-thet))
	    else
	      mo_ach_corr(ic)=0.0
	    endif
c
	    if (lout) write (output_io,1100) ic,100.*pro00,100.*pro01,
     &	                          100.*pro11,nneig,mo_ach_corr(ic)
c
	  else
	    ier_num = -19
	    ier_typ = ER_CHEM
	  endif
	  if (ier_num.ne.0) return
	enddo
c
c------ Save results to res[i]
c
	res_para(0)=chem_ncor
	do ic=1,chem_ncor
	  if (res_para(0).lt.maxpar_res) then
	    res_para(ic)=mo_ach_corr(ic)
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (  ' Calculating correlations ',/,
     &	          '    Molecule types : A = ',I4,' and B = ',I4,' ',//,
     &	          4x,'Neig.',5x,'AA',9x,'AB',9x,'BB',9x,
     &	          '# pairs    correlation',/,4x,65('-'))
1100	format (5x,i3,3x,3(f6.2,' % ',2x),1x,i8,6x,f7.4)
c
	end
c*****7*****************************************************************
	subroutine chem_neighbour(jatom,ic,iatom,patom,natom,maxw)
c+
c	Determine neighbours from given atom index 'jatom'.
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'atom_env.inc'
	include		'param.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw
c
	integer		iatom(0:maxw),jatom,natom,ic
	real		patom(3,0:maxw)
c
	real		u(3),v(3),w(3),uu(3)
	real		offset(3),dummy
	integer		jcell(3),icell(3),isite,jsite
	integer		i,j,k,ii,iv,katom
	logical 	lok
c
	real		do_bang
c
	natom = 0
	dummy = -1

c
c------ ------------------------------------------------------
c------ Mode distance
c------ ------------------------------------------------------
c
	if (chem_ctyp(ic).eq.CHEM_DIST) then
	  do j=1,3
	    u(j)=cr_pos(j,jatom)
	    w(j)=0.0
	  enddo
	  call do_find_env(1,dummy,1,u,chem_rmin(ic),chem_rmax(ic),
     &	                   chem_quick,chem_period)
	  if (atom_env(0).ne.0) then
	    do j=1,atom_env(0)
	      if (chem_cang(ic)) then
c
c------ ------- Check neighbouring angle and symmetry
c
	        do k=1,chem_nnei(ic)
	          do ii=1,3
	            v(ii) =atom_pos(ii,j)-cr_pos(ii,jatom)
	            uu(ii)=chem_neig(ii,k,ic)
	          enddo
	          if (abs(do_bang(.true.,v,w,uu)).lt.
     &	                          chem_wink_sigma(ic)) then
	            natom          = natom + 1
	            if (natom.le.maxw) then
	              iatom(natom)   = atom_env(j)
	              patom(1,natom) = atom_pos(1,j)
	              patom(2,natom) = atom_pos(2,j)
	              patom(3,natom) = atom_pos(3,j)
	            else
	              ier_num = -23
	              ier_typ = ER_CHEM
	              return
	            endif
	          endif
	        enddo
c
c------ ------- Check just distance
c
	      else
	        natom          = natom + 1
	        if (natom.le.maxw) then
	          iatom(natom)   = atom_env(j)
	          patom(1,natom) = atom_pos(1,j)
	          patom(2,natom) = atom_pos(2,j)
	          patom(3,natom) = atom_pos(3,j)
	        else
	          ier_num = -23
	          ier_typ = ER_CHEM
	          return
	        endif
	      endif
	    enddo
	  endif
c
c------ ------------------------------------------------------
c------ Mode vector
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_VEC) then
	  call indextocell (jatom,jcell,jsite)
	  do i=1,chem_nvec(ic)
	    iv = chem_use_vec(i,ic)
	    if (jsite.eq.chem_cvec(1,iv)) then
	      icell(1)=jcell(1)+chem_cvec(3,iv)
	      icell(2)=jcell(2)+chem_cvec(4,iv)
	      icell(3)=jcell(3)+chem_cvec(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cvec(2,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	    elseif(jsite.eq.chem_cvec(2,iv)) then
	      icell(1)=jcell(1)-chem_cvec(3,iv)
	      icell(2)=jcell(2)-chem_cvec(4,iv)
	      icell(3)=jcell(3)-chem_cvec(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cvec(1,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	    endif
	  enddo
c
c------ ------------------------------------------------------
c------ Mode angular neighbours
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_ANG) then
	  call indextocell (jatom,jcell,jsite)
	  do i=1,chem_nwin(ic)
	    iv = chem_use_win(i,ic)
	    if (jsite.eq.chem_cwin(1,iv)) then
	      natom          = natom + 1
	      iatom(natom)   = jatom
	      patom(1,natom) = cr_pos(1,jatom)
	      patom(2,natom) = cr_pos(2,jatom)
	      patom(3,natom) = cr_pos(3,jatom)
	      icell(1)=jcell(1)+chem_cwin(3,iv)
	      icell(2)=jcell(2)+chem_cwin(4,iv)
	      icell(3)=jcell(3)+chem_cwin(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(2,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	      icell(1)=jcell(1)+chem_cwin(7,iv)
	      icell(2)=jcell(2)+chem_cwin(8,iv)
	      icell(3)=jcell(3)+chem_cwin(9,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(6,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	    elseif (jsite.eq.chem_cwin(2,iv)) then
	      icell(1)=jcell(1)-chem_cwin(3,iv)
	      icell(2)=jcell(2)-chem_cwin(4,iv)
	      icell(3)=jcell(3)-chem_cwin(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(1,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	      natom          = natom + 1
	      iatom(natom)   = jatom
	      patom(1,natom) = cr_pos(1,jatom)
	      patom(2,natom) = cr_pos(2,jatom)
	      patom(3,natom) = cr_pos(3,jatom)
	      icell(1)=jcell(1)-chem_cwin(3,iv)+chem_cwin(7,iv)
	      icell(2)=jcell(2)-chem_cwin(4,iv)+chem_cwin(8,iv)
	      icell(3)=jcell(3)-chem_cwin(5,iv)+chem_cwin(9,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(6,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	    elseif (jsite.eq.chem_cwin(6,iv)) then
	      icell(1)=jcell(1)-chem_cwin(7,iv)
	      icell(2)=jcell(2)-chem_cwin(8,iv)
	      icell(3)=jcell(3)-chem_cwin(9,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(1,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	      icell(1)=jcell(1)-chem_cwin(7,iv)+chem_cwin(3,iv)
	      icell(2)=jcell(2)-chem_cwin(8,iv)+chem_cwin(4,iv)
	      icell(3)=jcell(3)-chem_cwin(9,iv)+chem_cwin(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(2,iv)
	        call celltoindex (icell,isite,katom)
	        natom          = natom + 1
	        iatom(natom)   = katom
	        patom(1,natom) = cr_pos(1,katom) + offset(1)
	        patom(2,natom) = cr_pos(2,katom) + offset(2)
	        patom(3,natom) = cr_pos(3,katom) + offset(3)
	      endif
	      natom          = natom + 1
	      iatom(natom)   = jatom
	      patom(1,natom) = cr_pos(1,jatom)
	      patom(2,natom) = cr_pos(2,jatom)
	      patom(3,natom) = cr_pos(3,jatom)
	    endif
	  enddo
	endif
CDBG
CDBG	write(*,*) 'CHEM_N natom,iatom ',natom,(iatom(i),i=1,natom)
CDBG	do j=1,natom
CDBG	  write(*,*) 'CHEM_N patom       ',(patom(i,j),i=1,3)
CDBG	enddo
c
	end
c*****7*****************************************************************
	subroutine chem_neighbour_multi(jatom,ic,iatom,patom,natom,
     &	                                 ncent,maxw)
c+
c	Determine neighbours from given atom index 'jatom'.
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'atom_env.inc'
	include		'param.inc'
	include		'errlist.inc'
	include 	'chem.inc'
	include		'debug.inc'
c
	integer		maxw
c
	integer		jatom,ic
	integer		iatom(0:maxw,CHEM_MAX_CENT)
	integer		natom(CHEM_MAX_CENT)
	integer		ncent
	real		patom(3,0:maxw,CHEM_MAX_CENT)
	real		dist(CHEM_MAX_CENT)
	real		werte(MAX_ATOM_ENV)
c
	real		u(3),v(3),w(3),uu(3)
	real		offset(3),dummy
	integer		jcell(3),icell(3),isite,jsite
	integer		i,j,k,l,ii,iv,katom,ianz
	integer		icent,nnew
	logical		laccept
	logical 	lok
	logical		lis_neig
	logical		ldbg
c
	logical		atom_allowed
	real		do_bang
	real		do_blen
c
	do i=1,CHEM_MAX_CENT
	  natom(i) = 0
CDBG	  do j=0,maxw
CDBG	    iatom(j,i) = 0
CDBG	    patom(1,j,i) = 0
CDBG	    patom(2,j,i) = 0
CDBG	    patom(3,j,i) = 0
CDBG	  enddo
	enddo
	dummy = -1
	ncent = 1
c
CDBG
	ldbg = .false.
CDBG	if(jatom.eq.5                  ) then
CDBG	  write(*,*) ' chem_ctyp(ic) ',chem_ctyp(ic)
CDBG	  ldbg = .true.
CDBG	endif
CDBG	dbg  = ldbg
CDBG	dbg  = .false.
c
c------ ------------------------------------------------------
c------ Mode distance
c------ ------------------------------------------------------
c
	if (chem_ctyp(ic).eq.CHEM_DIST) then
	  ncent        = 1
	  natom(ncent) = 0
	  do j=1,3
	    u(j)=cr_pos(j,jatom)
	    w(j)=0.0
	  enddo
	  call do_find_env(1,dummy,1,u,chem_rmin(ic),chem_rmax(ic),
     &	                   chem_quick,chem_period)
	  if (atom_env(0).ne.0) then
	    do j=1,atom_env(0)
	      if (chem_cang(ic)) then
c
c------ ------- Check neighbouring angle and symmetry
c
	        do k=1,chem_nnei(ic)
	          do ii=1,3
	            v(ii) =atom_pos(ii,j)-cr_pos(ii,jatom)
	            uu(ii)=chem_neig(ii,k,ic)
	          enddo
CDBG	if(jatom.eq.98 .and. cr_iscat(atom_env(j)).eq.8) then
CDBG	  dummy = do_bang(.true.,v,w,uu)
CDBG	  write(*,*) 'Atom  Typ ',98,cr_iscat(98) ,(cr_pos(ii,98),ii=1,3)
CDBG	  write(*,*) 'Nachbar   ',atom_env(j),cr_iscat(atom_env(j)),
CDBG     &	              (atom_pos(ii,j),ii=1,3)
CDBG	  write(*,*) ' v,Angle  ',v,dummy
CDBG	endif
	          if (abs(do_bang(.true.,v,w,uu)).lt.
     &	                          chem_wink_sigma(ic)) then
	            natom(ncent)          = natom(ncent) + 1
	            if (natom(ncent).le.maxw) then
	              iatom(natom(ncent),ncent)   = atom_env(j)
	              patom(1,natom(ncent),ncent) = atom_pos(1,j)
	              patom(2,natom(ncent),ncent) = atom_pos(2,j)
	              patom(3,natom(ncent),ncent) = atom_pos(3,j)
	            else
	              ier_num = -23
	              ier_typ = ER_CHEM
	              return
	            endif
	          endif
	        enddo
c
c------ ------- Check just distance
c
	      else
	        natom(ncent)          = natom(ncent) + 1
	        if (natom(ncent).le.maxw) then
	          iatom(natom(ncent),ncent)   = atom_env(j)
	          patom(1,natom(ncent),ncent) = atom_pos(1,j)
	          patom(2,natom(ncent),ncent) = atom_pos(2,j)
	          patom(3,natom(ncent),ncent) = atom_pos(3,j)
	        else
	          ier_num = -23
	          ier_typ = ER_CHEM
	          return
	        endif
	      endif
	    enddo
	  endif
	  if(natom(ncent).gt.0) then
	    iatom(  0,ncent) = jatom
	    patom(1,0,ncent) = cr_pos(1,jatom)
	    patom(2,0,ncent) = cr_pos(2,jatom)
	    patom(3,0,ncent) = cr_pos(3,jatom)
	  endif
c
c------ ------------------------------------------------------
c------ Mode distance-range (New mode for mmc)
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_RANGE) then
	    i  = 1
	    iv = chem_use_ran(i,ic)
c
c------	--Check whether central atom is allowed
c
	  if(chem_cran_cent(0,iv).eq.-1) then
	    laccept = .true.
	  else
	    ianz = chem_cran_cent(0,iv)
	    do j=1,chem_cran_cent(0,iv)
	      werte(j) = chem_cran_cent(j,iv)
	    enddo
	    laccept = atom_allowed(jatom,werte,ianz,maxw)
	  endif
CDBG
	if(ldbg) then
	write(*,*) ' ic       ',ic
	write(*,*) ' werte    ',ianz,'|',(werte(j),j=1,ianz)
	write(*,*) ' jatom    ',jatom
	write(*,*) ' cr_iscat ',cr_iscat(jatom)
	write(*,*) ' laccept  ',laccept
	endif
	  if(.not. laccept) return
	if(ldbg) then
	write(*,*) ' jatom    ',jatom
	write(*,*) ' cr_iscat ',cr_iscat(jatom)
	endif
c
	  ncent        = 1
	  natom(ncent) = 0
	  do j=1,3
	    u(j)=cr_pos(j,jatom)
	    w(j)=0.0
	  enddo
	  dist(1) = 0.0
c
c------	--Check whether neighbour atom is allowed
c
	  if(chem_cran_neig(0,iv).eq.-1) then
	    werte(1) = -1
	    ianz     = 1
	  else
	    ianz = chem_cran_neig(0,iv)
	    do j=1,chem_cran_neig(0,iv)
	      werte(j) = chem_cran_neig(j,iv)
	    enddo
	  endif
CDBG
	if(ldbg) then
	write(*,*) ' ianz, werte ',ianz,'|',(werte(j),j=1,ianz)
	write(*,*) ' rmin,rmax   ',chem_cran_rmin(iv),chem_cran_rmax(iv)
	write(*,*) ' u           ',u
	endif
	  call do_find_env(1,dummy,1,u,chem_cran_rmin(iv),
     &	                   chem_cran_rmax(iv),
     &	                   chem_quick,chem_period)
	if(ldbg) then
	write(*,*) ' atom_env(0) ',atom_env(0)
	endif
	  if (atom_env(0).ne.0) then
	    do j=1,atom_env(0)
	      k = atom_env(j)
	      laccept = atom_allowed(k,werte,ianz,maxw)
	      if(laccept) then
CDBG
	if(ldbg) then
	write(*,*) ' neighbour ',jatom
	write(*,*) ' angle?    ',chem_cran_cang(iv)
	endif
c
	      if (chem_cran_cang(iv)) then
c
c------ ------- Check neighbouring angle and symmetry
c
	        do k=1,chem_cran_nuvw(iv)
	          do ii=1,3
	            v(ii) =atom_pos(ii,j)-cr_pos(ii,jatom)
	            uu(ii)=chem_cran_uvw(ii,k,iv)
	          enddo
	          dummy = do_blen(.true.,w,v)
CDBG
	if(ldbg) then
	  write(*,*) 'Nachbar   ',atom_env(j),cr_iscat(atom_env(j)),
     &	              (atom_pos(ii,j),ii=1,3)
	  write(*,*) ' v,len,an ',v,dummy,abs(do_bang(.true.,v,w,uu))
	endif
	          if (abs(do_bang(.true.,v,w,uu)).lt. 
     &	              chem_cran_wsig(iv)                  ) then
	            natom(ncent)          = natom(ncent) + 1
	            if (natom(ncent).le.maxw) then
	ii = 1
	do while(dummy.gt.dist(ii) .and. ii.lt.natom(ncent))
	  ii = ii + 1
	enddo
	do l = natom(ncent)-1,ii,-1
	  iatom(  l+1,ncent) = iatom(  l,ncent)
	  patom(1,l+1,ncent) = patom(1,l,ncent)
	  patom(2,l+1,ncent) = patom(2,l,ncent)
	  patom(3,l+1,ncent) = patom(3,l,ncent)
	  dist (  l+1      ) = dist (  l      )
	enddo
	              iatom(  ii,ncent) = atom_env(j)
	              patom(1,ii,ncent) = atom_pos(1,j)
	              patom(2,ii,ncent) = atom_pos(2,j)
	              patom(3,ii,ncent) = atom_pos(3,j)
	              dist (  ii      ) = dummy
CDBG	              iatom(natom(ncent),ncent)   = atom_env(j)
CDBG	              patom(1,natom(ncent),ncent) = atom_pos(1,j)
CDBG	              patom(2,natom(ncent),ncent) = atom_pos(2,j)
CDBG	              patom(3,natom(ncent),ncent) = atom_pos(3,j)
CDBG
	if(ldbg) then
	do l=1,natom(ncent)
	  write(*,*) ' l,i,p ',l,iatom(l,ncent),(patom(ii,l,ncent),ii=1,3),
     &	                       dist(l)
	enddo
	endif
	            else
	              ier_num = -23
	              ier_typ = ER_CHEM
	              return
	            endif
	          endif
	        enddo
c
c------ ------- Check just distance
c
	      else
	        natom(ncent)          = natom(ncent) + 1
	        if (natom(ncent).le.maxw) then
	ii = 1
	do while(dummy.gt.dist(ii) .and. ii.lt.natom(ncent))
	  ii = ii + 1
	enddo
	do l = natom(ncent)-1,ii,-1
	  iatom(  l+1,ncent) = iatom(  l,ncent)
	  patom(1,l+1,ncent) = patom(1,l,ncent)
	  patom(2,l+1,ncent) = patom(2,l,ncent)
	  patom(3,l+1,ncent) = patom(3,l,ncent)
	  dist (  l+1      ) = dist (  l      )
	enddo
	              iatom(  ii,ncent) = atom_env(j)
	              patom(1,ii,ncent) = atom_pos(1,j)
	              patom(2,ii,ncent) = atom_pos(2,j)
	              patom(3,ii,ncent) = atom_pos(3,j)
	              dist (  ii      ) = dummy
CDBG	          iatom(natom(ncent),ncent)   = atom_env(j)
CDBG	          patom(1,natom(ncent),ncent) = atom_pos(1,j)
CDBG	          patom(2,natom(ncent),ncent) = atom_pos(2,j)
CDBG	          patom(3,natom(ncent),ncent) = atom_pos(3,j)
	        else
	          ier_num = -23
	          ier_typ = ER_CHEM
	          return
	        endif
	      endif
	      endif
	    enddo
	  endif
	  if(natom(ncent).gt.0) then
	    iatom(  0,ncent) = jatom
	    patom(1,0,ncent) = cr_pos(1,jatom)
	    patom(2,0,ncent) = cr_pos(2,jatom)
	    patom(3,0,ncent) = cr_pos(3,jatom)
	  endif
c
c------	--Only the shortest vectors are to be considered
c
	  if(chem_cran_short(iv)) then
	if(ldbg) then
	  write(*,*) ' natom(ncent) ',  natom(ncent)
	  write(*,*) ' nshort       ', chem_cran_nshort(iv)
	endif
	    do l=chem_cran_nshort(iv)+1,natom(ncent)
	      iatom(  l,ncent) = 0
	      patom(1,l,ncent) = 0.0
	      patom(2,l,ncent) = 0.0
	      patom(3,l,ncent) = 0.0
	    enddo
	    natom(ncent) = min(natom(ncent),chem_cran_nshort(iv))
	  endif
	    
c
c------ ------------------------------------------------------
c------ Mode vector
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_VEC) then
	  ncent = 0
	  call indextocell (jatom,jcell,jsite)
	  do i=1,chem_nvec(ic)
	    ncent = ncent + 1
	    iatom(natom(ncent),ncent) = jatom
	    patom(1,0,ncent) = cr_pos(1,jatom)
	    patom(2,0,ncent) = cr_pos(2,jatom)
	    patom(3,0,ncent) = cr_pos(3,jatom)
	    iv = chem_use_vec(i,ic)
	    if (jsite.eq.chem_cvec(1,iv)) then
	      icell(1)=jcell(1)+chem_cvec(3,iv)
	      icell(2)=jcell(2)+chem_cvec(4,iv)
	      icell(3)=jcell(3)+chem_cvec(5,iv)
	      lok=.false.
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cvec(2,iv)
	        call celltoindex (icell,isite,katom)
	        natom(ncent)          = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = katom
	        patom(1,natom(ncent),ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,natom(ncent),ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,natom(ncent),ncent) = cr_pos(3,katom) +offset(3)
	      endif
	    elseif(jsite.eq.chem_cvec(2,iv)) then
	      icell(1)=jcell(1)-chem_cvec(3,iv)
	      icell(2)=jcell(2)-chem_cvec(4,iv)
	      icell(3)=jcell(3)-chem_cvec(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cvec(1,iv)
	        call celltoindex (icell,isite,katom)
	        natom(ncent)   = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = katom
	        patom(1,natom(ncent),ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,natom(ncent),ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,natom(ncent),ncent) = cr_pos(3,katom) +offset(3)
	      endif
	    endif
	    if(natom(ncent).eq.0) then
	      ncent = ncent - 1
	    endif
	  enddo
c
c------ ------------------------------------------------------
c------ Mode angular neighbours
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_ANG) then
	  ncent = 0
	  call indextocell (jatom,jcell,jsite)
CDBG
	if(ldbg) then
	  write(*,*) ' Starte Loop ueber definierte Winkel'
	  write(*,*) ' Atom# Typ Site ',jatom,cr_iscat(jatom),jsite
	endif
	  do i=1,chem_nwin(ic)
	    ncent = ncent + 1
	    iv = chem_use_win(i,ic)
CDBG
	if(ldbg) then
	  write(*,*) ' i, ncent,iv,chem_cwin(1,iv) ',i,ncent,iv,chem_cwin(1,iv)
	endif
	    if (jsite.eq.chem_cwin(1,iv)) then
c
c------	------Selected atom is center of the angle
c
	      iatom(0,ncent)   = jatom
	      patom(1,0,ncent) = cr_pos(1,jatom)
	      patom(2,0,ncent) = cr_pos(2,jatom)
	      patom(3,0,ncent) = cr_pos(3,jatom)
	      icell(1)=jcell(1)+chem_cwin(3,iv)
	      icell(2)=jcell(2)+chem_cwin(4,iv)
	      icell(3)=jcell(3)+chem_cwin(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(2,iv)
	        call celltoindex (icell,isite,katom)
	        natom(ncent)   = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = katom
	        patom(1,natom(ncent),ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,natom(ncent),ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,natom(ncent),ncent) = cr_pos(3,katom) +offset(3)
	      endif
	      icell(1)=jcell(1)+chem_cwin(7,iv)
	      icell(2)=jcell(2)+chem_cwin(8,iv)
	      icell(3)=jcell(3)+chem_cwin(9,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(6,iv)
	        call celltoindex (icell,isite,katom)
	        natom(ncent)   = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = katom
	        patom(1,natom(ncent),ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,natom(ncent),ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,natom(ncent),ncent) = cr_pos(3,katom) +offset(3)
	      endif
	    elseif (jsite.eq.chem_cwin(2,iv)) then
c
c------	------Selected atom is first neighbour of the central atom 
c
	      icell(1)=jcell(1)-chem_cwin(3,iv)
	      icell(2)=jcell(2)-chem_cwin(4,iv)
	      icell(3)=jcell(3)-chem_cwin(5,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
c------	--------The central atom is within the boundary condition
	        isite = chem_cwin(1,iv)
	        call celltoindex (icell,isite,katom)
	        iatom(0,ncent)   = katom
	        patom(1,0,ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,0,ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,0,ncent) = cr_pos(3,katom) +offset(3)
	        natom(ncent)          = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = jatom
	        patom(1,natom(ncent),ncent) = cr_pos(1,jatom)
	        patom(2,natom(ncent),ncent) = cr_pos(2,jatom)
	        patom(3,natom(ncent),ncent) = cr_pos(3,jatom)
	        icell(1)=jcell(1)-chem_cwin(3,iv)+chem_cwin(7,iv)
	        icell(2)=jcell(2)-chem_cwin(4,iv)+chem_cwin(8,iv)
	        icell(3)=jcell(3)-chem_cwin(5,iv)+chem_cwin(9,iv)
	        call check_bound (icell,offset,chem_period,lok)
	        if (lok) then
	          isite = chem_cwin(6,iv)
	          call celltoindex (icell,isite,katom)
	          natom(ncent)   = natom(ncent) + 1
	          iatom(natom(ncent),ncent)   = katom
	          patom(1,natom(ncent),ncent) = cr_pos(1,katom) +offset(1)
	          patom(2,natom(ncent),ncent) = cr_pos(2,katom) +offset(2)
	          patom(3,natom(ncent),ncent) = cr_pos(3,katom) +offset(3)
	        endif
	      else
c------	--------The central atom is outside the (periodic) boundary
	        natom(ncent) = 0
	      endif
	    elseif (jsite.eq.chem_cwin(6,iv)) then
c
c------	------Selected atom is second neighbour of the central atom 
c
	      icell(1)=jcell(1)-chem_cwin(7,iv)
	      icell(2)=jcell(2)-chem_cwin(8,iv)
	      icell(3)=jcell(3)-chem_cwin(9,iv)
	      call check_bound (icell,offset,chem_period,lok)
	      if (lok) then
	        isite = chem_cwin(1,iv)
	        call celltoindex (icell,isite,katom)
	        iatom(0,ncent)   = katom
	        patom(1,0,ncent) = cr_pos(1,katom) +offset(1)
	        patom(2,0,ncent) = cr_pos(2,katom) +offset(2)
	        patom(3,0,ncent) = cr_pos(3,katom) +offset(3)
	        icell(1)=jcell(1)-chem_cwin(7,iv)+chem_cwin(3,iv)
	        icell(2)=jcell(2)-chem_cwin(8,iv)+chem_cwin(4,iv)
	        icell(3)=jcell(3)-chem_cwin(9,iv)+chem_cwin(5,iv)
	        call check_bound (icell,offset,chem_period,lok)
	        if (lok) then
	          isite = chem_cwin(2,iv)
	          call celltoindex (icell,isite,katom)
	          natom(ncent)          = natom(ncent) + 1
	          iatom(natom(ncent),ncent)   = katom
	          patom(1,natom(ncent),ncent) = cr_pos(1,katom) + offset(1)
	          patom(2,natom(ncent),ncent) = cr_pos(2,katom) + offset(2)
	          patom(3,natom(ncent),ncent) = cr_pos(3,katom) + offset(3)
	        endif
	        natom(ncent)          = natom(ncent) + 1
	        iatom(natom(ncent),ncent)   = jatom
	        patom(1,natom(ncent),ncent) = cr_pos(1,jatom)
	        patom(2,natom(ncent),ncent) = cr_pos(2,jatom)
	        patom(3,natom(ncent),ncent) = cr_pos(3,jatom)
	      else
c------	--------The central atom is outside the (periodic) boundary
	        natom(ncent) = 0
	      endif
	    endif
	    if(natom(ncent).eq.0) then
	      ncent = ncent - 1
	    endif
	  enddo
c
c------ ------------------------------------------------------
c------ Mode environment
c------ ------------------------------------------------------
c
	elseif (chem_ctyp(ic).eq.CHEM_ENVIR) then
	  ncent = 0
	  do j=1,3
	    u(j)=cr_pos(j,jatom)
	    w(j)=0.0
	  enddo
c
c------	--Loop over all environment definitions used by current neighbor definition
c
	  do i=1,chem_nenv(ic)
c
c------	----The selected atom is the central atom of environment definition
c
	    if(cr_iscat(jatom).eq.chem_cenv(0,chem_use_env(i,ic)) .or. 
     &	       -1             .eq.chem_cenv(0,chem_use_env(i,ic))) then
	      do j=1,chem_env_neig(chem_use_env(i,ic))
	        werte(j) = chem_cenv(j,chem_use_env(i,ic))
	      enddo
	      ianz = chem_env_neig(chem_use_env(i,ic))
	      call do_find_env(ianz,werte,maxw,u,chem_rmin_env(ic),
     &	                     chem_rmax_env(ic),
     &	                     chem_quick,chem_period)
	      if (atom_env(0).ne.0) then
c
c------	--------Neighbours were found, add to list of atoms
c
	        ncent = ncent + 1
	        do j=1,atom_env(0)
	          if (natom(ncent).lt.maxw) then
	            natom(ncent)          = natom(ncent) + 1
	            iatom(natom(ncent),ncent)   = atom_env(j)
	            patom(1,natom(ncent),ncent) = atom_pos(1,j)
	            patom(2,natom(ncent),ncent) = atom_pos(2,j)
	            patom(3,natom(ncent),ncent) = atom_pos(3,j)
	          else
	            natom(ncent) = 0
	            ncent        = ncent - 1
	            ier_num = -23
	            ier_typ = ER_CHEM
	            return
	          endif
	        enddo
	        iatom(0,ncent)   = jatom
	        patom(1,0,ncent) = cr_pos(1,jatom)
	        patom(2,0,ncent) = cr_pos(2,jatom)
	        patom(3,0,ncent) = cr_pos(3,jatom)
	      endif
	    else
c
c------	----The selected atom is a neighbour atom of environment definition
c
	      lis_neig = .false.
	      do j=1,chem_env_neig(chem_use_env(i,ic))
	        if(cr_iscat(jatom).eq.
     &	           chem_cenv(j,chem_use_env(i,ic))) then
	          lis_neig = .true.
	        endif
	      enddo
	      if(lis_neig) then
	        werte(1) = chem_cenv(0,chem_use_env(i,ic))
	        ianz     = 1
	        call do_find_env(ianz,werte,maxw,u,chem_rmin_env(ic),
     &	                       chem_rmax_env(ic),
     &	                       chem_quick,chem_period)
c
c-----	------- Store the true central atoms in patom
c
	        do j=1,atom_env(0)
	          iatom(0,ncent+j)   = atom_env(j)
	          patom(1,0,ncent+j) = atom_pos(1,j)
	          patom(2,0,ncent+j) = atom_pos(2,j)
	          patom(3,0,ncent+j) = atom_pos(3,j)
	        enddo
	        nnew = atom_env(0)
	        do k=1,nnew
	          do j=1,chem_env_neig(chem_use_env(i,ic))
	            werte(j) = chem_cenv(j,chem_use_env(i,ic))
	          enddo
	          do j=1,3
	            u(j) = patom(j,0,ncent+1)
	          enddo
	          ianz = chem_env_neig(chem_use_env(i,ic))
	          call do_find_env(ianz,werte,maxw,u,chem_rmin_env(ic),
     &	                         chem_rmax_env(ic),
     &	                         chem_quick,chem_period)
	          if (atom_env(0).ne.0) then
	            ncent = ncent + 1
	            do j=1,atom_env(0)
	              if (natom(ncent).lt.maxw) then
	                natom(ncent)          = natom(ncent) + 1
	                iatom(natom(ncent),ncent)   = atom_env(j)
	                patom(1,natom(ncent),ncent) = atom_pos(1,j)
	                patom(2,natom(ncent),ncent) = atom_pos(2,j)
	                patom(3,natom(ncent),ncent) = atom_pos(3,j)
	              else
	                natom(ncent) = 0
	                ncent = ncent - 1
	                ier_num = -23
	                ier_typ = ER_CHEM
	                return
	              endif
	            enddo
	          endif
	        enddo
	      endif
	    endif
	  enddo
	endif
CDBG
	if(ldbg) then
	write(*,*) 'CHEM_NEIGHBOUR_MULTI'
	write(*,*) 'cr_pos(j,jatom) ,typ',
     &             (cr_pos(j,jatom),j=1,3),cr_iscat(jatom)
	write(*,*) ' ncent          ',ncent
	write(*,*) 'ic              ',ic
	do icent = 1,ncent
	  if(natom(icent).gt.0) then
	  write(*,*) ' Central atom   ',(patom(i,0,icent),i=1,3)
	  write(*,*) 'CHEM_N natom    ',natom(icent)
	  write(*,*) 'iatom           ',(iatom(i,icent),i=0,natom(icent))
	  do j=0,natom(icent)
	    write(*,*) 'CHEM_N patom       ',(patom(i,j,icent),i=1,3)
	  enddo
	  else
	    write(*,*) ' No neighbours for definition ',icent
	  endif
	enddo
	endif
CDBG	dbg  = .false.
c
	end
c*****7*****************************************************************
	subroutine chem_neighbour_mol(jmol,ic,imol,nmol,maxww)
c+
c	Determine neighbours from given molecule index 'jmol'.
c-
	implicit	none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include		'molecule.inc'
	include 	'rmc.inc'
	include 	'chem.inc'
c
	integer       	maxww,maxw
	parameter    	(maxw=chem_max_neig)
c
	real		patom(3,0:maxw)
	integer		imol(0:maxww),jmol,nmol,ic
	integer		iatom(0:maxw),jatom,natom
	integer		i,im
	logical		no
c
	nmol = 0
c
c------ Get origin of selected molecule and find neighbours
c
	jatom = mole_cont(mole_off(jmol)+1)
	call chem_neighbour (jatom,ic,iatom,patom,natom,maxw)
c
c------ Convert found atoms to molecules
c
	if (natom.gt.0) then
	  do i=1,natom
	    no = .true.
	    im = 0
	    do while (no .and. im.lt.mole_num_mole)
	      im = im + 1
	      no = (iatom(i).ne.mole_cont(mole_off(im)+1))
	    enddo 
c
	    if (.not. no) then
	      nmol       = nmol + 1
	      if (nmol.le.chem_max_neig) then
	        imol(nmol) = im
	      else
	        ier_num = -23
	        ier_typ = ER_CHEM
	        return
	      endif
	    endif
	  enddo
	endif
c
	end
c*****7*****************************************************************
	subroutine chem_blen(iianz,jjanz,werte,wwerte,maxw)
c+
c	Calculate bond length distribution within crystal
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'atom_env.inc'
	include		'param.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw,iianz,jjanz
	real		werte(maxw),wwerte(maxw)
c
	integer		i,j,k,is,js,ibin
	integer		bl_anz(0:maxscat,0:maxscat)
	real		u(3)
	real		bl_min(0:maxscat,0:maxscat)
	real		bl_max(0:maxscat,0:maxscat)
	real		bl_sum(0:maxscat,0:maxscat)
	real		bl_s2 (0:maxscat,0:maxscat)
	real		bl_ave,bl_sig
c
	character*9	at_name_i
	character*9	at_name_j
	character*9	at_name
	logical		atom_allowed
c
c------ write output
c
	write (output_io,500) chem_blen_cut,chem_fname,chem_bin
c
c------ Reset arrays
c
	do i=0,cr_nscat
	  do j=0,cr_nscat
	    bl_sum(i,j) =     0.0
	    bl_s2 (i,j) =     0.0
	    bl_anz(i,j) =     0
	    bl_min(i,j) =  1000.0
	    bl_max(i,j) = -1000.0
	  enddo
	enddo
	do i=1,chem_max_bin
	  chem_hist(i)=0
	enddo
c
c------ Start the calculation
c
	do i=1,cr_natoms
	  if (atom_allowed(i,werte,iianz,maxw)) then
	    do j=1,3
	      u(j)=cr_pos(j,i)
	    enddo
	    call do_find_env (jjanz,wwerte,maxw,u,chem_blen_cut(1),
     &	                      chem_blen_cut(2),chem_quick,chem_period)
	    if (ier_num.ne.0) return
	    do k=1,atom_env(0)
	      is = cr_iscat(i)
	      js = cr_iscat(atom_env(k))
	      bl_sum(is,js) = bl_sum(is,js) + res_para(k)
	      bl_s2 (is,js) = bl_s2 (is,js) + res_para(k)**2
	      bl_anz(is,js) = bl_anz(is,js) + 1
	      bl_min(is,js) = min (bl_min(is,js),res_para(k))
	      bl_max(is,js) = max (bl_max(is,js),res_para(k))

       	      ibin = int ((res_para(k)-chem_blen_cut(1))*chem_bin/ 
     &	                 (chem_blen_cut(2)-chem_blen_cut(1)))+1
	      chem_hist(ibin)=chem_hist(ibin)+1
	    enddo
	  endif
	enddo
c
c------ write output
c
	do i=0,cr_nscat
	  do j=i,cr_nscat
	    if (bl_anz(i,j).ne.0 .or. bl_anz(j,i).ne.0 ) then
	      bl_ave=(bl_sum(i,j)+bl_sum(j,i))/(bl_anz(i,j)+bl_anz(j,i))
	      bl_sig=(bl_s2 (i,j)+bl_s2 (j,i))/(bl_anz(i,j)+bl_anz(j,i))
	      bl_sig=bl_sig-(bl_ave**2)
	      if (bl_sig.gt.0.0) then
	        bl_sig = sqrt(bl_sig)
	      else
	        bl_sig = 0.0
	      endif
	      at_name_i = at_name(i)
	      at_name_j = at_name(j)
	      write (output_io,1000) at_name_i ,at_name_j ,bl_ave,bl_sig,
     &	                          min(bl_min(i,j),bl_min(j,i)),
     &	                          max(bl_max(i,j),bl_max(j,i))
	    endif
	  enddo
	enddo
c
c------ write histogramm
c
	open (unit=43,file=chem_fname,status='unknown')
	do i=1,chem_bin
	  write (43,5000) chem_blen_cut(1) + 
     &	            (chem_blen_cut(2)-chem_blen_cut(1))*(i-1)/chem_bin,
     &	             chem_hist(i)
	enddo
	close (43)
c
 400	format (/,' Calculating bond-length distibution (Mode: QUICK)')
 410	format (/,' Calculating bond-length distibution (Mode: EXACT)')
 500	format (' Calculating bond-length distibution',/,
     &	        '    Allowed range : ',F5.2,' A to ',F5.2,
     &	        '  A / File : ',A12,' (',I4,' pts)',/)
1000	format ('    ',A9,'- ',A9,': d = ',F6.3,' +- ',F6.3,' A ',
     &	        '(Min = ',F6.3,', Max = ',F6.3,')')
5000	format (F8.3,I12)
	end
c*****7*****************************************************************
	subroutine chem_bang(iianz,jjanz,kkanz,werte,wwerte,uwerte,maxw)
c+
c	Calculate bond angle distribution within crystal
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'atom_env.inc'
	include		'param.inc'
	include		'errlist.inc'
	include 	'chem.inc'
c
	integer		maxw,iianz,jjanz,kkanz
	real		werte(maxw),wwerte(maxw),uwerte(maxw)
c
	integer		i,j,k,l,is,js,ibin
	integer		ba_anz(0:maxscat,0:maxscat)
	integer         ba_env(0:MAX_ATOM_ENV)
	logical         lspace
c
c	von der relativen Reihenfolge der beiden Statements haengt es ab,
c	ob der zweite Nachbar gefunden wird oder nicht !?!?
c
CDBG	real            ba_pos(3,MAX_ATOM_ENV)
CDBG	real		u(3),v(3),w(3),angle
	real		u(3),v(3),w(3),angle
	real            ba_pos(3,MAX_ATOM_ENV)

	real		ba_min(0:maxscat,0:maxscat)
	real		ba_max(0:maxscat,0:maxscat)
	real		ba_sum(0:maxscat,0:maxscat)
	real		ba_s2 (0:maxscat,0:maxscat)
	real		ba_ave,ba_sig
c
	character*9	at_name_i
	character*9	at_name_j
	character*9	at_name
	logical		atom_allowed
	real            do_bang
c
	lspace = .true.
c
c------ write output
c
	write (output_io,500) chem_bang_cut,chem_blen_cut,
     &	                      chem_fname,chem_bin
c
c------ Reset arrays
c
	do i=0,cr_nscat
	  do j=0,cr_nscat
	    ba_sum(i,j) =     0.0
	    ba_s2 (i,j) =     0.0
	    ba_anz(i,j) =     0
	    ba_min(i,j) =  1000.0
	    ba_max(i,j) = -1000.0
	  enddo
	enddo
	do i=1,chem_max_bin
	  chem_hist(i)=0
	enddo
c
c------ Start the calculation
c
	do i=1,cr_natoms
	  if (atom_allowed(i,werte,iianz,maxw)) then
	    do j=1,3
	      u(j)=cr_pos(j,i)
	    enddo
	    call do_find_env (jjanz,wwerte,maxw,u,chem_blen_cut(1),
     &	                      chem_blen_cut(2),chem_quick,chem_period)
	    if (ier_num.ne.0) return
c
c------	----copy environment of first atom type
c
	    do k=0,atom_env(0)
	      ba_env(k) = atom_env(k)
	      do j=1,3
	        ba_pos(j,k) = atom_pos(j,k)
	      enddo
	    enddo
	    call do_find_env (kkanz,uwerte,maxw,u,chem_blen_cut(1),
     &	                      chem_blen_cut(2),chem_quick,chem_period)
	    if (ier_num.ne.0) return
c
c------	----For all atoms in the environment, except atom i itself
c
	    do k=1,ba_env(0)
	      if(ba_env(k).ne.i) then
	        is = cr_iscat(ba_env(k))
	        do j=1,3
	          v(j)=ba_pos(j,k)
	        enddo
c
c------	--------For all second neighbor atoms in the environment, 
c	        except atom i and first neighbor atom 
c
	        do l=1,atom_env(0)
	          if(atom_env(l).ne.i .and. 
     &	             atom_env(l).ne.ba_env(k)) then
	            js = cr_iscat(atom_env(l))
	            do j=1,3
	              w(j)=atom_pos(j,l)
	            enddo
	            angle = do_bang(lspace,v,u,w)
	            ba_sum(is,js) = ba_sum(is,js) + angle
	            ba_s2 (is,js) = ba_s2 (is,js) + angle**2
	            ba_anz(is,js) = ba_anz(is,js) + 1
	            ba_min(is,js) = min (ba_min(is,js),angle)
	            ba_max(is,js) = max (ba_max(is,js),angle)

       	            ibin = int ((angle-chem_bang_cut(1))*chem_bin/ 
     &	                 (chem_bang_cut(2)-chem_bang_cut(1)))+1
	            chem_hist(ibin)=chem_hist(ibin)+1
	          endif
	        enddo
	      endif
	    enddo
	  endif
	enddo
c
c------ write output
c
	do i=0,cr_nscat
	  do j=i,cr_nscat
	    if (ba_anz(i,j).ne.0 .or. ba_anz(j,i).ne.0 ) then
	      ba_ave=(ba_sum(i,j)+ba_sum(j,i))/(ba_anz(i,j)+ba_anz(j,i))
	      ba_sig=(ba_s2 (i,j)+ba_s2 (j,i))/(ba_anz(i,j)+ba_anz(j,i))
	      ba_sig=ba_sig-(ba_ave**2)
	      if (ba_sig.gt.0.0) then
	        ba_sig = sqrt(ba_sig)
	      else
	        ba_sig = 0.0
	      endif
	      at_name_i = at_name(i)
	      at_name_j = at_name(j)
	      write (output_io,1000) at_name_i ,at_name_j ,ba_ave,ba_sig,
     &	                          min(ba_min(i,j),ba_min(j,i)),
     &	                          max(ba_max(i,j),ba_max(j,i))
	    endif
	  enddo
	enddo
c
c------ write histogramm
c
	open (unit=43,file=chem_fname,status='unknown')
	do i=1,chem_bin
	  write (43,5000) chem_bang_cut(1) + 
     &	            (chem_bang_cut(2)-chem_bang_cut(1))*(i-1)/chem_bin,
     &	             chem_hist(i)
	enddo
	close (43)
c
 400	format (/,' Calculating bond-angle distibution (Mode: QUICK)')
 410	format (/,' Calculating bond-angle distibution (Mode: EXACT)')
 500	format (' Calculating bond-angle distibution',/,
     &	        '    Allowed range : ',F6.2,'   to ',F6.2,' Degrees',/,
     &	        '    Allowed length: ',F6.2,' A to ',F6.2,
     &	        '  A / File : ',A12,' (',I4,' pts)',/)
1000	format ('    ',A9,'- ',A9,': a = ',F7.3,' +- ',F7.3,' Deg ',
     &	        '(Min = ',F7.3,', Max = ',F7.3,')')
5000	format (F8.3,I12)
	end
c*****7*****************************************************************
	subroutine chem_elem (lout)
c+
c	Show information about elements/rel. amounts within crystal
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include		'param.inc'
	include 	'chem.inc'
c
	real		proz
	integer		natom(0:MAXSCAT)
	integer		i
	logical		lout
c
	character*9	at_name_i
	character*9	at_name
c
c------ reset counters, ...
c
	do i=0,cr_nscat
	  natom(i)=0
	enddo
c
c------ get size of model crystal, rel. amount of elements
c
	do i=1,cr_natoms
	  natom(cr_iscat(i))=natom(cr_iscat(i))+1
	enddo
c
c------ write output
c
	if (lout) write (output_io,1000) (cr_icc(i),i=1,3)
	if (lout) write (output_io,1100) cr_natoms,cr_ncatoms,cr_nscat
	res_para(0)=float(cr_nscat)+1
	do i=0,cr_nscat
	  proz=float(natom(i))/cr_natoms
	  if (lout) then
	    at_name_i = at_name(i)
	    write (output_io,1200) at_name_i ,proz,natom(i)
	  endif
	  if (i.le.maxpar_res) then
	    res_para(i+1)=proz
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (' Size of the crystal (unit cells) : ',2(I4,' x '),I4)
1100	format (' Total number of atoms            : ',I6,/
     &	        ' Number of atoms per unit cell    : ',I6,/
     &	        ' Number of different atoms        : ',I6,/)
1200	format ('    Element : ',A9,' rel. abundance : ',F5.3,
     &	        '  (',I6,' atoms)')
	end
c*****7*****************************************************************
	subroutine chem_mole (lout)
c+
c	Show information about molecules/rel. amounts within crystal
c-
	implicit	none
c
	include		'config.inc'
	include		'prompt.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
	include		'errlist.inc'
	include		'param.inc'
	include		'molecule.inc'
	include 	'chem.inc'
c
	real		proz
	integer		nmole(0:MOLE_MAX_TYPE)
	integer		i
	logical		lout
c
c------ reset counters, ...
c
	do i=1,mole_num_type
	  nmole(i)=0
	enddo
c
c------ get relative amounts of molecule types
c
	do i=1,mole_num_mole
	  nmole(mole_type(i))=nmole(mole_type(i))+1
	enddo
c
c------ write output
c
	if (lout) then
	  write (output_io,1000) (cr_icc(i),i=1,3)
	  write (output_io,1100) mole_num_mole,mole_num_unit,
     &	                       mole_num_type
	endif
c
	res_para(0)=float(mole_num_type)
	do i=1,mole_num_type
	  proz=float(nmole(i))/mole_num_mole
	  if (lout) then
	    write (output_io,1200) i,proz,nmole(i)
	  endif
	  if (i.le.maxpar_res) then
	    res_para(i+1)=proz
	  else
	    ier_typ = ER_CHEM
	    ier_num = -2
	  endif
	enddo
c
1000	format (' Size of the crystal (unit cells)  : ',2(I4,' x '),I4)
1100	format (' Total number of molecules         : ',I6,/
     &	        ' Number of molecules per unit cell : ',I6,/
     &	        ' Number of different molecules     : ',I6,/)
1200	format ('    Mol. type : ',I4,5X,' rel. abundance : ',F5.3,
     &	        '  (',I6,' molecules)')
	end
c*****7***************************************************************
	character*9 function at_name(iscat)
c+
c	This function builds the atom name as XX(iscat) to be 
c	able to distinguish between different atom types with
c	the same name.
c-
	implicit      none
c
	include		'config.inc'
	include		'crystal.inc'
	include		'crystal.cmm'
c
	character*5	istr
	integer		iscat,il,is
c
	integer		len_str
c
	if (iscat.ge.100) then
	  write(istr,1000) iscat
	elseif (iscat.ge.10) then
	  write(istr,1100) iscat
	else 
	  write(istr,1200) iscat
	endif
c
	il=len_str(cr_at_lis(iscat))
	is=len_str(istr)
c
	at_name=cr_at_lis(iscat)(1:il)//istr(1:is)
c
1000	format ('(',I3,')')
1100	format ('(',I2,')')
1200	format ('(',I1,')')
	end
c*****7***************************************************************
	subroutine errlist_chem
c-
c	Displays error messages for the error type CHEM
c+
	implicit      none
c
	include      'errlist.inc'
c
	integer       iu,io
	parameter    (IU=-26,IO=0)
c
	character*41  ERROR(IU:IO)
c
	DATA ERROR/
     &  'Invalid correlation environment index given',      !-26
     &  'Invalid range for bond-angle histogramm    ',      !-25
     &  'Invalid correlation angle index given      ',      !-24
     &  'Too many neighbouring atoms/molecules      ',      !-23
     &  'Command not available in molecule mode     ',      !-22
     &  'Molecule types need to be different        ',      !-21
     &  'No molecules present in crystal            ',      !-20
     &  'No neighbouring molecules found            ',      !-19
     &  'Correlation fields require same # vectors  ',      !-18
     &  'Correlation fields require same mode       ',      !-17
     &  'Failed to apply periodic boundaries        ',      !-16
     &  'No displacement directions selected        ',      !-15
     &  'Invalid neighbour definition selected      ',      !-14
     &  'Correlation direction invalid              ',      !-13
     &  'Too many neighbour definitions             ',      !-12
     &  'No neighbours defined                      ',      !-11
     &  'Invalid crystal site or atom index given   ',      !-10
     &  'Invalid correlation vector index given     ',      ! -9
     &  'No neighbouring atoms found                ',      ! -8
     &  'Atoms need to be different                 ',      ! -7
     &  'Atom name ALL not allowed for this command ',      ! -6
     &  'Too many different atoms found             ',      ! -5
     &  'Invalid SIGMA entered                      ',      ! -4
     &  'Invalid range for bond-length histogramm   ',      ! -3
     &  'Not enough space for all results in res[]  ',      ! -2 
     &  'Too many points for histogramm             ',      ! -1 
     &  ' '       !  0
     &	/
c
	call disp_error ('CHEM',error,iu,io)
	end
C*****7****************************************************************
